@@ -42,6 +42,7 @@
 #include "dbread.h"
 #include "boxplots.h"
 #include "gretl_www.h"
+#include "gretl_scalar.h"
 
 #ifdef WIN32
 # include <windows.h>
@@ -73,11 +74,11 @@ int data_status;
 char linebak[MAXLINE];      /* for storing comments */
 char *line_read;
 
-static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo);
+static int exec_line (ExecState *s, DATASET *dset);
 static int push_input_file (FILE *fp);
 static FILE *pop_input_file (void);
-static int saved_object_action (const char *line, double ***pZ,
-				DATAINFO *pdinfo, MODEL **models,
+static int saved_object_action (const char *line, 
+				DATASET *dset, 
 				PRN *prn);
 
 static void usage (int err)
@@ -98,6 +99,10 @@ static void usage (int err)
 	     "Example of run mode usage:\n"
 	     " gretlcli -r myfile.inp\n"));
 
+    printf("\nSpecial batch-mode option:\n"
+	   " --scriptopt=<value> sets a scalar value, accessible to a script\n"
+	   " under the name \"scriptopt\"\n\n");
+
     if (err) {
 	exit(EXIT_FAILURE);
     } else {
@@ -159,48 +164,45 @@ static void nls_init (void)
     build_path(LOCALEDIR, gretl_home(), "locale", NULL);
 # endif /* WIN32 */
 
-    setlocale (LC_ALL, "");
-    bindtextdomain (PACKAGE, LOCALEDIR);
-    textdomain (PACKAGE); 
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE); 
     iso_gettext("@CLI_INIT");
 
-    putenv("LC_NUMERIC=");
+    gretl_setenv("LC_NUMERIC", "");
     setlocale(LC_NUMERIC, "");
     reset_local_decpoint();
 #endif /* ENABLE_NLS */
 }
 
-static int cli_clear_data (CMD *cmd, double ***pZ, DATAINFO *pdinfo,
-			   MODEL **models)
+static int cli_clear_data (CMD *cmd, DATASET *dset, MODEL *model)
 {
     gretlopt clearopt = 0;
     int err = 0;
 
     if (cmd->ci == CLEAR) {
 	clearopt = cmd->opt;
-    } else {
-	if (cmd->opt & OPT_P) {
-	    /* --preserve: clear dataset only */
-	    clearopt = OPT_D;
-	}
+    } else if (cmd->opt & OPT_P) {
+	/* --preserve: clear dataset only */
+	clearopt = OPT_D;
     }
 
     if (!(clearopt & OPT_O)) {
-	/* OPT_O means "other", i.e. don't scrub the dataset */
+	/* OPT_O means "other", i.e. leave the dataset 
+	   in place but clear all other stuff
+	*/
 	*datafile = 0;
 
-	if (pZ != NULL && *pZ != NULL) {
-	    err = restore_full_sample(pZ, pdinfo, NULL); 
-	    free_Z(*pZ, pdinfo); 
-	    *pZ = NULL;
+	if (dset->Z != NULL) {
+	    err = restore_full_sample(dset, NULL); 
+	    free_Z(dset); 
 	}
 
-	clear_datainfo(pdinfo, CLEAR_FULL);
+	clear_datainfo(dset, CLEAR_FULL);
 	data_status = 0;
     }
 
-    clear_model(models[0]);
-    clear_model(models[1]);
+    clear_model(model);
 
     if (clearopt & OPT_D) {
 	libgretl_session_cleanup(SESSION_CLEAR_DATASET);
@@ -300,13 +302,17 @@ static int maybe_get_input_line_continuation (char *line)
 	*tmp = '\0';
 
 	if (batch || runit) {
-	    fgets(tmp, MAXLINE - 1, fb);
+	    char *test = fgets(tmp, MAXLINE, fb);
+
+	    if (test == NULL) {
+		break;
+	    }
 	} else {
 #ifdef HAVE_READLINE
 	    rl_gets(&line_read, "> ");
 	    strcpy(tmp, line_read);
 #else
-	    fgets(tmp, MAXLINE - 1, stdin); 
+	    fgets(tmp, MAXLINE, stdin); 
 #endif /* HAVE_READLINE */
 	}
 
@@ -342,15 +348,15 @@ int main (int argc, char *argv[])
 #ifdef WIN32
     char *callname = argv[0];
 #endif
-    double **Z = NULL;
-    DATAINFO *datainfo = NULL;
-    MODEL **models = NULL;
+    DATASET *dset = NULL;
+    MODEL *model = NULL;
     ExecState state;
     char *line = NULL;
     int quiet = 0;
     int load_datafile = 1;
     char filearg[MAXLEN];
     char runfile[MAXLEN];
+    double scriptval = NADBL;
     CMD cmd;
     PRN *prn;
     int err = 0;
@@ -365,8 +371,8 @@ int main (int argc, char *argv[])
     rl_bind_key(0x18, ctrl_x);
 #endif
 
-    datainfo = datainfo_new();
-    if (datainfo == NULL) {
+    dset = datainfo_new();
+    if (dset == NULL) {
 	noalloc();
     }
 
@@ -375,7 +381,7 @@ int main (int argc, char *argv[])
     } else {
 	gretlopt opt;
 
-	err = parseopt(&argc, &argv, &opt, filearg);
+	err = parseopt(&argc, &argv, &opt, &scriptval, filearg);
 
 	if (!err && (opt & (OPT_DBOPEN | OPT_WEBDB))) {
 	    /* catch GUI-only options */
@@ -481,19 +487,19 @@ int main (int argc, char *argv[])
 	    exit(EXIT_FAILURE);
 	    break;
 	case GRETL_NATIVE_DATA:
-	    err = gretl_get_data(datafile, &Z, datainfo, OPT_NONE, prn);
+	    err = gretl_get_data(datafile, dset, OPT_NONE, prn);
 	    break;
 	case GRETL_XML_DATA:
-	    err = gretl_read_gdt(datafile, &Z, datainfo, OPT_NONE, prn);
+	    err = gretl_read_gdt(datafile, dset, OPT_NONE, prn);
 	    break;
 	case GRETL_CSV:
-	    err = import_csv(datafile, &Z, datainfo, OPT_NONE, prn);
+	    err = import_csv(datafile, dset, OPT_NONE, prn);
 	    break;
 	case GRETL_XLS:
 	case GRETL_GNUMERIC:
 	case GRETL_ODS:
 	    err = import_spreadsheet(datafile, ftype, NULL, NULL,
-				     &Z, datainfo, OPT_NONE, prn);
+				     dset, OPT_NONE, prn);
 	    break;
 	case GRETL_DTA:
 	case GRETL_SAV:
@@ -501,7 +507,7 @@ int main (int argc, char *argv[])
 	case GRETL_JMULTI:
 	case GRETL_OCTAVE:
 	case GRETL_WF1:
-	    err = import_other(datafile, ftype, &Z, datainfo, 
+	    err = import_other(datafile, ftype, dset, 
 			       OPT_NONE, prn);
 	    break;
 	case GRETL_SCRIPT:
@@ -529,21 +535,24 @@ int main (int argc, char *argv[])
 	}
     }
 
-    /* allocate memory for models */
-    models = allocate_working_models(2);
-    if (models == NULL) {
+    /* allocate memory for model */
+    model = allocate_working_model();
+    if (model == NULL) {
 	noalloc(); 
     }
 
     gretl_cmd_init(&cmd);
-    gretl_exec_state_init(&state, 0, line, &cmd, models, prn);
+    gretl_exec_state_init(&state, 0, line, &cmd, model, prn);
     set_debug_read_func(get_interactive_line);
 
     /* print list of variables */
     if (data_status) {
-	varlist(datainfo, prn);
+	varlist(dset, prn);
     }
 
+    /* define "scriptopt" */
+    gretl_scalar_add("scriptopt", scriptval);
+
     /* check for help file */
     if (!batch) {
 	const char *hpath = helpfile_path(GRETL_HELPFILE);
@@ -579,7 +588,7 @@ int main (int argc, char *argv[])
 	    set_gretl_echo(0);
 	} 
 	sprintf(line, "run %s\n", runfile);
-	err = exec_line(&state, &Z, datainfo);
+	err = exec_line(&state, dset);
     }
 
     /* main command loop */
@@ -591,7 +600,7 @@ int main (int argc, char *argv[])
 	}
 
 	if (gretl_execute_loop()) { 
-	    if (gretl_loop_exec(&state, &Z, datainfo)) {
+	    if (gretl_loop_exec(&state, dset)) {
 		return 1;
 	    }
 	} else {
@@ -612,7 +621,7 @@ int main (int argc, char *argv[])
 
 	strcpy(linecopy, line);
 	tailstrip(linecopy);
-	err = exec_line(&state, &Z, datainfo);
+	err = exec_line(&state, dset);
     } /* end of get commands loop */
 
     if (!err) {
@@ -625,13 +634,13 @@ int main (int argc, char *argv[])
     if (makepkg && !err) {
 	switch_ext(filearg, runfile, "gfn");
 	sprintf(line, "makepkg %s\n", filearg);
-	exec_line(&state, &Z, datainfo);
+	exec_line(&state, dset);
     }
 
     /* leak check -- try explicitly freeing all memory allocated */
 
-    destroy_working_models(models, 2);
-    destroy_dataset(Z, datainfo);
+    destroy_working_model(model);
+    destroy_dataset(dset);
 
     if (fb != stdin && fb != NULL) {
 	fclose(fb);
@@ -667,12 +676,12 @@ static void cli_exec_callback (ExecState *s, void *ptr,
     /* otherwise, no-op */
 }
 
-static int cli_renumber_series (const char *s, double **Z, 
-				DATAINFO *pdinfo, PRN *prn)
+static int cli_renumber_series (const char *s, DATASET *dset, 
+				PRN *prn)
 {
-    int err, fixmax = highest_numbered_var_in_saved_object(pdinfo);
+    int err, fixmax = highest_numbered_var_in_saved_object(dset);
 
-    err = renumber_series_with_checks(s, fixmax, Z, pdinfo, prn);
+    err = renumber_series_with_checks(s, fixmax, dset, prn);
     if (err) {
 	errmsg(err, prn);
     }
@@ -698,8 +707,8 @@ static int cli_try_http (const char *s, char *fname, int *http)
     return err;
 }
 
-static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
-			    DATAINFO *pdinfo, MODEL **models,
+static int cli_open_append (CMD *cmd, const char *line, 
+			    DATASET *dset, MODEL *model,
 			    PRN *prn)
 {
     char newfile[MAXLEN] = {0};
@@ -748,24 +757,24 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     }
 
     if (!dbdata && cmd->ci != APPEND) {
-	cli_clear_data(cmd, pZ, pdinfo, models);
+	cli_clear_data(cmd, dset, model);
     } 
 
     if (ftype == GRETL_CSV) {
-	err = import_csv(newfile, pZ, pdinfo, cmd->opt, prn);
+	err = import_csv(newfile, dset, cmd->opt, prn);
     } else if (SPREADSHEET_IMPORT(ftype)) {
 	err = import_spreadsheet(newfile, ftype, cmd->list, cmd->extra,
-				 pZ, pdinfo, cmd->opt, prn);
+				 dset, cmd->opt, prn);
     } else if (OTHER_IMPORT(ftype)) {
-	err = import_other(newfile, ftype, pZ, pdinfo, cmd->opt, prn);
+	err = import_other(newfile, ftype, dset, cmd->opt, prn);
     } else if (ftype == GRETL_XML_DATA) {
-	err = gretl_read_gdt(newfile, pZ, pdinfo, cmd->opt, prn);
+	err = gretl_read_gdt(newfile, dset, cmd->opt, prn);
     } else if (ftype == GRETL_ODBC) {
 	err = set_odbc_dsn(line, prn);
     } else if (dbdata) {
 	err = set_db_name(newfile, ftype, prn);
     } else {
-	err = gretl_get_data(newfile, pZ, pdinfo, cmd->opt, prn);
+	err = gretl_get_data(newfile, dset, cmd->opt, prn);
     }
 
     if (err) {
@@ -779,8 +788,8 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 
     data_status = 1;
 
-    if (pdinfo->v > 0 && !dbdata && !(cmd->opt & OPT_Q)) {
-	varlist(pdinfo, prn);
+    if (dset->v > 0 && !dbdata && !(cmd->opt & OPT_Q)) {
+	varlist(dset, prn);
     }
 
     if (http) {
@@ -800,16 +809,16 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
    see also gui_exec_line() in gui2/library.c
 */
 
-static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
+static int exec_line (ExecState *s, DATASET *dset)
 {
     char *line = s->line;
     CMD *cmd = s->cmd;
     PRN *prn = s->prn;
-    MODEL **models = s->models;
+    MODEL *model = s->model;
     int old_runit = runit;
     unsigned char eflag;
     char runfile[MAXLEN];
-    int k, err = 0;
+    int err = 0;
 
     if (string_is_blank(line)) {
 	return 0;
@@ -828,11 +837,12 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     if (!s->in_comment && !cmd->context) {
 	/* catch requests relating to saved objects, which are not
 	   really "commands" as such */
-	k = saved_object_action(line, pZ, pdinfo, models, prn);
-	if (k == 1) {
-	    return 0; /* action was OK, or ignored */
-	} else if (k == -1) {
+	int action = saved_object_action(line, dset, prn);
+
+	if (action == OBJ_ACTION_INVALID) {
 	    return 1; /* action was faulty */
+	} else if (action != OBJ_ACTION_NONE) {
+	    return 0; /* action was OK (and handled), or ignored */
 	}
     }
 
@@ -843,7 +853,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	/* if we're stacking commands for a loop, parse "lightly" */
 	err = get_command_index(line, cmd);
     } else {
-	err = parse_command_line(line, cmd, pZ, pdinfo);
+	err = parse_command_line(line, cmd, dset);
     }
 
     if (err) {
@@ -889,14 +899,14 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	if (gretl_echo_on() && (!gretl_compiling_loop() || batch || runit)) {
 	    eflag = gretl_compiling_loop()? CMD_STACKING : CMD_BATCH_MODE;
 	    /* straight visual echo */
-	    echo_cmd(cmd, pdinfo, line, eflag, prn);
+	    echo_cmd(cmd, dset, line, eflag, prn);
 	}
-	err = gretl_loop_append_line(s, pZ, pdinfo);
+	err = gretl_loop_append_line(s, dset);
 	if (err) {
 	    errmsg(err, prn);
 	} else if (!batch && !runit) {
 	    /* echo to record */
-	    echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
+	    echo_cmd(cmd, dset, line, CMD_RECORDING, cmdprn);
 	}
 	return err;
     }
@@ -907,7 +917,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    ; /* don't echo */
 	} else {
 	    eflag = (batch || runit)? CMD_BATCH_MODE : CMD_CLI;
-	    echo_cmd(cmd, pdinfo, line, eflag, prn);
+	    echo_cmd(cmd, dset, line, eflag, prn);
 	}
     }
 
@@ -918,7 +928,6 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     switch (cmd->ci) {
 
     case DELEET:
-	k = 0;
 	if (cmd->opt & OPT_D) {
 	    err = db_delete_series_by_name(cmd->param, prn);
 	} else if (*cmd->param != '\0') {
@@ -926,16 +935,20 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	} else if (get_list_by_name(cmd->extra)) {
 	    err = delete_list_by_name(cmd->extra);
 	} else {
-	    err = dataset_drop_listed_variables(cmd->list, pZ, pdinfo, 
-					    &k, prn);
+	    int nv = 0;
+
+	    err = dataset_drop_listed_variables(cmd->list, dset, 
+						&nv, prn);
+	    if (!err && nv > 0) {
+		pputs(prn, _("Take note: variables have been renumbered"));
+		pputc(prn, '\n');
+		maybe_list_vars(dset, prn);
+	    }
+		
 	}
 	if (err) {
 	    errmsg(err, prn);
-	} else if (k) {
-	    pputs(prn, _("Take note: variables have been renumbered"));
-	    pputc(prn, '\n');
-	    maybe_list_vars(pdinfo, prn);
-	}
+	} 
 	break;
 
     case HELP:
@@ -944,24 +957,21 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     case OPEN:
     case APPEND:
-	err = cli_open_append(cmd, line, pZ, pdinfo, models, prn);
+	err = cli_open_append(cmd, line, dset, model, prn);
 	break;
 
     case NULLDATA:
-	k = gretl_int_from_string(cmd->param, &err);
-	if (!err && k < 2) {
+	if (cmd->order < 1) {
 	    err = 1;
-	}
-	if (err) {
 	    pputs(prn, _("Data series length count missing or invalid\n"));
-	    break;
-	}
-	cli_clear_data(cmd, pZ, pdinfo, models);
-	err = open_nulldata(pZ, pdinfo, data_status, k, prn);
-	if (err) { 
-	    errmsg(err, prn);
 	} else {
-	    data_status = 1;
+	    cli_clear_data(cmd, dset, model);
+	    err = open_nulldata(dset, data_status, cmd->order, prn);
+	    if (err) { 
+		errmsg(err, prn);
+	    } else {
+		data_status = 1;
+	    }
 	}
 	break;
 
@@ -1058,28 +1068,28 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	break;
 
     case CLEAR:
-	err = cli_clear_data(cmd, pZ, pdinfo, models);
+	err = cli_clear_data(cmd, dset, model);
 	break;
 
     case DATAMOD:
 	if (cmd->aux == DS_CLEAR) {
-	    err = cli_clear_data(cmd, pZ, pdinfo, models);
+	    err = cli_clear_data(cmd, dset, model);
 	    pputs(prn, _("Dataset cleared\n"));
 	    break;
 	} else if (cmd->aux == DS_RENUMBER) {
-	    err = cli_renumber_series(cmd->param, *pZ, pdinfo, prn);
+	    err = cli_renumber_series(cmd->param, dset, prn);
 	    break;
 	}
 	/* else fall-through intended */
 
     default:
-	err = gretl_cmd_exec(s, pZ, pdinfo);
+	err = gretl_cmd_exec(s, dset);
 	break;
     }
 
     if (!err && cmd->ci != QUIT && gretl_echo_on() && !batch && !old_runit) {
 	/* record a successful interactive command */
-	echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
+	echo_cmd(cmd, dset, line, CMD_RECORDING, cmdprn);
     }
 
     if (err) {
