@@ -38,7 +38,7 @@ static void doubleclick_action (windata_t *vwin)
 {
     switch (vwin->role) {
     case MAINWIN:
-	if (datainfo != NULL && datainfo->n > 0) {
+	if (dataset != NULL && dataset->n > 0) {
 	    display_var();
 	}
 	break;
@@ -221,18 +221,16 @@ void file_save (windata_t *vwin, int ci)
     case SAVE_SCRIPT:
     case SAVE_GP_CMDS:
     case SAVE_R_CMDS:
-	file_selector(ci, FSEL_DATA_VWIN, vwin);
-	break;
     case SAVE_DATA:
     case SAVE_DATA_AS:
-    case SAVE_DBDATA:
-    case SAVE_FUNCTIONS:	
+	file_selector(ci, FSEL_DATA_VWIN, vwin);
+	break;
     case EXPORT_CSV:
-    case EXPORT_R:
-    case EXPORT_OCTAVE:
-    case EXPORT_DAT:
-    case EXPORT_JM:
-	data_save_selection_wrapper(ci);
+    case EXPORT:
+	data_export_selection_wrapper(ci);
+	break;
+    case SAVE_FUNCTIONS:
+	functions_selection_wrapper();
 	break;
     case SAVE_TEX:
     case SAVE_TEXT:
@@ -245,20 +243,10 @@ void file_save (windata_t *vwin, int ci)
 
 static int fsave_code (const gchar *s)
 {
-    if (!strcmp(s, "SaveAsGdt"))
+    if (!strcmp(s, "SaveDataAs"))
 	return SAVE_DATA_AS;
-    if (!strcmp(s, "SaveAsDb"))
-	return SAVE_DBDATA;
-    if (!strcmp(s, "ExportCSV"))
-	return EXPORT_CSV;
-    if (!strcmp(s, "ExportR"))
-	return EXPORT_R;
-    if (!strcmp(s, "ExportOctave"))
-	return EXPORT_OCTAVE;
-    if (!strcmp(s, "ExportPcGive"))
-	return EXPORT_DAT;
-    if (!strcmp(s, "ExportJMulTi"))
-	return EXPORT_JM;
+    if (!strcmp(s, "ExportData"))
+	return EXPORT;
     if (!strcmp(s, "NewGfn"))
 	return SAVE_FUNCTIONS;
 
@@ -411,6 +399,8 @@ static int selector_callback_code (const gchar *s)
 	return SCATTERS;
     if (!strcmp(s, "MultiTS"))
 	return TSPLOTS;
+    if (!strcmp(s, "GR_FBOX"))
+	return GR_FBOX;
     if (!strcmp(s, "VLAGSEL"))
 	return VLAGSEL;
     if (!strcmp(s, "ConfEllipse"))
@@ -440,9 +430,9 @@ void selector_callback (GtkAction *action, gpointer data)
 			 do_vector_model, ci);
     } else if (ci == VLAGSEL) {
 	selection_dialog(_("gretl: VAR lag selection"), do_vector_model, ci);
-    } else if (ci == GR_XY || ci == GR_IMP || ci == GR_DUMMY
-	       || ci == SCATTERS || ci == GR_3D
-	       || ci == GR_XYZ) {
+    } else if (ci == GR_XY || ci == GR_IMP || ci == GR_DUMMY ||
+	       ci == SCATTERS || ci == GR_3D || ci == GR_XYZ ||
+	       ci == GR_FBOX) {
 	int (*selfunc)() = NULL;
 
 	switch (ci) {
@@ -462,6 +452,9 @@ void selector_callback (GtkAction *action, gpointer data)
 	case SCATTERS:
 	    selfunc = do_scatters;
 	    break;
+	case GR_FBOX:
+	    selfunc = do_factorized_boxplot;
+	    break;
 	default:
 	    return;
 	}
@@ -501,8 +494,6 @@ static int gretl_callback_code (const gchar *s)
 	return GSETMISS;
     if (!strcmp(s, "GR_BOX")) 
 	return GR_BOX;
-    if (!strcmp(s, "GR_NBOX")) 
-	return GR_NBOX;
     if (!strcmp(s, "gmm")) 
 	return GMM;
     if (!strcmp(s, "mle")) 
@@ -536,7 +527,7 @@ void gretl_callback (GtkAction *action, gpointer data)
     switch (cmd) {
     case GENR:
 	title = N_("gretl: add var");
-	if (datainfo->n > 5000) {
+	if (dataset->n > 5000) {
 	    query = N_("Enter formula for new variable");
 	} else {
 	    query = N_("Enter formula for new variable\n"
@@ -549,7 +540,7 @@ void gretl_callback (GtkAction *action, gpointer data)
 	title = N_("gretl: missing code");
 	dynquery = g_strdup_printf(_("Enter value to be read as \"missing\"\n"
 				     "for the variable \"%s\""), 
-				   datainfo->varname[mdata->active_var]);
+				   dataset->varname[mdata->active_var]);
 	okfunc = do_variable_setmiss;
 	break;
     case GSETMISS:
@@ -558,7 +549,6 @@ void gretl_callback (GtkAction *action, gpointer data)
 	okfunc = do_global_setmiss;
 	break;
     case GR_BOX:
-    case GR_NBOX:
 	title = N_("gretl: boxplots");
 	query = N_("Specify variables to plot:");
 	okfunc = do_box_graph;
@@ -619,6 +609,29 @@ void gretl_callback (GtkAction *action, gpointer data)
     }
 }
 
+void menu_boxplot_callback (int varnum)
+{
+    const char *opts[] = {
+	N_("Simple boxplot"),
+	N_("With confidence interval for median"),
+	N_("Factorized")
+    };
+    int ret;
+
+    ret = radio_dialog(_("gretl: define graph"), NULL, 
+		       opts, 3, 0, 0);
+
+    if (ret == 0) {
+	do_boxplot_var(varnum, OPT_NONE);
+    } else if (ret == 1) {
+	do_boxplot_var(varnum, OPT_O);
+    } else if (ret == 2) {
+	selector_set_varnum(varnum);
+	selection_dialog(_("gretl: define graph"), 
+			 do_factorized_boxplot, GR_FBOX);
+    }
+}
+
 void revise_nl_model (MODEL *pmod)
 {
     const char *title = NULL;
@@ -670,7 +683,7 @@ void genr_callback (void)
 {
     const char *msg;
 
-    if (datainfo->n > 5000) {
+    if (dataset->n > 5000) {
 	msg = N_("Enter formula for new variable");
     } else {
 	msg = N_("Enter formula for new variable\n"
@@ -707,7 +720,7 @@ void newdata_callback (void)
 	return;
     }
 
-    if (open_nulldata(&Z, datainfo, data_status, n, NULL)) {
+    if (open_nulldata(dataset, data_status, n, NULL)) {
 	errbox(_("Failed to create empty data set"));
 	return;
     }
