@@ -25,6 +25,7 @@
 #include "datafiles.h"
 #include "gretl_www.h"
 #include "gretl_untar.h"
+#include "gretl_xml.h"
 #include "menustate.h"
 #include "treeutils.h"
 #include "winstack.h"
@@ -103,10 +104,10 @@ static void update_statusline (windata_t *vwin, const char *s)
     g_free(tmp);
 }
 
-static void set_time_series (DATAINFO *pdinfo)
+static void set_time_series (DATASET *dset)
 {
-    if (pdinfo->pd != 1 || strcmp(pdinfo->stobs, "1")) { 
-	pdinfo->structure = TIME_SERIES;
+    if (dset->pd != 1 || strcmp(dset->stobs, "1")) { 
+	dset->structure = TIME_SERIES;
     }
 }
 
@@ -158,7 +159,7 @@ static int gui_get_remote_db_data (windata_t *vwin, SERIESINFO *sinfo,
     return err;
 }
 
-static void display_dbdata (const double **dbZ, DATAINFO *dbinfo)
+static void display_dbdata (DATASET *dbset)
 {
     PRN *prn;
     int width = 36;
@@ -167,31 +168,30 @@ static void display_dbdata (const double **dbZ, DATAINFO *dbinfo)
 	return;
     }
 
-    if (dbinfo->v > 1) {
+    if (dbset->v > 1) {
 	width = 72;
     }
 
-    printdata(NULL, NULL, dbZ, dbinfo, OPT_O, prn);
+    printdata(NULL, NULL, dbset, OPT_O, prn);
     view_buffer(prn, width, 350, _("gretl: display database series"), PRINT,
 		NULL); 
 }
 
-static void graph_dbdata (const double **dbZ, DATAINFO *dbinfo)
+static void graph_dbdata (DATASET *dbset)
 {
     int *list;
     int err;
 
-    list = gretl_consecutive_list_new(1, dbinfo->v - 1);
+    list = gretl_consecutive_list_new(1, dbset->v - 1);
     if (list == NULL) {
 	nomem();
 	return;
     }
 
-    if (dbinfo->structure == CROSS_SECTION) {
-	err = boxplots(list, dbZ, dbinfo, OPT_NONE);
+    if (dbset->structure == CROSS_SECTION) {
+	err = boxplots(list, dbset, OPT_NONE);
     } else {
-	err = gnuplot(list, NULL, dbZ, dbinfo,
-		      OPT_G | OPT_O | OPT_T);
+	err = gnuplot(list, NULL, dbset, OPT_G | OPT_O | OPT_T);
     }
 
     if (err) {
@@ -243,7 +243,7 @@ static int expand_data_dialog (int src_pd, int targ_pd, int *interpol)
 
 static int obs_overlap_check (SERIESINFO *sinfo)
 {
-    int err = db_range_check(sinfo, datainfo);
+    int err = db_range_check(sinfo, dataset);
 
     if (err) {
 	gui_errmsg(err);
@@ -256,13 +256,13 @@ static int pd_convert_check (SERIESINFO *sinfo)
 { 
     int err = 0;
 
-    if (sinfo->pd < datainfo->pd) {
+    if (sinfo->pd < dataset->pd) {
 	if (sinfo->pd != 1 && sinfo->pd != 4 && 
-	    datainfo->pd != 4 && datainfo->pd != 12) {
+	    dataset->pd != 4 && dataset->pd != 12) {
 	    err = 1;
 	} 
-    } else if (sinfo->pd > datainfo->pd) {
-	if (datainfo->pd != 1 && datainfo->pd != 4 && sinfo->pd != 12) {
+    } else if (sinfo->pd > dataset->pd) {
+	if (dataset->pd != 1 && dataset->pd != 4 && sinfo->pd != 12) {
 	    err = 1;
 	}
     }
@@ -287,28 +287,50 @@ static const char *compact_method_string (CompactMethod method)
     }
 }
 
+static const char *trimmed_db_name (const char *fname)
+{
+    const char *s = fname;
+
+    if (strstr(fname, gretl_binbase()) != NULL) {
+	s = strrchr(fname, SLASH);
+	if (s != NULL) {
+	    return s + 1;
+	}
+    }
+
+    return NULL;
+}
+
 static void record_db_open_command (dbwrapper *dw)
 {
     if (dw->fname != NULL) {
+	int quotes = (strchr(dw->fname, ' ') != NULL);
 	gchar *cmdline;
 
 	if (dw->dbtype == GRETL_PCGIVE_DB) {
-	    cmdline = g_strdup_printf("open %s.bn7", dw->fname);
+	    if (quotes) {
+		cmdline = g_strdup_printf("open \"%s.bn7\"", dw->fname);
+	    } else {
+		cmdline = g_strdup_printf("open %s.bn7", dw->fname);
+	    }
 	} else if (dw->dbtype == GRETL_NATIVE_DB) {
-	    const char *tmp = gretl_binbase();
-	    int n = strlen(tmp);
+	    const char *tmp = trimmed_db_name(dw->fname);
 
-	    if (!strncmp(dw->fname, tmp, n)) {
-		tmp = dw->fname + n;
-		if (*tmp == SLASH) tmp++;
+	    if (tmp != NULL) {
 		cmdline = g_strdup_printf("open %s.bin", tmp);
+	    } else if (quotes) {
+		cmdline = g_strdup_printf("open \"%s.bin\"", dw->fname);
 	    } else {
 		cmdline = g_strdup_printf("open %s.bin", dw->fname);
 	    }
 	} else if (dw->dbtype == GRETL_NATIVE_DB_WWW) {
 	    cmdline = g_strdup_printf("open %s --www", dw->fname);
 	} else {
-	    cmdline = g_strdup_printf("open %s", dw->fname);
+	    if (quotes) {
+		cmdline = g_strdup_printf("open \"%s\"", dw->fname);
+	    } else {
+		cmdline = g_strdup_printf("open %s", dw->fname);
+	    }
 	}
 
 	record_command_verbatim(cmdline);
@@ -347,8 +369,8 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	    continue;
 	}
 
-	if (sinfo->pd < datainfo->pd && !warned) {
-	    resp = expand_data_dialog(sinfo->pd, datainfo->pd, &interpol);
+	if (sinfo->pd < dataset->pd && !warned) {
+	    resp = expand_data_dialog(sinfo->pd, dataset->pd, &interpol);
 	    if (resp != GRETL_YES) {
 		return 0;
 	    }
@@ -356,8 +378,8 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	}
 
 	/* is there already a var of this name? */
-	dbv = series_index(datainfo, sinfo->varname);
-	if (dbv < datainfo->v) {
+	dbv = series_index(dataset, sinfo->varname);
+	if (dbv < dataset->v) {
 	    if (dw->nv == 1) {
 		resp = yes_no_dialog ("gretl",                      
 				      _("There is already a variable of this name\n"
@@ -368,34 +390,34 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	    }
 	    overwrite = 1;
 	    /* pick up on pre-registered compaction method? */
-	    if (COMPACT_METHOD(datainfo, dbv) != COMPACT_NONE) {
-		method = COMPACT_METHOD(datainfo, dbv);
+	    if (COMPACT_METHOD(dataset, dbv) != COMPACT_NONE) {
+		method = COMPACT_METHOD(dataset, dbv);
 	    }
 	}
 
-	if (!overwrite && dataset_add_series(1, &Z, datainfo)) {
+	if (!overwrite && dataset_add_series(1, dataset)) {
 	    nomem();
 	    return 1;
 	}
 
-	if (sinfo->pd < datainfo->pd) {
+	if (sinfo->pd < dataset->pd) {
 	    /* the series needs to be expanded */
-	    xvec = expand_db_series(dbZ[v], sinfo, datainfo->pd, interpol);
-	} else if (sinfo->pd > datainfo->pd) {
+	    xvec = expand_db_series(dbZ[v], sinfo, dataset->pd, interpol);
+	} else if (sinfo->pd > dataset->pd) {
 	    /* the series needs to be compacted */
 	    compact = 1;
 	    if (!chosen) {
-		data_compact_dialog(vwin->main, sinfo->pd, &datainfo->pd, NULL, 
+		data_compact_dialog(vwin->main, sinfo->pd, &dataset->pd, NULL, 
 				    &method, NULL);
 		if (method == COMPACT_NONE) {
 		    if (!overwrite) {
-			dataset_drop_last_variables(1, &Z, datainfo);
+			dataset_drop_last_variables(1, dataset);
 		    }
 		    return 0;
 		}
 		chosen = 1;
 	    }
-	    xvec = compact_db_series(dbZ[v], sinfo, datainfo->pd, 
+	    xvec = compact_db_series(dbZ[v], sinfo, dataset->pd, 
 				     method);
 	} else {  
 	    /* the frequency does not need adjustment */
@@ -410,7 +432,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	if (xvec == NULL) {
 	    nomem();
 	    if (!overwrite) {
-		dataset_drop_last_variables(1, &Z, datainfo);
+		dataset_drop_last_variables(1, dataset);
 	    }
 	    return 1;
 	}
@@ -428,14 +450,14 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	g_free(cmdline);
 
 	/* common stuff for adding a var */
-	strcpy(datainfo->varname[dbv], sinfo->varname);
-	strcpy(VARLABEL(datainfo, dbv), sinfo->descrip);
-	get_db_padding(sinfo, datainfo, &pad1, &pad2);
+	strcpy(dataset->varname[dbv], sinfo->varname);
+	strcpy(VARLABEL(dataset, dbv), sinfo->descrip);
+	get_db_padding(sinfo, dataset, &pad1, &pad2);
 
 	if (pad1 > 0) {
 	    fprintf(stderr, "Padding at start, %d obs\n", pad1);
 	    for (t=0; t<pad1; t++) {
-		Z[dbv][t] = NADBL;
+		dataset->Z[dbv][t] = NADBL;
 	    }
 	    start = pad1;
 	} else {
@@ -443,22 +465,22 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	}
 
 	if (pad2 > 0) {
-	    int n = datainfo->n;
+	    int n = dataset->n;
 
 	    fprintf(stderr, "Padding at end, %d obs\n", pad2);
 	    for (t=n-1; t>=n-1-pad2; t--) {
-		Z[dbv][t] = NADBL;
+		dataset->Z[dbv][t] = NADBL;
 	    }
 	    stop = n - pad2;
 	} else {
-	    stop = datainfo->n;
+	    stop = dataset->n;
 	}
 
 	/* fill in actual data values */
 	fprintf(stderr, "Filling in values from %d to %d\n", start, stop - 1);
 	for (t=start; t<stop; t++) {
 	    x = xvec[t - pad1];
-	    Z[dbv][t] = (x == DBNA)? NADBL : x;
+	    dataset->Z[dbv][t] = (x == DBNA)? NADBL : x;
 	}
 
 	free(xvec);
@@ -467,24 +489,21 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
     return 0;
 }
 
-static void 
-add_dbdata (windata_t *vwin, double **dbZ, DATAINFO *dbinfo,
-	    dbwrapper *dw, int *freeit)
+static void add_dbdata (windata_t *vwin, DATASET *dbset,
+			dbwrapper *dw, int *freeit)
 {
     SERIESINFO *sinfo;
-    int i, t, err = 0;
+    int i, err = 0;
 
     if (data_status) { 
 	/* we already have data in gretl's workspace */
-	add_db_series_to_dataset(vwin, dbZ, dw);
+	add_db_series_to_dataset(vwin, dbset->Z, dw);
     } else {  
 	/* no data open: start new data set from db */
 	gchar *cmdline;
 
-	destroy_dataset(Z, datainfo);
-
-	Z = dbZ;
-	datainfo = dbinfo;
+	destroy_dataset(dataset);
+	dataset = dbset;
 	*freeit = 0;
 
 	record_db_open_command(dw);
@@ -492,12 +511,8 @@ add_dbdata (windata_t *vwin, double **dbZ, DATAINFO *dbinfo,
 	for (i=1; i<=dw->nv && !err; i++) {
 	    sinfo = &dw->sinfo[i-1];
 
-	    for (t=0; t<datainfo->n; t++) {
-		Z[i][t] = dbZ[i][t];
-	    }
-	    
-	    strcpy(datainfo->varname[i], sinfo->varname);
-	    strcpy(VARLABEL(datainfo, i), sinfo->descrip);
+	    strcpy(dataset->varname[i], sinfo->varname);
+	    strcpy(VARLABEL(dataset, i), sinfo->descrip);
 	    
 	    cmdline = g_strdup_printf("data %s", sinfo->varname);
 	    record_command_verbatim(cmdline);
@@ -608,10 +623,9 @@ static int diffdate (double d1, double d0, int pd)
     return x;
 }
 
-static DATAINFO *new_dataset_from_dbwrapper (dbwrapper *dw,
-					     double ***pZ)
+static DATASET *new_dataset_from_dbwrapper (dbwrapper *dw)
 {
-    DATAINFO *dinfo = NULL;
+    DATASET *dset = NULL;
     SERIESINFO *sinfo;
     char stobs[OBSLEN], endobs[OBSLEN];
     double xd, xdmax = 0, xdmin = NADBL;
@@ -663,29 +677,28 @@ static DATAINFO *new_dataset_from_dbwrapper (dbwrapper *dw,
     fprintf(stderr, "min(sd0) = %g, stobs='%s', n = %d\n", xdmin, 
 	    stobs, nmax);
 
-    dinfo = create_new_dataset(pZ, dw->nv + 1, nmax, 0);
+    dset = create_new_dataset(dw->nv + 1, nmax, 0);
     
-    if (dinfo != NULL) {
-	dinfo->pd = dw->sinfo[0].pd;
+    if (dset != NULL) {
+	dset->pd = dw->sinfo[0].pd;
 
-	strcpy(dinfo->stobs, stobs);
-	strcpy(dinfo->endobs, endobs);
+	strcpy(dset->stobs, stobs);
+	strcpy(dset->endobs, endobs);
 
-	colonize_obs(dinfo->stobs);
-	colonize_obs(dinfo->endobs);
+	colonize_obs(dset->stobs);
+	colonize_obs(dset->endobs);
 
-	dinfo->sd0 = xdmin;
-	set_time_series(dinfo);
+	dset->sd0 = xdmin;
+	set_time_series(dset);
     }
 
-    return dinfo;
+    return dset;
 }
 
 static void gui_get_db_series (windata_t *vwin, int cmd)
 {
     int dbcode = vwin->role;
-    DATAINFO *dbinfo = NULL;
-    double **dbZ = NULL;
+    DATASET *dbset = NULL;
     dbwrapper *dw;
     int freeit = 1;
     int i, err = 0;
@@ -695,8 +708,8 @@ static void gui_get_db_series (windata_t *vwin, int cmd)
 	return;
     }
 
-    dbinfo = new_dataset_from_dbwrapper(dw, &dbZ);
-    if (dbinfo == NULL) {
+    dbset = new_dataset_from_dbwrapper(dw);
+    if (dbset == NULL) {
 	dbwrapper_destroy(dw);
 	nomem();
 	return;
@@ -706,13 +719,13 @@ static void gui_get_db_series (windata_t *vwin, int cmd)
 	SERIESINFO *sinfo = &dw->sinfo[i];
 
 	if (dbcode == NATIVE_SERIES) { 
-	    err = get_native_db_data(vwin->fname, sinfo, dbZ);
+	    err = get_native_db_data(vwin->fname, sinfo, dbset->Z);
 	} else if (dbcode == REMOTE_SERIES) {
-	    err = gui_get_remote_db_data(vwin, sinfo, dbZ);
+	    err = gui_get_remote_db_data(vwin, sinfo, dbset->Z);
 	} else if (dbcode == RATS_SERIES) {
-	    err = get_rats_db_data(vwin->fname, sinfo, dbZ);
+	    err = get_rats_db_data(vwin->fname, sinfo, dbset->Z);
 	} else if (dbcode == PCGIVE_SERIES) {
-	    err = get_pcgive_db_data(vwin->fname, sinfo, dbZ);
+	    err = get_pcgive_db_data(vwin->fname, sinfo, dbset->Z);
 	}
 
 	if (cmd == DB_IMPORT && err == DB_MISSING_DATA) {
@@ -724,23 +737,22 @@ static void gui_get_db_series (windata_t *vwin, int cmd)
 	    goto bailout;
 	} 
 
-	strcpy(dbinfo->varname[i+1], sinfo->varname);
-	strcpy(VARLABEL(dbinfo, i+1), sinfo->descrip);
+	strcpy(dbset->varname[i+1], sinfo->varname);
+	strcpy(VARLABEL(dbset, i+1), sinfo->descrip);
     }
 
     if (cmd == DB_DISPLAY) {
-	display_dbdata((const double **) dbZ, dbinfo);
+	display_dbdata(dbset);
     } else if (cmd == DB_GRAPH) {
-	graph_dbdata((const double **) dbZ, dbinfo);
+	graph_dbdata(dbset);
     } else if (cmd == DB_IMPORT) { 
-	add_dbdata(vwin, dbZ, dbinfo, dw, &freeit);
+	add_dbdata(vwin, dbset, dw, &freeit);
     }
 
  bailout:
 
     if (freeit) {
-	free_Z(dbZ, dbinfo);
-	free_datainfo(dbinfo);
+	destroy_dataset(dbset);
     }
 
     dbwrapper_destroy(dw);
@@ -1879,6 +1891,36 @@ static void offer_db_open (char *target)
     }
 }
 
+#ifdef OSX_BUILD
+
+static int get_target_in_home (char *targ, int code, 
+			       const char *objname,
+			       const char *ext)
+{
+    const char *suppdir = gretl_app_support_dir();
+    int err = 0;
+
+    fprintf(stderr, "suppdir='%s'\n", suppdir);
+
+    if (suppdir == NULL || *suppdir == '\0') {
+	err = E_FOPEN;
+    } else {
+	if (code == REMOTE_FUNC_FILES) {
+	    sprintf(targ, "%sfunctions/%s%s", suppdir, objname, ext);
+	} else if (code == REMOTE_DB) {
+	    sprintf(targ, "%sdb/%s%s", suppdir, objname, ext);
+	} else if (code == REMOTE_DATA_PKGS) {
+	    sprintf(targ, "%sdata/%s%s", suppdir, objname, ext);
+	} else {
+	    sprintf(targ, "%s%s%s", suppdir, objname, ext);
+	}
+    }
+
+    return err;
+}
+
+#endif
+
 enum {
     REAL_INSTALL,
     TMP_INSTALL
@@ -1893,10 +1935,11 @@ static char *get_writable_target (int code, int op, char *objname,
 				  gboolean zipfile)
 {
     const char *ext;
-    FILE *fp;
     char *targ;
     int err = 0;
 
+    fprintf(stderr, "get_writable_target\n");
+
     targ = mymalloc(MAXLEN);
     if (targ == NULL) {
 	return NULL;
@@ -1910,14 +1953,27 @@ static char *get_writable_target (int code, int op, char *objname,
 	ext = ".tar.gz";
     }
 
+#ifdef OSX_BUILD
+    /* we prefer writing to ~/Library/Application Support
+       rather than /Applications/Gretl.app 
+    */
+    if (op != TMP_INSTALL) {
+	err = get_target_in_home(targ, code, objname, ext);
+	goto path_done;
+    }
+#endif
+
     if (code == REMOTE_DB) {
-	build_path(targ, gretl_binbase(), objname, ext);
+	char dbdir[MAXLEN];
+
+	build_path(dbdir, gretl_home(), "db", NULL);
+	build_path(targ, dbdir, objname, ext);
     } else if (code == REMOTE_DATA_PKGS) {
 	char pkgdir[MAXLEN];
 
 	get_default_dir(pkgdir, SAVE_DATA_PKG);
 	build_path(targ, pkgdir, objname, ext);
-    } else {
+    } else if (code == REMOTE_FUNC_FILES) {
 	if (op == TMP_INSTALL) {
 	    build_path(targ, gretl_dotdir(), "dltmp", NULL);
 	    err = gretl_tempname(targ);
@@ -1929,16 +1985,22 @@ static char *get_writable_target (int code, int op, char *objname,
 	}
     } 
 
+#ifdef OSX_BUILD
+ path_done:
+#endif
+
     if (err) {
 	free(targ);
 	targ = NULL;
     } else {
+	FILE *fp;
+
 	errno = 0;
 	fp = gretl_fopen(targ, "w");
 
 	if (fp == NULL) {
-	    if (errno == EACCES && code != REMOTE_FUNC_FILES) { 
-		/* write to user dir instead? */
+	    if (errno == EACCES) { 
+		/* write to working dir instead? */
 		build_path(targ, gretl_workdir(), objname, ext);
 	    } else {
 		file_write_errbox(targ);
@@ -2081,8 +2143,11 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 	    err = retrieve_remote_function_package(objname, path);
 	}
     } else if (vwin->role == REMOTE_DATA_PKGS) {
-	err = retrieve_remote_datafiles_package(objname, path);
-    } else {
+	gchar *tarname = g_strdup_printf("%s.tar.gz", objname);
+
+	err = retrieve_remote_datafiles_package(tarname, path);
+	g_free(tarname);
+    } else if (vwin->role == REMOTE_DB) {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 	err = retrieve_remote_db(objname, path, GRAB_NBO_DATA);
 #else
@@ -2265,23 +2330,23 @@ int write_db_description (const char *binname, const char *descrip)
 }
 
 static int
-read_db_files_in_dir (DIR *dir, int dbtype, const char *dbdir, 
+read_db_files_in_dir (DIR *dir, int dbtype, const char *path, 
 		      GtkListStore *store, GtkTreeIter *iter)
 {
     struct dirent *dirent;
     const char *fname;
     char *descrip;
-    int n, ndb = 0;
+    int len, ndb = 0;
 
     while ((dirent = readdir(dir)) != NULL) {
 	fname = dirent->d_name;
-	n = strlen(fname);
-	if (!g_ascii_strcasecmp(fname + n - 4, ".bin")) {
-	    descrip = real_get_db_description(NULL, fname, dbdir);
+	len = strlen(fname);
+	if (!g_ascii_strcasecmp(fname + len - 4, ".bin")) {
+	    descrip = real_get_db_description(NULL, fname, path);
 	    if (descrip != NULL) {
 		gtk_list_store_append(store, iter);
 		gtk_list_store_set(store, iter, 0, fname, 1, descrip, 
-				   2, dbdir, -1);
+				   2, path, -1);
 		g_free(descrip);
 		ndb++;
 	    }
@@ -2295,62 +2360,41 @@ static void get_local_object_status (const char *fname, int role,
 				     const char **status, 
 				     time_t remtime)
 {
-    char filedir[MAXLEN];
     char fullname[MAXLEN];
     struct stat fbuf;
-    int err;
+    char **dirs;
+    int type, found = 0;
+    int i, n_dirs;
+    int err = 0;
 
     if (role == REMOTE_DB) {
-	build_path(fullname, gretl_binbase(), fname, NULL);
+	type = DB_SEARCH;
     } else if (role == REMOTE_DATA_PKGS) {
-	build_path(filedir, gretl_home(), "data", NULL);
-	build_path(fullname, filedir, fname, NULL);
+	type = DATA_SEARCH;
+    } else if (role == REMOTE_FUNC_FILES) {
+	type = FUNCS_SEARCH;
     } else {
-	build_path(filedir, gretl_home(), "functions", NULL);
-	build_path(fullname, filedir, fname, NULL);
+	*status = N_("Unknown: access error");
+	return;
     }
 
-    if ((err = gretl_stat(fullname, &fbuf)) == -1) {
-	if (errno == ENOENT) {
-#ifdef G_OS_WIN32 /* FIXME? */
-	    *status = N_("Not installed");
-#else
-	    /* try user dir too, if not on Windows */
-	    if (role == REMOTE_DB) {
-		build_path(fullname, gretl_workdir(), fname, NULL);
-	    } else if (role == REMOTE_DATA_PKGS) {
-		build_path(fullname, gretl_workdir(), fname, NULL);
-	    } else {
-		build_path(filedir, gretl_workdir(), "functions", NULL);
-		build_path(fullname, filedir, fname, NULL);
-	    }
-	    err = gretl_stat(fullname, &fbuf);
-	    if (err == -1) {
-		if (role == REMOTE_FUNC_FILES || role == REMOTE_DATA_PKGS) {
-		    /* try default working dir */
-		    const char *dw = gretl_default_workdir();
+    dirs = get_plausible_search_dirs(type, &n_dirs);
 
-		    if (dw != NULL) {
-			if (role == REMOTE_FUNC_FILES) {
-			    build_path(filedir, dw, "functions", NULL);
-			    build_path(fullname, filedir, fname, NULL);
-			} else {
-			    build_path(fullname, dw, fname, NULL);
-			}
-			err = stat(fullname, &fbuf);
-		    }
-		} 
-		if (err == -1) {
-		    *status = N_("Not installed");
-		}
-	    } 
-#endif
-	} else {
-	    *status = N_("Unknown: access error");
-	}
+    for (i=0; i<n_dirs; i++) {
+	build_path(fullname, dirs[i], fname, NULL);
+	errno = 0;
+	if (gretl_stat(fullname, &fbuf) == 0) {
+	    found = 1;
+	    break;
+	} else if (errno != ENOENT) {
+	    err = 1;
+	    break;
+	} 
     }
 
-    if (!err) {
+    free_strings_array(dirs, n_dirs);
+
+    if (found) {
 	double dt;
 
 	dt = difftime(remtime, fbuf.st_ctime);
@@ -2362,6 +2406,10 @@ static void get_local_object_status (const char *fname, int role,
 	} else {
 	    *status = N_("Up to date");
 	}
+    } else if (err) {
+	*status = N_("Unknown: access error");
+    } else {
+	*status = N_("Not installed");
     }
 }
 
@@ -2621,6 +2669,116 @@ gint populate_remote_db_list (windata_t *vwin)
     return err;
 }
 
+static int get_addon_info (xmlNodePtr node, xmlDocPtr doc, char **S)
+{
+    xmlNodePtr cur = node->xmlChildrenNode;
+    int err = 0;
+
+    while (cur != NULL) {
+	if (!xmlStrcmp(cur->name, (XUC) "version")) {
+	    gretl_xml_node_get_trimmed_string(cur, doc, &S[1]);
+	} else if (!xmlStrcmp(cur->name, (XUC) "date")) {
+	    gretl_xml_node_get_trimmed_string(cur, doc, &S[2]);
+	} else if (!xmlStrcmp(cur->name, (XUC) "description")) {
+	    gretl_xml_node_get_trimmed_string(cur, doc, &S[4]);
+	}
+
+	cur = cur->next;
+    }
+
+    if (S[1] == NULL || S[2] == NULL || S[4] == NULL) {
+	err = E_DATA;
+    } else {
+	char *path = gretl_function_package_get_path(S[0], PKG_SUBDIR);
+
+	if (path != NULL) {
+	    S[3] = gretl_strdup(_("Installed"));
+	    free(path);
+	} else {
+	    S[3] = gretl_strdup(_("Not installed"));
+	}
+    }
+
+    return err;
+}
+
+gint populate_remote_addons_list (windata_t *vwin)
+{
+    xmlDocPtr doc;
+    xmlNodePtr node = NULL;
+    char *getbuf = NULL;
+    int n = 0, err = 0;
+
+    err = query_sourceforge("/addons-data/addons.xml", &getbuf);
+    if (err) {
+	show_network_error(NULL);
+	free(getbuf);
+	return err;
+    }
+
+    xmlKeepBlanksDefault(0);
+
+    doc = xmlParseMemory(getbuf, strlen(getbuf));
+    if (doc == NULL) {
+	err = E_DATA;
+    } else {
+	node = xmlDocGetRootElement(doc);
+	if (node == NULL || xmlStrcmp(node->name, (XUC) "gretl-addons")) {
+	    err = E_DATA;
+	}
+    }
+
+    if (!err) {
+	GtkListStore *store;
+	GtkTreeIter iter;
+	char *S[5] = {0};
+	int i;
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model 
+			       (GTK_TREE_VIEW(vwin->listbox)));
+	gtk_list_store_clear(store);
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	node = node->xmlChildrenNode;
+
+	while (node != NULL && !err) {
+	    if (!xmlStrcmp(node->name, (XUC) "gretl-addon")) {
+		gretl_xml_get_prop_as_string(node, "name", &S[0]);
+		if (S[0] != NULL && 
+		    (err = get_addon_info(node, doc, S)) == 0) {
+		    gtk_list_store_append(store, &iter);
+		    gtk_list_store_set(store, &iter, 
+				       0, S[0], 1, S[1],
+				       2, S[2], 3, S[3], 
+				       4, S[4], -1);
+		    for (i=0; i<5; i++) {
+			free(S[i]);
+			S[i] = NULL;
+		    }
+		    n++;
+		} else {
+		    err = E_DATA;
+		}
+	    } 
+	    node = node->next;
+	}
+    }
+
+    if (err) {
+	gui_errmsg(err);
+    } else if (n == 0) {
+	warnbox(_("No function packages found"));
+	err = 1;
+    }
+
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+    }       
+    
+    free(getbuf);
+
+    return err;
+}
+
 /* Fill a list box with names, version numbers, and short descriptions
    of function packages, retrieved from server.
 */
@@ -2794,85 +2952,42 @@ gint populate_remote_data_pkg_list (windata_t *vwin)
     return err;
 }
 
-static DIR *open_gretl_db_dir (const char *dbdir)
-{
-    DIR *dir = NULL;
-    int tries = 0;
-
-    while (dir == NULL) {
-#ifdef G_OS_WIN32 
-	dir = win32_opendir(dbdir);
-#else
-	dir = opendir(dbdir);
-#endif
-	if (dir == NULL) {
-	    errbox(_("Can't open folder %s"), dbdir);
-	    if (++tries == 2 || options_dialog(TAB_DBS, "binbase", mdata->main)) {
-		/* canceled */
-		return NULL;
-	    }
-	}
-    }
-
-    return dir;
-}
-
 gint populate_dbfilelist (windata_t *vwin)
 {
     GtkListStore *store;
     GtkTreeIter iter;
-    const char *dbdir;
-    DIR *dir = NULL;
-    int ndb = 0;
+    char **dirnames;
+    DIR *dir;
+    int i, n_dirs, ndb = 0;
+    int err = 0;
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
     gtk_list_store_clear(store);
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
-    /* pick up any databases in the shared gretl dir */
-    dbdir = gretl_binbase();
-    dir = open_gretl_db_dir(dbdir);
-    if (dir != NULL) {
-	ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
-	closedir(dir);
-    }
+    dirnames = get_plausible_search_dirs(DB_SEARCH, &n_dirs);
 
-    /* pick up any databases in the user's personal dir */
-    dbdir = gretl_workdir();
-#ifdef G_OS_WIN32 
-    dir = win32_opendir(dbdir);
-#else
-    dir = opendir(dbdir);
-#endif
-    if (dir != NULL) {
-	ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
-	closedir(dir);
-    }
-
-    /* and in the default working dir? */
-    dbdir = gretl_default_workdir();
-    if (dbdir != NULL) {
-#ifdef G_OS_WIN32 
-	dir = win32_opendir(dbdir);
-#else
-	dir = opendir(dbdir);
-#endif
+    for (i=0; i<n_dirs; i++) {
+	dir = gretl_opendir(dirnames[i]);
 	if (dir != NULL) {
-	    ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
+	    ndb += read_db_files_in_dir(dir, vwin->role, dirnames[i], store, &iter);
 	    closedir(dir);
 	}
-    }
+    }  
+
+    free_strings_array(dirnames, n_dirs);
 
     if (ndb == 0) {
 	errbox(_("No database files found"));
+	err = 1;
     }
 
-    return (ndb == 0);
+    return err;
 }
 
 void set_db_dir_callback (windata_t *vwin, char *path)
 {
-    DIR *dir = opendir(path);
+    DIR *dir = gretl_opendir(path);
     int ndb = 0;
 
     if (dir != NULL) {
@@ -2880,9 +2995,9 @@ void set_db_dir_callback (windata_t *vwin, char *path)
 	GtkTreeIter iter;
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
-
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 	ndb = read_db_files_in_dir(dir, vwin->role, path, store, &iter);
+	closedir(dir);
     }
 
     if (ndb == 0) {
@@ -2894,9 +3009,9 @@ static void set_compact_info_from_default (int method)
 {
     int i;
 
-    for (i=1; i<datainfo->v; i++) {
-	if (COMPACT_METHOD(datainfo, i) == COMPACT_NONE) {
-	    COMPACT_METHOD(datainfo, i) = method;
+    for (i=1; i<dataset->v; i++) {
+	if (COMPACT_METHOD(dataset, i) == COMPACT_NONE) {
+	    COMPACT_METHOD(dataset, i) = method;
 	}
     }
 }
@@ -2912,11 +3027,11 @@ void do_compact_data_set (void)
 	return;
     }
 
-    if (dated_seven_day_data(datainfo)) {
+    if (dated_seven_day_data(dataset)) {
 	pmonstart = &monstart;
     }
 
-    data_compact_dialog(mdata->main, datainfo->pd, &newpd, pmonstart, 
+    data_compact_dialog(mdata->main, dataset->pd, &newpd, pmonstart, 
 			&method, &repday);
 
     if (method == COMPACT_NONE) {
@@ -2924,7 +3039,7 @@ void do_compact_data_set (void)
 	return;
     }
 
-    err = compact_data_set(&Z, datainfo, newpd, method, monstart, repday);
+    err = compact_data_set(dataset, newpd, method, monstart, repday);
 
     if (err) {
 	gui_errmsg(err);
@@ -2943,16 +3058,16 @@ void do_expand_data_set (void)
     }
 
     /* supported: annual to quarterly or quarterly to monthly */
-    newpd = (datainfo->pd == 1)? 4 : 12;
+    newpd = (dataset->pd == 1)? 4 : 12;
 
-    data_expand_dialog(mdata->main, datainfo->pd, &interpol);
+    data_expand_dialog(mdata->main, dataset->pd, &interpol);
 
     if (interpol < 0) {
 	/* canceled */
 	return;
     }
 
-    err = expand_data_set(&Z, datainfo, newpd, interpol);
+    err = expand_data_set(dataset, newpd, interpol);
 
     if (err) {
 	gui_errmsg(err);
@@ -2960,4 +3075,3 @@ void do_expand_data_set (void)
 	mark_dataset_as_modified();
     }
 }
-
