@@ -97,7 +97,8 @@ struct fpkg_response {
 
 #define REMOTE_ACTION(c) (c == REMOTE_DB || \
                           c == REMOTE_FUNC_FILES || \
-                          c == REMOTE_DATA_PKGS)
+                          c == REMOTE_DATA_PKGS || \
+                          c == REMOTE_ADDONS)
 
 static void
 read_fn_files_in_dir (DIR *dir, const char *path, 
@@ -288,27 +289,21 @@ static int compare_colls (const void *a, const void *b)
 {
     const file_collection *ca = *(const file_collection **) a;
     const file_collection *cb = *(const file_collection **) b;
-     
-    return strcmp(ca->title, cb->title);
+
+    if (!strcmp(ca->title, "Gretl")) {
+	return -1;
+    } else if (!strcmp(cb->title, "Gretl")) {
+	return 1;
+    } else {
+	return strcmp(ca->title, cb->title);
+    }
 }
 
 static void collection_stack_sort (file_collection **colls, int n)
 {
-    file_collection *tmp;
-    int i;
-
-    for (i=0; i<n; i++) {
-	if (!strcmp(colls[i]->title, "Gretl")) {
-	    if (i > 0) {
-		tmp = colls[0];
-		colls[0] = colls[i];
-		colls[i] = tmp;
-	    }
-	    break;
-	}
+    if (n >= 2) {
+	qsort(colls, n, sizeof *colls, compare_colls);
     }
-
-    qsort(colls + 1, n - 1, sizeof *colls, compare_colls);
 }
 
 static file_collection *collection_stack (file_collection *coll, int op)
@@ -387,6 +382,7 @@ static file_collection *pop_ps_collection (void)
 void destroy_file_collections (void)
 {
     collection_stack(NULL, STACK_DESTROY);
+    maybe_add_packages_to_model_menus(NULL);
 }
 
 static void reset_data_stack (void)
@@ -446,28 +442,26 @@ static int dont_go_there (const char *s)
     return ret;
 }
 
-static int seek_file_collections (int location)
+static int seek_file_collections (const char *basedir, int type)
 {
     char *tmp = NULL;
-    DIR *dir, *try;  
-    struct dirent *dirent;
     char *subdir;
-    int i = 0, err = 0;
+    DIR *dir, *try;      
+    struct dirent *dirent;
+    int err = 0;
 
-    if (location == DATA_SEARCH) {
-	tmp = gretl_strdup_printf("%sdata", gretl_home());
-    } else if (location == SCRIPT_SEARCH) {
-	tmp = gretl_strdup_printf("%sscripts", gretl_home());
-    } else if (location == USER_SEARCH) {
-	tmp = gretl_strdup(gretl_workdir());
+    if (type == DATA_SEARCH) {
+	tmp = gretl_strdup_printf("%sdata", basedir);
+    } else if (type == SCRIPT_SEARCH) {
+	tmp = gretl_strdup_printf("%sscripts", basedir);
+    } else if (type == USER_SEARCH) {
+	tmp = gretl_strdup(basedir);
 	trim_slash(tmp);
     } else {
 	return 1;
     }
 
- user_search_2:
-
-    dir = opendir(tmp);
+    dir = gretl_opendir(tmp);
     if (dir == NULL) {
 	free(tmp);
 	return 1;
@@ -484,7 +478,7 @@ static int seek_file_collections (int location)
 	    } else {
 		subdir = tmp;
 	    }
-	    try = opendir(subdir);
+	    try = gretl_opendir(subdir);
 	    if (try != NULL) {
 #if COLL_DEBUG
 		fprintf(stderr, " trying in subdir '%s'\n", subdir);
@@ -501,13 +495,6 @@ static int seek_file_collections (int location)
     closedir(dir);
     free(tmp);
 
-    if (location == USER_SEARCH && i++ == 0) {
-	tmp = gretl_strdup(gretl_default_workdir());
-	if (tmp != NULL) {
-	    goto user_search_2;
-	}
-    }
-
     return err;
 }
 
@@ -551,12 +538,27 @@ static int build_file_collections (void)
     static int err;
 
     if (!built) {
-	err = seek_file_collections(DATA_SEARCH);
+	err = seek_file_collections(gretl_home(), DATA_SEARCH);
 	if (!err) {
-	    err = seek_file_collections(SCRIPT_SEARCH);
+	    err = seek_file_collections(gretl_home(), SCRIPT_SEARCH);
+	}
+#ifdef OSX_BUILD
+	if (!err) {
+	    err = seek_file_collections(gretl_app_support_dir(), DATA_SEARCH);
 	}
 	if (!err) {
-	    err = seek_file_collections(USER_SEARCH);
+	    err = seek_file_collections(gretl_app_support_dir(), SCRIPT_SEARCH);
+	}	
+#endif
+	if (!err) {
+	    err = seek_file_collections(gretl_workdir(), USER_SEARCH);
+	}
+	if (!err) {
+	    const char *wd = maybe_get_default_workdir();
+
+	    if (wd != NULL) {
+		err = seek_file_collections(wd, USER_SEARCH);
+	    }
 	}
 	if (!err) {
 	    sort_data_stack();
@@ -839,9 +841,24 @@ windata_t *display_function_package_data (const char *pkgname,
     return vwin;
 }
 
+/* on adding files to gfn view window: ensure that the GTK
+   selection stays in sync with the "active_var" ID
+*/
+
+static void fix_selected_row (GtkTreeModel *model,
+			      GtkTreePath *path,
+			      GtkTreeIter *iter,
+			      gpointer data)
+{
+    gint idx = gtk_tree_path_get_indices(path)[0];
+    windata_t *vwin = data;
+
+    vwin->active_var = idx;
+}
+
 void set_funcs_dir_callback (windata_t *vwin, char *path)
 {
-    DIR *dir = opendir(path);
+    DIR *dir = gretl_opendir(path);
 
     if (dir != NULL) {
 	const char *opts[] = {
@@ -885,6 +902,10 @@ void set_funcs_dir_callback (windata_t *vwin, char *path)
 	read_fn_files_in_dir(dir, path, store, &iter, &nfn, &maxlen);
 
 	if (nfn > nfn0) {
+	    GtkTreeSelection *sel;
+
+	    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(vwin->listbox));
+	    gtk_tree_selection_selected_foreach(sel, fix_selected_row, vwin);
 	    infobox(_("Found %d function file(s)"), nfn - nfn0);
 	    g_object_set_data(G_OBJECT(vwin->listbox), "nfn", GINT_TO_POINTER(nfn));
 	    if (resp > 0) {
@@ -926,7 +947,8 @@ static void browser_functions_handler (windata_t *vwin, int task)
     }
 
 #if 1
-    fprintf(stderr, "browser_functions_handler: path='%s'\n", path);
+    fprintf(stderr, "browser_functions_handler: active=%d, pkgname='%s'\n"
+	    "path='%s'\n", vwin->active_var, pkgname, path);
 #endif
 
     if (task == LOAD_FN_PKG) {
@@ -940,8 +962,13 @@ static void browser_functions_handler (windata_t *vwin, int task)
     } else if (task == EDIT_FN_PKG) {
 	edit_function_package(path);
     } else if (task == CALL_FN_PKG) {
+	/* note: the double-click default */
 	call_function_package(path, vwin, &err);
-    }
+	if (err == FN_NO_DATA) {
+	    display_function_package_data(pkgname, path, VIEW_PKG_INFO);
+	    err = 0;
+	}
+    } 
 
     g_free(pkgname);
 
@@ -950,6 +977,78 @@ static void browser_functions_handler (windata_t *vwin, int task)
     }
 }
 
+static void show_addon_info (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+    gchar *pkgname = NULL;
+    gchar *descrip = NULL;
+    gchar *status = NULL;
+    int v = vwin->active_var;
+
+    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), v, 0, &pkgname);
+    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), v, 4, &descrip);
+    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), v, 3, &status);
+    
+    if (pkgname == NULL || descrip == NULL) {
+	gui_errmsg(E_DATA);
+    } else {
+	gchar *local = NULL;
+
+	if (status != NULL && !strcmp(status, _("Installed"))) {
+	    char *path = gretl_function_package_get_path(pkgname, PKG_SUBDIR);
+	    gchar *ver = NULL, *date = NULL;
+	    fnpkg *pkg = NULL;
+	    int err = 0;
+
+	    if (path != NULL) {
+		pkg = get_function_package_by_filename(path, &err);
+		err = function_package_get_properties(pkg, "version", &ver,
+						      "date", &date,
+						      NULL);
+		if (!err) {
+		    local = g_strdup_printf("Installed version is %s (%s)", 
+					    ver, date);
+		    g_free(ver);
+		    g_free(date);
+		}
+		free(path);
+	    }
+	}
+	if (local != NULL) {
+	    infobox("%s:\n%s\n%s", pkgname, descrip, local);
+	    g_free(local);
+	} else {
+	    infobox("%s:\n%s", pkgname, descrip);
+	}
+    }
+
+    g_free(pkgname);
+    g_free(descrip);
+    g_free(status);
+}
+
+static void install_addon_callback (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+    gchar *pkgname = NULL;
+    int v = vwin->active_var;
+
+    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), v, 0, &pkgname);
+    
+    if (pkgname == NULL) {
+	gui_errmsg(E_DATA);
+    } else {
+	int err = download_addon(pkgname, NULL);
+
+	if (!err) {
+	    list_store_set_string(GTK_TREE_VIEW(vwin->listbox), v, 3,
+				  _("Installed"));
+	} 
+
+	g_free(pkgname);
+    }
+}
+
 void browser_load_func (GtkWidget *w, gpointer data)
 {
     windata_t *vwin = (windata_t *) data;
@@ -1059,9 +1158,9 @@ windata_t *get_local_viewer (int remote_role)
     return vwin;
 }
 
-static void new_package_callback (GtkWidget *w , gpointer p)
+static void new_package_callback (GtkWidget *w, gpointer p)
 {
-    data_save_selection_wrapper(SAVE_FUNCTIONS);
+    functions_selection_wrapper();
 }
 
 static void close_files_viewer (GtkWidget *w, windata_t *vwin)
@@ -1108,7 +1207,7 @@ static void build_funcfiles_popup (windata_t *vwin)
 	    add_popup_item(_("New"), vwin->popup, 
 			   G_CALLBACK(new_package_callback), 
 			   vwin);
-	} else {
+	} else if (vwin->role == REMOTE_FUNC_FILES) {
 	    /* files on server: limited menu */
 	    add_popup_item(_("Info"), vwin->popup, 
 			   G_CALLBACK(pkg_info_from_server), 
@@ -1116,7 +1215,16 @@ static void build_funcfiles_popup (windata_t *vwin)
 	    add_popup_item(_("Install"), vwin->popup, 
 			   G_CALLBACK(install_file_from_server), 
 			   vwin);
+	} else {
+	    /* addons */
+	    add_popup_item(_("Info"), vwin->popup, 
+			   G_CALLBACK(show_addon_info), 
+			   vwin);
+	    add_popup_item(_("Install"), vwin->popup, 
+			   G_CALLBACK(install_addon_callback), 
+			   vwin);
 	}
+	    
     }
 }
 
@@ -1212,11 +1320,7 @@ enum {
 
 static GretlToolItem files_items[] = {
     { N_("Open"),           GTK_STOCK_OK, NULL, BTN_OPEN },
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
-    { N_("Select directory"), GTK_STOCK_OPEN, NULL, BTN_DIR },
-#else
     { N_("Select directory"), GTK_STOCK_DIRECTORY, NULL, BTN_DIR },
-#endif
     { N_("Edit"),           GTK_STOCK_EDIT,       G_CALLBACK(browser_edit_func), BTN_EDIT },
     { N_("Info"),           GTK_STOCK_INFO,       NULL,                          BTN_INFO },
     { N_("View code"),      GTK_STOCK_PROPERTIES, G_CALLBACK(show_function_code), BTN_CODE },
@@ -1244,6 +1348,10 @@ static int files_item_get_callback (GretlToolItem *item, int role)
     } else if (local_funcs_item(item->flag)) {
 	return (role == FUNC_FILES);
     } else if (item->flag == BTN_INST) {
+	if (role == REMOTE_ADDONS) {
+	    item->func = G_CALLBACK(install_addon_callback);
+	    return 1;
+	}
 	return (role == REMOTE_DB || 
 		role == REMOTE_FUNC_FILES ||
 		role == REMOTE_DATA_PKGS);
@@ -1267,7 +1375,9 @@ static int files_item_get_callback (GretlToolItem *item, int role)
 	    item->func = G_CALLBACK(show_function_info);
 	} else if (role == REMOTE_FUNC_FILES) {
 	    item->func = G_CALLBACK(pkg_info_from_server);
-	} 
+	} else if (role == REMOTE_ADDONS) {
+	    item->func = G_CALLBACK(show_addon_info);
+	}
     } else if (item->flag == BTN_INDX) {
 	/* index: databases only */
 	if (role == NATIVE_DB) {
@@ -1324,14 +1434,6 @@ static void make_files_toolbar (windata_t *vwin)
     gtk_widget_show_all(hbox);
 }
 
-#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 8
-/* we'll have to fake g_get_host_name */
-static const gchar *g_get_host_name (void)
-{
-    return _("local machine");
-}
-#endif
-
 static gchar *files_title (int code)
 {
     static char hname[48];
@@ -1356,6 +1458,8 @@ static gchar *files_title (int code)
     return ret;
 }
 
+/* handle drag of pointer from remote database window */
+
 static void  
 db_window_handle_drag  (GtkWidget *widget,
 			GdkDragContext *context,
@@ -1366,7 +1470,6 @@ db_window_handle_drag  (GtkWidget *widget,
 			guint time,
 			gpointer p)
 {
-    /* handle drag of pointer from remote database window */
     if (info == GRETL_REMOTE_DB_PTR && data != NULL) {
 	GdkAtom type = gtk_selection_data_get_data_type(data);
 	
@@ -1378,6 +1481,8 @@ db_window_handle_drag  (GtkWidget *widget,
     }
 }
 
+/* handle drag of pointer from remote function package window */
+
 static void  
 pkg_window_handle_drag  (GtkWidget *widget,
 			 GdkDragContext *context,
@@ -1388,7 +1493,6 @@ pkg_window_handle_drag  (GtkWidget *widget,
 			 guint time,
 			 gpointer p)
 {
-    /* handle drag of pointer from remote function package window */
     if (info == GRETL_REMOTE_FNPKG_PTR && data != NULL) {
 	GdkAtom type = gtk_selection_data_get_data_type(data);
 
@@ -1463,6 +1567,8 @@ void display_files (int code, gpointer p)
 	title = g_strdup(_("gretl: function packages on server"));
     } else if (code == REMOTE_DATA_PKGS) {
 	title = g_strdup(_("gretl: data packages on server"));
+    } else if (code == REMOTE_ADDONS) {
+	title = g_strdup(_("gretl: addons"));
     }
 
     vwin = gretl_browser_new(code, title, 0);
@@ -1507,7 +1613,8 @@ void display_files (int code, gpointer p)
 			     vwin->popup);
 	}
 	reset_data_stack();
-    } else if (code == FUNC_FILES || code == REMOTE_FUNC_FILES) {
+    } else if (code == FUNC_FILES || code == REMOTE_FUNC_FILES ||
+	       code == REMOTE_ADDONS) {
 	build_funcfiles_popup(vwin);
 	g_signal_connect(G_OBJECT(vwin->listbox), "button-press-event",
 			 G_CALLBACK(popup_menu_handler), 
@@ -1522,7 +1629,7 @@ void display_files (int code, gpointer p)
 	g_signal_connect(G_OBJECT(vwin->listbox), "button-press-event",
 			 G_CALLBACK(popup_menu_handler), 
 			 vwin->popup);
-    }	
+    } 
 
     if (REMOTE_ACTION(code)) {
 	GtkWidget *hbox;
@@ -1589,6 +1696,8 @@ static int display_files_code (const gchar *s)
 	return FUNC_FILES;
     if (!strcmp(s, "RemoteGfn"))
 	return REMOTE_FUNC_FILES;
+    if (!strcmp(s, "SFAddons"))
+	return REMOTE_ADDONS;
     return 0;
 }
 
@@ -1828,9 +1937,9 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
     GtkListStore *store;
     GtkTreeIter iter;
     char **dnames = NULL;
-    int ndirs = 0;
     DIR *dir;
-    int i, nfn, maxlen = 0;
+    int i, n_dirs = 0;
+    int nfn, maxlen = 0;
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
     nfn = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(vwin->listbox), "nfn"));
@@ -1842,13 +1951,13 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 
 	gtk_tree_model_get_iter_first(model, &iter);
 	gtk_tree_model_get(model, &iter, 4, &dirname, -1);
-	strings_array_add(&dnames, &ndirs, dirname);
+	strings_array_add(&dnames, &n_dirs, dirname);
 	g_free(dirname);
 
 	while (gtk_tree_model_iter_next(model, &iter)) {
 	    gtk_tree_model_get(model, &iter, 4, &dirname, -1);
-	    if (!dirname_done(dnames, ndirs, dirname)) {
-		strings_array_add(&dnames, &ndirs, dirname);
+	    if (!dirname_done(dnames, n_dirs, dirname)) {
+		strings_array_add(&dnames, &n_dirs, dirname);
 	    }
 	    g_free(dirname);
 	}
@@ -1858,9 +1967,9 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
     gtk_list_store_clear(store);
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
-    if (ndirs > 0) {
-	for (i=0; i<ndirs; i++) {
-	    dir = opendir(dnames[i]);
+    if (n_dirs > 0) {
+	for (i=0; i<n_dirs; i++) {
+	    dir = gretl_opendir(dnames[i]);
 	    if (dir != NULL) {
 		read_fn_files_in_dir(dir, dnames[i], store, &iter,
 				     &nfn, &maxlen);
@@ -1868,21 +1977,20 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 	    }
 	}
 
-	free_strings_array(dnames, ndirs);
+	free_strings_array(dnames, n_dirs);
     } else {
-	char fndir[FILENAME_MAX];
-	int i = 0;
+	dnames = get_plausible_search_dirs(FUNCS_SEARCH, &n_dirs);
 
-	while (get_plausible_functions_dir(fndir, i++)) {
-	    if (*fndir != '\0') {
-		dir = opendir(fndir);
-		if (dir != NULL) {
-		    read_fn_files_in_dir(dir, fndir, store, &iter,
-					 &nfn, &maxlen);
-		    closedir(dir);
-		}
+	for (i=0; i<n_dirs; i++) {
+	    dir = gretl_opendir(dnames[i]);
+	    if (dir != NULL) {
+		read_fn_files_in_dir(dir, dnames[i], store, &iter,
+				     &nfn, &maxlen);
+		closedir(dir);
 	    }
 	}
+
+	free_strings_array(dnames, n_dirs);
     }	    
 
     if (nfn == 0) {
@@ -1976,6 +2084,8 @@ gint populate_filelist (windata_t *vwin, gpointer p)
 	return populate_remote_data_pkg_list(vwin);
     } else if (vwin->role == FUNC_FILES) {
 	return populate_func_list(vwin, p);
+    } else if (vwin->role == REMOTE_ADDONS) {
+	return populate_remote_addons_list(vwin);
     } else {
 	return read_file_descriptions(vwin, p);
     }
@@ -2018,6 +2128,12 @@ static GtkWidget *files_window (windata_t *vwin)
 	N_("Summary"), 
 	N_("Local status")
     };
+    const char *addons_titles[] = {
+	N_("Package"), 
+	N_("Version"),
+	N_("Date"),
+	N_("Local status")
+    };
 
     GType types_2[] = {
 	G_TYPE_STRING,
@@ -2042,6 +2158,13 @@ static GtkWidget *files_window (windata_t *vwin)
 	G_TYPE_STRING,
 	G_TYPE_BOOLEAN  /* hidden boolean: zipfile? */
     };
+    GType types_5c[] = {
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_STRING  /* hidden string: description */
+    };
 
     const char **titles = data_titles;
     GType *types = types_2;
@@ -2091,6 +2214,13 @@ static GtkWidget *files_window (windata_t *vwin)
 	hidden_col = TRUE;
 	full_width = 580;
 	break;
+    case REMOTE_ADDONS:
+	titles = addons_titles;
+	cols = 5;
+	types = types_5c;
+	hidden_col = TRUE;
+	full_width = 400;
+	break;	
     default:
 	break;
     }
@@ -2108,6 +2238,10 @@ static GtkWidget *files_window (windata_t *vwin)
 		      types, titles, use_tree);
     gtk_widget_show(box);
 
+#ifndef G_OS_WIN32
+    set_wm_icon(vwin->main);
+#endif
+
     return box;
 }
 
