@@ -314,11 +314,6 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 	gtk_window_set_title(GTK_WINDOW(d), "gretl");
     }
 
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 7)
-    g_signal_connect(G_OBJECT(d), "key-press-event", 
-		     G_CALLBACK(esc_kills_window), NULL);
-#endif
-
     aa = gtk_dialog_get_action_area(GTK_DIALOG(d));
     gtk_button_box_set_layout(GTK_BUTTON_BOX(aa), GTK_BUTTONBOX_END);
     set_dialog_border_widths(d);
@@ -489,7 +484,7 @@ struct dialog_t_ {
     GtkWidget *edit;
     GtkWidget *popup;
     gpointer data;
-    gint code;
+    gint ci;
     gint blocking;
     gretlopt opt;
 };
@@ -511,25 +506,21 @@ static void destroy_dialog_data (GtkWidget *w, gpointer data)
     if (active_edit_text) active_edit_text = NULL;
 }
 
-static void cancel_on_delete (GtkDialog *d, int resp, int *c)
+static gboolean cancel_on_delete (GtkDialog *d, int resp, int *c)
 {
     if (resp == GTK_RESPONSE_NONE || 
 	resp == GTK_RESPONSE_DELETE_EVENT ||
 	resp == GTK_RESPONSE_CANCEL) {
 	*c = 1;
     }
+
+    return FALSE;
 }
 
 gboolean esc_kills_window (GtkWidget *w, GdkEventKey *key, 
 			   gpointer unused)
 {
     if (key->keyval == GDK_Escape) { 
-#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 7
-	/* remedial action for old gtk */
-	if (GTK_IS_DIALOG(w)) {
-	    gtk_dialog_response(GTK_DIALOG(w), GTK_RESPONSE_CANCEL);
-	}
-#endif
         gtk_widget_destroy(w);
 	return TRUE;
     } else {
@@ -537,9 +528,9 @@ gboolean esc_kills_window (GtkWidget *w, GdkEventKey *key,
     }
 }
 
-static dialog_t *
-dialog_data_new (gpointer p, gint code, const char *title,
-		 int *canceled)
+static dialog_t *dialog_data_new (gpointer p, int ci, 
+				  const char *title,
+				  int *canceled)
 {
     dialog_t *d = mymalloc(sizeof *d);
     GtkWidget *aa;
@@ -549,16 +540,16 @@ dialog_data_new (gpointer p, gint code, const char *title,
     }
 
     d->data = p;
-    d->code = code;
+    d->ci = ci;
     d->opt = OPT_NONE;
     d->dialog = gtk_dialog_new();
     d->popup = NULL;
 
     if (canceled != NULL) {
+	d->blocking = 1;
 	g_signal_connect(G_OBJECT(d->dialog), "delete-event",
 			 G_CALLBACK(cancel_on_delete), 
 			 canceled);
-	d->blocking = 1;
     } else {
 	d->blocking = 0;
     }
@@ -613,7 +604,7 @@ static void set_edit_save_buf (const char *buf, int code)
 
 static int dlg_text_set_previous (dialog_t *d)
 {
-    if (d->code == edit_save_code && 
+    if (d->ci == edit_save_code && 
 	edit_save_buf != NULL) {
 	textview_set_text(d->edit, edit_save_buf);
 	return 1;
@@ -632,7 +623,7 @@ static int dlg_text_set_gmm_skel (dialog_t *d)
 	"  params\n"
 	"end gmm\n";
 
-    if (d->code == GMM && edit_save_buf == NULL) {
+    if (d->ci == GMM && edit_save_buf == NULL) {
 	textview_set_text(d->edit, skel);
 	textview_set_cursor_at_line(d->edit, 1);
 	return 1;
@@ -664,7 +655,7 @@ gchar *edit_dialog_special_get_text (dialog_t *dlg)
 	gtk_widget_destroy(dlg->dialog);
     }
 
-    set_edit_save_buf(buf, dlg->code);
+    set_edit_save_buf(buf, dlg->ci);
 
     return buf;
 }
@@ -676,7 +667,7 @@ const gchar *edit_dialog_get_text (dialog_t *dlg)
     buf = gtk_entry_get_text(GTK_ENTRY(dlg->edit));
 
     if (buf == NULL || *buf == '\0') {
-	if (dlg->code != CORRGM) {
+	if (dlg->ci != CORRGM) {
 	    gtk_widget_destroy(dlg->dialog);
 	}
 	return NULL;
@@ -687,7 +678,7 @@ const gchar *edit_dialog_get_text (dialog_t *dlg)
 
 int edit_dialog_get_action (const dialog_t *dlg)
 {
-    return dlg->code;
+    return dlg->ci;
 }
 
 gretlopt edit_dialog_get_opt (const dialog_t *dlg)
@@ -749,7 +740,7 @@ static void dlg_text_set_from_sys (equation_system *sys,
 	return;
     }
 
-    print_equation_system_info(sys, datainfo, OPT_NONE, prn);
+    print_equation_system_info(sys, dataset, OPT_NONE, prn);
     buf = gretl_print_get_buffer(prn);
     textview_set_text(d->edit, buf);
     gretl_print_destroy(prn);
@@ -831,11 +822,11 @@ static GtkWidget *build_edit_popup (dialog_t *d)
     menu = gtk_menu_new();
 
     for (i=0; i<n_items; i++) {
-	if (d->code == NLS && (i != ADD_DERIV)) {
+	if (d->ci == NLS && (i != ADD_DERIV)) {
 	    continue;
-	} else if (d->code == MLE && (i != ADD_DERIV)) {
+	} else if (d->ci == MLE && (i != ADD_DERIV)) {
 	    continue;
-	} else if (d->code == SYSTEM) {
+	} else if (d->ci == SYSTEM) {
 	    if (i == ADD_DERIV) continue;
 	    if ((i == ADD_ENDO_LIST || i == ADD_INSTR_LIST) &&
 		edit_has_list(d->edit, i)) {
@@ -921,27 +912,30 @@ static GtkWidget *dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
 {
     GtkWidget *b = NULL;
 
-    if (opt == OPT_T) {
+    if (dlg->ci == GR_BOX && opt == OPT_O) {
+	b = gretl_option_check_button(_("Show interval for median"),
+				      &dlg->opt, opt);
+    } else if (opt == OPT_T) {
 	b = gretl_option_check_button(_("Iterated estimation"),
-				      &dlg->opt, OPT_T); /* OPT_T vs OPT_I?? */
+				      &dlg->opt, opt); /* OPT_T vs OPT_I?? */
 	if (pmod != NULL && (pmod->opt & OPT_I)) {
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), TRUE);
 	}
     } else if (opt == OPT_V) {
 	b = gretl_option_check_button(_("Show details of iterations"),
-				      &dlg->opt, OPT_V);
+				      &dlg->opt, opt);
     } else if (opt == OPT_R) {
 	b = gretl_option_check_button(_("Robust standard errors"),
-				      &dlg->opt, OPT_R);
+				      &dlg->opt, opt);
 	if (pmod != NULL && (pmod->opt & OPT_R)) {
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), TRUE);
 	}
     } else if (opt == OPT_B) {
 	b = gretl_option_check_button(_("Use bootstrap"),
-				      &dlg->opt, OPT_B);
+				      &dlg->opt, opt);
     } else if (opt == OPT_F) {
 	b = gretl_option_check_button(_("Show full restricted estimates"),
-				      &dlg->opt, OPT_F);
+				      &dlg->opt, opt);
     } 
 
     if (b != NULL) {
@@ -974,6 +968,7 @@ static void combo_opt_changed (GtkComboBox *box, combo_opts *opts)
 }
 
 GtkWidget *gretl_opts_combo_full (combo_opts *opts, int deflt, 
+				  const int *masked, 
 				  GCallback callback,
 				  gpointer calldata)
 {
@@ -984,7 +979,9 @@ GtkWidget *gretl_opts_combo_full (combo_opts *opts, int deflt,
 
     combo = gtk_combo_box_text_new();
     for (i=0; opts->strs[i] != NULL; i++) {
-	combo_box_append_text(combo, _(opts->strs[i]));
+	if (!in_gretl_list(masked, i)) {
+	    combo_box_append_text(combo, _(opts->strs[i]));
+	}
     }
     gtk_combo_box_set_active(GTK_COMBO_BOX(combo), deflt);
     g_signal_connect(G_OBJECT(combo), "changed",
@@ -999,7 +996,14 @@ GtkWidget *gretl_opts_combo_full (combo_opts *opts, int deflt,
 
 GtkWidget *gretl_opts_combo (combo_opts *opts, int deflt)
 {
-    return gretl_opts_combo_full(opts, deflt, NULL, NULL);
+    return gretl_opts_combo_full(opts, deflt, NULL, NULL, NULL);
+}
+
+GtkWidget *gretl_opts_combo_masked (combo_opts *opts, int deflt,
+				    const int *masked)
+{
+    return gretl_opts_combo_full(opts, deflt, masked,
+				 NULL, NULL);
 }
 
 void depopulate_combo_box (GtkComboBox *box)
@@ -1018,7 +1022,7 @@ static void mle_gmm_iters_dialog (GtkWidget *w, dialog_t *d)
     double tol;
     int cancel = 0;
 
-    BFGS_defaults(&maxit, &tol, d->code);
+    BFGS_defaults(&maxit, &tol, d->ci);
     lmem = libset_get_int(LBFGS_MEM);
 
     if (maxit <= 0) {
@@ -1168,7 +1172,7 @@ static void dlg_display_sys (dialog_t *d)
     GtkWidget *w;
     int hsize = 62;
 
-    if (d->code == ESTIMATE) {
+    if (d->ci == ESTIMATE) {
 	/* estimating an existing system, not editing */
 	GtkWidget *vbox;
 
@@ -1179,7 +1183,7 @@ static void dlg_display_sys (dialog_t *d)
 	gtk_widget_show(w);
     }
 
-    d->edit = dlg_text_edit_new(&hsize, d->code == SYSTEM);
+    d->edit = dlg_text_edit_new(&hsize, d->ci == SYSTEM);
     dialog_table_setup(d, hsize);
     dlg_text_set_from_sys(sys, d); 
     gretl_dialog_set_resizeable(d->dialog, TRUE);
@@ -1380,6 +1384,8 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	dialog_option_switch(top_vbox, d, OPT_B, NULL);
     } else if (ci == RESTRICT && vecm_model_window(okptr)) {
 	dialog_option_switch(top_vbox, d, OPT_F, NULL);
+    } else if (ci == GR_BOX) {
+	dialog_option_switch(top_vbox, d, OPT_O, NULL);
     }
 
     if (varclick == VARCLICK_INSERT_ID) { 
@@ -1524,24 +1530,7 @@ gchar *combo_box_get_active_text (gpointer p)
 {
     gchar *ret = NULL;
     
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
-    GtkWidget *w = gtk_bin_get_child(GTK_BIN(p));
-
-    if (GTK_IS_ENTRY(w)) {
-	const gchar *s = gtk_entry_get_text(GTK_ENTRY(w));
-
-	if (s != NULL) {
-	    ret = g_strdup(s);
-	}
-    } else {
-	GtkComboBox *box = GTK_COMBO_BOX(p);
-	GtkTreeModel *model = gtk_combo_box_get_model(box);
-	GtkTreeIter iter;
-
-	gtk_combo_box_get_active_iter(box, &iter);
-	gtk_tree_model_get(model, &iter, 0, &ret, -1);
-    }
-#elif GTK_MAJOR_VERSION >= 3
+#if GTK_MAJOR_VERSION >= 3
     ret = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(p)); 
 #else /* gtk 2 >= 2.6 */
     ret = gtk_combo_box_get_active_text(GTK_COMBO_BOX(p));
