@@ -38,10 +38,6 @@
 # include "gretlwin32.h"
 #endif
 
-#define IS_DAT_ACTION(i) (i == SAVE_DATA || \
-                          i == SAVE_DATA_AS || \
-                          i == OPEN_DATA)
-
 #define OPEN_DATA_ACTION(i)  (i == OPEN_DATA || \
                               i == OPEN_CSV || \
                               i == OPEN_OCTAVE || \
@@ -75,6 +71,7 @@
 #define GDT_ACTION(i) (i == SAVE_DATA || \
                        i == SAVE_DATA_AS || \
                        i == SAVE_BOOT_DATA || \
+                       i == EXPORT_GDT || \
                        i == OPEN_DATA || \
                        i == APPEND_DATA)
 
@@ -87,7 +84,7 @@ struct extmap {
 };
 
 static struct extmap action_map[] = {
-    { SAVE_DBDATA,       ".bin" },
+    { EXPORT_DB,         ".bin" },
     { SAVE_SCRIPT,       ".inp" },
     { SAVE_FUNCTIONS_AS, ".inp" },
     { SAVE_CMD_LOG,      ".inp" },
@@ -138,34 +135,6 @@ static struct extmap action_map[] = {
     { FILE_OP_MAX,       NULL }
 };
 
-static gretlopt save_action_to_opt (int action, gpointer p)
-{
-    gretlopt opt = OPT_NONE;
-
-    switch (action) {
-    case SAVE_DATA:
-    case SAVE_DATA_AS:
-    case SAVE_BOOT_DATA: opt = OPT_Z; break;
-    case SAVE_DBDATA:    opt = OPT_D; break;
-    case EXPORT_OCTAVE:  opt = OPT_M; break;
-    case EXPORT_R:       opt = OPT_R; break;
-    case EXPORT_CSV:     opt = OPT_C; break;
-    case EXPORT_DAT:     opt = OPT_G; break; /* PcGive */
-    case EXPORT_JM:      opt = OPT_J; break; /* JMulti */
-    default: break;
-    }
-
-    if (action == SAVE_DATA_AS && session_file_is_open()) {
-	opt |= OPT_X; /* "exporting" to GDT */
-    }
-
-    if (p != NULL) {
-	opt |= GPOINTER_TO_INT(p);
-    }
-
-    return opt;
-}
-
 static const char *get_extension_for_action (int action, gpointer data)
 {
     const char *s = NULL;
@@ -258,7 +227,7 @@ static int post_process_savename (char *fname, int action, gpointer data)
 		strcat(fname, ext);
 	    }
 	}
-    }    
+    } 
 
     return err;
 }
@@ -360,6 +329,7 @@ static void filesel_save_prn_buffer (PRN *prn, const char *fname)
 static void filesel_open_script (const char *fname, windata_t *vwin)
 {
     if (vwin != NULL) {
+	/* we're called from an existing script editor window */
 	strcpy(tryfile, fname);
 	sourceview_insert_file(vwin, fname);
     } else if (has_suffix(fname, ".R")) {
@@ -565,7 +535,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	return;
     }
 
-    if (EXPORT_ACTION(action, src) || 
+    if (EXPORT_ACTION(action, src) || action == EXPORT_GDT ||
 	(action > END_SAVE_DATA && action < END_SAVE_OTHER)) {
 	/* saving CSV, graphs etc.; check overwrite */
 	if (overwrite_stop(fname)) {
@@ -582,7 +552,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
     } else if (src == FSEL_DATA_PRN) {
 	filesel_save_prn_buffer((PRN *) data, fname);
     } else if (SAVE_DATA_ACTION(action)) {
-	err = do_store(fname, save_action_to_opt(action, data));
+	err = do_store(fname, action);
     } else if (action == SAVE_GNUPLOT) {
 	save_graph_to_file(data, fname);
     } else if (action == SAVE_GRAPHIC) {
@@ -634,6 +604,28 @@ static char *get_filter_suffix (int action, gpointer data, char *suffix)
     return suffix;
 }
 
+static void maybe_upcase_filter_pattern (GtkFileFilter *filter,
+					 const char *s)
+{
+    char *p, tmp[16];
+    int changed = 0;
+
+    strcpy(tmp, s);
+    p = tmp + 1;
+
+    while (*p) {
+	if (islower((unsigned char) *p)) {
+	    *p = toupper(*p);
+	    changed = 1;
+	}
+	p++;
+    }
+
+    if (changed) {
+	gtk_file_filter_add_pattern(filter, tmp);
+    }
+}
+
 static GtkFileFilter *get_file_filter (int action, gpointer data)
 {
     GtkFileFilter *filter;
@@ -643,6 +635,9 @@ static GtkFileFilter *get_file_filter (int action, gpointer data)
     get_filter_suffix(action, data, suffix);
     gtk_file_filter_add_pattern(filter, suffix);
 
+    /* support Windows stupidity */
+    maybe_upcase_filter_pattern(filter, suffix);
+
     return filter;
 }
 
@@ -732,40 +727,53 @@ static void filesel_maybe_set_current_name (GtkFileChooser *filesel,
     } else if (action == SAVE_LABELS) {
 	gtk_file_chooser_set_current_name(filesel, "labels.txt");
     }
-	
 }
 
 static void filesel_add_filter (GtkWidget *filesel,
 				const char *desc, 
-				const char *pat)
+				const char *pat,
+				int *maxlen)
 {
     GtkFileFilter *filt = gtk_file_filter_new();
 
-    gtk_file_filter_set_name(filt, desc);
+    if (maxlen != NULL) {
+	int n = g_utf8_strlen(desc, -1);
+	
+	if (n > *maxlen) {
+	    *maxlen = n;
+	}
+    }
+
+    gtk_file_filter_set_name(filt, _(desc));
     gtk_file_filter_add_pattern(filt, pat);
+
+    maybe_upcase_filter_pattern(filt, pat);
+
     gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filt);
 }
 
 static void filesel_set_filters (GtkWidget *filesel, int action,
-				 FselDataSrc src, gpointer data)
+				 FselDataSrc src, gpointer data,
+				 int *maxlen)
 {
     if (action == OPEN_CSV || action == APPEND_CSV) {
-	filesel_add_filter(filesel, _("CSV files (*.csv)"), "*.csv");
-	filesel_add_filter(filesel, _("ASCII files (*.txt)"), "*.txt");
-	filesel_add_filter(filesel, _("all files (*.*)"), "*");
+	filesel_add_filter(filesel, N_("CSV files (*.csv)"), "*.csv", maxlen);
+	filesel_add_filter(filesel, N_("ASCII files (*.txt)"), "*.txt", maxlen);
+	filesel_add_filter(filesel, N_("all files (*.*)"), "*", maxlen);
+    } else if (action == OPEN_XLS || action == APPEND_XLS) {
+	filesel_add_filter(filesel, N_("Excel files (*.xls)"), "*.xls", maxlen);
+	filesel_add_filter(filesel, N_("Excel files (*.xlsx)"), "*.xlsx", maxlen);
     } else if (action == OPEN_SCRIPT) {
-	filesel_add_filter(filesel, _("gretl script files (*.inp)"), "*.inp");
-	if (src != FSEL_DATA_FNPKG) {
-	    filesel_add_filter(filesel, _("GNU R files (*.R)"), "*.R");
-	    filesel_add_filter(filesel, _("gnuplot files (*.plt)"), "*.plt");
-	    filesel_add_filter(filesel, _("GNU Octave files (*.m)"), "*.m");
-	    if (ox_support) {
-		filesel_add_filter(filesel, _("Ox files (*.ox)"), "*.ox");
-	    }
+	filesel_add_filter(filesel, N_("gretl script files (*.inp)"), "*.inp", maxlen);
+	filesel_add_filter(filesel, N_("GNU R files (*.R)"), "*.R", maxlen);
+	filesel_add_filter(filesel, N_("gnuplot files (*.plt)"), "*.plt", maxlen);
+	filesel_add_filter(filesel, N_("GNU Octave files (*.m)"), "*.m", maxlen);
+	if (ox_support) {
+	    filesel_add_filter(filesel, N_("Ox files (*.ox)"), "*.ox", maxlen);
 	}
     } else if (action == OPEN_LABELS) {
-	filesel_add_filter(filesel, _("ASCII files (*.txt)"), "*.txt");
-	filesel_add_filter(filesel, _("all files (*.*)"), "*");
+	filesel_add_filter(filesel, N_("ASCII files (*.txt)"), "*.txt", maxlen);
+	filesel_add_filter(filesel, N_("all files (*.*)"), "*", maxlen);
     } else {
 	GtkFileFilter *filter = get_file_filter(action, data);
 
@@ -781,30 +789,64 @@ static void remember_folder (GtkFileChooser *chooser, char *savedir)
     g_free(folder);
 }
 
+/* this is a hack to work around breakage in GTK 2.24.6 */
+
+static void resize_combo (GtkWidget *w, gpointer data)
+{
+    gint *ivals = data;
+
+    if (ivals[0]) {
+	return;
+    } else if (GTK_IS_COMBO_BOX(w)) {
+	gtk_widget_set_size_request(w, ivals[1] * 8, -1);
+	ivals[0] = 1;
+    } else if (GTK_IS_CONTAINER(w)) {
+	gtk_container_foreach(GTK_CONTAINER(w), resize_combo,
+			      data);
+    } 
+}
+
+static void fix_filter_combo_size (GtkWidget *filesel, int maxlen)
+{
+    GtkWidget *ca = gtk_dialog_get_content_area(GTK_DIALOG(filesel));
+    gint ivals[] = { 0, maxlen };
+
+    gtk_container_foreach(GTK_CONTAINER(ca), resize_combo, ivals);
+}
+
 static void gtk_file_selector (int action, FselDataSrc src, 
 			       gpointer data, GtkWidget *parent) 
 {
     static char savedir[MAXLEN];
     char startdir[MAXLEN];
     GtkWidget *filesel;
-    GtkFileChooserAction fa;
+    GtkFileChooserAction fsel_action;
     const gchar *okstr;
     int remember = get_keep_folder();
+    int max_filter_len = 0;
+    int *lenptr = NULL;
     gint response;
 
+    if (gtk_major_version == 2 &&
+	gtk_minor_version == 24 &&
+	gtk_micro_version == 6) {
+	/* broken GTK version, 2.24.6 */
+	lenptr = &max_filter_len;
+    }
+
     if (SET_DIR_ACTION(action)) {
-	fa = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+	fsel_action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 	okstr = GTK_STOCK_OK;
 	remember = 0;
     } else if (action == SET_PROG) {
-	fa = GTK_FILE_CHOOSER_ACTION_OPEN;
+	fsel_action = GTK_FILE_CHOOSER_ACTION_OPEN;
 	okstr = GTK_STOCK_OK;
 	remember = 0;
     } else if (action < END_OPEN) {
-	fa = GTK_FILE_CHOOSER_ACTION_OPEN;
+	fsel_action = GTK_FILE_CHOOSER_ACTION_OPEN;
 	okstr = GTK_STOCK_OPEN;
     } else {
-	fa = GTK_FILE_CHOOSER_ACTION_SAVE;
+	fsel_action = GTK_FILE_CHOOSER_ACTION_SAVE;
 	okstr = GTK_STOCK_SAVE;
 	if (action == SAVE_FUNCTIONS || 
 	    action == SAVE_DATA_PKG ||
@@ -824,10 +866,12 @@ static void gtk_file_selector (int action, FselDataSrc src,
 #endif
 
     if (parent == NULL) {
+	/* by default the file selector is parented by the
+	   gretl main window */
 	parent = mdata->main;
     }
 
-    filesel = gtk_file_chooser_dialog_new(NULL, GTK_WINDOW(parent), fa,
+    filesel = gtk_file_chooser_dialog_new(NULL, GTK_WINDOW(parent), fsel_action,
 					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					  okstr, GTK_RESPONSE_ACCEPT,
 					  NULL);
@@ -837,12 +881,16 @@ static void gtk_file_selector (int action, FselDataSrc src,
     if (SET_DIR_ACTION(action)) {
 	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), startdir);
     } else {
-	filesel_set_filters(filesel, action, src, data);
+	filesel_set_filters(filesel, action, src, data, lenptr);
 	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
 	filesel_maybe_set_current_name(GTK_FILE_CHOOSER(filesel), action,
 				       src, data);
     }
 
+    if (max_filter_len > 0) {
+	fix_filter_combo_size(filesel, max_filter_len);
+    }
+
     response = gtk_dialog_run(GTK_DIALOG(filesel));
 
     if (response == GTK_RESPONSE_ACCEPT) {
