@@ -18,6 +18,7 @@
  */
 
 #include "gretl.h"
+#include "version.h"
 #include "dlgutils.h"
 #include "selector.h"
 #include "gretl_func.h"
@@ -28,13 +29,14 @@
 #include "gretl_string_table.h"
 #include "gretl_scalar.h"
 #include "gretl_bundle.h"
+#include "gretl_xml.h"
 #include "database.h"
 #include "guiprint.h"
 #include "ssheet.h"
 #include "datafiles.h"
 #include "toolbar.h"
 #include "obsbutton.h"
-#include "version.h"
+#include "fncall.h"
 
 #define FCDEBUG 0
 
@@ -117,6 +119,8 @@ static call_info *cinfo_new (fnpkg *pkg, windata_t *vwin)
 
     cinfo->pkg = pkg;
     cinfo->vwin = vwin;
+    cinfo->dlg = NULL;
+    cinfo->top_hbox = NULL;
 
     cinfo->publist = NULL;
     cinfo->iface = -1;
@@ -340,10 +344,10 @@ static int probably_stochastic (int v)
 {
     int ret = 1;
 
-    if (sample_size(datainfo) >= 3) {
-	int t = datainfo->t1;
-	double d1 = Z[v][t+1] - Z[v][t];
-	double d2 = Z[v][t+2] - Z[v][t+1];
+    if (sample_size(dataset) >= 3) {
+	int t = dataset->t1;
+	double d1 = dataset->Z[v][t+1] - dataset->Z[v][t];
+	double d2 = dataset->Z[v][t+2] - dataset->Z[v][t+1];
 
 	if (d1 == floor(d1) && d2 == d1) {
 	    ret = 0;
@@ -383,13 +387,13 @@ static GList *add_series_names (GList *list)
 {
     int i;
 
-    for (i=1; i<datainfo->v; i++) {
-	if (!var_is_hidden(datainfo, i)) {
-	    list = g_list_append(list, (gpointer) datainfo->varname[i]);
+    for (i=1; i<dataset->v; i++) {
+	if (!var_is_hidden(dataset, i)) {
+	    list = g_list_append(list, (gpointer) dataset->varname[i]);
 	} 
     }
 
-    list = g_list_append(list, (gpointer) datainfo->varname[0]);
+    list = g_list_append(list, (gpointer) dataset->varname[0]);
 
     return list;
 }
@@ -760,10 +764,16 @@ static void update_xlist_arg (GtkComboBox *combo,
 
 static GtkWidget *xlist_int_selector (call_info *cinfo, int i)
 {
-    MODEL *pmod = cinfo->vwin->data;
-    int *xlist = NULL;
+    MODEL *pmod;
+    int *xlist;
     GtkWidget *combo;
 
+    if (cinfo->vwin == NULL || cinfo->vwin->data == NULL) {
+	return NULL;
+    }
+
+    pmod = cinfo->vwin->data;
+
     combo = gtk_combo_box_text_new();
     widget_set_int(combo, "argnum", i);
     g_signal_connect(G_OBJECT(combo), "changed",
@@ -778,7 +788,7 @@ static GtkWidget *xlist_int_selector (call_info *cinfo, int i)
 	for (i=1; i<=xlist[0]; i++) {
 	    vi = xlist[i];
 	    if (vi > 0) {
-		s = datainfo->varname[xlist[i]];
+		s = dataset->varname[xlist[i]];
 		combo_box_append_text(combo, s);
 	    }
 	}
@@ -829,7 +839,7 @@ static GtkWidget *spin_arg_selector (call_info *cinfo, int i,
     adj = (GtkAdjustment *) gtk_adjustment_new(initv, minv, maxv, 
 					       1, 1, 0);
     if (type == GRETL_TYPE_OBS) {
-	spin = obs_button_new(adj, datainfo);
+	spin = obs_button_new(adj, dataset);
     } else {
 	spin = gtk_spin_button_new(adj, 1, 0);
     }
@@ -854,8 +864,8 @@ static GtkWidget *int_arg_selector (call_info *cinfo, int i,
     if (type == GRETL_TYPE_OBS) {
 	/* the incoming vals will be 1-based */
 	minv = (na(dminv) || dminv < 1)? 0 : (int) dminv - 1;
-	maxv = (na(dmaxv) || dmaxv > datainfo->n)? 
-	    (datainfo->n - 1) : (int) dmaxv - 1;
+	maxv = (na(dmaxv) || dmaxv > dataset->n)? 
+	    (dataset->n - 1) : (int) dmaxv - 1;
     } else {
 	minv = (na(dminv))? INT_MIN : (int) dminv;
 	maxv = (na(dmaxv))? INT_MAX : (int) dmaxv;
@@ -901,6 +911,11 @@ static GtkWidget *double_arg_selector (call_info *cinfo, int i)
     }
     g_free(tmp);
 
+    if (deflt > maxv) {
+	/* note that default may be NADBL */
+	deflt = minv;
+    }
+
     adj = (GtkAdjustment *) gtk_adjustment_new(deflt, minv, maxv, 
 					       step, step, 0);
     spin = gtk_spin_button_new(adj, 1, ndec);
@@ -976,7 +991,7 @@ static void arg_combo_set_default (call_info *cinfo,
 	int ok = 0;
 
 	if (series_arg(ptype)) {
-	    int v = current_series_index(datainfo, name);
+	    int v = current_series_index(dataset, name);
 
 	    if (v > 0 && probably_stochastic(v)) {
 		ok = !already_set_as_default(cinfo, name, ptype);
@@ -1006,7 +1021,7 @@ static GtkWidget *combo_arg_selector (call_info *cinfo, int ptype, int i)
     GList *list = NULL;
     GtkWidget *combo;
     GtkWidget *entry;
-    int null_OK = 0;
+    int k = 0, null_OK = 0;
 
     combo = combo_box_text_new_with_entry();
     entry = gtk_bin_get_child(GTK_BIN(combo));
@@ -1024,20 +1039,35 @@ static GtkWidget *combo_arg_selector (call_info *cinfo, int ptype, int i)
     list = get_selection_list(ptype);
     if (list != NULL) {
 	set_combo_box_strings_from_list(combo, list);
-	if (null_OK) {
-	    combo_box_append_text(combo, "null");	    
-	}
 	arg_combo_set_default(cinfo, GTK_COMBO_BOX(combo), 
 			      list, ptype);
+	k = g_list_length(list);
 	g_list_free(list);
     } 
 
-    if (ptype == GRETL_TYPE_INT || ptype == GRETL_TYPE_DOUBLE) {
+    if (null_OK) {
+	combo_box_append_text(combo, "null");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), k);	
+    }
+
+    if (ptype == GRETL_TYPE_INT) {
 	double x = fn_param_default(cinfo->func, i);
 
 	if (!na(x)) {
 	    gchar *tmp = g_strdup_printf("%g", x);
 
+	    gtk_entry_set_text(GTK_ENTRY(entry), tmp);
+	    g_free(tmp);
+	} 
+    } else if (ptype == GRETL_TYPE_DOUBLE &&
+	       fn_param_has_default(cinfo->func, i)) {
+	double x = fn_param_default(cinfo->func, i);
+
+	if (na(x)) {
+	    gtk_entry_set_text(GTK_ENTRY(entry), "NA");
+	} else {
+	    gchar *tmp = g_strdup_printf("%g", x);
+
 	    gtk_entry_set_text(GTK_ENTRY(entry), tmp);
 	    g_free(tmp);
 	}
@@ -1089,10 +1119,9 @@ static int spinnable_scalar_arg (call_info *cinfo, int i)
     const ufunc *func = cinfo->func;
     double mi = fn_param_minval(func, i);
     double ma = fn_param_maxval(func, i);
-    double df = fn_param_default(func, i);
     double s = fn_param_step(func, i);
 
-    return !na(mi) && !na(ma) && !na(df) && !na(s);
+    return !na(mi) && !na(ma) && !na(s);
 }
 
 static void set_close_on_OK (GtkWidget *b, gpointer p)
@@ -1347,7 +1376,7 @@ static int function_data_check (call_info *cinfo)
     int i, err = 0;
 
     if (cinfo->dreq != FN_NODATA_OK) {
-	if (datainfo == NULL || datainfo->v == 0) {
+	if (dataset == NULL || dataset->v == 0) {
 	    warnbox(_("Please open a data file first"));
 	    return 1;
 	}
@@ -1358,7 +1387,7 @@ static int function_data_check (call_info *cinfo)
 
 	if (type == GRETL_TYPE_SERIES || type == GRETL_TYPE_LIST ||
 	    type == GRETL_TYPE_SERIES_REF) {
-	    if (datainfo == NULL || datainfo->v == 0) {
+	    if (dataset == NULL || dataset->v == 0) {
 		warnbox(_("Please open a data file first"));
 		err = 1;
 		break;
@@ -1416,11 +1445,11 @@ static int maybe_add_amp (call_info *cinfo, int i, PRN *prn, int *add)
 	    }
 	}
     } else if (t == GRETL_TYPE_SERIES_REF) {
-	if (current_series_index(datainfo, s) < 0) {
+	if (current_series_index(dataset, s) < 0) {
 	    char cmd[32];
 
 	    sprintf(cmd, "series %s", s);
-	    err = generate(cmd, &Z, datainfo, OPT_Q, NULL);
+	    err = generate(cmd, dataset, OPT_Q, NULL);
 	    if (!err) {
 		pprintf(prn, "? %s\n", cmd);
 	    }
@@ -1455,7 +1484,7 @@ static int pre_process_args (call_info *cinfo, PRN *prn)
 	if (should_addressify_var(cinfo, i)) {
 	    sprintf(auxname, "FNARG%d", i + 1);
 	    sprintf(auxline, "genr %s=%s", auxname, cinfo->args[i]);
-	    err = generate(auxline, &Z, datainfo, OPT_NONE, NULL);
+	    err = generate(auxline, dataset, OPT_NONE, NULL);
 	    if (!err) {
 		g_free(cinfo->args[i]);
 		cinfo->args[i] = g_strdup(auxname);
@@ -1506,7 +1535,8 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
     const char *funname;
     gretl_bundle *bundle = NULL;
     int attach_bundle = 0;
-    int orig_v = datainfo->v;
+    int no_print = 0;
+    int orig_v = dataset->v;
     int i, err = 0;
 
     funname = user_function_name_by_index(cinfo->iface);
@@ -1545,11 +1575,15 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 	pprintf(prn, "? %s\n", fnline);
     }
 
+#if FCDEBUG
+    fprintf(stderr, "fnline: %s\n", fnline);
+#endif
+
     /* note: gretl_exec_state_init zeros the first byte of the
        supplied 'line' */
 
     gretl_exec_state_init(&state, SCRIPT_EXEC, NULL, get_lib_cmd(),
-			  models, prn);
+			  model, prn);
     state.line = fnline;
 
     if (cinfo->flags & MODEL_CALL) {
@@ -1557,11 +1591,15 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
     }
 
 #if USE_GTK_SPINNER
-    start_wait_for_output(cinfo->top_hbox, 0); 
+    if (cinfo->top_hbox != NULL) {
+	start_wait_for_output(cinfo->top_hbox, 0);
+    }
 #endif
-    err = gui_exec_line(&state, &Z, datainfo);
+    err = gui_exec_line(&state, dataset);
 #if USE_GTK_SPINNER
-    stop_wait_for_output(cinfo->top_hbox);
+    if (cinfo->top_hbox != NULL) {
+	stop_wait_for_output(cinfo->top_hbox);
+    }
 #endif
 
     if (cinfo->flags & MODEL_CALL) {
@@ -1585,7 +1623,17 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 	}
     }
 
-    if (user_func_is_noprint(cinfo->func) && !err) {
+    no_print = user_func_is_noprint(cinfo->func);
+
+    if (!err && bundle != NULL && no_print) {
+	gretl_print_reset_buffer(prn);
+	if (try_exec_bundle_print_function(bundle, prn)) {
+	    /* flag printing of bundled result */
+	    no_print = 0;
+	} 
+    }
+
+    if (!err && no_print) {
 	gretl_print_destroy(prn);
     } else {
 	view_buffer(prn, 80, 400, funname, 
@@ -1597,7 +1645,7 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 	gui_errmsg(err);
     } 
 
-    if (datainfo->v != orig_v) {
+    if (dataset->v != orig_v) {
 	mark_dataset_as_modified();
 	populate_varlist();
     }
@@ -1684,7 +1732,7 @@ static void fncall_exec_callback (GtkWidget *w, call_info *cinfo)
 	    gretl_print_destroy(prn);
 	}
 
-	if (close_on_OK) {
+	if (cinfo->dlg != NULL && close_on_OK) {
 	    gtk_widget_destroy(cinfo->dlg);
 	}
     }
@@ -1706,6 +1754,23 @@ static void maybe_set_gui_interface (const char *gname,
     }
 }
 
+static int need_model_check (call_info *cinfo)
+{
+    int i, err = 0;
+
+    for (i=0; i<cinfo->n_params; i++) {
+	if (fn_param_uses_xlist(cinfo->func, i)) {
+	    if (cinfo->vwin == NULL || cinfo->vwin->role != VIEW_MODEL) {
+		err = E_DATA;
+		errbox(_("This function needs a model in place"));
+		break;
+	    }
+	}
+    }
+
+    return err;
+}
+
 /* call to execute a function from the specified package: we do
    this only for locally installed packages */
 
@@ -1722,7 +1787,7 @@ void call_function_package (const char *fname, windata_t *vwin,
     if (err) {
 	gui_errmsg(err); /* FIXME error not very informative? */
 	if (loaderr != NULL) {
-	    *loaderr = 1;
+	    *loaderr = FN_NO_LOAD;
 	}
 	return;
     }
@@ -1750,9 +1815,17 @@ void call_function_package (const char *fname, windata_t *vwin,
 
     if (!err) {
 	/* do we have suitable data in place? */
-	err = check_function_needs(datainfo, cinfo->dreq, minver);
+	err = check_function_needs(dataset, cinfo->dreq, minver);
 	if (err) {
-	    gui_errmsg(err);
+	    if (loaderr != NULL) {
+		/* coming from package listing window */
+		gui_warnmsg(err);
+		*loaderr = FN_NO_DATA;
+		cinfo_free(cinfo);
+		return;
+	    } else {
+		gui_errmsg(err);
+	    }
 	}
     }
 
@@ -1808,7 +1881,17 @@ void call_function_package (const char *fname, windata_t *vwin,
     }
 
     if (!err) {
-	function_call_dialog(cinfo);
+	/* FIXME this check should come earlier? */
+	err = need_model_check(cinfo);
+    }
+
+    if (!err) {
+	if (fn_n_params(cinfo->func) == 0) {
+	    /* no arguments to be gathered */
+	    fncall_exec_callback(NULL, cinfo);
+	} else {
+	    function_call_dialog(cinfo);
+	}
     } else {
 	cinfo_free(cinfo);
     }
@@ -1827,9 +1910,8 @@ void function_call_cleanup (void)
    packed into @aname, after a colon. 
 */
 
-int exec_bundle_plot_function (void *ptr, const char *aname)
+int exec_bundle_plot_function (gretl_bundle *b, const char *aname)
 {
-    gretl_bundle *b = ptr;
     ufunc *func;
     fnargs *args = NULL;
     const char *bname;
@@ -1883,7 +1965,7 @@ int exec_bundle_plot_function (void *ptr, const char *aname)
 	PRN *prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
 
 	err = gretl_function_exec(func, args, GRETL_TYPE_NONE,
-				  &Z, datainfo, NULL, NULL, prn);
+				  dataset, NULL, NULL, prn);
 	gretl_print_destroy(prn);
     }
 
@@ -1960,7 +2042,7 @@ int try_exec_bundle_print_function (gretl_bundle *b, PRN *prn)
 	gretl_exec_state_init(&state, SCRIPT_EXEC, NULL, get_lib_cmd(),
 			      NULL, prn);
 	state.line = fnline;
-	err = gui_exec_line(&state, &Z, datainfo);
+	err = gui_exec_line(&state, dataset);
 
 	if (err) {
 	    gui_errmsg(err);
@@ -1972,112 +2054,49 @@ int try_exec_bundle_print_function (gretl_bundle *b, PRN *prn)
     return ret;
 }
 
-#define OLS_BUNDLE_DEMO 0
+/* get a listing of available addons along with (a) the 
+   name of the versioned subdirectory containing the 
+   most recent usable version (given the gretl version)
+   and (b) the date of that package version, taken from its 
+   spec file. E.g.
 
-#if OLS_BUNDLE_DEMO
-
-/* This is quite complicated, because we're bypassing the "standard"
-   mechanism for a function call, and instead using output from the
-   gretl model selection dialog and constructing a "direct" call to
-   the function, as is done inside geneval.c. There's probably a
-   simpler way of doing this.
+   gig 1.9.5 2011-04-22
+   ivpanel 1.9.4 2011-02-10
 */
 
-static int ols_bundle_callback (selector *sr)
+int query_addons (void)
 {
-    const char *funname = "ols_pack";
-    const char *pkgname = "olsbundle";
-    char *path = NULL;
-    ufunc *uf = NULL;
-    fnargs *args = NULL;
-    int yno, err = 0;
+    gchar *query;
+    char *buf = NULL;
+    int v1, v2, v3;
+    int err = 0;
 
-    if (selector_error(sr)) {
-	return 1;
+    sscanf(GRETL_VERSION, "%d.%d.%d", &v1, &v2, &v3);
+    if (strstr(GRETL_VERSION, "cvs")) {
+	/* benefit of the doubt */
+	v3++;
     }
+    
+    query = g_strdup_printf("/addons-data/pkginfo.php?gretl_version=%d.%d.%d",
+			    v1, v2, v3);
+    err = query_sourceforge(query, &buf);
+    g_free(query);
 
-    /* get path to package; load package if not already loaded;
-       get specific function from package */
-
-    path = gretl_function_package_get_path(pkgname, PKG_ALL);
-    if (path == NULL) {
-	gretl_errmsg_sprintf("Couldn't find package '%s'", pkgname);
+    if (!err && buf == NULL) {
+	/* shouldn't happen */
 	err = E_DATA;
-    } else {
-	fnpkg *pkg = get_function_package_by_filename(path, &err);
-
-	if (!err) {
-	    uf = get_function_from_package(funname, pkg);
-	}
-	if (uf == NULL) {
-	    gretl_errmsg_sprintf("Couldn't find function '%s'", funname);
-	    err = E_DATA;
-	}
-	free(path);
-    }
-
-    if (err) {
-	gui_errmsg(err);
-	return err;
-    } else {
-	/* args: get ID of dependent var plus X list */
-	const char *buf = selector_list(sr);
-
-	if (buf == NULL) {
-	    err = E_DATA;
-	} else {
-	    int *list = gretl_list_from_string(buf, &err);
-
-	    if (!err) {
-		yno = list[1];
-		gretl_list_delete_at_pos(list, 1);
-		remember_list(list, "arg1temp", NULL);
-		free(list);
-	    }
-	}
     }
 
     if (!err) {
-	/* push args to function */
-	args = fn_args_new();
-	if (args == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    err = push_fn_arg(args, GRETL_TYPE_USERIES, &yno);
-	    if (!err) {
-		err = push_fn_arg(args, GRETL_TYPE_LIST, "arg1temp");
-	    }
-	}
+	/* make use of info! */
+	infobox(buf);
     }
 
-    if (!err) {
-	/* execute function and retrieve bundle */
-	gretl_bundle *bundle = NULL;
-	PRN *prn = NULL;
-
-	err = bufopen(&prn);
-
-	if (!err) {
-	    err = gretl_function_exec(uf, args, GRETL_TYPE_BUNDLE, &Z, datainfo, 
-				      &bundle, NULL, prn);
-	}
-	if (!err) {
-	    view_buffer(prn, 80, 400, funname, VIEW_BUNDLE, bundle);
-	}
-    }
-
-    if (err) {
-	gui_errmsg(err);
-    }
-
-    fn_args_free(args);
-    delete_list_by_name("arg1temp");
+    free(buf);
 
     return err;
 }
 
-#endif /* OLS_BUNDLE_DEMO */
-
 static int query_addons_dir (const char *pkgname, char *sfdir)
 {
     gchar *query;
@@ -2123,7 +2142,7 @@ static int query_addons_dir (const char *pkgname, char *sfdir)
     return err;
 }
 
-static int download_addon (const char *pkgname, char **local_path)
+int download_addon (const char *pkgname, char **local_path)
 {
     const char *SF = "http://downloads.sourceforge.net/"
 	"project/gretl/addons";
@@ -2162,9 +2181,11 @@ static int download_addon (const char *pkgname, char **local_path)
 	    strcat(path, SLASHSTR);
 	    strcat(path, pkgname);
 	    strcat(path, ".gfn");
-	    *local_path = gretl_strdup(path);
-	    if (*local_path == NULL) {
-		err = E_ALLOC;
+	    if (local_path != NULL) {
+		*local_path = gretl_strdup(path);
+		if (*local_path == NULL) {
+		    err = E_ALLOC;
+		}
 	    }
 	}
 	g_free(uri);
@@ -2174,6 +2195,51 @@ static int download_addon (const char *pkgname, char **local_path)
     return err;
 }
 
+struct addon_info_ {
+    char *pkgname;
+    char *label;
+    char *menupath;
+    char *filepath;
+    PkgType ptype;
+    int modelwin;
+};
+
+typedef struct addon_info_ addon_info;
+
+static addon_info *addons;
+static int n_addons;
+
+static void destroy_addons_info (void)
+{
+    if (addons != NULL && n_addons > 0) {
+	int i;
+
+	for (i=0; i<n_addons; i++) {
+	    free(addons[i].pkgname);
+	    free(addons[i].label);
+	    free(addons[i].menupath);
+	    free(addons[i].filepath);
+	}
+	free(addons);
+    }
+
+    addons = NULL;
+    n_addons = 0;
+}
+
+static addon_info *retrieve_addon_info (const gchar *pkgname)
+{
+    int i;
+
+    for (i=0; i<n_addons; i++) {
+	if (!strcmp(pkgname, addons[i].pkgname)) {
+	    return &addons[i];
+	}
+    }
+
+    return NULL;
+}
+
 /* Callback for a menu item representing a "addon" function package, 
    whose name (e.g. "gig") is attached to @action. We first see if 
    we can find the full path to the corresponding gfn file; if so we
@@ -2183,19 +2249,19 @@ static int download_addon (const char *pkgname, char **local_path)
 static void gfn_menu_callback (GtkAction *action, windata_t *vwin)
 {
     const gchar *pkgname = gtk_action_get_name(action);
-    char *path = NULL;
+    addon_info *addon;
 
-#if OLS_BUNDLE_DEMO
-    if (!strcmp(pkgname, "olsbundle")) {
-	selection_dialog(_("gretl: specify model"), 
-			 ols_bundle_callback, OLS);
+    addon = retrieve_addon_info(pkgname);
+    if (addon == NULL) {
+	/* "can't happen" */
 	return;
     }
-#endif
 
-    path = gretl_function_package_get_path(pkgname, PKG_ADDON);
+    if (addon->filepath == NULL) {
+	addon->filepath = gretl_function_package_get_path(pkgname, addon->ptype);
+    }
 
-    if (path == NULL) {
+    if (addon->filepath == NULL) {
 	gchar *msg = g_strdup_printf(_("The %s package was not found, or is not "
 				       "up to date.\nWould you like to try "
 				       "downloading it now?"), pkgname);
@@ -2203,16 +2269,85 @@ static void gfn_menu_callback (GtkAction *action, windata_t *vwin)
 
 	g_free(msg);
 	if (resp == GRETL_YES) {
-	    download_addon(pkgname, &path);
+	    download_addon(pkgname, &addon->filepath);
 	}
     }
 
-    if (path != NULL) {
-	call_function_package(path, vwin, NULL);
-	free(path);
+    if (addon->filepath != NULL) {
+	call_function_package(addon->filepath, vwin, NULL);
     }
 }
 
+static int read_addons_info (void)
+{
+    char fname[FILENAME_MAX];
+    xmlDocPtr doc = NULL;
+    xmlNodePtr cur = NULL;
+    int err, n = 0;
+
+    sprintf(fname, "%sfunctions%cpackages.xml", 
+	    gretl_home(), SLASH);
+
+    err = gretl_xml_open_doc_root(fname, "gretl-package-info", &doc, &cur);
+    if (err) {
+	return err;
+    } 
+
+    cur = cur->xmlChildrenNode;
+
+    while (cur != NULL && !err) {
+        if (!xmlStrcmp(cur->name, (XUC) "package")) {
+	    int mw = gretl_xml_get_prop_as_bool(cur, "model-window");
+	    int top = gretl_xml_get_prop_as_bool(cur, "toplev");
+	    xmlChar *name, *desc, *path;
+
+	    name = xmlGetProp(cur, (XUC) "name");
+	    desc = xmlGetProp(cur, (XUC) "label");
+	    path = xmlGetProp(cur, (XUC) "path");
+
+	    if (name == NULL || desc == NULL || path == NULL) {
+		err = 1;
+	    } else {
+		addons = myrealloc(addons, (n+1) * sizeof *addons);
+
+		if (addons == NULL) {
+		    err = 1;
+		} else {
+		    addons[n].pkgname = (char *) name;
+		    addons[n].label = (char *) desc;
+		    addons[n].menupath = (char *) path;
+		    addons[n].filepath = NULL;
+		    addons[n].ptype = (top)? PKG_TOPLEV : PKG_SUBDIR;
+		    addons[n].modelwin = mw;
+		    n++;
+		} 
+	    }
+
+	    if (err) {
+		free(name);
+		free(desc);
+		free(path);
+	    }		
+	}
+
+	if (!err) {
+	    cur = cur->next;
+	}
+    }
+
+    if (err) {
+	destroy_addons_info();
+    } else {
+	n_addons = n;
+    }
+
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+    }
+
+    return err;
+}
+
 /* For recognized "addon" function packages: given the internal
    package name (e.g. "gig") and a menu path where we'd like
    it to appear (e.g. "/menubar/Model/TSModels" -- see the
@@ -2221,9 +2356,7 @@ static void gfn_menu_callback (GtkAction *action, windata_t *vwin)
    which see above.
 */
 
-static void add_package_to_menu (const char *pkgname,
-				 const char *label,
-				 const char *menupath,
+static void add_package_to_menu (addon_info *addon,
 				 windata_t *vwin)
 {
     static GtkActionEntry pkg_item = {
@@ -2231,53 +2364,24 @@ static void add_package_to_menu (const char *pkgname,
     };
     GtkActionGroup *actions;
 
-    pkg_item.name = pkgname;
-    if (label != NULL) {
-	pkg_item.label = label;
+    pkg_item.name = addon->pkgname;
+    if (addon->label != NULL) {
+	pkg_item.label = _(addon->label);
     } else {
-	pkg_item.label = pkgname;
+	pkg_item.label = addon->pkgname;
     }
 
     gtk_ui_manager_add_ui(vwin->ui, gtk_ui_manager_new_merge_id(vwin->ui),
-			  menupath, pkg_item.label, pkgname,
+			  addon->menupath, pkg_item.label, pkg_item.name,
 			  GTK_UI_MANAGER_MENUITEM, 
 			  FALSE);
 
-    actions = gtk_action_group_new(pkgname);
+    actions = gtk_action_group_new(pkg_item.name);
     gtk_action_group_add_actions(actions, &pkg_item, 1, vwin);
     gtk_ui_manager_insert_action_group(vwin->ui, actions, 0);
     g_object_unref(actions);
 }
 
-struct addon_info {
-    const char *pkgname;
-    const char *label;
-    const char *menupath;
-};
-
-/* put the recognized gretl addons into the appropriate menus */
-
-void maybe_add_packages_to_menus (windata_t *vwin)
-{
-    struct addon_info addons[] = {
-	{ "gig", "GARCH variants", "/menubar/Model/TSModels" },
-	{ "ivpanel", "Panel IV model", "/menubar/Model/PanelModels" }
-    };
-    int i, n = sizeof addons / sizeof addons[0];
-
-    for (i=0; i<n; i++) {
-	add_package_to_menu(addons[i].pkgname,
-			    addons[i].label,
-			    addons[i].menupath,
-			    vwin);
-    }
-
-#if OLS_BUNDLE_DEMO
-    add_package_to_menu("olsbundle", "OLS bundle",
-			"/menubar/Model", vwin);
-#endif
-}
-
 /* run a package's gui-precheck function to determine if
    it's OK to add its GUI interface to a gretl
    model-window menu
@@ -2297,7 +2401,7 @@ static int precheck_error (ufunc *func, windata_t *vwin)
     prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
     set_genr_model_from_vwin(vwin);
     err = gretl_function_exec(func, args, GRETL_TYPE_DOUBLE,
-			      &Z, datainfo, &check_err, NULL, prn);
+			      dataset, &check_err, NULL, prn);
     unset_genr_model();
     gretl_print_destroy(prn);
     fn_args_free(args);
@@ -2309,58 +2413,104 @@ static int precheck_error (ufunc *func, windata_t *vwin)
     return err;
 }
 
-void maybe_add_packages_to_model_menus (windata_t *vwin)
+static int maybe_add_model_pkg (addon_info *addon,
+				windata_t *vwin)
 {
-    struct addon_info model_addons[] = {
-	{ "bandplot", "Confidence band plot", "/menubar/Graphs" }
-    };
-    int i, n = sizeof model_addons / sizeof model_addons[0];
-    char *path;
+    int dreq, mreq, minver = 0;
+    gchar *precheck = NULL;
+    fnpkg *pkg;
+    int err = 0;
 
-    for (i=0; i<n; i++) {
-	struct addon_info *addon = &model_addons[i];
-	int dreq, minver = 0;
-	gchar *precheck = NULL;
-	fnpkg *pkg;
-	int err = 0;
-
-	path = gretl_function_package_get_path(addon->pkgname, 
-					       PKG_ADDON);
+    if (addon->filepath == NULL) {
+	addon->filepath = gretl_function_package_get_path(addon->pkgname, 
+							  addon->ptype);
+    }
 	
-	if (path == NULL) {
-	    continue;
-	}
+    if (addon->filepath == NULL) {
+	fprintf(stderr, "%s: couldn't find it\n", addon->pkgname);
+	return 0;
+    } else {
+	fprintf(stderr, "found %s\n", addon->filepath);
+    }
 
-	pkg = get_function_package_by_filename(path, &err);
+    pkg = get_function_package_by_filename(addon->filepath, &err);
 
-	if (!err) {
-	    err = function_package_get_properties(pkg,
-						  "data-requirement", &dreq,
-						  "min-version", &minver,
-						  "gui-precheck", &precheck,
-						  NULL);
-	}
+    if (!err) {
+	err = function_package_get_properties(pkg,
+					      "data-requirement", &dreq,
+					      "model-requirement", &mreq,
+					      "min-version", &minver,
+					      "gui-precheck", &precheck,
+					      NULL);
+    }
 
-	if (!err) {
-	    if (precheck != NULL) {
-		ufunc *func = get_function_from_package(precheck, pkg);
-	    
-		if (func == NULL || precheck_error(func, vwin)) {
-		    fprintf(stderr, "precheck failed\n");
-		    err = 1;
-		}
-	    } else {
-		err = check_function_needs(datainfo, dreq, minver);
+    if (!err) {
+	if (mreq > 0) {
+	    MODEL *pmod = vwin->data;
+
+	    if (pmod->ci != mreq) {
+		fprintf(stderr, "%s: model-requirement (%s) not met\n",  
+			addon->pkgname, gretl_command_word(mreq));
+		err = 1;
 	    }
 	}
-
 	if (!err) {
-	    add_package_to_menu(addon->pkgname, addon->label, 
-				addon->menupath, vwin);
+	    err = check_function_needs(dataset, dreq, minver);
 	}
+	if (!err && precheck != NULL) {
+	    ufunc *func = get_function_from_package(precheck, pkg);
+	    
+	    if (func == NULL || precheck_error(func, vwin)) {
+		fprintf(stderr, "precheck failed\n");
+		err = 1;
+	    }
+	} 
+    }
 
-	g_free(precheck);
-	free(path);
+    if (!err) {
+	add_package_to_menu(addon, vwin);
+    }
+
+    g_free(precheck);
+
+    return err;
+}
+
+/* put recognized gretl addons (other than those that
+   live in model menus) into the appropriate menus */
+
+void maybe_add_packages_to_menus (windata_t *vwin)
+{
+    int i;
+
+    if (addons == NULL) {
+	read_addons_info();
+    }
+
+    for (i=0; i<n_addons; i++) {
+	if (!addons[i].modelwin) {
+	    add_package_to_menu(&addons[i], vwin);
+	}
     }
 }
 
+void maybe_add_packages_to_model_menus (windata_t *vwin)
+{
+    int i;
+
+    if (vwin == NULL) {
+	/* clean-up signal */
+	destroy_addons_info();
+	return;
+    }
+
+    if (addons == NULL) {
+	read_addons_info();
+    }
+
+    for (i=0; i<n_addons; i++) {
+	if (addons[i].modelwin) {
+	    maybe_add_model_pkg(&addons[i], vwin);
+	}
+    }
+}
