@@ -18,6 +18,7 @@
  */
 
 #include "gretl.h"
+#include "gretl_xml.h"
 #include "dlgutils.h"
 #include "datafiles.h"
 #include "textbuf.h"
@@ -374,7 +375,14 @@ static void login_cancel (GtkWidget *w, login_info *linfo)
 static gboolean update_active_func (GtkComboBox *menu, 
 				    function_info *finfo)
 {
-    int i = gtk_combo_box_get_active(menu);
+    int i = 0;
+
+    if (menu != NULL) {
+	i = gtk_combo_box_get_active(menu);
+	if (i < 0) {
+	    i = 0;
+	}
+    }
 
     if (i < finfo->n_pub) {
 	finfo->active = finfo->pubnames[i];
@@ -716,10 +724,10 @@ static char **get_function_names (const int *list, int *err)
    relative to the existing lists of function names in @finfo.
 */
 
-static int finfo_set_function_names (function_info *finfo,
-				     const int *publist,
-				     const int *privlist,
-				     int *changed)
+static int finfo_reset_function_names (function_info *finfo,
+				       const int *publist,
+				       const int *privlist,
+				       int *changed)
 {
     char **pubnames = NULL;
     char **privnames = NULL;
@@ -752,17 +760,36 @@ static int finfo_set_function_names (function_info *finfo,
     if (!*changed) {
 	free_strings_array(pubnames, npub);
 	free_strings_array(privnames, npriv);
-    }
-
-    if (!err && *changed) {
-	free(finfo->pubnames);
+    } else if (!err) {
+	free_strings_array(finfo->pubnames, finfo->n_pub);
 	finfo->pubnames = pubnames;
 	finfo->n_pub = npub;
-	finfo->active = finfo->pubnames[0];
-
-	free(finfo->privnames);
+	free_strings_array(finfo->privnames, finfo->n_priv);
 	finfo->privnames = privnames;
 	finfo->n_priv = npriv;
+	finfo->active = finfo->pubnames[0];
+    }
+
+    return err;
+}
+
+static int finfo_set_function_names (function_info *finfo,
+				     const int *publist,
+				     const int *privlist)
+{
+    int npriv = (privlist == NULL)? 0 : privlist[0];
+    int err = 0;
+
+    finfo->pubnames = get_function_names(publist, &err);
+    if (!err) {
+	finfo->n_pub = publist[0];
+    }
+
+    if (!err && npriv > 0) {
+	finfo->privnames = get_function_names(privlist, &err);
+	if (!err) {
+	    finfo->n_priv = npriv;
+	}
     }
 
     return err;
@@ -789,9 +816,8 @@ static void func_selector_set_strings (function_info *finfo,
 
 static GtkWidget *active_func_selector (function_info *finfo)
 {
-    GtkWidget *ifmenu;
+    GtkWidget *ifmenu = gtk_combo_box_text_new();
 
-    ifmenu = gtk_combo_box_text_new();
     func_selector_set_strings(finfo, ifmenu);
     gtk_widget_show_all(ifmenu);
 
@@ -955,25 +981,21 @@ static void insert_today (GtkWidget *w, GtkWidget *entry)
 static gint today_popup (GtkWidget *entry, GdkEventButton *event,
 			 GtkWidget **popup)
 {
-    GdkModifierType mods = parent_get_pointer_mask(entry);
+    if (*popup == NULL) {
+	GtkWidget *menu = gtk_menu_new();
+	GtkWidget *item;
 
-    if (RIGHT_CLICK(mods)) {
-	if (*popup == NULL) {
-	    GtkWidget *menu = gtk_menu_new();
-	    GtkWidget *item;
-
-	    item = gtk_menu_item_new_with_label(_("Insert today's date"));
-	    g_signal_connect(G_OBJECT(item), "activate",
-			     G_CALLBACK(insert_today), entry);
-	    gtk_widget_show(item);
-	    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-	    *popup = menu;
-	}
-
-	gtk_menu_popup(GTK_MENU(*popup), NULL, NULL, NULL, NULL, 
-		       event->button, event->time);
+	item = gtk_menu_item_new_with_label(_("Insert today's date"));
+	g_signal_connect(G_OBJECT(item), "activate",
+			 G_CALLBACK(insert_today), entry);
+	gtk_widget_show(item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	*popup = menu;
     }
 
+    gtk_menu_popup(GTK_MENU(*popup), NULL, NULL, NULL, NULL, 
+		   event->button, event->time);
+
     return TRUE;
 }
 
@@ -1141,6 +1163,8 @@ static void finfo_dialog (function_info *finfo)
 			 G_CALLBACK(update_active_func), finfo);
     } 
 
+    update_active_func(NULL, finfo);
+
     /* save package as script button */
     button = gtk_button_new_with_label(_("Save as script"));
     gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
@@ -1320,9 +1344,9 @@ static int validate_package_file (const char *fname, int verbose)
     xmlDtdPtr dtd;
     int err = 0;
 
-    doc = xmlParseFile(fname);
-    if (doc == NULL) {
-	errbox("Couldn't parse %s", fname);
+    err = gretl_xml_open_doc_root(fname, NULL, &doc, NULL);
+    if (err) {
+	gui_errmsg(err);
 	return 1;
     }
 
@@ -1718,26 +1742,26 @@ int save_function_package_spec (const char *fname, gpointer p)
 
 static int get_lists_from_selector (int **l1, int **l2)
 {
-    int *list;
+    gchar *liststr = get_selector_storelist();
     int err = 0;
 
-    if (storelist == NULL) {
-	return E_DATA;
-    }
-
-    list = gretl_list_from_string(storelist, &err);
+    if (liststr == NULL) {
+	err = E_DATA;
+    } else {
+	int *list = gretl_list_from_string(liststr, &err);
     
-    if (!err) {
-	if (gretl_list_has_separator(list)) {
-	    err = gretl_list_split_on_separator(list, l1, l2);
-	} else {
-	    *l1 = list;
-	    list = NULL;
+	if (list != NULL) {
+	    if (gretl_list_has_separator(list)) {
+		err = gretl_list_split_on_separator(list, l1, l2);
+	    } else {
+		*l1 = list;
+		list = NULL;
+	    }
+	    free(list);
 	}
+	g_free(liststr);
     }
 
-    free(list);
-
     return err;
 }
 
@@ -1750,7 +1774,6 @@ void edit_new_function_package (void)
     function_info *finfo;
     int *publist = NULL;
     int *privlist = NULL;
-    int changed = 0;
     int err = 0;
 
     err = get_lists_from_selector(&publist, &privlist);
@@ -1764,8 +1787,7 @@ void edit_new_function_package (void)
     }
 
     if (!err) {
-	err = finfo_set_function_names(finfo, publist, privlist,
-				       &changed);
+	err = finfo_set_function_names(finfo, publist, privlist);
 	if (err) {
 	    gretl_errmsg_set("Couldn't find the requested functions");
 	}
@@ -1804,8 +1826,8 @@ void revise_function_package (void *p)
     printlist(privlist, "new privlist");
 #endif
 
-    err = finfo_set_function_names(finfo, publist, privlist,
-				   &changed);
+    err = finfo_reset_function_names(finfo, publist, privlist,
+				     &changed);
 
     if (!err && changed) {
 	depopulate_combo_box(GTK_COMBO_BOX(finfo->codesel));
@@ -1822,7 +1844,6 @@ void edit_function_package (const char *fname)
     function_info *finfo;
     int *publist = NULL;
     int *privlist = NULL;
-    int changed = 0;
     fnpkg *pkg;
     const char *p;
     int err = 0;
@@ -1854,8 +1875,7 @@ void edit_function_package (const char *fname)
 					  NULL);
 
     if (!err && publist != NULL) {
-	err = finfo_set_function_names(finfo, publist, privlist,
-				       &changed);
+	err = finfo_set_function_names(finfo, publist, privlist);
     }
 
 #if PKG_DEBUG
