@@ -20,7 +20,6 @@
 /* gretl.c : main for gretl */
 
 #include "gretl.h"
-#include "version.h"
 #include "gretl_func.h"
 #include "gretl_xml.h"
 #include "libset.h"
@@ -45,20 +44,25 @@
 #include "varinfo.h"
 #include "dlgutils.h"
 #include "fncall.h"
+#include "selector.h"
 
 #include <dirent.h>
 
 #ifndef G_OS_WIN32
 # include <unistd.h>
 # include <sys/types.h>
+# include <signal.h>
 # include "../pixmaps/gretl.xpm"  /* program icon for X */
 #else
 # include <windows.h>
 # include "gretlwin32.h"
 #endif
 
-#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6
-# define OLD_GTK
+#define GUI_DEBUG 0
+
+#if GUI_DEBUG
+# include "version.h"
+# include "build.h"
 #endif
 
 /* update.c */
@@ -66,7 +70,7 @@ extern int silent_update_query (void);
 extern int update_query (void); 
 
 /* functions private to gretl.c */
-static GtkWidget *make_main_window (void);
+static void make_main_window (void);
 
 static gboolean main_popup_handler (GtkWidget *w, GdkEventButton *event,
 				    gpointer data);
@@ -77,6 +81,7 @@ static void restore_sample_callback (void);
 static void show_sample_callback (void);
 static void mdata_select_all (void);
 static void mdata_select_list (void);
+static int gui_query_stop (void);
 
 GtkTargetEntry gretl_drag_targets[] = {
     { "text/uri-list",  0, GRETL_FILENAME },
@@ -106,86 +111,6 @@ static int optdebug;
 static gchar *param_msg = 
     N_("\nYou may supply the name of a data file on the command line");
 
-#ifdef OLD_GTK /* handle absence of GOptionEntry */
-
-struct start_opts {
-    const char *longopt;
-    char shortopt;
-    const char *optstr;
-    const char *filetype;
-};
-
-static struct start_opts options[] = {
-    { "run",     'r', N_("open a script file on startup"), "SCRIPT" },
-    { "db",      'd', N_("open a database on startup"),    "DATABASE"},
-    { "webdb",   'w', N_("open a remote (web) database on startup"), "REMOTE_DB" },
-    { "pkg",     'p', N_("open (edit) a function package on startup"), "FUNCPKG" },
-    { "english", 'e', N_("force use of English"), NULL },
-    { "basque",  'q', N_("force use of Basque"), NULL },
-    { "dump",    'c', N_("dump gretl configuration to file"), NULL },
-    { "version", 'v', N_("print version information"), NULL }, 
-    { NULL, 0, NULL, NULL },
-};
-
-static void gui_help (void) 
-{
-    int i;
-
-    putchar('\n');
-
-    for (i=0; options[i].longopt != NULL; i++) {
-	if (options[i].filetype != NULL) {
-	    printf("gretl --%s %s : %s\n", options[i].longopt, options[i].filetype, 
-		   _(options[i].optstr));
-	} else {
-	    printf("gretl --%s : %s\n", options[i].longopt, _(options[i].optstr));
-	}
-    }
-
-    printf("%s\n", _(param_msg));
-}
-
-static void old_gretl_init (int *pargc, char ***pargv, char *filearg)
-{
-    static char fname[FILENAME_MAX];
-    gretlopt opt = 0;
-    int err;
-
-    err = parseopt(pargc, pargv, &opt, fname);
-
-    if (opt & OPT_ENGLISH) {
-	opteng = 1;
-    } else if (opt & OPT_BASQUE) {
-	optbasque = 1;
-    }
-
-    if (err || (opt & (OPT_HELP | OPT_VERSION))) {
-	int ecode = (err)? EXIT_FAILURE : EXIT_SUCCESS;
-
-	gui_logo(NULL);
-	if (opt != OPT_VERSION) {
-	    gui_help();
-	}
-	exit(ecode);
-    } else if (opt & OPT_DUMP) {
-	optdump = 1;
-    } else if (opt & OPT_RUNIT) {
-	optrun = fname;
-    } else if (opt & OPT_DBOPEN) {
-	optdb = fname;
-    } else if (opt & OPT_WEBDB) {
-	optwebdb = fname;
-    } else if (opt & OPT_FNPKG) {
-	optpkg = fname;
-    } else if (*fname != '\0') {
-	/* got a data file argument? */
-	*pargc += 1;
-	strncat(filearg, fname, MAXLEN - 1);
-    }
-}
-
-#else /* newer GLib/GTK */
-
 static GOptionEntry options[] = {
     { "run", 'r', 0, G_OPTION_ARG_FILENAME, &optrun, 
       N_("open a script file on startup"), "SCRIPT" },
@@ -210,17 +135,14 @@ static GOptionEntry options[] = {
     { NULL, '\0', 0, 0, NULL, NULL, NULL },
 };
 
-#endif /* old vs new GTK switch */
-
 windata_t *mdata;
-DATAINFO *datainfo;
+DATASET *dataset;
 
 char datafile[MAXLEN];
 char scriptfile[MAXLEN];
 char tryfile[MAXLEN];
 
-double **Z;                 /* data set */
-MODEL **models;             /* gretl models structs */
+MODEL *model;             /* gretl models struct */
 
 int data_status, orig_vars;
 float gui_scale;
@@ -269,15 +191,14 @@ static void options_dialog_callback (void)
     options_dialog(0, NULL, mdata->main);
 }
 
-void open_code_callback (GtkAction *action)
+static void open_script_callback (void)
 {
-    const gchar *s = gtk_action_get_name(action);
+    file_selector(OPEN_SCRIPT, FSEL_DATA_NONE, NULL);
+}
 
-    if (!strcmp(s, "OpenScript")) {
-	file_selector(OPEN_SCRIPT, FSEL_DATA_NONE, NULL);
-    } else if (!strcmp(s, "OpenSession")) {
-	file_selector(OPEN_SESSION, FSEL_DATA_NONE, NULL);
-    }
+static void open_session_callback (void)
+{
+    file_selector(OPEN_SESSION, FSEL_DATA_NONE, NULL);
 }
 
 static void edit_package_callback (GtkAction *action, gpointer p)
@@ -346,32 +267,31 @@ static int script_type (const char *fname)
     }
 }
 
-static void fix_dbname (char *db)
+static void maybe_fix_dbname (char *dbname)
 {
     FILE *fp = NULL;
 
-    if (strstr(db, ".bin") == NULL &&
-	strstr(db, ".rat") == NULL &&
-	strstr(db, ".RAT") == NULL &&
-	strstr(db, ".bn7") == NULL) {
-	strcat(db, ".bin");
+    if (strstr(dbname, ".bin") == NULL &&
+	strstr(dbname, ".rat") == NULL &&
+	strstr(dbname, ".RAT") == NULL &&
+	strstr(dbname, ".bn7") == NULL) {
+	strcat(dbname, ".bin");
     }
 
-    fp = gretl_fopen(db, "rb");
-
-    if (fp == NULL) {
-	const char *bbase = gretl_binbase();
-
-	if (strstr(db, bbase) == NULL) {
-	    char tmp[MAXLEN];
-
-	    strcpy(tmp, db);
-	    build_path(db, bbase, tmp, NULL);
-	}
-    }
+    fp = gretl_fopen(dbname, "rb");
 
     if (fp != NULL) {
 	fclose(fp);
+    } else if (!g_path_is_absolute(dbname)) {
+	gchar *tmp = g_build_path(SLASHSTR, gretl_home(), "db",
+				  dbname, NULL);
+
+	fp = gretl_fopen(tmp, "rb");
+	if (fp != NULL) {
+	    fclose(fp);
+	    strcpy(dbname, tmp);
+	}
+	g_free(tmp);
     }
 }
 
@@ -387,11 +307,10 @@ static void real_nls_init (void)
 static void real_nls_init (void)
 {
     char localedir[MAXSTR];
-    char *loc;
 
     build_path(localedir, gretl_home(), "locale", NULL);
-    loc = setlocale(LC_ALL, "");
-    set_gretl_charset(loc);
+    setlocale(LC_ALL, "");
+    set_gretl_charset();
     bindtextdomain(PACKAGE, localedir);
     textdomain(PACKAGE);
     bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -403,7 +322,7 @@ static void real_nls_init (void)
 {
     char *gretlhome = getenv("GRETL_HOME");
     char localedir[MAXSTR];
-    char *p, *loc;
+    char *p;
 
     if (gretlhome == NULL) {
 	return;
@@ -415,9 +334,8 @@ static void real_nls_init (void)
 	strcpy(p, "share/locale");
     }
 
-    /* FIXME GUI language choice? */
-    loc = setlocale(LC_ALL, "");
-    set_gretl_charset(loc);
+    setlocale(LC_ALL, "");
+    set_gretl_charset();
     bindtextdomain(PACKAGE, localedir);
     textdomain(PACKAGE);
     bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -427,10 +345,8 @@ static void real_nls_init (void)
 
 static void real_nls_init (void)
 {
-    char *loc;
-
-    loc = setlocale(LC_ALL, "");
-    set_gretl_charset(loc);
+    setlocale(LC_ALL, "");
+    set_gretl_charset();
     bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
     bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -438,13 +354,16 @@ static void real_nls_init (void)
 
 #endif /* NLS init variants */
 
-void nls_init (void)
+void gui_nls_init (void)
 {
     char *mylang = getenv("GRETL_LANG");
 
     if (mylang != NULL) {
 	if (!g_ascii_strcasecmp(mylang, "english") ||
-	    !g_ascii_strcasecmp(mylang, "C")) return;
+	    !g_ascii_strcasecmp(mylang, "C")) {
+	    /* don't set up translation */
+	    return;
+	}
     } 
 
     real_nls_init();
@@ -466,6 +385,11 @@ static void record_filearg (char *targ, const char *src)
     }
 }
 
+static void usr1_handler (int signal) 
+{
+    fprintf(stderr, "Hello from gretl's signal handler: got SIGUSR1\n");
+}
+
 #endif
 
 /* callback from within potentially lengthy libgretl
@@ -482,7 +406,7 @@ static void gui_show_activity (void)
 
 static int have_data (void)
 {
-    return datainfo != NULL && datainfo->v > 0;
+    return dataset != NULL && dataset->v > 0;
 }
 
 int main (int argc, char **argv)
@@ -493,15 +417,15 @@ int main (int argc, char **argv)
     int ftype = 0;
     char auxname[MAXLEN];
     char filearg[MAXLEN];
-#ifndef OLD_GTK
     GError *opterr = NULL;
-#endif
 
 #ifdef G_OS_WIN32
     win32_set_gretldir(callname);
+#else
+    signal(SIGUSR1, usr1_handler);
 #endif
 
-    nls_init();
+    gui_nls_init();
 
     *tryfile = '\0';
     *scriptfile = '\0';
@@ -509,16 +433,15 @@ int main (int argc, char **argv)
     *auxname = '\0';
     *filearg = '\0';
 
-#ifdef OLD_GTK 
-    gtk_init(&argc, &argv);
-    old_gretl_init(&argc, &argv, filearg);
-#else
+#if GUI_DEBUG
+    fprintf(stderr, "starting gretl %s, %s\n", GRETL_VERSION, BUILD_DATE);
+#endif
+
     gtk_init_with_args(&argc, &argv, _(param_msg), options, "gretl", &opterr);
     if (opterr != NULL) {
 	g_print("%s\n", opterr->message);
 	exit(EXIT_FAILURE);
     }
-#endif
 
 #ifdef G_OS_WIN32
     /* let's call this before doing libgretl_init */
@@ -543,7 +466,7 @@ int main (int argc, char **argv)
 	get_runfile(optrun);
     } else if (optdb != NULL) {
 	strncat(auxname, optdb, MAXLEN - 1);
-	fix_dbname(auxname);
+	maybe_fix_dbname(auxname);
     } else if (optwebdb != NULL) {
 	strncat(auxname, optdb, MAXLEN - 1);
     } else if (optpkg != NULL) {
@@ -557,18 +480,24 @@ int main (int argc, char **argv)
 	force_language(LANG_EU);
     }
 
+#if GUI_DEBUG
+    fprintf(stderr, "finished option processing\n");
+#endif
+
+    /* set libgretl callbacks */
     set_workdir_callback(gui_set_working_dir);
     set_show_activity_func(gui_show_activity);
+    set_query_stop_func(gui_query_stop);
 
     /* allocate data information struct */
-    datainfo = datainfo_new();
-    if (datainfo == NULL) {
+    dataset = datainfo_new();
+    if (dataset == NULL) {
 	noalloc();
     }
 
     /* allocate memory for models */
-    models = allocate_working_models(3);
-    if (models == NULL) {
+    model = allocate_working_model();
+    if (model == NULL) {
 	noalloc();
     } 
 
@@ -578,6 +507,10 @@ int main (int argc, char **argv)
     session_init();
     init_fileptrs();
 
+#if GUI_DEBUG
+    fprintf(stderr, "finished miscellaneous init functions\n");
+#endif
+
     if (argc > 1) {
 	/* Process what is presumably a filename argument
 	   given on the command line (by now any options will
@@ -587,7 +520,9 @@ int main (int argc, char **argv)
 	int err = 0;
 
 	prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
-	if (err) exit(EXIT_FAILURE);
+	if (err) {
+	    exit(EXIT_FAILURE);
+	}
 
 	if (*filearg == '\0') {
 	    /* not already registered */
@@ -611,18 +546,16 @@ int main (int argc, char **argv)
 
 	switch (ftype) {
 	case GRETL_NATIVE_DATA:
-	    err = gretl_get_data(datafile, &Z, datainfo, 
-				 OPT_NONE, prn);
+	    err = gretl_get_data(datafile, dataset, OPT_NONE, prn);
 	    break;
 	case GRETL_XML_DATA:
-	    err = gretl_read_gdt(datafile, &Z, datainfo, 
-				 OPT_NONE, prn);
+	    err = gretl_read_gdt(datafile, dataset, OPT_NONE, prn);
 	    break;
 	case GRETL_CSV:
-	    err = import_csv(datafile, &Z, datainfo, 
-			     OPT_NONE, prn);
+	    err = import_csv(datafile, dataset, OPT_NONE, prn);
 	    break;
 	case GRETL_XLS:
+	case GRETL_XLSX:    
 	case GRETL_GNUMERIC:
 	case GRETL_ODS:
 	case GRETL_DTA:
@@ -644,7 +577,7 @@ int main (int argc, char **argv)
 	    strcpy(auxname, datafile);
 	    *tryfile = '\0';
 	    *datafile = '\0';
-	    fix_dbname(auxname);
+	    maybe_fix_dbname(auxname);
 	    optdb = auxname;
 	    break;
 	case GRETL_UNRECOGNIZED:
@@ -669,22 +602,40 @@ int main (int argc, char **argv)
 	gretl_print_destroy(prn);
     }
 
+#if GUI_DEBUG
+    fprintf(stderr, "about to build GUI...\n");
+#endif
+
     /* create the GUI */
-    gretl_tooltips_init();
     gretl_stock_icons_init();
+#if GUI_DEBUG
+    fprintf(stderr, " done gretl_stock_icons_init\n");
+#endif
     make_main_window();
 
+#if GUI_DEBUG
+    fprintf(stderr, "done make_main_window\n");
+#endif
+
     if (have_data()) {
 	/* redundant? */
-	set_sample_label(datainfo);
+	set_sample_label(dataset);
     }
 
     add_files_to_menus();
 
+#if GUI_DEBUG
+    fprintf(stderr, "done add_files_to_menus\n");
+#endif
+
     session_menu_state(FALSE);
     restore_sample_state(FALSE);
     main_menubar_state(FALSE);
 
+#if GUI_DEBUG
+    fprintf(stderr, "done setting GUI state\n");
+#endif
+
     /* FIXME run init script, if found? */
 
     if (have_data()) {
@@ -712,6 +663,19 @@ int main (int argc, char **argv)
 	silent_update_query(); 
     }
 
+#if 0
+    fprintf(stderr, " at last step");
+    if (optdb != NULL) {
+	fprintf(stderr, ": optdb != NULL\n");
+    } else if (optwebdb != NULL) {
+	fprintf(stderr, ": optwebdb != NULL\n");
+    } else if (optpkg != NULL) {
+	fprintf(stderr, ": optpkg != NULL\n");
+    } else {
+	fputc('\n', stderr);
+    }
+#endif
+
     /* try opening specified database or package */
     if (optdb != NULL) {
 	open_named_db_index(auxname);
@@ -721,23 +685,23 @@ int main (int argc, char **argv)
 	edit_package_at_startup(auxname);
     }
 
+#if GUI_DEBUG
+    fprintf(stderr, "calling gtk_main()\n");
+#endif
+
     /* Enter the event loop */
     gtk_main();
 
     /* clean up before exiting */
     free_session();
 
-    if (Z != NULL) {
-	free_Z(Z, datainfo);
-    }
-
-    destroy_working_models(models, 3);
+    destroy_working_model(model);
 
     library_command_free();
     libgretl_cleanup();
 
     if (data_status) {
-	free_datainfo(datainfo);
+	destroy_dataset(dataset);
     }
 
     destroy_file_collections();
@@ -816,7 +780,7 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	}
     }
 
-    if (datainfo->v == 0) {
+    if (dataset->v == 0) {
 	goto suppress;
     }
 
@@ -960,14 +924,14 @@ void populate_varlist (void)
     gtk_tree_store_clear(store);
     gtk_tree_model_get_iter_first(model, &iter);
 
-    for (i=0; i<datainfo->v; i++) {
+    for (i=0; i<dataset->v; i++) {
 	int pv = 0;
 
-	if (var_is_hidden(datainfo, i)) {
+	if (var_is_hidden(dataset, i)) {
 	    continue;
 	}
-	if (i > 0 && (is_standard_lag(i, datainfo, &pv) ||
-		      is_dummy_child(i, datainfo, &pv))) {
+	if (i > 0 && (is_standard_lag(i, dataset, &pv) ||
+		      is_dummy_child(i, dataset, &pv))) {
 	    if (pv > 0) {
 		GtkTreeIter child_iter, parent_iter;
 
@@ -977,8 +941,8 @@ void populate_varlist (void)
 		    sprintf(id, "%d", i);
 		    gtk_tree_store_set(store, &child_iter, 
 				       0, id, 
-				       1, datainfo->varname[i],
-				       2, VARLABEL(datainfo, i),
+				       1, dataset->varname[i],
+				       2, VARLABEL(dataset, i),
 				       -1);	
 		}	
 	    }
@@ -989,8 +953,8 @@ void populate_varlist (void)
 	    sprintf(id, "%d", i);
 	    gtk_tree_store_set(store, &iter, 
 			       0, id, 
-			       1, datainfo->varname[i],
-			       2, VARLABEL(datainfo, i),
+			       1, dataset->varname[i],
+			       2, VARLABEL(dataset, i),
 			       -1);
 	}
     } 
@@ -1025,7 +989,7 @@ void populate_varlist (void)
     select = gtk_tree_view_get_selection(view);
     gtk_tree_selection_select_iter(select, &iter);
 
-    if (datainfo->v > 1) {
+    if (dataset->v > 1) {
 	GtkTreePath *path;
 
 	sprintf(id, "%d", pos);
@@ -1148,21 +1112,23 @@ void clear_varlist (GtkWidget *widget)
 
 static float get_gui_scale (void)
 {
-    GtkSettings *settings;
-    gchar *fontname = NULL;
-    int fsize;
+    GtkSettings *settings = gtk_settings_get_default();
     float scale = 1.0;
 
-    settings = gtk_settings_get_default();
-    g_object_get(G_OBJECT(settings), "gtk-font-name", &fontname, NULL);
+    if (settings != NULL) {
+	gchar *fontname = NULL;
+	int fsize;
 
-    if (fontname != NULL) {
-	if (sscanf(fontname, "%*s %d", &fsize) == 1) {
-	    if (fsize > 10 && fsize < 100) {
-		scale = fsize / 10.0;
+	g_object_get(G_OBJECT(settings), "gtk-font-name", &fontname, NULL);
+
+	if (fontname != NULL) {
+	    if (sscanf(fontname, "%*s %d", &fsize) == 1) {
+		if (fsize > 10 && fsize < 100) {
+		    scale = fsize / 10.0;
+		}
 	    }
+	    g_free(fontname);
 	}
-	g_free(fontname);
     }
 
     return scale;
@@ -1184,7 +1150,7 @@ mainwin_config (GtkWidget *w, GdkEventConfigure *event, gpointer p)
    the screen dimensions
 */
 
-static void scale_main_window (void)
+static void set_main_window_scale (void)
 {
     GdkScreen *s = gdk_screen_get_default();
 
@@ -1208,7 +1174,7 @@ static void scale_main_window (void)
     }
 }
 
-static GtkWidget *make_main_window (void) 
+static void make_main_window (void) 
 {
     GtkWidget *main_vbox;
     GtkWidget *box, *dlabel;
@@ -1235,7 +1201,7 @@ static GtkWidget *make_main_window (void)
 	/* set default window size */
 	mainwin_width = 580 * gui_scale;
 	mainwin_height = 420 * gui_scale;
-	scale_main_window();
+	set_main_window_scale();
     }
 
     mdata->main = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -1275,13 +1241,16 @@ static GtkWidget *make_main_window (void)
     dlabel = gtk_label_new(_(" No datafile loaded ")); 
     g_object_set_data(G_OBJECT(mdata->main), "dlabel", dlabel);
 
-    /* will hold the list of variables */
+    /* this will hold the list of variables */
     box = gtk_vbox_new(FALSE, 0);
 
     align = gtk_alignment_new(0, 0, 0, 0);
     gtk_box_pack_start(GTK_BOX(box), align, FALSE, FALSE, 0);
-    gtk_widget_show(align);
     gtk_container_add(GTK_CONTAINER(align), dlabel);
+
+#if GUI_DEBUG
+    fprintf(stderr, " adding main-window listbox...\n");
+#endif
    
     vwin_add_list_box(mdata, GTK_BOX(box), 3, FALSE, types, titles, 1);
 
@@ -1302,39 +1271,59 @@ static GtkWidget *make_main_window (void)
     mdata->status = gtk_label_new("");
     gtk_box_pack_start(GTK_BOX(main_vbox), mdata->status, FALSE, TRUE, 0);
 
+#if GUI_DEBUG
+    fprintf(stderr, " finalizing main window...\n");
+#endif
+
     /* put stuff into list box, activate menus */
     if (have_data()) {
 	populate_varlist();
     }
 
+#if GUI_DEBUG
+    fprintf(stderr, "  step 1 done\n");
+#endif
+
     /* get a monospaced font for various windows */
     set_fixed_font();
 
-    /* and a proportional font for menus, etc */
+#if GUI_DEBUG
+    fprintf(stderr, "  set_fixed_font done\n");
+#endif
+
+    /* and a proportional font for menus, etc. */
     set_app_font(NULL);
 
+#if GUI_DEBUG
+    fprintf(stderr, "  set_app_font done\n");
+#endif
+
+    add_mainwin_toolbar(main_vbox);
+
+#if GUI_DEBUG
+    fprintf(stderr, "  add_mainwin_toolbar done\n");
+#endif
+
     gtk_widget_show_all(mdata->main); 
 
-    /* create gretl toolbar */
-    show_toolbar();
+#if GUI_DEBUG
+    fprintf(stderr, "  gtk_widget_show_all done\n");
+#endif
 
     if (winsize && main_x >= 0 && main_y >= 0) {
 	gtk_window_move(GTK_WINDOW(mdata->main), main_x, main_y);
     }
 
-    return main_vbox;
-}
-
-static void iconview_callback (void)
-{
-    view_session();
+#if GUI_DEBUG
+    fprintf(stderr, "  possible window_move done\n");
+#endif
 }
 
 GtkActionEntry main_entries[] = {
     /* File */
     { "File",         NULL, N_("_File"), NULL, NULL, NULL }, 
-    { "OpenData",       NULL, N_("_Open data"), NULL, NULL, NULL }, 
-    { "OpenGdt",        GTK_STOCK_OPEN, N_("_User file..."), NULL, NULL, G_CALLBACK(open_data) },
+    { "OpenData",     NULL, N_("_Open data"), NULL, NULL, NULL }, 
+    { "OpenGdt",      GTK_STOCK_OPEN, N_("_User file..."), NULL, NULL, G_CALLBACK(open_data) },
     { "DisplayDataFiles", GTK_STOCK_OPEN, N_("_Sample file..."), "", NULL, G_CALLBACK(show_files) },
     { "ImportData",   NULL, N_("_Import"), NULL, NULL, NULL }, 
     { "OpenCSV",      NULL, N_("_text/CSV..."), NULL, NULL, G_CALLBACK(open_data) },
@@ -1359,27 +1348,18 @@ GtkActionEntry main_entries[] = {
     { "AppendDTA",      NULL, N_("_Stata..."), NULL, NULL, G_CALLBACK(open_data) },
     { "AppendSAV",      NULL, N_("_SPSS..."), NULL, NULL, G_CALLBACK(open_data) },
     { "AppendSAS",      NULL, N_("_SAS (xport)..."), NULL, NULL, G_CALLBACK(open_data) },
-
     { "AppendJMulTi",   NULL, N_("_JMulTi..."), NULL, NULL, G_CALLBACK(open_data) },
 
     { "SaveData",  GTK_STOCK_SAVE, N_("_Save data"), "<control>S", NULL, G_CALLBACK(auto_store) },
-    { "SaveDataAs", GTK_STOCK_SAVE_AS, N_("Save data _as"), NULL, NULL, NULL }, 
-    { "SaveAsGdt", NULL, N_("_Standard format..."), NULL, NULL, G_CALLBACK(fsave_callback) },
-    { "SaveAsDb",  NULL, N_("_Database..."), NULL, NULL, G_CALLBACK(fsave_callback) },
-
-    { "ExportData", NULL, N_("_Export data"), NULL, NULL, NULL },
-    { "ExportCSV",    NULL, N_("_CSV..."), NULL, NULL, G_CALLBACK(fsave_callback) },
-    { "ExportR",      NULL, N_("GNU _R..."), NULL, NULL, G_CALLBACK(fsave_callback) },
-    { "ExportOctave", NULL, N_("_Octave..."), NULL, NULL, G_CALLBACK(fsave_callback) },
-    { "ExportJMulTi", NULL, N_("_JMulTi..."), NULL, NULL, G_CALLBACK(fsave_callback) },
-    { "ExportPcGive", NULL, N_("_PcGive..."), NULL, NULL, G_CALLBACK(fsave_callback) },
+    { "SaveDataAs", GTK_STOCK_SAVE_AS, N_("Save data _as..."), NULL, NULL, G_CALLBACK(fsave_callback) }, 
+    { "ExportData", NULL, N_("_Export data..."), NULL, NULL, G_CALLBACK(fsave_callback) },
 
     { "MailData", GRETL_STOCK_MAIL, N_("Send To..."), NULL, NULL, G_CALLBACK(email_data) },
     { "NewData", GTK_STOCK_NEW, N_("_New data set"), NULL, NULL, G_CALLBACK(newdata_callback) },
     { "ClearData", GTK_STOCK_CLEAR, N_("C_lear data set"), NULL, NULL, G_CALLBACK(verify_clear_data) },
     { "WorkingDir", NULL, N_("_Working directory..."), NULL, NULL, G_CALLBACK(working_dir_dialog) },
     { "ScriptFiles", NULL, N_("_Script files"), NULL, NULL, NULL },
-    { "OpenScript", GTK_STOCK_OPEN, N_("_User file..."), "", NULL, G_CALLBACK(open_code_callback) },
+    { "OpenScript", GTK_STOCK_OPEN, N_("_User file..."), "", NULL, G_CALLBACK(open_script_callback) },
     { "DisplayScripts", GTK_STOCK_OPEN, N_("_Practice file..."), "", NULL, G_CALLBACK(show_files) },
     { "NewScript", GTK_STOCK_NEW, N_("_New script"), "", NULL, NULL },
     { "GretlScript", NULL, N_("gretl script"), NULL, NULL, G_CALLBACK(new_script_callback) },
@@ -1389,7 +1369,7 @@ GtkActionEntry main_entries[] = {
     { "OxScript", NULL, N_("Ox program"), NULL, NULL, G_CALLBACK(new_script_callback) },
 
     { "SessionFiles", NULL, N_("_Session files"), NULL, NULL, NULL },
-    { "OpenSession", GTK_STOCK_OPEN, N_("_Open session..."), "", NULL, G_CALLBACK(open_code_callback) },
+    { "OpenSession", GTK_STOCK_OPEN, N_("_Open session..."), "", NULL, G_CALLBACK(open_session_callback) },
     { "SaveSession", GTK_STOCK_SAVE, N_("_Save session"), "", NULL, 
       G_CALLBACK(save_session_callback) },
     { "SaveSessionAs", GTK_STOCK_SAVE_AS, N_("Save session _as..."), NULL, NULL, 
@@ -1456,7 +1436,7 @@ GtkActionEntry main_entries[] = {
 
     /* View */
     { "View", NULL, N_("_View"), NULL, NULL, NULL },
-    { "IconView", NULL, N_("_Icon view"), NULL, NULL, G_CALLBACK(iconview_callback) },
+    { "IconView", NULL, N_("_Icon view"), NULL, NULL, G_CALLBACK(view_session) },
     { "EditScalars", NULL, N_("_Scalars"), NULL, NULL, G_CALLBACK(edit_scalars) },   
     { "Windows", NULL, N_("_Windows"), NULL, NULL, NULL },
     { "GraphVars", NULL, N_("_Graph specified vars"), NULL, NULL, NULL },
@@ -1466,7 +1446,7 @@ GtkActionEntry main_entries[] = {
     { "FactorPlot", NULL, N_("X-Y with _factor separation..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "FrischPlot", NULL, N_("X-Y with _control..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "GR_BOX", NULL, N_("_Boxplots..."), NULL, NULL, G_CALLBACK(gretl_callback) },
-    { "GR_NBOX", NULL, N_("_Notched boxplots..."), NULL, NULL, G_CALLBACK(gretl_callback) },
+    { "GR_FBOX", NULL, N_("_Factorized boxplot..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "GR_QQ", NULL, N_("_Q-Q plot..."), NULL, NULL, G_CALLBACK(menu_op_action) },
     { "ThreeDPlot", NULL, N_("_3D plot..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "MultiPlots", NULL, N_("_Multiple graphs"), NULL, NULL, NULL },
@@ -1612,10 +1592,11 @@ GtkActionEntry main_entries[] = {
     /* Help */
     { "Help", NULL, N_("_Help"), NULL, NULL, NULL },
     { "TextCmdRef", GRETL_STOCK_BOOK, N_("_Command reference"), NULL, NULL, G_CALLBACK(plain_text_cmdref) },
-    { "FuncRef",   NULL, N_("Function reference"), NULL, NULL, G_CALLBACK(genr_funcs_ref) },
+    { "FuncRef",   NULL, N_("_Function reference"), NULL, NULL, G_CALLBACK(genr_funcs_ref) },
     { "UserGuide", GRETL_STOCK_PDF, N_("_User's guide"), NULL, NULL, G_CALLBACK(display_pdf_help) },
     { "PDFCmdRef", GRETL_STOCK_PDF, N_("_Command reference"), NULL, NULL, G_CALLBACK(display_pdf_help) },
     { "UpdateCheck", GTK_STOCK_NETWORK, N_("Check for _updates"), NULL, NULL, G_CALLBACK(update_query) },
+    { "SFAddons", NULL, N_("Check for _addons"), NULL, NULL, G_CALLBACK(show_files) },
     { "About", GTK_STOCK_ABOUT, N_("_About gretl"), NULL, NULL, G_CALLBACK(about_dialog) }
 };
 
@@ -1673,21 +1654,23 @@ static void add_conditional_items (windata_t *vwin)
 
 static gchar *get_main_ui (void)
 {
-    char fname[FILENAME_MAX];
     gchar *main_ui = NULL;
+    gchar *fname;
+    int err;
 
-    sprintf(fname, "%sui%cgretlmain.xml", gretl_home(), SLASH);
-    gretl_file_get_contents(fname, &main_ui, NULL);
+    fname = g_strdup_printf("%sui%cgretlmain.xml", gretl_home(), SLASH);
+    err = gretl_file_get_contents(fname, &main_ui, NULL);
+    g_free(fname);
 
-    return main_ui;
+    return err ? NULL : main_ui;
 }
 
 static int set_up_main_menu (void)
 {
     GtkActionGroup *actions;
-    GtkWidget *dataitem;
     gchar *main_ui = NULL;
     GError *error = NULL;
+    int err = 0;
 
     main_ui = get_main_ui();
     if (main_ui == NULL) {
@@ -1709,18 +1692,29 @@ static int set_up_main_menu (void)
     if (!gtk_ui_manager_add_ui_from_string(mdata->ui, main_ui, -1, &error)) {
 	g_message("building menus failed: %s", error->message);
 	g_error_free(error);
+	err = 1;
     } else {
 	add_conditional_items(mdata);
+	mdata->mbar = gtk_ui_manager_get_widget(mdata->ui, "/menubar");
+	if (mdata->mbar == NULL) {
+	    fprintf(stderr, "/menubar widget is NULL!\n");
+	    err = 1;
+	} else {
+	    GtkWidget *dataitem;
+
+	    dataitem = gtk_ui_manager_get_widget(mdata->ui, "/menubar/Data");
+	    if (dataitem == NULL) {
+		fprintf(stderr, "/menubar/Data widget is NULL!\n");
+		err = 1;
+	    } else {
+		g_signal_connect(G_OBJECT(dataitem), "activate",
+				 G_CALLBACK(check_var_labels_state), mdata);
+		g_free(main_ui);
+	    }
+	}
     }
 
-    g_free(main_ui);
-    mdata->mbar = gtk_ui_manager_get_widget(mdata->ui, "/menubar");
-
-    dataitem = gtk_ui_manager_get_widget(mdata->ui, "/menubar/Data");
-    g_signal_connect(G_OBJECT(dataitem), "activate",
-		     G_CALLBACK(check_var_labels_state), mdata);
-
-    return 0;
+    return err;
 }
 
 void raise_main_window (void)
@@ -1747,9 +1741,8 @@ static const gchar *window_list_icon (int role)
 	id = GRETL_STOCK_MODEL;
     } else if (role == CONSOLE) {
 	id = GRETL_STOCK_CONSOLE;
-    } else if (role == EDIT_SCRIPT || 
-	       role == EDIT_PKG_CODE ||
-	       role == EDIT_PKG_SAMPLE) {
+    } else if (role >= EDIT_HEADER && 
+	       role < EDIT_MAX) {
 	id = GTK_STOCK_EDIT;
     } else if (role == GNUPLOT) {
 	id = GRETL_STOCK_SCATTER;
@@ -1761,6 +1754,8 @@ static const gchar *window_list_icon (int role)
 	id = GRETL_STOCK_CALC;
     } else if (role == VIEW_SCRIPT) {
 	id = GTK_STOCK_EXECUTE;
+    } else if (role == OPEN_SESSION) {
+	id = GRETL_STOCK_ICONS;
     }
 
     return id;
@@ -1843,7 +1838,7 @@ void add_window_list_item (GtkWidget *w, int role)
     guint merge_id;
 
     if (window_list == NULL) {
-	/* create the window group */
+	/* create the window_list group */
 	window_list = gtk_action_group_new("WindowList");
 	gtk_ui_manager_insert_action_group(mdata->ui, window_list, 0);
     }
@@ -1947,12 +1942,12 @@ void window_list_popup (GtkWidget *src)
     }
 }
 
-int gui_restore_sample (double ***pZ, DATAINFO *pdinfo)
+int gui_restore_sample (DATASET *dset)
 {
     int err = 0;
 
-    if (pZ != NULL && *pZ != NULL) {
-	err = restore_full_sample(pZ, pdinfo, NULL);
+    if (dset != NULL && dset->Z != NULL) {
+	err = restore_full_sample(dset, NULL);
 	if (err) {
 	    gui_errmsg(err);
 	} else {
@@ -1965,10 +1960,10 @@ int gui_restore_sample (double ***pZ, DATAINFO *pdinfo)
 
 static void restore_sample_callback (void)
 {
-    int err = gui_restore_sample(&Z, datainfo); 
+    int err = gui_restore_sample(dataset); 
 
     if (!err) {
-	set_sample_label(datainfo);    
+	set_sample_label(dataset);    
 	gretl_command_strcpy("smpl --full");
 	check_and_record_command();
     }
@@ -1983,7 +1978,7 @@ static void show_sample_callback (void)
 	return;
     }
 
-    print_sample_status(datainfo, prn);
+    print_sample_status(dataset, prn);
     title = g_strdup_printf("gretl: %s", _("sample status"));
     view_buffer(prn, 78, 360, title, PRINT, NULL);
     g_free(title);
@@ -2064,7 +2059,11 @@ void set_wm_icon (GtkWidget *w)
 {
     GdkPixbuf *icon = gdk_pixbuf_new_from_xpm_data(gretl_xpm);
 
-    gtk_window_set_icon(GTK_WINDOW(w), icon);
+    if (icon != NULL) {
+	gtk_window_set_icon(GTK_WINDOW(w), icon);
+    } else {
+	fprintf(stderr, "Couldn't create icon for gretl_xpm\n");
+    }
 }
 
 #endif
@@ -2158,18 +2157,12 @@ static void auto_store (void)
 	/* the data file is embedded in a session file */
 	save_session_dataset();
     } else {
-	/* by default, use gzip compression */
-	gretlopt oflag = OPT_Z;
-
-	/* but if there's already a datafile, and it's not gzipped, then
-	   arrange for the new file to be uncompressed too
-	*/
-	if (*datafile && !is_gzipped(datafile)) {
-	    oflag = OPT_NONE;
-	}
+	/* ensure there's no stale selection around */
+	set_selector_storelist(NULL);
 
 	if ((data_status & USER_DATA) && has_suffix(datafile, ".gdt")) {
-	    do_store(datafile, oflag);
+	    /* bypass filename selection */
+	    do_store(datafile, SAVE_DATA);
 	} else {
 	    file_selector(SAVE_DATA, FSEL_DATA_NONE, NULL);
 	}
@@ -2227,3 +2220,30 @@ main_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer data)
 
     return FALSE;
 }
+
+static int script_stopper (int set)
+{
+    static int stop;
+    int ret = 0;
+
+    if (set) {
+	/* set the stop signal */
+	stop = 1;
+    } else if (stop) {
+	warnbox(_("Execution aborted by request"));
+	ret = 1;
+	stop = 0;
+    }
+
+    return ret;
+}
+
+static int gui_query_stop (void)
+{
+    return script_stopper(0);
+}
+
+void do_stop_script (GtkWidget *w, windata_t *vwin)
+{
+    script_stopper(1);
+}
