@@ -42,23 +42,11 @@
 #include <time.h>
 
 #ifndef _WIN32
-# if GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 10
-#  define GTK_PRINTING
-# endif
-#endif
-
-#if defined(_WIN32) || defined(GTK_PRINTING)
-# define NATIVE_PRINTING
-#endif
-
-#if GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 8
-# define USE_GTK_SPINNER 1
+# define GTK_PRINTING
 #endif
 
 #define GNULL (gconstpointer) NULL
 
-#include <gtksourceview/gtksourceview.h>
-
 /* fixups for GTK3 */
 
 #if GTK_MAJOR_VERSION >= 3
@@ -109,16 +97,14 @@
 #define MAXSTR FILENAME_MAX
 
 /* basic global program vars */
-extern double **Z;
-extern DATAINFO *datainfo;
-extern MODEL **models;
+extern DATASET *dataset;
+extern MODEL *model;
 
 /* global counters */
 extern int orig_vars;
 
 /* global state variables */
 extern int data_status;
-extern char *storelist;
 extern float gui_scale;
 
 /* global filenames */
@@ -169,12 +155,13 @@ int mdata_active_var (void);
 void populate_varlist (void);
 void clear_varlist (GtkWidget *widget);
 void mdata_select_last_var (void);
-int gui_restore_sample (double ***pZ, DATAINFO *pdinfo);
+int gui_restore_sample (DATASET *dset);
 
 void add_window_list_item (GtkWidget *w, int role);
 void window_list_popup (GtkWidget *src);
 void raise_main_window (void);
 int get_n_listed_windows (void);
+void do_stop_script (GtkWidget *w, windata_t *vwin);
 
 /* functions defined in files other than gretl.c */
 void about_dialog (void);
