@@ -25,7 +25,7 @@ enum extra_cmds {
     GR_DUMMY,
     GR_XYZ,
     GR_BOX,
-    GR_NBOX,
+    GR_FBOX,
     GR_3D,
     COMPACT,
     EXPAND, 
@@ -59,6 +59,7 @@ enum extra_cmds {
     EDIT_OX,
     EDIT_OCTAVE,
     EDIT_X12A,
+    EDIT_MAX,
     CLI_HELP,
     GUI_HELP,
     CLI_HELP_EN,
@@ -121,6 +122,7 @@ enum file_ops {
     OPEN_OCTAVE,
     OPEN_GNUMERIC,
     OPEN_XLS,
+    OPEN_XLSX,
     OPEN_ODS,
     OPEN_WF1,
     OPEN_DTA,
@@ -135,6 +137,7 @@ enum file_ops {
     APPEND_OCTAVE,
     APPEND_GNUMERIC,
     APPEND_XLS,
+    APPEND_XLSX,
     APPEND_ODS,
     APPEND_WF1,
     APPEND_DTA,
@@ -149,12 +152,13 @@ enum file_ops {
     END_OPEN,        /* marker for end of file open section */
     SAVE_DATA,
     SAVE_DATA_AS,
-    SAVE_DBDATA,
     EXPORT_OCTAVE,
     EXPORT_R,
     EXPORT_CSV,
     EXPORT_DAT,
     EXPORT_JM,
+    EXPORT_DB,
+    EXPORT_GDT,
     COPY_CSV,
     END_SAVE_DATA,  /* marker for end of data-saving section */
     SAVE_TEX,
@@ -202,6 +206,7 @@ enum browser_codes {
     RATS_SERIES,
     PCGIVE_SERIES,
     REMOTE_SERIES,
+    REMOTE_ADDONS,
     MAINWIN
 };
 
