@@ -228,7 +228,7 @@ static void dummy_output_handler (const gchar *log_domain,
 				  const gchar *message,
 				  gpointer user_data)
 {
-    fprintf(stderr, "%s : %s\n", log_domain, message);
+    return;
 }
 
 static void stderr_output_handler (const gchar *log_domain,
@@ -236,13 +236,15 @@ static void stderr_output_handler (const gchar *log_domain,
 				   const gchar *message,
 				   gpointer user_data)
 {
-    return;
+    fprintf(stderr, "%s : %s\n", log_domain, message);
 }
 
-static void set_g_warnings (int debug)
+static void set_g_logging (int debug)
 {
     void (*handler) (const gchar *, GLogLevelFlags,
 		     const gchar *, gpointer);
+     GLogLevelFlags flags = G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING |
+	 G_LOG_LEVEL_DEBUG;
 
     if (debug) {
 	handler = stderr_output_handler;
@@ -250,18 +252,11 @@ static void set_g_warnings (int debug)
 	handler = dummy_output_handler;
     }
 
-    g_log_set_handler("Gtk",
-		      G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING,
-		      (GLogFunc) handler, NULL);
-    g_log_set_handler("Gdk",
-		      G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING,
-		      (GLogFunc) handler, NULL);
-    g_log_set_handler("GLib",
-		      G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING,
-		      (GLogFunc) handler, NULL);
-    g_log_set_handler("Pango",
-		      G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING,
-		      (GLogFunc) handler, NULL);
+    g_log_set_handler("Gtk", flags, (GLogFunc) handler, NULL);
+    g_log_set_handler("Gdk", flags, (GLogFunc) handler, NULL);
+    g_log_set_handler("GLib", flags, (GLogFunc) handler, NULL);
+    g_log_set_handler("Pango", flags, (GLogFunc) handler, NULL);
+    g_log_set_handler("GtkSourceView", flags, (GLogFunc) handler, NULL);
 }
 
 char *default_windows_menu_fontspec (void)
@@ -363,7 +358,7 @@ void gretl_win32_debug_init (int debug)
         redirect_io_to_console();
     }
 
-    set_g_warnings(debug);
+    set_g_logging(debug);
 }
 
 /* carry out some Windows-specific start-up tasks, and
@@ -375,7 +370,10 @@ void gretl_win32_init (const char *progname, int debug)
     wimp_init();
     read_win32_config(debug);
     set_gretl_startdir();
-    ws_startup(); 
+    ws_startup();
+    if (debug) {
+	fprintf(stderr, "gretl_win32_init: done\n");
+    }
 }
 
 static int win_copy_buf (const char *buf, int fmt, size_t buflen)
@@ -672,7 +670,9 @@ static int win32_open_arg (const char *arg, char *ext)
 
 		lstrcat(p, " ");
 		lstrcat(p, arg);
-		if (WinExec(key, SW_SHOW) < 32) err = 1;
+		if (WinExec(key, SW_SHOW) < 32) {
+		    err = 1;
+		}
 	    }
 	}
     }
