@@ -99,7 +99,7 @@ static struct gui_help_item gui_help_items[] = {
     { GR_XYZ,         "controlled" },
     { BXPLOT,         "boxplots" },
     { GR_BOX,         "boxplots" },
-    { GR_NBOX,        "boxplots" },
+    { GR_FBOX,        "boxplots" },
     { GR_3D,          "3-D" },
     { ONLINE,         "online" },
     { MARKERS,        "markers" },
@@ -239,21 +239,13 @@ static int compat_command_number (const char *s)
     return -1;
 }
 
-static void set_en_help_file (int gui)
+static void set_en_help_file (int h)
 {
-    const char *helpfile;
     char *tmp, *p;
 
-    if (gui) {
-	helpfile = helpfile_path(GRETL_HELPFILE);
-    } else {
-	helpfile = helpfile_path(GRETL_CMD_HELPFILE);
-    }
-
-    tmp = malloc(strlen(helpfile) + 1);
+    tmp = gretl_strdup(helpfile_path(h));
 
     if (tmp != NULL) {
-	strcpy(tmp, helpfile);
 #ifdef G_OS_WIN32
 	p = strrchr(tmp, '_');
 	if (p != NULL) strcpy(p, ".txt");
@@ -261,7 +253,7 @@ static void set_en_help_file (int gui)
 	p = strrchr(tmp, '.');
 	if (p != NULL) *p = 0;
 #endif
-	if (gui) {
+	if (h == GRETL_HELPFILE) {
 	    en_gui_helpfile = tmp;
 	} else {
 	    en_cli_helpfile = tmp;
@@ -274,25 +266,23 @@ void helpfile_init (void)
     const char *hpath = helpfile_path(GRETL_HELPFILE);
     char *p;
 
+    translated_helpfile = 0;
+
 #ifdef G_OS_WIN32
     p = strrchr(hpath, '_');
     if (p != NULL && strncmp(p, "_hlp", 4)) { 
 	translated_helpfile = 1;
-    } else {
-	translated_helpfile = 0;
-    }
+    } 
 #else
     p = strrchr(hpath, '.');
     if (p != NULL && strcmp(p, ".hlp")) { 
 	translated_helpfile = 1;
-    } else {
-	translated_helpfile = 0;
-    }
+    } 
 #endif
 
-    if (translated_helpfile == 1) {
-	set_en_help_file(0);
-	set_en_help_file(1);
+    if (translated_helpfile) {
+	set_en_help_file(GRETL_CMD_HELPFILE);
+	set_en_help_file(GRETL_HELPFILE);
     }
 }
 
@@ -741,7 +731,7 @@ static void normalize_base (GtkWidget *w, gpointer p)
     gtk_widget_modify_base(w, GTK_STATE_SELECTED, NULL);
 }
 
-static void notify_not_found (GtkWidget *entry)
+void notify_string_not_found (GtkWidget *entry)
 {
     GdkColor color;
 
@@ -880,6 +870,33 @@ static int all_lower_case (const char *s)
     return 1;
 }
 
+static int maybe_go_to_page (windata_t *vwin)
+{
+    gboolean ret = FALSE;
+    int idx, pos = 0; 
+
+    if (vwin->role == FUNCS_HELP) {
+	/* looking for a function */
+	idx = function_help_index_from_word(needle);
+	if (idx > 0) {
+	    pos = help_pos_from_index(idx, vwin->role);
+	}
+    } else {
+	/* looking for a command */
+	idx = command_help_index(needle);
+	if (idx > 0) {
+	    pos = help_pos_from_index(idx, vwin->role);
+	}
+    } 
+
+    if (pos > 0) {
+	real_do_help(idx, pos, vwin->role);
+	ret = TRUE;
+    }    
+
+    return ret;
+}
+
 /* respond to Enter key in the 'finder' entry */
 
 static void vwin_finder_callback (GtkEntry *entry, windata_t *vwin)
@@ -906,32 +923,10 @@ static void vwin_finder_callback (GtkEntry *entry, windata_t *vwin)
 	found = real_find_in_listbox(vwin, needle, sensitive, 0);
     }
 
-    if (!found) {
-	if (vwin->role == CLI_HELP || vwin->role == CLI_HELP_EN) {
-	    /* are we looking for a command? */
-	    int idx = command_help_index(needle);
-
-	    if (idx > 0) {
-		int pos = help_pos_from_index(idx, vwin->role);
-
-		if (pos > 0) {
-		    real_do_help(idx, pos, vwin->role);
-		    found = TRUE;
-		}
-	    }
-	} else if (vwin->role == FUNCS_HELP) {
-	    /* are we looking for a function? */
-	    int idx = function_help_index_from_word(needle);
-
-	    if (idx > 0) {
-		int pos = help_pos_from_index(idx, vwin->role);
-
-		if (pos > 0) {
-		    real_do_help(idx, pos, vwin->role);
-		    found = TRUE;
-		}
-	    }
-	}
+    if (!found && (vwin->role == CLI_HELP || 
+		   vwin->role == CLI_HELP_EN ||
+		   vwin->role == FUNCS_HELP)) {
+	found = maybe_go_to_page(vwin);
     }
 
     if (!found && search_all) {
@@ -942,7 +937,7 @@ static void vwin_finder_callback (GtkEntry *entry, windata_t *vwin)
     }
 
     if (!found) {
-	notify_not_found(GTK_WIDGET(entry));
+	notify_string_not_found(GTK_WIDGET(entry));
     }
 }
 
@@ -1632,9 +1627,9 @@ static void find_in_text (GtkWidget *button, GtkWidget *dialog)
 
     found = real_find_in_text(GTK_TEXT_VIEW(vwin->text), needle,
 			      sensitive, TRUE, FALSE);
-
+    
     if (!found) {
-	notify_not_found(find_entry);
+	notify_string_not_found(find_entry);
     }
 }
 
@@ -1782,7 +1777,7 @@ static void find_in_listbox (GtkWidget *w, GtkWidget *dialog)
     found = real_find_in_listbox(vwin, needle, sensitive, vnames);
 
     if (!found) {
-	notify_not_found(find_entry);
+	notify_string_not_found(find_entry);
     }
 }
 
@@ -1872,13 +1867,14 @@ static void find_string_dialog (void (*findfunc)(), windata_t *vwin)
 
     hbox = gtk_dialog_get_action_area(GTK_DIALOG(find_dialog));
 
-    /* cancel button */
-    button = cancel_button(hbox);
+    /* Close button */
+    button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+    gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(cancel_find), find_dialog);
     gtk_widget_show(button);
 
-    /* find button */
+    /* Find button */
     button = gtk_button_new_from_stock(GTK_STOCK_FIND);
     gtk_widget_set_can_default(button, TRUE);
     gtk_container_add(GTK_CONTAINER(hbox), button);
