@@ -73,7 +73,9 @@ void do_run_script (GtkWidget *w, windata_t *vwin);
 
 void run_script_fragment (windata_t *vwin, gchar *buf);
 
-void gui_errmsg (const int errcode);
+void gui_errmsg (int errcode);
+
+void gui_warnmsg (int errcode);
 
 void register_graph (PRN *prn);
 
@@ -268,6 +270,8 @@ void do_graph_var (int varnum);
 
 void do_boxplot_var (int varnum, gretlopt opt);
 
+int do_factorized_boxplot (selector *sr);
+
 void ts_plot_callback (void);
 
 int do_scatters (selector *sr);
@@ -304,7 +308,7 @@ void do_new_script (int code);
 
 void new_script_callback (GtkAction *action);
 
-int do_store (char *mydatfile, gretlopt oflag);
+int do_store (char *mydatfile, int action);
 
 void view_latex (PRN *prn);
 
@@ -312,7 +316,7 @@ void save_latex (PRN *prn, const char *fname);
 
 void do_save_text (char *fname, MODEL *pmod);
 
-int gui_exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo);
+int gui_exec_line (ExecState *s, DATASET *dset);
 
 int check_and_record_command (void);
 
