@@ -10,6 +10,7 @@ const char *special_keyword[] = {
     "return", 
     "while",
     "elif",
+    "eval",
     "const",
     "3sls",
     "liml",
@@ -29,6 +30,8 @@ const char *gretl_data_types[] = {
     "series",
     "matrix",
     "list",
+    "string",
+    "bundle",
     "null",
     "void",
     NULL
@@ -403,10 +406,10 @@ int main (int argc, char **argv)
 {
     if (argc == 2 && !strcmp(argv[1], "--emacs")) {
 	output_emacs_block();
-    } else if (argc == 2 && !strcmp(argv[1], "--lang2")) {
-	output_lang2_file();
-    } else {
+    } else if (argc == 2 && !strcmp(argv[1], "--lang1")) {
 	output_lang1_file();
+    } else {
+	output_lang2_file();
     }
 
     return 0;
