@@ -86,14 +86,15 @@ typedef struct SESSION_GRAPH_ SESSION_GRAPH;
 typedef struct gui_obj_ gui_obj;
 
 enum {
-    SESSION_CHANGED  = 1 << 0,
-    SESSION_SAVED    = 1 << 1
+    SESSION_CHANGED    = 1 << 0,
+    SESSION_SAVED      = 1 << 1
 };
 
 struct SESSION_ {
     char name[MAXLEN];
     char dirname[MAXLEN];
     int status;
+    int show_notes;
     int nmodels;
     int ngraphs;
     int ntexts;
@@ -185,7 +186,6 @@ static char *graph_items[] = {
 
 static char *dataset_items[] = {
     N_("Edit"),
-    N_("Save..."),
     N_("Export as CSV..."),
     N_("Copy as CSV...")
 };
@@ -826,10 +826,10 @@ int gui_add_graph_to_session (char *fname, char *fullname, int type)
     char graphname[MAXSAVENAME];
     int err = 0;
 
-    if (type != GRETL_OBJ_PLOT && Z == NULL) {
+    if (type != GRETL_OBJ_PLOT && (dataset == NULL || dataset->Z == NULL)) {
 	/* we may be called via the "stats calculator" when
 	   there's no dataset yet */
-	err = open_nulldata(&Z, datainfo, 0, 10, NULL);
+	err = open_nulldata(dataset, 0, 10, NULL);
 	if (err) {
 	    gui_errmsg(err);
 	    return 1;
@@ -1362,7 +1362,7 @@ void do_open_session (void)
     }
 
     if (!err) {
-	err = gretl_read_gdt(gdtname, &Z, datainfo, OPT_B, NULL);
+	err = gretl_read_gdt(gdtname, dataset, OPT_B, NULL);
     }
 
     if (err) {
@@ -1402,13 +1402,13 @@ void do_open_session (void)
     err = maybe_read_settings_file(fname);
 
     if (sinfo.resample_n > 0) {
-	err = dataset_resample(sinfo.resample_n, sinfo.seed, &Z, 
-			       datainfo);
+	err = dataset_resample(sinfo.resample_n, sinfo.seed, 
+			       dataset);
     } else if (sinfo.mask != NULL) {
-	err = restrict_sample_from_mask(sinfo.mask, &Z, datainfo,
+	err = restrict_sample_from_mask(sinfo.mask, dataset,
 					OPT_NONE);
 	if (!err) {
-	    datainfo->restriction = sinfo.restriction;
+	    dataset->restriction = sinfo.restriction;
 	    sinfo.restriction = NULL;
 	}
     }
@@ -1418,13 +1418,13 @@ void do_open_session (void)
 	goto bailout;
     }
 
-    datainfo->t1 = sinfo.t1;
-    datainfo->t2 = sinfo.t2;
+    dataset->t1 = sinfo.t1;
+    dataset->t2 = sinfo.t2;
 
     register_data(OPENED_VIA_SESSION);
 
     if (sinfo.mask != NULL) {
-	set_sample_label(datainfo);
+	set_sample_label(dataset);
     }
 
     sinfo_free_data(&sinfo);
@@ -1445,6 +1445,10 @@ void do_open_session (void)
 	view_session();
 	mark_session_saved();
 	session_switch_log_location(LOG_OPEN);
+
+	if (session.show_notes) {
+	    edit_session_notes();
+	}
     }
 }
 
@@ -1568,7 +1572,7 @@ int highest_numbered_variable_in_session (void)
 	}
 	type = session.models[i]->type;
 	if (type == GRETL_OBJ_EQN) {
-	    mvm = highest_numbered_var_in_model((MODEL *) ptr, datainfo);
+	    mvm = highest_numbered_var_in_model((MODEL *) ptr, dataset);
 	    if (mvm > vmax) {
 		vmax = mvm;
 	    }
@@ -1579,7 +1583,7 @@ int highest_numbered_variable_in_session (void)
 	    }
 	} else if (type == GRETL_OBJ_SYS) {
 	    mvm = highest_numbered_var_in_system((equation_system *) ptr, 
-						 datainfo);
+						 dataset);
 	    if (mvm > vmax) {
 		vmax = mvm;
 	    }
@@ -1598,12 +1602,12 @@ void gui_clear_dataset (void)
 {
     *datafile = 0;
 
-    if (Z != NULL) {
-	free_Z(Z, datainfo);
-	Z = NULL;
+    if (dataset->Z != NULL) {
+	free_Z(dataset);
+	dataset->Z = NULL;
     } 
 
-    clear_datainfo(datainfo, CLEAR_FULL);
+    clear_datainfo(dataset, CLEAR_FULL);
 
     clear_varlist(mdata->listbox);
     clear_sample_label();
@@ -1613,15 +1617,13 @@ void gui_clear_dataset (void)
     main_menubar_state(FALSE);
 }
 
-static void session_clear_data (double ***pZ, DATAINFO *pdinfo)
+static void session_clear_data (DATASET *pdinfo)
 {
-    gui_restore_sample(pZ, pdinfo);
+    gui_restore_sample(pdinfo);
     gui_clear_dataset();
 
-    /* clear protected models */
-    clear_model(models[0]);
-    clear_model(models[1]);
-    clear_model(models[2]);
+    /* clear protected model */
+    clear_model(model);
 
     free_command_stack(); 
     set_model_count(0);
@@ -1636,9 +1638,11 @@ void close_session (gretlopt opt)
     fprintf(stderr, "close_session: starting cleanup\n");
 #endif
 
-    if (datainfo != NULL && datainfo->v > 0) {
-	logcode = LOG_CLOSE;
-	session_clear_data(&Z, datainfo); 
+    if (!(opt & OPT_O)) {
+	if (dataset != NULL && dataset->v > 0) {
+	    logcode = LOG_CLOSE;
+	    session_clear_data(dataset); 
+	}
     }
 
     free_session();
@@ -1655,6 +1659,7 @@ void close_session (gretlopt opt)
     }
 
     session.status = 0;
+    session.show_notes = 0;
     commands_recorded = 0;
 
     winstack_destroy();
@@ -1665,6 +1670,8 @@ void close_session (gretlopt opt)
 
     if (opt & OPT_P) {
 	libgretl_session_cleanup(SESSION_CLEAR_DATASET);
+    } else if (opt & OPT_O) {
+	libgretl_session_cleanup(SESSION_CLEAR_OTHER);
     } else {
 	libgretl_session_cleanup(SESSION_CLEAR_ALL);
     }
@@ -1697,29 +1704,28 @@ static int real_save_session_dataset (const char *dname)
 {
     char tmpname[MAXLEN];
     char *mask = NULL;
-    int save_t1 = datainfo->t1;
-    int save_t2 = datainfo->t2;
+    int save_t1 = dataset->t1;
+    int save_t2 = dataset->t2;
     int write_err = 0;
     int err = 0;
 
     /* we need to retrieve and save the full version of the dataset */
 
     if (complex_subsampled()) {
-	mask = copy_datainfo_submask(datainfo, &err);
-	if (!err && dataset_is_resampled(datainfo)) {
+	mask = copy_datainfo_submask(dataset, &err);
+	if (!err && dataset_is_resampled(dataset)) {
 	    /* can't happen? */
 	    mask = NULL;
 	}
     }
 
     if (!err) {
-	err = restore_full_sample(&Z, datainfo, NULL);
+	err = restore_full_sample(dataset, NULL);
     }
 
     if (!err) {
 	session_file_make_path(tmpname, dname);
-	write_err = gretl_write_gdt(tmpname, NULL, (const double **) Z, 
-				    datainfo, OPT_NONE, 1);
+	write_err = gretl_write_gdt(tmpname, NULL, dataset, OPT_NONE, 1);
 	fprintf(stderr, "Save session datafile as '%s', err = %d\n",
 		tmpname, write_err);
     }
@@ -1727,13 +1733,13 @@ static int real_save_session_dataset (const char *dname)
     if (mask != NULL) {
 	/* reset the prior subsample */
 	if (!err) {
-	    err = restrict_sample_from_mask(mask, &Z, datainfo, OPT_NONE);
+	    err = restrict_sample_from_mask(mask, dataset, OPT_NONE);
 	}
 	free(mask);
     }
 
-    datainfo->t1 = save_t1;
-    datainfo->t2 = save_t2;
+    dataset->t1 = save_t1;
+    dataset->t2 = save_t2;
 
     if (!err) {
 	err = write_err;
@@ -1878,7 +1884,7 @@ int save_session (char *fname)
 	    session_name_from_session_file(session.name, fname);
 	    strcpy(sessionfile, fname);
 	    data_status |= SESSION_DATA;
-	    set_sample_label(datainfo);
+	    set_sample_label(dataset);
 	}
 	mark_session_saved();
     }
@@ -1886,6 +1892,25 @@ int save_session (char *fname)
     return err;
 }
 
+void session_notes_callback (GtkWidget *w, gpointer p)
+{
+    const char *opts[] = {
+	N_("Display notes on opening session file")
+    };
+    int active[] = {0};
+    int resp;
+
+    active[0] = session.show_notes;
+
+    resp = checks_only_dialog("gretl", NULL, opts, 1,
+			      active, 0);
+
+    if (resp >= 0 && session.show_notes != active[0]) {
+	session.show_notes = active[0];
+	mark_session_changed();
+    }
+}
+
 void save_session_callback (GtkAction *action)
 {
     int as_is = 0;
@@ -2032,12 +2057,12 @@ static int display_session_model (SESSION_MODEL *sm)
     if (sm->type == GRETL_OBJ_EQN) {
 	MODEL *pmod = (MODEL *) sm->ptr;
 
-	printmodel(pmod, datainfo, OPT_NONE, prn);
+	printmodel(pmod, dataset, OPT_NONE, prn);
 	view_model(prn, pmod, 78, 400, sm->name);
     } else if (sm->type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) sm->ptr;
 
-	gretl_VAR_print(var, datainfo, OPT_NONE, prn);
+	gretl_VAR_print(var, dataset, OPT_NONE, prn);
 	view_buffer(prn, 78, 450, sm->name, var->ci, var);
     } else if (sm->type == GRETL_OBJ_SYS) {
 	equation_system *sys = (equation_system *) sm->ptr;
@@ -2242,36 +2267,52 @@ static int delete_session_object (gui_obj *obj)
     }
 
     session_delete_icon(obj);
+    mark_session_changed();
 
     return 0;
 }
 
+/* run a sanity check on deleting a session object (e.g.
+   model, graph) before proceeding */
+
 static void maybe_delete_session_object (gui_obj *obj)
 {
-    gpointer p = NULL;
-    gchar *msg;
+    int busy = 0;
 
-    if (obj->sort == GRETL_OBJ_EQN || obj->sort == GRETL_OBJ_SYS || 
-	obj->sort == GRETL_OBJ_VAR) {
-	SESSION_MODEL *mod = obj->data;
+    if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
+	SESSION_GRAPH *graph = (SESSION_GRAPH *) obj->data;
+	char fullname[MAXLEN];
 
-	p = mod->ptr;
+	session_file_make_path(fullname, graph->fname);
+	busy = plot_file_is_busy(fullname);
+	if (!busy && match_window_by_filename(fullname)) {
+	    busy = 1;
+	}
     } else {
-	p = obj->data;
+	gpointer p = NULL;
+
+	if (obj->sort == GRETL_OBJ_EQN || obj->sort == GRETL_OBJ_SYS || 
+	    obj->sort == GRETL_OBJ_VAR) {
+	    SESSION_MODEL *mod = obj->data;
+
+	    p = mod->ptr;
+	} else {
+	    p = obj->data;
+	}
+
+	busy = (p != NULL && winstack_match_data(p));
     }
 
-    if (p != NULL && winstack_match_data(p)) {
+    if (busy) {
 	warnbox(_("Please close this object's window first"));
-	return;
+    } else {	    
+	gchar *msg = g_strdup_printf(_("Really delete %s?"), obj->name);
+
+	if (yes_no_dialog(_("gretl: delete"), msg, 0) == GRETL_YES) {
+	    delete_session_object(obj);
+	}
+	g_free(msg);
     }
-
-    msg = g_strdup_printf(_("Really delete %s?"), obj->name);
-
-    if (yes_no_dialog(_("gretl: delete"), msg, 0) == GRETL_YES) {
-	delete_session_object(obj);
-    }
-
-    g_free(msg);
 }
 
 static gui_obj *get_gui_obj_by_data (void *targ)
@@ -2988,8 +3029,6 @@ static void data_popup_callback (GtkWidget *widget, gpointer data)
 
     if (!strcmp(item, _("Edit"))) {
 	show_spreadsheet(SHEET_EDIT_DATASET);
-    } else if (!strcmp(item, _("Save..."))) {
-	file_save(mdata, SAVE_DATA);
     } else if (!strcmp(item, _("Export as CSV..."))) {
 	file_save(mdata, EXPORT_CSV);
     } else if (!strcmp(item, _("Copy as CSV..."))) {
@@ -3008,14 +3047,15 @@ static void scalars_popup_callback (GtkWidget *widget, gpointer data)
     }
 }
 
-static void object_set_window_title (windata_t *vwin, gui_obj *obj)
+static gchar *object_get_window_title (gui_obj *obj)
 {
-    if (vwin != NULL && obj != NULL) {
-	gchar *title = g_strdup_printf("gretl: %s", obj->name);
+    gchar *title = NULL;
 
-	gtk_window_set_title(GTK_WINDOW(vwin->main), title);
-	g_free(title);	
+    if (obj != NULL) {
+	title = g_strdup_printf("gretl: %s", obj->name);
     }
+
+    return title;
 }
 
 static void object_popup_callback (GtkWidget *widget, gpointer data)
@@ -3042,13 +3082,16 @@ static void object_popup_callback (GtkWidget *widget, gpointer data)
 	if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
 	    SESSION_GRAPH *graph = (SESSION_GRAPH *) obj->data;
 	    char fullname[MAXLEN];
+	    gchar *title;
 	    windata_t *vwin;
 
 	    gretl_chdir(gretl_dotdir());
 	    session_file_make_path(fullname, graph->fname);
 	    remove_png_term_from_plot_by_name(fullname);
-	    vwin = view_file(fullname, 1, 0, 78, 400, EDIT_GP);
-	    object_set_window_title(vwin, obj);
+	    title = object_get_window_title(obj);
+	    vwin = view_file_with_title(fullname, 1, 0, 78, 400, 
+					EDIT_GP, title);
+	    g_free(title);
 	    /* add flag so we can mark the session as modified
 	       if the plot file is changed */
 	    vwin->flags |= VWIN_SESSION_GRAPH;
@@ -3508,6 +3551,8 @@ void view_session (void)
     gtk_widget_show(hbox);
     gtk_widget_show(iconview);
 
+    add_window_list_item(iconview, OPEN_SESSION);
+
     gtk_container_foreach(GTK_CONTAINER(scroller), 
 			  (GtkCallback) white_bg_style, 
 			  NULL);
