@@ -27,12 +27,8 @@ static int check_graph_file (const char *fname, int type)
 	    fgets(line, sizeof line, fp);
 	    fclose(fp);
 	    if (!strncmp(line, "# boxplot generated", 19)) {
-		err = gnuplot_from_boxplot(fullname);
-		if (err) {
-		    fprintf(stderr, "Failed to convert old boxplot file\n");
-		} else {
-		    fprintf(stderr, "Converted old boxplot file OK\n");
-		}
+		fprintf(stderr, "Ignoring old boxplot file\n");
+		err = 1;
 	    }
 	} else {
 	    fclose(fp);
@@ -226,7 +222,6 @@ static int rebuild_session_model (const char *fname,
 				  (type == GRETL_OBJ_VAR)? "gretl-VAR" :
 				  "gretl-equation-system", &doc, &node);
     if (err) {
-	fprintf(stderr, "Failed on gretl_xml_open_doc_root\n");
 	return err;
     }
 
@@ -237,11 +232,11 @@ static int rebuild_session_model (const char *fname,
     }
 
     if (type == GRETL_OBJ_EQN) {
-	ptr = gretl_model_from_XML(node, doc, datainfo, &err);
+	ptr = gretl_model_from_XML(node, doc, dataset, &err);
     } else if (type == GRETL_OBJ_VAR) {
-	ptr = gretl_VAR_from_XML(node, doc, datainfo, &err);
+	ptr = gretl_VAR_from_XML(node, doc, dataset, &err);
     } else {
-	ptr = equation_system_from_XML(node, doc, datainfo, &err);
+	ptr = equation_system_from_XML(node, doc, &err);
     }
 
     xmlFreeDoc(doc);
@@ -357,9 +352,6 @@ static int get_session_datafile_name (const char *fname, struct sample_info *sin
     xmlChar *tmp;
     int err = 0;
 
-    LIBXML_TEST_VERSION
-	xmlKeepBlanksDefault(0);
-
     err = gretl_xml_open_doc_root(fname, "gretl-session", &doc, &cur);
     if (err) {
 	gui_errmsg(err);
@@ -376,7 +368,6 @@ static int get_session_datafile_name (const char *fname, struct sample_info *sin
 
     if (doc != NULL) {
 	xmlFreeDoc(doc);
-	xmlCleanupParser();
     }
 
     return err;
@@ -394,14 +385,10 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
     int object_errs = 0;
     int err = 0;
 
-    LIBXML_TEST_VERSION
-	xmlKeepBlanksDefault(0);
-
     err = gretl_xml_open_doc_root(fname, "gretl-session", &doc, &cur);
-
     if (err) {
 	gui_errmsg(err);
-	return 1;
+	return err;
     }
 
     /* Now walk the tree */
@@ -473,6 +460,12 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
 		(char *) xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
 	    if (session.notes == NULL) {
 		object_errs++;
+	    } else {
+		tmp = xmlGetProp(cur, (XUC) "auto-show");
+		if (tmp != NULL) {
+		    session.show_notes = 1;
+		    free(tmp);
+		}
 	    }
 	}
 	if (!err) {
@@ -482,7 +475,6 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
 
     if (doc != NULL) {
 	xmlFreeDoc(doc);
-	xmlCleanupParser();
     }
 
     if (!err && object_errs > 0) {
@@ -748,8 +740,8 @@ static int write_session_xml (const char *datname)
 	fputs("<gretl-session>\n", fp);
     }
 
-    fprintf(fp, " <sample t1=\"%d\" t2=\"%d\"/>\n", datainfo->t1, datainfo->t2);
-    write_datainfo_submask(datainfo, fp);
+    fprintf(fp, " <sample t1=\"%d\" t2=\"%d\"/>\n", dataset->t1, dataset->t2);
+    write_datainfo_submask(dataset, fp);
 
     nmodels = session.nmodels;
     tabmodels = model_table_n_models();
@@ -889,7 +881,11 @@ static int write_session_xml (const char *datname)
     fputs(" </texts>\n", fp);
 
     if (session.notes != NULL) {
-	fputs("<notes>", fp);
+	if (session.show_notes) {
+	    fputs("<notes auto-show=\"true\">", fp);
+	} else {
+	    fputs("<notes>", fp);
+	}
 	gretl_xml_put_raw_string(session.notes, fp);
 	fputs("</notes>\n", fp);
     } 
