@@ -18,6 +18,7 @@
  */
 
 #include "gretl.h"
+#include "version.h"
 #include "filelists.h"
 #include "gretl_www.h"
 #include "dlgutils.h"
@@ -28,7 +29,6 @@
 #include "ssheet.h"
 
 #include "libset.h"
-#include "version.h"
 #include "texprint.h"
 
 #include <unistd.h>
@@ -294,7 +294,7 @@ const char *get_scriptpage (void)
 
 int autoicon_on (void)
 {
-    if (datainfo != NULL && datainfo->v > 0) {
+    if (dataset != NULL && dataset->v > 0) {
 	return autoicon;
     } else {
 	return 0;
@@ -386,7 +386,9 @@ void set_app_font (const char *fontname)
 {
     GtkSettings *settings;
 
-    if (fontname != NULL && *fontname == 0) return;
+    if (fontname != NULL && *fontname == '\0') {
+	return;
+    }
 
     settings = gtk_settings_get_default();
 
@@ -394,8 +396,10 @@ void set_app_font (const char *fontname)
     g_object_set(G_OBJECT(settings), "gtk-menu-images", TRUE, NULL);
 
     if (fontname == NULL) {
+	/* just loading the default font */
 	g_object_set(G_OBJECT(settings), "gtk-font-name", appfontname, NULL);
     } else {
+	/* loading a user-specified font: check that it works */
 	GtkWidget *w;
 	PangoFontDescription *pfd;
 	PangoContext *pc;
@@ -407,6 +411,7 @@ void set_app_font (const char *fontname)
 	pfont = pango_context_load_font(pc, pfd);
 
 	if (pfont != NULL) {
+	    /* OK, found it */
 	    strcpy(appfontname, fontname);
 	    g_object_set(G_OBJECT(settings), "gtk-font-name", appfontname, NULL);
 	}
@@ -438,41 +443,6 @@ static int write_OK (gchar *dirname)
     return ok;
 }
 
-static void get_pkg_dir (char *dirname, int action)
-{
-    const char *subdir = NULL;
-    int ok = 0;
-
-    if (action == SAVE_FUNCTIONS) {
-	subdir = "functions";
-    } else if (action == SAVE_DATA_PKG) {
-	subdir = "data";
-    } else {
-	return;
-    }
-
-    /* try 'system' location first */
-
-    sprintf(dirname, "%s%s", gretl_home(), subdir);
-    ok = write_OK(dirname);
-
-    if (!ok) {
-	/* try user's filespace */
-	const char *wdir = gretl_default_workdir();
-
-	if (wdir == NULL) {
-	    wdir = gretl_workdir();
-	}
-
-	sprintf(dirname, "%s%s", wdir, subdir);
-	ok = write_OK(dirname);
-    }
-
-    if (!ok) {
-	*dirname = '\0';
-    }
-}
-
 void set_gretl_startdir (void)
 {
     if (usecwd) {
@@ -504,12 +474,47 @@ void set_gretl_startdir (void)
     }
 }
 
+static void get_pkg_save_dir (char *dirname, int action)
+{
+    const char *subdir = NULL;
+    int ok = 0;
+
+    if (action == SAVE_FUNCTIONS) {
+	subdir = "functions";
+    } else if (action == SAVE_DATA_PKG) {
+	subdir = "data";
+    } else {
+	return;
+    }
+
+    /* try 'system' location first */
+
+    sprintf(dirname, "%s%s", gretl_home(), subdir);
+    ok = write_OK(dirname);
+
+    if (!ok) {
+	/* try user's filespace */
+	const char *wdir = maybe_get_default_workdir();
+
+	if (wdir == NULL) {
+	    wdir = gretl_workdir();
+	}
+
+	sprintf(dirname, "%s%s", wdir, subdir);
+	ok = write_OK(dirname);
+    }
+
+    if (!ok) {
+	*dirname = '\0';
+    }
+}
+
 void get_default_dir (char *s, int action)
 {
     *s = '\0';
 
     if (action == SAVE_FUNCTIONS || action == SAVE_DATA_PKG) {
-	get_pkg_dir(s, action);
+	get_pkg_save_dir(s, action);
     } else if (action == OPEN_RATS_DB) {
 	strcpy(s, gretl_ratsbase());
     } else {
@@ -534,7 +539,7 @@ static const char *get_reg_base (const char *key)
 
 #ifdef OSX_BUILD
 static int alt_ok (const char *prog)
-{ 
+{
     char *p, test[MAXSTR];
     int tr = strstr(prog, "tramo") != NULL;
     int ok;
@@ -574,7 +579,7 @@ static int tramo_ok = 0;
 
 int get_tramo_ok (void)
 {
-    return tramo_ok && tramo_ts(datainfo);
+    return tramo_ok && tramo_ts(dataset);
 }
 
 static void set_tramo_status (void)
@@ -617,7 +622,7 @@ static int x12a_ok = 0;
 
 int get_x12a_ok (void)
 {
-    return x12a_ok && x12_ts(datainfo);
+    return x12a_ok && x12_ts(dataset);
 }
 
 static void set_x12a_status (void)
@@ -935,14 +940,14 @@ void set_path_callback (char *setvar, char *setting)
 
 static void browse_button_callback (GtkWidget *w, RCVAR *rc)
 {
-    GtkWidget *top = g_object_get_data(G_OBJECT(w), "parent");
+    GtkWidget *parent = g_object_get_data(G_OBJECT(w), "parent");
     int code = SET_PROG;
 
     if (strstr(rc->description, "directory") != NULL) {
 	code = SET_DIR;
     }
 
-    file_selector_with_parent(code, FSEL_DATA_MISC, rc->var, top);
+    file_selector_with_parent(code, FSEL_DATA_MISC, rc->var, parent);
 }
 
 static GtkWidget *make_path_browse_button (RCVAR *rc, GtkWidget *w)
@@ -1201,7 +1206,6 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		    gtk_widget_set_sensitive(rc_vars[i-1].widget, FALSE);
 		}
 	    }
-
 	} else if (rc->flags & BOOLSET) { 
 	    /* radio-button dichotomy */
 	    int rcval = *(int *) (rc->var);
@@ -1570,9 +1574,12 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 	    } else if ((rcvar->flags & USERSET) || 
 		       (rcvar->flags & MACHSET) ||
 		       (rcvar->flags & MACHSET)) {
-		const gchar *str = gtk_entry_get_text(GTK_ENTRY(w));
+		gchar *str = entry_box_get_trimmed_text(w);
 
-		rcvar_set_string(rcvar, str, &changed);
+		if (str != NULL) {
+		    rcvar_set_string(rcvar, str, &changed);
+		    g_free(str);
+		} 
 	    } else if (rcvar->flags & LISTSET) {
 		GtkComboBox *box = GTK_COMBO_BOX(w);
 
@@ -1598,6 +1605,11 @@ static void apply_changes (GtkWidget *widget, gpointer data)
     maybe_revise_tramo_x12a_status();
 #endif
 
+    if (use_proxy && *dbproxy == '\0') {
+	/* fix inconsistency */
+	use_proxy = 0;
+    }
+
     write_rc(); /* note: calls gretl_update_paths */
 
     /* register these for session using libset apparatus */
@@ -1694,11 +1706,7 @@ static int dir_exists (const char *dname, FILE *fp)
     DIR *test;
     int ok = 0;
 
-#ifdef G_OS_WIN32
-    test = win32_opendir(dname);
-#else
-    test = opendir(dname);
-#endif
+    test = gretl_opendir(dname);
 
     if (test != NULL) {
 	ok = 1;
@@ -1769,7 +1777,7 @@ static int common_read_rc_setup (void)
 
     err = gretl_set_paths(&paths, set_paths_opt);
     if (err) {
-	/* tell the user, then turn of the special alarm */
+	/* tell the user, then turn off the special alarm */
 	gui_errmsg(err);
 	set_gretl_alarm(0);
     }
@@ -2022,6 +2030,10 @@ int read_win32_config (int debug)
     set_fixed_font();
     set_app_font(NULL);
 
+    if (debug) {
+	fprintf(stderr, "read_win32_config: returning %d\n", err);
+    }
+
     return err;
 }
 
@@ -2034,40 +2046,39 @@ int read_win32_config (int debug)
 
 static int read_gretlrc (void) 
 {
-    char line[MAXLEN], key[32], linevar[MAXLEN];
-    int got_recent = 0;
-    FILE *fp;
-
-    fp = fopen(rcfile, "r");
+    FILE *fp = fopen(rcfile, "r");
 
     if (fp == NULL) {
 	fprintf(stderr, "Couldn't read %s\n", rcfile);
-	return common_read_rc_setup();
-    }
+    } else {
+	char line[MAXLEN], key[32], linevar[MAXLEN];
+	int got_recent = 0;
 
-    while (fgets(line, sizeof line, fp) != NULL) {
-	if (line[0] == '#') {
-	    continue;
-	}
-	if (!strncmp(line, "recent", 6)) {
-	    got_recent = 1;
-	    break;
-	}
-	if (sscanf(line, "%s", key) == 1) {
-	    strcpy(linevar, line + strlen(key) + 3); 
-	    chopstr(linevar); 
-	    if (*linevar) {
-		find_and_set_rc_var(key, linevar);
+	while (fgets(line, sizeof line, fp)) {
+	    if (*line == '#') {
+		continue;
 	    }
+	    if (!strncmp(line, "recent", 6)) {
+		got_recent = 1;
+		break;
+	    }
+	    if (sscanf(line, "%31s", key) == 1) {
+		*linevar = '\0';
+		strncat(linevar, line + strlen(key) + 3, MAXLEN-1); 
+		chopstr(linevar); 
+		if (*linevar != '\0') {
+		    find_and_set_rc_var(key, linevar);
+		}
+	    }
+	}	
+
+	if (got_recent) {
+	    rc_read_file_lists(fp, line);
 	}
-    }	
 
-    if (got_recent) {
-	rc_read_file_lists(fp, line);
+	fclose(fp);
     }
 
-    fclose(fp);
-
     return common_read_rc_setup();
 }
 
@@ -2084,88 +2095,9 @@ static int fontsel_code (GtkAction *action)
     }
 }
 
-/* font selection: non-Windows, gtk-2.0 version first */
+/* font selection: Windows version first */
 
-#ifndef G_OS_WIN32
-
-static void font_selection_ok (GtkWidget *w, GtkFontselHackDialog *fs)
-{
-    guint which = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(fs), "which"));
-    gchar *fontname;
-
-    fontname = gtk_fontsel_hack_dialog_get_font_name(fs);
-
-    if (*fontname == 0) {
-	g_free(fontname);
-	gtk_widget_destroy(GTK_WIDGET(fs));
-	return;
-    }
-
-    if (which == FIXED_FONT_SELECTION) {
-	strcpy(fixedfontname, fontname);
-	set_fixed_font();
-	write_rc();
-    } else if (which == APP_FONT_SELECTION) {
-	set_app_font(fontname);
-	write_rc();
-    }
-
-    g_free(fontname);
-    gtk_widget_destroy(GTK_WIDGET(fs));
-}
-
-void font_selector (GtkAction *action)
-{
-    static GtkWidget *fontsel = NULL;
-    int which = fontsel_code(action);
-    int filter = GTK_FONT_HACK_LATIN;
-    char *title = NULL;
-    const char *fontname = NULL;
-
-    if (fontsel != NULL) {
-	gtk_window_present(GTK_WINDOW(fontsel));
-        return;
-    }
-
-    if (which == FIXED_FONT_SELECTION) {
-	title = _("Font for gretl output windows");
-	filter = GTK_FONT_HACK_LATIN_MONO;
-	fontname = fixedfontname;
-    } else if (which == APP_FONT_SELECTION) {
-	title = _("Font for menus and labels");
-	fontname = appfontname;
-    }
-
-    fontsel = gtk_fontsel_hack_dialog_new(title);
-    gtk_fontsel_hack_dialog_set_filter
-	(GTK_FONTSEL_HACK_DIALOG(fontsel), filter);
-    gtk_fontsel_hack_dialog_set_font_name 
-	(GTK_FONTSEL_HACK_DIALOG(fontsel), fontname); 
-    g_object_set_data(G_OBJECT(fontsel), "which", GINT_TO_POINTER(which));
-
-    gtk_window_set_position(GTK_WINDOW(fontsel), GTK_WIN_POS_MOUSE);
-
-    g_signal_connect(G_OBJECT(fontsel), "destroy",
-		     G_CALLBACK(gtk_widget_destroyed),
-		     &fontsel);
-    g_signal_connect(G_OBJECT(fontsel), "destroy",
-		     G_CALLBACK(gtk_main_quit),
-		     NULL);
-    g_signal_connect(G_OBJECT(gtk_fontsel_hack_dialog_ok_button(fontsel)),
-		     "clicked", 
-		     G_CALLBACK(font_selection_ok),
-		     fontsel);
-    g_signal_connect(G_OBJECT(gtk_fontsel_hack_dialog_cancel_button(fontsel)),
-		     "clicked", 
-		     G_CALLBACK(delete_widget),
-		     fontsel);
-
-    gtk_widget_show(fontsel);
-
-    gtk_main();
-}
-
-#else /* end non-win32 font selection, start win32 */
+#ifdef G_OS_WIN32
 
 void font_selector (GtkAction *action)
 {
@@ -2192,7 +2124,76 @@ void font_selector (GtkAction *action)
     }
 }
 
-#endif /* end win32 font selection */
+#else /* not MS Windows */
+
+static void font_selection_ok (GtkWidget *w, GtkFontselHackDialog *fs)
+{
+    gchar *fontname;
+
+    fontname = gtk_fontsel_hack_dialog_get_font_name(fs);
+
+    if (fontname != NULL && *fontname != '\0') {
+	int filter = gtk_fontsel_hack_dialog_get_filter(fs); 
+
+	if (filter == GTK_FONT_HACK_LATIN_MONO) {
+	    strcpy(fixedfontname, fontname);
+	    set_fixed_font();
+	    write_rc();
+	} else if (filter == GTK_FONT_HACK_LATIN) {
+	    set_app_font(fontname);
+	    write_rc();
+	}
+    }
+
+    g_free(fontname);
+    gtk_widget_destroy(GTK_WIDGET(fs));
+}
+
+void font_selector (GtkAction *action)
+{
+    static GtkWidget *fontsel = NULL;
+    int filter, which = fontsel_code(action);
+    char *title = NULL;
+    const char *fontname = NULL;
+
+    if (fontsel != NULL) {
+	gtk_window_present(GTK_WINDOW(fontsel));
+        return;
+    }
+
+    if (which == FIXED_FONT_SELECTION) {
+	title = _("Font for gretl output windows");
+	filter = GTK_FONT_HACK_LATIN_MONO;
+	fontname = fixedfontname;
+    } else if (which == APP_FONT_SELECTION) {
+	title = _("Font for menus and labels");
+	filter = GTK_FONT_HACK_LATIN;
+	fontname = appfontname;
+    }
+
+    fontsel = gtk_fontsel_hack_dialog_new(title);
+
+    gtk_fontsel_hack_dialog_set_filter(GTK_FONTSEL_HACK_DIALOG(fontsel), 
+				       filter);
+    gtk_fontsel_hack_dialog_set_font_name(GTK_FONTSEL_HACK_DIALOG(fontsel), 
+					  fontname); 
+
+    gtk_window_set_position(GTK_WINDOW(fontsel), GTK_WIN_POS_MOUSE);
+
+    g_signal_connect(G_OBJECT(fontsel), "destroy",
+		     G_CALLBACK(gtk_widget_destroyed),
+		     &fontsel);
+    g_signal_connect(G_OBJECT(gtk_fontsel_hack_dialog_ok_button(fontsel)),
+		     "clicked", G_CALLBACK(font_selection_ok),
+		     fontsel);
+    g_signal_connect(G_OBJECT(gtk_fontsel_hack_dialog_cancel_button(fontsel)),
+		     "clicked", G_CALLBACK(delete_widget),
+		     fontsel);
+
+    gtk_widget_show(fontsel);
+}
+
+#endif /* end font selection dialog switch */
 
 void update_persistent_graph_colors (void)
 {
