@@ -19,20 +19,19 @@
 
 #include "gretl.h"
 #include "textbuf.h"
-#include "gretl_func.h"
 #include "toolbar.h"
 #include "dlgutils.h"
+#include "gretl_func.h"
 
-#if defined(USE_GTKSOURCEVIEW_2) || defined(USE_GTKSOURCEVIEW_3)
-# define NEWER_SOURCEVIEW
+#ifdef G_OS_WIN32
+# include "gretlwin32.h" /* for browser_open() */
 #endif
 
 #include <gtksourceview/gtksourceview.h>
-#include <gtksourceview/gtksourcelanguage.h>
-#ifdef NEWER_SOURCEVIEW
-# include <gtksourceview/gtksourcelanguagemanager.h>
-#else
-# include <gtksourceview/gtksourcelanguagesmanager.h>
+#include <gtksourceview/gtksourcelanguagemanager.h>
+#include <gtksourceview/gtksourceprintcompositor.h>
+#ifdef PKGBUILD
+# include <gtksourceview/gtksourcestyleschememanager.h>
 #endif
 
 #ifdef USE_GTKSOURCEVIEW_3
@@ -43,6 +42,7 @@
 #define SCRIPT_PAGE 998
 #define GFR_PAGE    997
 #define BIB_PAGE    996
+#define EXT_PAGE    995
 
 enum {
     PLAIN_TEXT,
@@ -275,52 +275,26 @@ int viewer_char_count (windata_t *vwin)
 
 void text_paste (GtkWidget *w, windata_t *vwin)
 {
-    gchar *undo_buf = textview_get_text(vwin->text);
-    gchar *old;
-
-    old = g_object_get_data(G_OBJECT(vwin->text), "undo");
-    g_free(old);
-
-    g_object_set_data(G_OBJECT(vwin->text), "undo", undo_buf);
-
     gtk_text_buffer_paste_clipboard(gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text)),
 				    gtk_clipboard_get(GDK_NONE),
 				    NULL, TRUE);
 }
 
+void text_redo (GtkWidget *w, windata_t *vwin)
+{
+    if (vwin->sbuf != NULL && gtk_source_buffer_can_redo(vwin->sbuf)) {
+	gtk_source_buffer_redo(vwin->sbuf);
+    } else {
+	warnbox(_("No redo information available"));
+    }
+}
+
 void text_undo (GtkWidget *w, windata_t *vwin)
 {
-    gchar *old = NULL;
-
-    if (vwin->sbuf != NULL) {
-	if (gtk_source_buffer_can_undo(vwin->sbuf)) {
-	    gtk_source_buffer_undo(vwin->sbuf);
-	} else {
-	    errbox(_("No undo information available"));
-	}
-	return;
-    }
-    
-    old = g_object_steal_data(G_OBJECT(vwin->text), "undo");
-
-    if (old == NULL) {
-	errbox(_("No undo information available"));
+    if (vwin->sbuf != NULL && gtk_source_buffer_can_undo(vwin->sbuf)) {
+	gtk_source_buffer_undo(vwin->sbuf);
     } else {
-	GtkTextBuffer *buf;
-	GtkTextIter start, end;
-	GtkTextMark *ins;
-
-	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
-	ins = gtk_text_buffer_get_insert(buf);
-
-	gtk_text_buffer_get_start_iter(buf, &start);
-	gtk_text_buffer_get_end_iter(buf, &end);
-	gtk_text_buffer_delete(buf, &start, &end);
-
-	gtk_text_buffer_insert(buf, &start, old, -1);
-	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(vwin->text), 
-				     ins, 0.0, TRUE, 0.1, 0.0);
-	g_free(old);
+	warnbox(_("No undo information available"));
     }
 }
 
@@ -329,9 +303,7 @@ int text_can_undo (windata_t *vwin)
     if (vwin->sbuf != NULL) {
 	return gtk_source_buffer_can_undo(vwin->sbuf);
     } else {
-	gchar *old = g_object_get_data(G_OBJECT(vwin->text), "undo");
-
-	return old != NULL;
+	return 0;
     }
 }
 
@@ -425,8 +397,6 @@ static int source_buffer_load_buf (GtkSourceBuffer *sbuf, const char *buf)
     return 0;
 }
 
-#ifdef NEWER_SOURCEVIEW
-
 static void sourceview_apply_language (windata_t *vwin)
 {
     GtkSourceLanguageManager *lm; 
@@ -435,6 +405,10 @@ static void sourceview_apply_language (windata_t *vwin)
 
     lm = g_object_get_data(G_OBJECT(vwin->sbuf), "languages-manager");
 
+    if (lm == NULL) {
+	return;
+    }
+
     if (vwin->role == EDIT_GP) {
 	id = "gnuplot";
     } else if (vwin->role == EDIT_R) {
@@ -456,40 +430,86 @@ static void sourceview_apply_language (windata_t *vwin)
     }
 }
 
-#else /* use gtksourceview-1.0 API */
-
-static void sourceview_apply_language (windata_t *vwin)
+static void begin_print (GtkPrintOperation *operation,
+                         GtkPrintContext *context,
+                         gpointer data)
 {
-    GtkSourceLanguagesManager *lm; 
-    GtkSourceLanguage *lang = NULL;
-    const char *mtype = NULL;
+    GtkSourcePrintCompositor *comp;
+    int n_pages;
 
-    lm = g_object_get_data(G_OBJECT(vwin->sbuf), "languages-manager");
+    comp = GTK_SOURCE_PRINT_COMPOSITOR(data);
 
-    if (vwin->role == EDIT_GP) {
-	mtype = "application/x-gnuplot";
-    } else if (vwin->role == EDIT_R) {
-	mtype = "text/x-R";
-    } else if (vwin->role == EDIT_OX) {
-	mtype = "text/x-c++src";
-    } else if (vwin->role == EDIT_OCTAVE) {
-	mtype = "text/x-octave";
-    } else {
-	mtype = "application/x-gretlscript";
-    }
+    while (!gtk_source_print_compositor_paginate(comp, context));
 
-    lang = gtk_source_languages_manager_get_language_from_mime_type(lm, mtype);
-
-    if (lang == NULL) {
-	fprintf(stderr, "*** gtksourceview: lang is NULL for mtype='%s'\n", mtype);
-	g_object_set(G_OBJECT(vwin->sbuf), "highlight", FALSE, NULL);
-    } else {
-	g_object_set(G_OBJECT(vwin->sbuf), "highlight", TRUE, NULL);
-	gtk_source_buffer_set_language(vwin->sbuf, lang);
-    }
+    n_pages = gtk_source_print_compositor_get_n_pages(comp);
+    gtk_print_operation_set_n_pages(operation, n_pages);
 }
 
-#endif
+static void draw_page (GtkPrintOperation *operation,
+		       GtkPrintContext *context,
+		       gint page_nr,
+		       gpointer data)
+{
+    GtkSourcePrintCompositor *comp =
+	GTK_SOURCE_PRINT_COMPOSITOR(data);
+
+    gtk_source_print_compositor_draw_page(comp, context,
+					  page_nr);
+}
+
+static GtkPrintSettings *settings = NULL;
+
+void sourceview_print (windata_t *vwin)
+{
+    GtkSourceView *view = GTK_SOURCE_VIEW(vwin->text);
+    GtkSourcePrintCompositor *comp;
+    GtkPrintOperation *print;
+    GtkPrintOperationResult res;
+    GError *error = NULL;
+
+    comp = gtk_source_print_compositor_new_from_view(view);
+    print = gtk_print_operation_new();
+
+    if (settings != NULL) {
+	gtk_print_operation_set_print_settings(print, settings);
+    } else {
+	gtk_source_print_compositor_set_right_margin(comp, 54, GTK_UNIT_POINTS);
+	gtk_source_print_compositor_set_left_margin(comp, 54, GTK_UNIT_POINTS);
+	gtk_source_print_compositor_set_top_margin(comp, 54, GTK_UNIT_POINTS);
+	gtk_source_print_compositor_set_bottom_margin(comp, 72, GTK_UNIT_POINTS);
+    }
+
+    g_signal_connect(print, "begin_print", G_CALLBACK(begin_print), comp);
+    g_signal_connect(print, "draw_page", G_CALLBACK(draw_page), comp);
+
+    res = gtk_print_operation_run(print, 
+				  GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
+				  GTK_WINDOW(vwin->main), 
+				  &error);
+
+    if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
+	GtkWidget *dlg; 
+
+	dlg = gtk_message_dialog_new(GTK_WINDOW(vwin->main),
+				     GTK_DIALOG_DESTROY_WITH_PARENT,
+				     GTK_MESSAGE_ERROR,
+				     GTK_BUTTONS_CLOSE,
+				     "Error printing file:\n%s",
+				     error->message);
+	g_signal_connect(dlg, "response", 
+			 G_CALLBACK(gtk_widget_destroy), NULL);
+	gtk_widget_show(dlg);
+	g_error_free(error);
+    } else if (res == GTK_PRINT_OPERATION_RESULT_APPLY) {
+	if (settings != NULL) {
+	    g_object_unref(settings);
+	}
+	settings = g_object_ref(gtk_print_operation_get_print_settings(print));
+    }
+
+    g_object_unref(print);
+    /* g_object_unref(comp); ?? */
+}
 
 static void 
 real_sourceview_insert (windata_t *vwin, const char *fname, const char *buf)
@@ -592,6 +612,58 @@ script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
     return ret;
 }
 
+#ifdef PKGBUILD
+
+/* Packages for Windows and OS X: gtksourceview needs to
+   be told where to find its language-specs and style
+   files. Note that we can perhaps avoid doing this if
+   we revise the directory structure inside these packages
+   in line with the "canonical" form, so that the
+   gtksourceview files are in ../share relative to the
+   executables (in ./bin).
+
+   Anyway, given that we're doing it for now, on Windows
+   we need to ensure that the "set_search_path" functions
+   are fed a UTF-8 pathname, since gtksourceview uses
+   g_open() internally and the Glib filename encoding is 
+   UTF-8 on Windows.
+*/
+
+static void set_sourceview_data_path (GtkSourceLanguageManager *lm)
+{
+    static int done;
+
+    if (lm != NULL && !done) {
+	GtkSourceStyleSchemeManager *mgr;
+	gchar *dirs[2] = {NULL, NULL};
+
+	dirs[0] = g_strdup_printf("%sgtksourceview", gretl_home());
+
+# ifdef G_OS_WIN32
+	if (!g_utf8_validate(dirs[0], -1, NULL)) {
+	    gsize bytes;
+	    gchar *tmp = g_locale_to_utf8(dirs[0], -1, NULL, &bytes, NULL);
+
+	    if (tmp != NULL) {
+		g_free(dirs[0]);
+		dirs[0] = tmp;
+	    } 
+	}
+# endif
+
+	gtk_source_language_manager_set_search_path(lm, dirs);
+
+	mgr = gtk_source_style_scheme_manager_get_default();
+	gtk_source_style_scheme_manager_set_search_path(mgr, dirs);
+	gtk_source_style_scheme_manager_force_rescan(mgr);
+
+	g_free(dirs[0]);
+	done = 1;
+    }
+}
+
+#endif /* PKGBUILD */
+
 #define gretl_script_role(r) (r == EDIT_SCRIPT || \
 			      r == VIEW_SCRIPT || \
 			      r == EDIT_PKG_CODE || \
@@ -600,26 +672,28 @@ script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 void create_source (windata_t *vwin, int hsize, int vsize, 
 		    gboolean editable)
 {
-#ifdef NEWER_SOURCEVIEW
-    GtkSourceLanguageManager *lm = gtk_source_language_manager_new();
-#else
-    GtkSourceLanguagesManager *lm = gtk_source_languages_manager_new();
-#endif
+    GtkSourceLanguageManager *lm = NULL;
     GtkSourceBuffer *sbuf;
     GtkTextView *view;
     int cw;
-    
-    sbuf = GTK_SOURCE_BUFFER(gtk_source_buffer_new(NULL));
-    g_object_ref(lm);
-    g_object_set_data_full(G_OBJECT(sbuf), "languages-manager",
-			   lm, (GDestroyNotify) g_object_unref); 
-    g_object_unref(lm); 
 
-#ifdef NEWER_SOURCEVIEW
-    gtk_source_buffer_set_highlight_matching_brackets(sbuf, TRUE);
-#else
-    gtk_source_buffer_set_check_brackets(sbuf, TRUE);
+    /* FIXME : do path stuff first, regardless of
+       whether or not we're doing highlighting
+       for this file?
+    */
+
+    if (textview_use_highlighting(vwin->role)) {
+	lm = gtk_source_language_manager_get_default();
+#ifdef PKGBUILD
+	set_sourceview_data_path(lm);
 #endif
+    }
+
+    sbuf = GTK_SOURCE_BUFFER(gtk_source_buffer_new(NULL));
+    if (lm != NULL) {
+	g_object_set_data(G_OBJECT(sbuf), "languages-manager", lm);
+    }
+    gtk_source_buffer_set_highlight_matching_brackets(sbuf, TRUE);
 
     vwin->text = gtk_source_view_new_with_buffer(sbuf);
     vwin->sbuf = sbuf;
@@ -762,9 +836,7 @@ static GtkTextTagTable *gretl_tags_new (void)
 
     tag = gtk_text_tag_new("code");
     g_object_set(tag, "family", "monospace", 
-#if GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 8
 		 "paragraph-background", "#e6f3ff",
-#endif
 		 NULL);
     gtk_text_tag_table_add(table, tag);
 
@@ -965,6 +1037,8 @@ void textview_insert_from_tempfile (windata_t *vwin, PRN *prn)
     }
 }
 
+#define TAGLEN 64
+
 static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter, 
 			 const char *text, gint page, 
 			 const char *indent)
@@ -972,12 +1046,13 @@ static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter,
     GtkTextTagTable *tab = gtk_text_buffer_get_tag_table(tbuf);
     GtkTextTag *tag;
     gchar *show = NULL;
-    gchar tagname[32];
+    gchar tagname[TAGLEN];
 
     if (page == GUIDE_PAGE) {
 	strcpy(tagname, "tag:guide");
-    } else if (page == SCRIPT_PAGE) {
-	strcpy(tagname, text);
+    } else if (page == SCRIPT_PAGE || page == EXT_PAGE) {
+	*tagname = '\0';
+	strncat(tagname, text, TAGLEN-1);
     } else if (page == BIB_PAGE) {
 	char *p = strrchr(text, ';');
 
@@ -997,7 +1072,7 @@ static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter,
 	if (page == GUIDE_PAGE || page == BIB_PAGE) {
 	    tag = gtk_text_buffer_create_tag(tbuf, tagname, "foreground", "blue", 
 					     "family", "sans", NULL);
-	} else if (page == SCRIPT_PAGE) {
+	} else if (page == SCRIPT_PAGE || page == EXT_PAGE) {
 	    tag = gtk_text_buffer_create_tag(tbuf, tagname, "foreground", "blue", 
 					     "family", "monospace", NULL);
 	} else if (indent != NULL) {
@@ -1133,6 +1208,25 @@ static int object_get_int (gpointer p, const char *key)
     return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(p), key));
 }
 
+static void open_external_link (GtkTextTag *tag)
+{
+    gchar *name = NULL;
+
+    g_object_get(G_OBJECT(tag), "name", &name, NULL);
+
+    if (name != NULL) {
+	if (strncmp(name, "http://", 7)) {
+	    gchar *url = g_strdup_printf("http://%s", name);
+
+	    browser_open(url);
+	    g_free(url);
+	} else {
+	    browser_open(name);
+	}
+	g_free(name);
+    }
+}
+
 static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 {
     GSList *tags = NULL, *tagp = NULL;
@@ -1152,6 +1246,8 @@ static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 		open_script_link(tag);
 	    } else if (page == BIB_PAGE) {
 		open_bibitem_link(tag, tview);
+	    } else if (page == EXT_PAGE) {
+		open_external_link(tag);
 	    } else {
 		int role = object_get_int(tview, "role");
 
@@ -1561,6 +1657,7 @@ gchar *textview_get_current_line (GtkWidget *view)
 {
     GtkTextBuffer *buf;
     GtkTextIter start, end;
+    gchar *ret = NULL;
 
     buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
     gtk_text_buffer_get_iter_at_mark(buf, &start, 
@@ -1568,9 +1665,19 @@ gchar *textview_get_current_line (GtkWidget *view)
     gtk_text_iter_set_line_offset(&start, 0);
     gtk_text_buffer_get_iter_at_mark(buf, &end, 
 				     gtk_text_buffer_get_insert(buf));
-    gtk_text_iter_forward_to_line_end(&end);
+    if (!gtk_text_iter_ends_line(&end)) {
+	/* N.B. don't skip on to the end of the _next_ line */
+	gtk_text_iter_forward_to_line_end(&end);
+    }
 
-    return gtk_text_buffer_get_text(buf, &start, &end, FALSE);
+    ret = gtk_text_buffer_get_text(buf, &start, &end, FALSE);
+    
+    if (string_is_blank(ret)) {
+	g_free(ret);
+	ret = NULL;
+    }
+
+    return ret;
 }
 
 static gchar *textview_get_current_line_with_newline (GtkWidget *view)
@@ -2042,18 +2149,19 @@ static int incremental_leading_spaces (const char *prevword,
 
 	adjust_indent(prevword, &this_indent, &next_indent);
 #if TABDEBUG > 1
-	fprintf(stderr, "adjust_indent 1: this=%d, next=%d\n",
-		this_indent, next_indent);
+	fprintf(stderr, "adjust_indent 1: prevword='%s', this=%d, next=%d\n",
+		prevword, this_indent, next_indent);
 #endif
 	prev_indent = this_indent;
 	if (*thisword != '\0') {
 	    adjust_indent(thisword, &this_indent, &next_indent);
 #if TABDEBUG > 1
-	    fprintf(stderr, "adjust_indent 2: this=%d\n", this_indent);
+	    fprintf(stderr, "adjust_indent 2: thisword='%s', this=%d\n", 
+		    thisword, this_indent);
 #endif
 	    this_indent -= prev_indent;
 #if TABDEBUG > 1
-	    fprintf(stderr, "adjust_indent 2: this=%d\n", this_indent);
+	    fprintf(stderr, "adjust_indent 3: this=%d\n", this_indent);
 #endif
 	} else {
 	    this_indent = next_indent - this_indent;
@@ -2116,8 +2224,7 @@ static int line_continues_previous (GtkTextBuffer *tbuf,
 }
 
 /* get "command word", max 8 characters: work backwards up script
-   to find this
- */
+   to find this */
 
 static char *get_previous_line_start_word (char *word, 
 					   GtkTextBuffer *tbuf,
@@ -2194,10 +2301,14 @@ static int maybe_insert_smart_tab (windata_t *vwin)
 	char prevword[9];
 	int i, nsp = 0, contd = 0;
 
-	*prevword = *thisword = '\0';
+	*thisword = '\0';
 
 	s = textview_get_current_line(vwin->text);
 	if (s != NULL) {
+#if TABDEBUG > 1
+	    fprintf(stderr, "*** maybe_insert_smart_tab: "
+		    "current line = '%s'\n", s);	    
+#endif
 	    sscanf(s, "%8s", thisword);
 	    g_free(s);
 	} 
@@ -2207,6 +2318,10 @@ static int maybe_insert_smart_tab (windata_t *vwin)
 	if (contd) {
 	    nsp += 2;
 	} else {
+#if TABDEBUG > 1
+	    fprintf(stderr, "*** maybe_insert_smart_tab: "
+		    "getting leading spaces\n");	    
+#endif
 	    nsp += incremental_leading_spaces(prevword, thisword);
 	}
 
@@ -2231,48 +2346,46 @@ static int maybe_insert_smart_tab (windata_t *vwin)
 
 /* On "Enter" in script editing, try to compute the correct indent
    level for the current line, and make an adjustment if it's not
-   already in place
+   already right. It would also be nice if we can place the
+   cursor at the appropriate indent on the next, blank line.
 */
 
 static gboolean script_electric_enter (windata_t *vwin)
 {
-    char *s;
+    char *s = NULL;
+    int targsp = 0;
+    gboolean ret = FALSE;
 
     if (!smarttab) {
 	return FALSE;
     }
 
-    s = textview_get_current_line(vwin->text);
+#if TABDEBUG
+    fprintf(stderr, "*** script_electric_enter\n");
+#endif
 
-    if (s == NULL) {
-	return FALSE;
-    } else if (*s == '\0') {
-	g_free(s);
-	return FALSE;
-    } else {
+    s = textview_get_current_line(vwin->text);
+ 
+    if (s != NULL && *s != '\0') {
+	/* work on the line that starts with @thisword, and
+	   is ended by the current Enter 
+	*/
 	GtkTextBuffer *tbuf;
 	GtkTextMark *mark;
 	GtkTextIter start, end;
 	char thisword[9];
 	char prevword[9];
-	int diff, nsp, incr;
-	int targsp = 0, contd = 0;
-
-	*thisword = *prevword = '\0';
-
-	sscanf(s, "%8s", thisword);
-	nsp = count_leading_spaces(s);
-	g_free(s);
-
-	if (*thisword == '\0') {
-	    return FALSE;
-	}
+	int i, diff, nsp, incr;
+	int k, contd = 0;
 
 	tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 	mark = gtk_text_buffer_get_insert(tbuf);
 	gtk_text_buffer_get_iter_at_mark(tbuf, &start, mark);
 	gtk_text_iter_set_line_offset(&start, 0);
 
+	*thisword = '\0';
+	sscanf(s, "%8s", thisword);
+	nsp = count_leading_spaces(s);
 	get_previous_line_start_word(prevword, tbuf, start, &targsp, &contd);
 
 #if TABDEBUG
@@ -2291,6 +2404,10 @@ static gboolean script_electric_enter (windata_t *vwin)
 	if (contd) {
 	    incr = 2;
 	} else {
+#if TABDEBUG > 1
+	    fprintf(stderr, "getting leading spaces ('%s', '%s')\n",
+		    prevword, thisword);
+#endif
 	    incr = incremental_leading_spaces(prevword, thisword);
 	}
 
@@ -2312,16 +2429,28 @@ static gboolean script_electric_enter (windata_t *vwin)
 	    gtk_text_iter_forward_chars(&end, diff);
 	    gtk_text_buffer_delete(tbuf, &start, &end);
 	} else if (diff < 0) {
-	    int i;
-
 	    diff = -diff;
 	    for (i=0; i<diff; i++) {
 		gtk_text_buffer_insert(tbuf, &start, " ", -1);
 	    }
 	}
+
+	/* try to arrange correct indent on the new line */
+	k = targsp + incremental_leading_spaces(thisword, "");
+	gtk_text_buffer_begin_user_action(tbuf);
+	gtk_text_buffer_get_iter_at_mark(tbuf, &start, mark);
+	gtk_text_buffer_insert(tbuf, &start, "\n", -1);
+	for (i=0; i<k; i++) {
+	    gtk_text_buffer_insert(tbuf, &start, " ", -1);
+	}
+	gtk_text_buffer_place_cursor(tbuf, &start);
+	gtk_text_buffer_end_user_action(tbuf);
+	ret = TRUE;
     }
 
-    return FALSE;
+    g_free(s);
+
+    return ret;
 }
 
 /* handler for the user pressing the Tab key when editing a script */
@@ -2538,6 +2667,7 @@ enum {
     INSERT_FIG,
     INSERT_REPL,
     INSERT_LIT,
+    INSERT_URL,
     INSERT_OPT,
     INSERT_ITAL,
     INSERT_SUP,
@@ -2625,6 +2755,8 @@ static int get_instruction_and_string (const char *p, char *str)
 	ins = INSERT_REPL;
     } else if (!strncmp(p, "lit", 3)) {
 	ins = INSERT_LIT;
+    } else if (!strncmp(p, "url", 3)) {
+	ins = INSERT_URL;
     } else if (!strncmp(p, "opt", 3)) {
 	ins = INSERT_OPT;
     } else if (!strncmp(p, "sup", 3)) {
@@ -2721,6 +2853,8 @@ insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
 		    itarg = function_help_index_from_word(targ);
 		}
 		insert_xlink(tbuf, iter, targ, itarg, indent);
+	    } else if (ins == INSERT_URL) {
+		insert_link(tbuf, iter, targ, EXT_PAGE, indent);
 	    } else if (ins == INSERT_PDFLINK) {
 		insert_link(tbuf, iter, targ, GUIDE_PAGE, indent);
 	    } else if (ins == INSERT_INPLINK) {
