@@ -53,13 +53,6 @@
 #include "../pixmaps/mini.split_v.xpm"
 #include "../pixmaps/mini.compass.xpm"
 #include "../pixmaps/mini.spreadsheet.xpm"
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 8)
-# include "../pixmaps/info_24.xpm"
-#endif
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
-# include "../pixmaps/edit_24.xpm"
-# include "../pixmaps/mini.edit.xpm"
-#endif
 
 /* for main-window toolbar */
 #include "../pixmaps/mini.calc.xpm"
@@ -95,92 +88,63 @@ enum {
     REFRESH_ITEM,
     OPEN_ITEM,
     SPLIT_ITEM,
-    EDITOR_ITEM
+    EDITOR_ITEM,
+    NOTES_ITEM,
+    STOP_ITEM
 } viewbar_flags;
 
 static GtkIconFactory *gretl_stock_ifac;
 
+struct stock_maker {
+    char **xpm;
+    const char *str;
+};
+
 void gretl_stock_icons_init (void)
 {
-    char **xpms[] = {
-#if NO_INFO_ICON
-	info_24_xpm,
-#endif
-#if NO_EDIT_ICON
-	edit_24_xpm,
-	mini_edit_xpm,
-#endif
-	mini_tex_xpm,
-	mail_16_xpm,
-	mini_tsplot_xpm,
-	mini_boxplot_xpm,
-	mini_pdf_xpm,
-	mini_manual_xpm,
-	mini_calc_xpm,
-	mini_sh_xpm,
-	mini_session_xpm,
-	mini_plot_xpm,
-	mini_model_xpm,
-	mini_func_xpm,
-	mini_pin_xpm,
-	mini_alpha_xpm,
-	mini_en_xpm,
-	mini_split_h_xpm,
-	mini_split_v_xpm,
-	mini_compass_xpm,
-	mini_spreadsheet_xpm,
-	mini_db_xpm
-    };
-    const char *stocks[] = {
-#if NO_INFO_ICON
-	GRETL_STOCK_INFO,
-#endif
-#if NO_EDIT_ICON
-	GRETL_STOCK_EDIT,    /* -> edit_24_xpm, 24 x 24 */
-	GRETL_STOCK_SCRIPT,  /* -> mini_edit_xpm, 16 x 16 */  
-#endif
-	GRETL_STOCK_TEX,
-	GRETL_STOCK_MAIL,
-	GRETL_STOCK_TS,
-	GRETL_STOCK_BOX,
-	GRETL_STOCK_PDF,
-	GRETL_STOCK_BOOK,
-	GRETL_STOCK_CALC,
-	GRETL_STOCK_CONSOLE,
-	GRETL_STOCK_ICONS,
-	GRETL_STOCK_SCATTER,
-	GRETL_STOCK_MODEL,
-	GRETL_STOCK_FUNC,
-	GRETL_STOCK_PIN,
-	GRETL_STOCK_ALPHA,
-	GRETL_STOCK_EN,
-	GRETL_STOCK_SPLIT_H,
-	GRETL_STOCK_SPLIT_V,
-	GRETL_STOCK_COMPASS,
-	GRETL_STOCK_SHEET,
-	GRETL_STOCK_DB
+    struct stock_maker stocks[] = {
+	{ mini_tex_xpm, GRETL_STOCK_TEX },
+	{ mail_16_xpm, GRETL_STOCK_MAIL },
+	{ mini_tsplot_xpm, GRETL_STOCK_TS },
+	{ mini_boxplot_xpm, GRETL_STOCK_BOX },
+	{ mini_pdf_xpm, GRETL_STOCK_PDF },
+	{ mini_manual_xpm, GRETL_STOCK_BOOK },
+	{ mini_calc_xpm, GRETL_STOCK_CALC },
+	{ mini_sh_xpm, GRETL_STOCK_CONSOLE },
+	{ mini_session_xpm, GRETL_STOCK_ICONS },
+	{ mini_plot_xpm, GRETL_STOCK_SCATTER },
+	{ mini_model_xpm, GRETL_STOCK_MODEL },
+	{ mini_func_xpm, GRETL_STOCK_FUNC },
+	{ mini_pin_xpm, GRETL_STOCK_PIN },
+	{ mini_alpha_xpm, GRETL_STOCK_ALPHA },
+	{ mini_en_xpm, GRETL_STOCK_EN },
+	{ mini_split_h_xpm, GRETL_STOCK_SPLIT_H },
+	{ mini_split_v_xpm, GRETL_STOCK_SPLIT_V },
+	{ mini_compass_xpm, GRETL_STOCK_COMPASS },
+	{ mini_spreadsheet_xpm, GRETL_STOCK_SHEET },
+	{ mini_db_xpm, GRETL_STOCK_DB}
     };
     int n = G_N_ELEMENTS(stocks);
 
     if (gretl_stock_ifac == NULL) {
 	GtkIconSource *source;
-	GtkIconSet *set;
+	GtkIconSet *iset;
 	GdkPixbuf *pbuf;
 	int i;
 
 	gretl_stock_ifac = gtk_icon_factory_new();
 
 	for (i=0; i<n; i++) {
-	    set = gtk_icon_set_new();
+	    iset = gtk_icon_set_new();
 	    source = gtk_icon_source_new();
 	    gtk_icon_source_set_size(source, GTK_ICON_SIZE_MENU);
-	    pbuf = gdk_pixbuf_new_from_xpm_data((const char **) xpms[i]);
+	    pbuf = gdk_pixbuf_new_from_xpm_data((const char **) stocks[i].xpm);
 	    gtk_icon_source_set_pixbuf(source, pbuf);
 	    g_object_unref(pbuf);
-	    gtk_icon_set_add_source(set, source);
+	    gtk_icon_set_add_source(iset, source);
 	    gtk_icon_source_free(source);
-	    gtk_icon_factory_add(gretl_stock_ifac, stocks[i], set);
-	    gtk_icon_set_unref(set);
+	    gtk_icon_factory_add(gretl_stock_ifac, stocks[i].str, iset);
+	    gtk_icon_set_unref(iset);
 	}
 
 	gtk_icon_factory_add_default(gretl_stock_ifac);
@@ -189,7 +153,7 @@ void gretl_stock_icons_init (void)
 
 /* callbacks for viewer window toolbar */
 
-void save_as_callback (GtkWidget *w, windata_t *vwin)
+static void save_as_callback (GtkWidget *w, windata_t *vwin)
 {
     guint u = 0;
 
@@ -247,8 +211,15 @@ static void mail_script_callback (GtkWidget *w, windata_t *vwin)
     send_file(vwin->fname);
 }
 
+/* callback for the "Open" icon in a script editing window,
+   which enables the user to switch to a different script
+*/
+
 static void file_open_callback (GtkWidget *w, windata_t *vwin)
 {
+    /* don't proceed unconditionally if there's unsaved
+       text in the window 
+    */
     if (query_save_text(NULL, NULL, vwin)) {
 	return;
     }
@@ -266,12 +237,24 @@ static void toolbar_new_callback (GtkWidget *w, windata_t *vwin)
     do_new_script(vwin->role);
 }
 
-#ifdef NATIVE_PRINTING
 static void window_print_callback (GtkWidget *w, windata_t *vwin)
 {
+#ifdef G_OS_WIN32
     window_print(NULL, vwin);
-}
+#else
+    if (textview_use_highlighting(vwin->role)) {
+	int resp = yes_no_dialog(NULL, _("Print with syntax highlighting?"), 1);
+
+	if (resp == GRETL_YES) {
+	    sourceview_print(vwin);
+	} else if (resp == GRETL_NO) {
+	    window_print(NULL, vwin);
+	}
+    } else {
+	window_print(NULL, vwin);
+    }
 #endif
+}
 
 static void window_help (GtkWidget *w, windata_t *vwin)
 {
@@ -364,7 +347,7 @@ static void add_matrix_callback (GtkWidget *w, windata_t *vwin)
 
 static void add_data_callback (GtkWidget *w, windata_t *vwin)
 {
-    int oldv = datainfo->v;
+    int oldv = dataset->v;
 
     if (vwin->role == PCA) {
 	add_pca_data(vwin);
@@ -376,7 +359,7 @@ static void add_data_callback (GtkWidget *w, windata_t *vwin)
 	add_fcast_data(vwin);
     }
 
-    if (datainfo->v > oldv) {
+    if (dataset->v > oldv) {
 	populate_varlist();
 	mark_dataset_as_modified();
     }	
@@ -614,27 +597,36 @@ static int edit_script_popup_item (GretlToolItem *item)
 
 static void set_plot_icon (GretlToolItem *item)
 {
-    if (dataset_is_time_series(datainfo)) {
+    if (dataset_is_time_series(dataset)) {
 	item->icon = GRETL_STOCK_TS;
     } else {
 	item->icon = GRETL_STOCK_BOX;
     }
 }
 
+static void vwin_cut_callback (GtkWidget *w, windata_t *vwin)
+{
+    gtk_text_buffer_cut_clipboard(gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text)),
+				  gtk_clipboard_get(GDK_NONE),
+				  TRUE);
+}
+
 static GretlToolItem viewbar_items[] = {
     { N_("New window"), GTK_STOCK_NEW, G_CALLBACK(toolbar_new_callback), OPEN_ITEM },
     { N_("Open..."), GTK_STOCK_OPEN, G_CALLBACK(file_open_callback), OPEN_ITEM },
     { N_("Save"), GTK_STOCK_SAVE, G_CALLBACK(vwin_save_callback), SAVE_ITEM },
     { N_("Save as..."), GTK_STOCK_SAVE_AS, G_CALLBACK(save_as_callback), SAVE_AS_ITEM },
-#ifdef NATIVE_PRINTING
     { N_("Print..."), GTK_STOCK_PRINT, G_CALLBACK(window_print_callback), 0 },
-#endif
+    { N_("Show/hide"), GRETL_STOCK_PIN, G_CALLBACK(session_notes_callback), NOTES_ITEM },
     { N_("Run"), GTK_STOCK_EXECUTE, G_CALLBACK(do_run_script), EXEC_ITEM },
+    { N_("Stop"), GTK_STOCK_STOP, G_CALLBACK(do_stop_script), STOP_ITEM },
+    { N_("Cut"), GTK_STOCK_CUT, G_CALLBACK(vwin_cut_callback), EDIT_ITEM }, 
     { N_("Copy"), GTK_STOCK_COPY, G_CALLBACK(vwin_copy_callback), COPY_ITEM }, 
     { N_("Paste"), GTK_STOCK_PASTE, G_CALLBACK(text_paste), EDIT_ITEM },
     { N_("Find..."), GTK_STOCK_FIND, G_CALLBACK(text_find), 0 },
     { N_("Replace..."), GTK_STOCK_FIND_AND_REPLACE, G_CALLBACK(text_replace), EDIT_ITEM },
     { N_("Undo"), GTK_STOCK_UNDO, G_CALLBACK(text_undo), EDIT_ITEM },
+    { N_("Redo"), GTK_STOCK_REDO, G_CALLBACK(text_redo), EDIT_ITEM },
     { N_("Sort"), GTK_STOCK_SORT_ASCENDING, G_CALLBACK(series_view_toggle_sort), SORT_ITEM },    
     { N_("Sort by..."), GTK_STOCK_SORT_ASCENDING, G_CALLBACK(multi_series_view_sort_by), SORT_BY_ITEM },
     { N_("Configure tabs..."), GTK_STOCK_PREFERENCES, G_CALLBACK(script_tabs_dialog), EDIT_SCRIPT_ITEM },
@@ -665,6 +657,10 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 		    r == VIEW_SCRIPT || \
 	            r == EDIT_PKG_SAMPLE)
 
+#define stop_ok(r) (r == EDIT_SCRIPT || \
+		    r == VIEW_SCRIPT || \
+	            r == EDIT_PKG_SAMPLE)
+
 #define open_ok(r) (vwin_editing_script(r))
 
 #define edit_ok(r) (vwin_editing_script(r) || \
@@ -711,7 +707,9 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
     int f = item->flag;
     int r = vwin->role;
 
-    if (!edit_ok(r) && f == EDIT_ITEM) {
+    if (!stop_ok(r) && f == STOP_ITEM) {
+	return NULL;
+    } else if (!edit_ok(r) && f == EDIT_ITEM) {
 	return NULL;
     } else if (!open_ok(r) && f == OPEN_ITEM) {
 	return NULL;
@@ -758,6 +756,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (f == SAVE_ITEM && !save_ok) {
 	return NULL;
+    } else if (r != EDIT_NOTES && f == NOTES_ITEM) {
+	return NULL;
     } else if (f == SAVE_AS_ITEM) {
 	if (!save_as_ok(r)) {
 	    return NULL;
@@ -783,31 +783,26 @@ GtkWidget *gretl_toolbar_new (void)
 
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 12)
 
-static GtkTooltips *gretl_tips;
-
-void gretl_tooltips_init (void)
-{
-    gretl_tips = gtk_tooltips_new();
-    gtk_tooltips_enable(gretl_tips); /* redundant? */
-}
-
 void gretl_tooltips_add (GtkWidget *w, const gchar *str)
 {
+    static GtkTooltips *gretl_tips;
+
+    if (gretl_tips == NULL) {
+	gretl_tips = gtk_tooltips_new();
+	gtk_tooltips_enable(gretl_tips); /* redundant? */
+    }
+
     gtk_tooltips_set_tip(gretl_tips, w, str, NULL);
 }
 
 #else /* new tooltips API */
 
-void gretl_tooltips_init (void)
-{
-}
-
 void gretl_tooltips_add (GtkWidget *w, const gchar *str)
 {
     gtk_widget_set_tooltip_text(w, str);
 }
 
-#endif
+#endif /* GTK variants */
 
 GtkToolItem *gretl_toolbar_insert (GtkWidget *tbar,
 				   GretlToolItem *item,
@@ -875,6 +870,10 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 	    if (strstr(vwin->fname, "script_tmp")) {
 		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
 	    }
+	} else if (item->flag == STOP_ITEM) {
+	    /* nothing to stop yet */
+	    g_object_set_data(G_OBJECT(vwin->mbar), "stop_button", button); 
+	    gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
 	}
     }
 
@@ -891,10 +890,6 @@ void vwin_add_viewbar (windata_t *vwin, ViewbarFlags flags)
 {
     GtkWidget *hbox;
 
-    if (gretl_stock_ifac == NULL) {
-	gretl_stock_icons_init();
-    }
-
     if ((flags & VIEWBAR_HAS_TEXT) || vwin->role == SCRIPT_OUT) {
 	g_object_set_data(G_OBJECT(vwin->main), "text_out", GINT_TO_POINTER(1));
     }
@@ -903,7 +898,6 @@ void vwin_add_viewbar (windata_t *vwin, ViewbarFlags flags)
     gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
 
     vwin->mbar = gretl_toolbar_new();
-
     viewbar_add_items(vwin, flags);
 
     gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
@@ -1008,7 +1002,7 @@ static void tbar_command_ref (void)
 static void tbar_xy_graph (void)
 {
     if (data_status) {
-	if (datainfo->v == 2) {
+	if (dataset->v == 2) {
 	    do_graph_var(mdata->active_var);
 	} else if (mdata_selection_count() == 2) {
 	    plot_from_selection(GR_XY);
@@ -1054,11 +1048,7 @@ static void tbar_show_funcs (GtkWidget *w, gpointer p)
 
 static GretlToolItem mainbar_items[] = {
     { N_("launch calculator"),  GRETL_STOCK_CALC,    G_CALLBACK(tbar_calc), 0 },
-#if NO_EDIT_ICON
-    { N_("new script"),         GRETL_STOCK_SCRIPT,  G_CALLBACK(tbar_new_script), 0 },
-#else
     { N_("new script"),         GTK_STOCK_EDIT,      G_CALLBACK(tbar_new_script), 0 },
-#endif
     { N_("open gretl console"), GRETL_STOCK_CONSOLE, G_CALLBACK(gretl_console), 0 },
     { N_("session icon view"),  GRETL_STOCK_ICONS,   G_CALLBACK(tbar_iconview), 0 },
     { N_("function packages"),  GRETL_STOCK_FUNC,    G_CALLBACK(tbar_show_funcs), 0 },
@@ -1070,14 +1060,12 @@ static GretlToolItem mainbar_items[] = {
     { N_("open dataset"),       GTK_STOCK_OPEN,      G_CALLBACK(tbar_open_data), 0 }
 };
 
-static void make_toolbar (GtkWidget *vbox)
+void add_mainwin_toolbar (GtkWidget *vbox)
 {
     GtkWidget *hbox, *toolbar;
     GretlToolItem *item;
     int i, n = G_N_ELEMENTS(mainbar_items);
 
-    gretl_stock_icons_init();
-
     toolbar = gretl_toolbar_new();
 
     for (i=0; i<n; i++) {
@@ -1088,13 +1076,4 @@ static void make_toolbar (GtkWidget *vbox)
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), toolbar, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-    gtk_widget_show_all(hbox);
 }
-
-void show_toolbar (void)
-{
-    GtkWidget *vbox = g_object_get_data(G_OBJECT(mdata->main), "vbox");
-
-    make_toolbar(vbox);
-}
-
