@@ -206,14 +206,14 @@ static void update_dialogs_from_varclick (int active_var)
     } else if (active_edit_name != NULL) {
 	edttext = gtk_entry_get_text(GTK_ENTRY(active_edit_name));
 	my_gtk_entry_append_text(GTK_ENTRY(active_edit_name), 
-				 datainfo->varname[active_var]);
+				 dataset->varname[active_var]);
     } else if (active_edit_text != NULL) {
 	GtkTextBuffer *tbuf;
 
 	tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(active_edit_text));
 	if (tbuf != NULL) {
 	    gtk_text_buffer_insert_at_cursor(tbuf, 
-					     datainfo->varname[active_var], 
+					     dataset->varname[active_var], 
 					     -1);
 	}
     }  
@@ -221,13 +221,15 @@ static void update_dialogs_from_varclick (int active_var)
 
 #ifdef OSX_BUILD
 
-/* Trying to support command-click as replacement for Ctrl-click
-   on OS X. But it doesn't work. I wonder why... */
+/* Support for command-click as replacement for Ctrl-click
+   on OS X */
 
 static gboolean maybe_do_meta_click (GtkWidget *widget, GtkTreeView *view,
 				     GtkTreePath *path)
 {
-    if (widget_get_pointer_mask(widget) & GDK_HYPER_MASK) {
+    GdkModifierType mods = widget_get_pointer_mask(widget);
+
+    if (mods & GDK_MOD2_MASK) {
 	/* The idea here is that we add to the current selection
 	   and return TRUE to block further processing of the
 	   click, so as to implement selection of non-contiguous
@@ -258,7 +260,7 @@ gboolean main_varclick (GtkWidget *widget, GdkEventButton *event,
     GtkTreePath *path;
     gboolean ret = FALSE;
 
-    if (datainfo == NULL || datainfo->n == 0) {
+    if (dataset == NULL || dataset->n == 0) {
 	return TRUE;
     }
 
@@ -437,11 +439,9 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
     if (tree) {
 	tstore = gtk_tree_store_newv(ncols, types);
 	vwin->listbox = view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(tstore));
-	g_object_unref(G_OBJECT(tstore));
     } else {
 	lstore = gtk_list_store_newv(ncols, types);
 	vwin->listbox = view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(lstore));
-	g_object_unref(G_OBJECT(lstore));
     }
 
     gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), TRUE);
@@ -553,6 +553,12 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
 					NULL, NULL);
     } 
 
+    if (lstore != NULL) {
+	g_object_unref(G_OBJECT(lstore));
+    } else if (tstore != NULL) {
+	g_object_unref(G_OBJECT(tstore));
+    }
+
     scroller = gtk_scrolled_window_new(NULL, NULL);
 
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
