@@ -25,14 +25,21 @@
                              !strcmp(s, "n.a.") || \
                              !strcmp(s, "na") || \
                              !strcmp(s, "N/A") || \
+			     !strcmp(s, "#N/A") || \
                              !strcmp(s, "NaN") || \
+                             !strcmp(s, ".NaN") || \
                              !strcmp(s, ".") || \
                              !strcmp(s, "..") || \
-                             !strncmp(s, "-999", 4))
+                             !strcmp(s, "-999") || \
+                             !strcmp(s, "-9999"))
 
 int import_obs_label (const char *s);
 
-int test_markers_for_dates (double ***pZ, DATAINFO *pdinfo, 
-			    char *skipstr, PRN *prn);
+int test_markers_for_dates (DATASET *dset, 
+			    int *reversed, 
+			    char *skipstr, 
+			    PRN *prn);
+
+void reverse_data (DATASET *dset, PRN *prn);
 
 #endif /* CSVDATA_H */
