@@ -201,6 +201,8 @@ struct str_table funcs[] = {
     { F_MAX,      "max" },
     { F_SD,       "sd" },
     { F_VCE,      "var" },
+    { F_SKEWNESS, "skewness" },
+    { F_KURTOSIS, "kurtosis" },
     { F_SST,      "sst" },
     { F_CNORM,    "cnorm" },
     { F_DNORM,    "dnorm" },
@@ -214,6 +216,7 @@ struct str_table funcs[] = {
     { F_PMAX,     "pmax" },      /* panel max */
     { F_PMEAN,    "pmean" },     /* panel mean */
     { F_PSD,      "psd" },       /* panel std dev */
+    { F_PSHRINK,  "pshrink" },
     { F_HPFILT,   "hpfilt" },    /* Hodrick-Prescott filter */
     { F_BKFILT,   "bkfilt" },    /* Baxter-King filter */
     { F_BWFILT,   "bwfilt" },    /* Butterworth filter */
@@ -245,6 +248,7 @@ struct str_table funcs[] = {
     { F_FFT,      "fft" },
     { F_FFTI,     "ffti" },
     { F_CMULT,    "cmult" },
+    { F_HDPROD,   "hdprod" },
     { F_CDIV,     "cdiv" },
     { F_MCOV,     "mcov" },
     { F_MCORR,    "mcorr" },
@@ -296,8 +300,10 @@ struct str_table funcs[] = {
     { F_PVAL,     "pvalue" },
     { F_CRIT,     "critical" },
     { F_RANDGEN,  "randgen" },
+    { F_MRANDGEN, "mrandgen" },
     { F_URCPVAL,  "urcpval" },
     { F_VALUES,   "values" },
+    { F_UNIQ,     "uniq" },
     { F_MSHAPE,   "mshape" },
     { F_SVD,      "svd" },
     { F_MOLS,     "mols" },
@@ -369,6 +375,8 @@ struct str_table funcs[] = {
     { F_INBUNDLE, "inbundle" },
     { F_STRSUB,   "strsub" },
     { F_COLNAME,  "colname" },
+    { F_RANDINT,  "randint" },
+    { F_NADARWAT, "nadarwat" },   /* Nadaraya-Watson */
     { 0,          NULL }
 };
 
@@ -790,8 +798,7 @@ NODE *obs_node (parser *p)
     } 
 
     if (special && !p->err) {
-	t = get_t_from_obs_string(word, (const double **) *p->Z, 
-				  p->dinfo);
+	t = get_t_from_obs_string(word, p->dset);
 	if (t >= 0) {
 	    /* convert to use-style 1-based index */
 	    t++;
@@ -880,7 +887,7 @@ static void look_up_word (const char *s, parser *p)
 	    if (p->idnum > 0) {
 		p->sym = DUM;
 	    } else {
-		if ((p->idnum = current_series_index(p->dinfo, s)) >= 0) {
+		if ((p->idnum = current_series_index(p->dset, s)) >= 0) {
 		    p->sym = USERIES;
 		} else if (!strcmp(s, "time")) {
 		    p->sym = DUM;
@@ -907,7 +914,7 @@ static void look_up_word (const char *s, parser *p)
 		    p->sym = VSTR;
 		    p->idstr = gretl_strdup(s);
 		} else if (p->targ == LIST &&
-			   varname_match_any(p->dinfo, s)) {
+			   varname_match_any(p->dset, s)) {
 		    p->sym = LIST;
 		    p->idstr = gretl_strdup(s);
 		} else if (!strcmp(s, "t")) {
@@ -941,7 +948,7 @@ static void look_up_word (const char *s, parser *p)
 			    t == M_UHAT || t == M_YHAT || \
 			    t == R_TEST_STAT || R_TEST_PVAL)
 
-static void word_check_next_char (const char *s, parser *p)
+static void word_check_next_char (parser *p)
 {
 #if LDEBUG
     if (p->ch) fprintf(stderr, "word_check_next_char: ch = '%c'\n", p->ch);
@@ -1004,7 +1011,7 @@ static void word_check_next_char (const char *s, parser *p)
 
     if (p->err) {
 	context_error(p->ch, p);
-    }
+    } 
 }
 
 static int is_word_char (parser *p)
@@ -1063,12 +1070,15 @@ static void getword (parser *p)
 	look_up_dollar_word(word, p);
     } else if (*word == '@') {
 	look_up_string_variable(word, p);
+    } else if (*word == '$' && word[1] == '\0' && p->ch == '[') {
+	p->sym = BUNDLE;
+	p->idstr = gretl_strdup("$");
     } else {
 	look_up_word(word, p);
     }
 
     if (!p->err && *word != '@') {
-	word_check_next_char(word, p);
+	word_check_next_char(p);
     }
 
 #if LDEBUG
@@ -1076,36 +1086,15 @@ static void getword (parser *p)
 #endif
 }
 
-static int doing_matrix_slice;
-
-void set_matrix_slice_on (void)
-{
-    doing_matrix_slice = 1;
-}
-
-void set_matrix_slice_off (void)
-{
-    doing_matrix_slice = 0;
-}
-
-static int doing_lag_parse;
-
-void set_lag_parse_on (void)
-{
-    doing_lag_parse = 1;
-}
-
-void set_lag_parse_off (void)
-{
-    doing_lag_parse = 0;
-}
-
-static int colon_ok (char *s, int n)
+static int colon_ok (parser *p, char *s, int n)
 {
     int i;
 
-    if (doing_matrix_slice) {
-	/* colon is a separator in this context */
+    if (p->flags & P_SLICING) {
+	/* calculating a matrix "slice": colon is a separator in 
+	   this context, cannot be part of a time/panel observation 
+	   string
+	*/
 #if LDEBUG
 	fprintf(stderr, "colon_ok: doing matrix slice\n");
 #endif
@@ -1129,17 +1118,17 @@ static int colon_ok (char *s, int n)
    packed into string 's' up to element 'i'
 */
 
-static int ok_dbl_char (int ch, char *s, int i)
+static int ok_dbl_char (parser *p, char *s, int i)
 {
     if (i < 0) {
 	return 1;
     }
 
-    if (ch >= '0' && ch <= '9') {
+    if (p->ch >= '0' && p->ch <= '9') {
 	return 1;
     }
 
-    switch (ch) {
+    switch (p->ch) {
     case '+':
     case '-':
 	return s[i] == 'e' || s[i] == 'E';
@@ -1152,7 +1141,7 @@ static int ok_dbl_char (int ch, char *s, int i)
 	    !strchr(s, ':');
     case ':':
 	/* allow for obs numbers in the form, e.g., "1995:10" */
-	return colon_ok(s, i);
+	return colon_ok(p, s, i);
     default:
 	break;
     }
@@ -1167,7 +1156,7 @@ static double getdbl (parser *p)
     int gotcol = 0;
     int i = 0;
 
-    while (ok_dbl_char(p->ch, xstr, i - 1) && i < NUMLEN - 1) {
+    while (ok_dbl_char(p, xstr, i - 1) && i < NUMLEN - 1) {
 	xstr[i++] = p->ch;
 	if (p->ch == ':') {
 	    gotcol = 1;
@@ -1185,10 +1174,13 @@ static double getdbl (parser *p)
 #endif
     
     if (gotcol) {
-	if (p->dinfo->pd == 1) {
+#if LDEBUG
+	fprintf(stderr, "getdbl: gotcol\n");
+#endif
+	if (p->dset->pd == 1) {
 	    p->err = E_PDWRONG;
 	} else {
-	    d = (double) dateton(xstr, p->dinfo);
+	    d = (double) dateton(xstr, p->dset);
 	    if (d < 0) {
 		p->err = E_DATA;
 		d = NADBL;
@@ -1208,7 +1200,7 @@ static double getdbl (parser *p)
 
 #define word_start_special(c) (c == '$' || c == '@' || c == '_')
 
-#define lag_range_sym(p) (doing_lag_parse && p->ch == 't' && \
+#define lag_range_sym(p) ((p->flags & P_LAGPRSE) && p->ch == 't' && \
                           *p->point == 'o' && \
 			  *(p->point + 1) == ' ')
 
@@ -1413,6 +1405,10 @@ void lex (parser *p)
 		p->sym = B_DOTLT;
 		parser_getc(p);
 		return;
+	    } else if (p->ch == '.') {
+		p->sym = B_ELLIP;
+		parser_getc(p);
+		return;
 	    } else {
 		/* not a "dot operator", so back up */
 		parser_ungetc(p);
@@ -1502,7 +1498,7 @@ const char *getsymb (int t, const parser *p)
 	if (t == NUM) {
 	    return fromdbl(p->xval); 
 	} else if (t == USERIES) {
-	    return p->dinfo->varname[p->idnum];
+	    return p->dset->varname[p->idnum];
 	} else if (t == USCALAR) {
 	    return p->idstr;
 	} else if (t == BUNDLE) {
