@@ -60,16 +60,18 @@ enum {
               B_DOTEQ,
               B_DOTGT,
   /* 30 */    B_DOTLT,
+              B_DOTASN,
               B_KRON,     /* Kronecker product */
               B_HCAT,     /* horizontal concatenation */
               B_VCAT,     /* vertical concatenation */
 	      B_LCAT,     /* list concatentation */
 	      B_LDIV,     /* matrix left division */
+	      B_ELLIP,    /* list-generating ellipsis */
 	      OP_MAX,     /* SEPARATOR: end of binary operators */
               G_LPR,      /* grouping: left paren */
-              G_RPR,      /* right paren */
+  /* 40 */    G_RPR,      /* right paren */
               G_LBR,      /* left bracket */
-  /* 40 */    G_RBR,      /* right bracket */
+              G_RBR,      /* right bracket */
               G_LCB,      /* left curly bracket */
 	      G_RCB,      /* right curly bracket */  
 	      P_COM,	  /* punctuation: comma */
@@ -77,9 +79,9 @@ enum {
 	      P_SEMI,	  /* semi-colon */
 	      P_COL,	  /* colon */
 	      PUNCT_MAX,  /* SEPARATOR: end of grouping and punctuation marks */
-              CON,	  /* named constant */
-              DUM,	  /* dummy variable */
-  /* 50 */    USCALAR,	  /* user variable, scalar */
+  /* 50 */    CON,	  /* named constant */
+	      DUM,	  /* dummy variable */
+              USCALAR,	  /* user variable, scalar */
               USERIES,    /* user variable, series */
 	      UMAT,	  /* user-defined matrix */
 	      UOBJ,	  /* user-defined object (e.g. model) */
@@ -87,9 +89,9 @@ enum {
 	      VEC,	  /* series, evaluated */
 	      IVEC,	  /* vector of integers, evaluated */
 	      LVEC,       /* list, as array of ints */
-	      MAT,	  /* matrix, evaluated */
+  /* 60 */    MAT,	  /* matrix, evaluated */
 	      OBS,	  /* observation from a series */
-  /* 60 */    MSL,	  /* matrix plus subspec */
+              MSL,	  /* matrix plus subspec */
               DMSL,	  /* "dollar" matrix plus subspec */
 	      DMSTR,	  /* "dollar" matrix plus string subspec */
 	      MSL2,	  /* unevaluated matrix subspec */
@@ -97,19 +99,19 @@ enum {
 	      SUBSL,	  /* row or column component of MSPEC */
 	      MDEF,	  /* explicit matrix definition {...} */
 	      LAG,        /* variable plus lag length */	  
-	      DVAR,	  /* $ "dataset" variable (mostly scalar or series) */
+  /* 70 */    DVAR,	  /* $ "dataset" variable (mostly scalar or series) */
 	      MVAR,	  /* $ model var (scalar, series, or matrix) */
-  /* 70 */    OVAR,	  /* object variable: variable "under" an object */
+              OVAR,	  /* object variable: variable "under" an object */
               LIST,	  /* reference to named list */
 	      LISTVAR,    /* variable in list */
 	      STR,	  /* string */
 	      BUNDLE,     /* gretl bundle (hash table) */
-	      BOBJ,       /* object from inside a bundle */
+	      BOBJ,       /* object inside a bundle */
 	      FARGS,	  /* set of n function arguments */
 	      EMPTY,      /* "null" */
-	      ABSENT,
+  /* 80 */    ABSENT,
 	      DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
-  /* 80 */    EROOT,	  /* dummy root for (...) expression */
+              EROOT,	  /* dummy root for (...) expression */
               UFUN,	  /* user-defined function */
 	      RFUN,       /* GNU R function */
               VSTR,       /* string variable */
@@ -169,6 +171,8 @@ enum {
     F_MAX,
     F_SD,
     F_VCE,	  /* variance */
+    F_SKEWNESS,
+    F_KURTOSIS,
     F_SST,
     F_CNORM,
     F_DNORM,
@@ -176,11 +180,6 @@ enum {
     F_GAMMA,	
     F_LNGAMMA,
     F_DIGAMMA,
-    F_PNOBS,
-    F_PMIN,
-    F_PMAX,
-    F_PMEAN,
-    F_PSD,
     F_IMAT,
     F_SUMR,
     F_SUMC,
@@ -222,7 +221,9 @@ enum {
     F_CRIT, 
     F_URCPVAL,
     F_RANDGEN,
+    F_MRANDGEN,
     F_VALUES,
+    F_UNIQ,
     F_NULLSPC,
     F_MEXP,
     F_MINC,
@@ -255,6 +256,7 @@ enum {
     F_ERRMSG,
     F_GETENV,
     F_NGETENV,
+    F_PSHRINK,
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
@@ -279,6 +281,7 @@ enum {
     F_PRINCOMP,
     F_QUANTILE,
     F_CMULT,	  /* complex multiplication */
+    F_HDPROD,     /* horizontal direct product */
     F_CDIV,	  /* complex division */
     F_MXTAB,
     F_MWRITE,
@@ -311,6 +314,12 @@ enum {
     F_ISCONST,
     F_INBUNDLE,
     F_COLNAME,
+    F_PNOBS,
+    F_PMIN,
+    F_PMAX,
+    F_PMEAN,
+    F_PSD,
+    F_RANDINT,
     F2_MAX,	  /* SEPARATOR: end of two-arg functions */
     F_BFGSMAX,
     F_MSHAPE,
@@ -335,6 +344,7 @@ enum {
     F_STRSUB,
     F_MLAG,
     F_EIGSOLVE,
+    F_NADARWAT,
     F3_MAX,       /* SEPARATOR: end of three-arg functions */
     F_BKFILT,
     F_MOLS,
@@ -380,7 +390,7 @@ enum {
 /* function with multiple args, character code for first arg */
 #define char0_func(s) (s == F_PVAL || s == F_CDF || s == F_INVCDF || \
 		       s == F_CRIT || s == F_RANDGEN || s == F_PDF ||	\
-		       s == F_BESSEL)
+		       s == F_BESSEL || s == F_MRANDGEN)
 
 /* functions taking a string arg in last position */
 #define string_last_func(s) (s == F_FDJAC || s == F_BFGSMAX || \
@@ -470,9 +480,9 @@ union val {
 };
 
 enum {
-    AUX_NODE    = 1 << 0, /* auxiliary: free on exit */
-    TMP_NODE    = 1 << 1, /* temporary: free content on exit */
-    PTR_NODE    = 1 << 2  /* node is compatible with P_LHPTR */
+    AUX_NODE = 1 << 0, /* auxiliary: free on exit */
+    TMP_NODE = 1 << 1, /* temporary: free content on exit */
+    PTR_NODE = 1 << 2  /* node is compatible with P_LHPTR */
 };
 
 struct node {
@@ -509,7 +519,9 @@ enum {
     P_LOOP    = 1 << 23, /* executing pre-built genr in loop context */
     P_SLAVE   = 1 << 24, /* running as "slave" of NLS/MLE/GMM */
     P_LHPTR   = 1 << 25, /* left-hand side: pointer type wanted */
-    P_MMASK   = 1 << 26  /* genr result is masked matrix */
+    P_MMASK   = 1 << 26, /* genr result is masked matrix */
+    P_SLICING = 1 << 27, /* calculating matrix slice (temporary) */
+    P_LAGPRSE = 1 << 28  /* parsing lag spec (temporary) */
 };
 
 struct lhinfo {
@@ -530,8 +542,7 @@ struct parser_ {
     const char *input; /* complete input string */
     const char *point; /* remaining unprocessed input */
     const char *rhs;   /* for use in labelling */
-    double ***Z;       /* convenience pointer to data array */
-    DATAINFO *dinfo;   /* convenience pointer to data info */
+    DATASET *dset;     /* convenience pointer to dataset */
     PRN *prn;          /* for printing messages */
     int flags;         /* various attributes (see above) */
     int targ;          /* target type */
@@ -573,17 +584,12 @@ void context_error (int c, parser *p);
 void undefined_symbol_error (const char *s, parser *p);
 const char *getsymb (int t, const parser *p);
 
-int realgen (const char *s, parser *p, double ***Z, 
-	     DATAINFO *pdinfo, PRN *prn, int flags);
+int realgen (const char *s, parser *p, DATASET *dset, 
+	     PRN *prn, int flags);
 void gen_save_or_print (parser *p, PRN *prn);
 void gen_cleanup (parser *p);
 void parser_free_aux_nodes (parser *p);
 
-void set_matrix_slice_on (void);
-void set_matrix_slice_off (void);
-void set_lag_parse_on (void);
-void set_lag_parse_off (void);
-
 /* name lookup functions */
 const char *constname (int c);
 const char *dvarname (int t);
@@ -595,17 +601,15 @@ int check_declarations (char ***pS, parser *p);
 
 /* in genfuncs.c, used only internally */
 int cross_sectional_stat (double *x, const int *list, 
-			  const double **Z, 
-			  const DATAINFO *pdinfo,
+			  const DATASET *dset,
 			  int f);
 int x_sectional_weighted_stat (double *x, const int *list, 
 			       const int *wlist,
-			       const double **Z, 
-			       const DATAINFO *pdinfo,
+			       const DATASET *dset,
 			       int f);
 
 /* in geneval.c, used only internally */
-double dvar_get_scalar (int i, const DATAINFO *pdinfo,
+double dvar_get_scalar (int i, const DATASET *dset,
 			char *label);
 
 /* helper functions for manual, gretl.lang file */
