@@ -23,8 +23,8 @@
 #include "gretl_string_table.h"
 
 #if GENDEBUG
-# define SDEBUG 1
-# define MDEBUG 1
+# define SDEBUG GENDEBUG
+# define MDEBUG GENDEBUG
 #else
 # define SDEBUG 0
 # define MDEBUG 0
@@ -71,7 +71,7 @@ static NODE *newref (parser *p, int t)
 	if (t == USERIES) {
 	    n->vnum = p->idnum;
 	    n->t = VEC;
-	    n->v.xvec = (*p->Z)[n->vnum];
+	    n->v.xvec = p->dset->Z[n->vnum];
 	} else {
 	    n->t = t;
 	    if (t == USCALAR || t == UMAT || t == UOBJ || 
@@ -350,15 +350,21 @@ static NODE *base (parser *p, NODE *up)
 	break;
     }
 
+#if 0 
+    /* This check ought to be redundant, and I believe it is 
+       now redundant, but I'm leaving it here for the present,
+       just in case. AC 2011-09-05
+    */
     if (p->err == 0 && p->sym != EOT && p->sym != QUERY &&
 	!(p->sym > U_MAX && p->sym < PUNCT_MAX)) {
 	/* catch high-level syntax errors: a "base" is followed
 	   by something that cannot really follow, e.g. a series
 	   is followed by another series
 	*/
-	fprintf(stderr, "gensytax: base: bad exit sym = %d\n", p->sym);
+	fprintf(stderr, "gensytax: base: bad exit-sym = %d\n", p->sym);
 	p->err = E_PARSE;
-    }
+    } 
+#endif
 
 #if SDEBUG
     notify("base", t, p);
@@ -400,7 +406,7 @@ static NODE *listvar_node (parser *p)
 
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
-	if (!strcmp(vname, p->dinfo->varname[v])) {
+	if (!strcmp(vname, p->dset->varname[v])) {
 	    /* found the variable */
 	    free(p->idstr);
 	    p->idstr = NULL;
@@ -734,17 +740,24 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
 		p->err = push_bn_node(t, n);
 	    }
 	    if (p->sym == P_COM) {
+		/* comma: on to the next column */
 		lex(p);
 	    } else if (p->sym == P_SEMI) {
+		/* semicolon: on to the next row */
 		n = newempty();
 		p->err = push_bn_node(t, n);
 		lex(p);
 	    } else if (p->sym == G_RCB) {
+		/* right curly bracket: reached the end */
 		if (p->ch == '[') {
 		    parser_ungetc(p);
 		    *sub = 1;
 		}
 		break;
+	    } else {
+		/* something that doesn't belong here */
+		cexp = '}';
+		break;
 	    }
 	}
     } else {
@@ -762,7 +775,13 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
     if (cexp && !p->err) {
 	expected_symbol_error(cexp, p);
     }
-}	
+}
+
+#define set_matrix_slice_on(p) (p->flags |= P_SLICING)
+#define set_matrix_slice_off(p) (p->flags &= ~P_SLICING)
+
+#define set_lag_parse_on(p) (p->flags |= P_LAGPRSE)
+#define set_lag_parse_off(p) (p->flags &= ~P_LAGPRSE)
 
 static void get_slice_parts (NODE *t, parser *p)
 {
@@ -772,7 +791,7 @@ static void get_slice_parts (NODE *t, parser *p)
     fprintf(stderr, "get_slice_parts, p->sym = %d\n", p->sym);
 #endif  
 
-    set_matrix_slice_on();
+    set_matrix_slice_on(p);
 
     if (p->sym == G_LBR) {
 	lex(p);
@@ -798,7 +817,7 @@ static void get_slice_parts (NODE *t, parser *p)
 	    t->v.b2.r = newempty();
 	    t->v.b2.r->t = ABSENT;
 	    lex(p);
-	    set_matrix_slice_off();
+	    set_matrix_slice_off(p);
 	    return;
 	}
 	if (p->sym == P_COM) {
@@ -836,7 +855,7 @@ static void get_slice_parts (NODE *t, parser *p)
 	expected_symbol_error(cexp, p);
     }
 
-    set_matrix_slice_off();
+    set_matrix_slice_off(p);
 }
 
 static void attach_child (NODE *parent, NODE *child, int k, int i,
@@ -1086,7 +1105,7 @@ static NODE *powterm (parser *p)
 	}
     } else if (sym == LAG || sym == OBS) {
 	if (sym == LAG) {
-	    set_lag_parse_on();
+	    set_lag_parse_on(p);
 	}
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
@@ -1095,7 +1114,7 @@ static NODE *powterm (parser *p)
 	    t->v.b2.r = base(p, t);
 	}
 	if (sym == LAG) {
-	    set_lag_parse_off();
+	    set_lag_parse_off(p);
 	}	
     } else if (sym == MSL || sym == DMSL) {
 	t = newb2(sym, NULL, NULL);
@@ -1246,7 +1265,7 @@ static void convert_pow_term (NODE *n)
 
 /* Test for whether the ' symbol must represent the unary
    transposition operator rather than binary transpose-multiply,
-   based on the following symbol.
+   based on the following symbol, @t.
 */
 
 #define must_be_unary(t) (t == EOT || \
@@ -1358,7 +1377,8 @@ static NODE *expr4 (parser *p)
     while (!p->err && (p->sym == B_ADD || p->sym == B_SUB || 
 		       p->sym == B_DOTADD || p->sym == B_DOTSUB ||
 		       p->sym == B_HCAT || p->sym == B_VCAT ||
-		       p->sym == B_LCAT || p->sym == B_RANGE)) {
+		       p->sym == B_LCAT || p->sym == B_RANGE ||
+		       p->sym == B_ELLIP)) {
 	t = newb2(p->sym, t, NULL);
 	if (t != NULL) {
 	    lex(p);
