@@ -62,17 +62,9 @@ struct saved_list_ {
     int level;
 };
 
-enum {
-    LIST_OMIT_TIGHT,
-    LIST_OMIT_SLOPPY
-};
-
 static saved_list **list_stack;
 static int n_lists;
 
-static int *real_gretl_list_omit (const int *orig, const int *omit, 
-				  int minpos, int mode, int *err);
-
 static saved_list *saved_list_new (const int *list, const char *name)
 {
     saved_list *sl = malloc(sizeof *sl);
@@ -175,6 +167,29 @@ const char *get_list_name_by_index (int idx)
     }
 }
 
+/**
+ * saved_list_get_name:
+ * @list: list whose name should be retrieved.
+ *
+ * Returns: the name of the specified saved list, or NULL if
+ * there is no match.
+ */
+
+const char *saved_list_get_name (const int *list)
+{
+    saved_list *sl;
+    int i;
+
+    for (i=0; i<n_lists; i++) {
+	sl = list_stack[i];
+	if (sl->list != NULL && sl->list == list) {
+	    return sl->name;
+	}
+    }   
+
+    return NULL;
+}
+
 /**
  * get_list_by_name:
  * @name: the name of the list to be found.
@@ -254,9 +269,7 @@ int subtract_from_list_by_name (const char *targ, const int *sub)
     if (sl == NULL) {
 	err = E_UNKVAR;
     } else {
-	int *tmp = real_gretl_list_omit(sl->list, sub, 1, 
-					LIST_OMIT_SLOPPY,
-					&err);
+	int *tmp = gretl_list_drop(sl->list, sub, &err);
 
 	if (!err) {
 	    free(sl->list);
@@ -470,64 +483,34 @@ int rename_saved_list (const char *orig, const char *newname)
 }
 
 /**
- * copy_named_list_as:
- * @orig: the name of the original list.
- * @newname: the name to be given to the copy.
+ * copy_list_as:
+ * @list: the list to copy.
+ * @name: the name to be given to the copy.
  *
- * If a saved list is found by the name @orig, a copy of
- * this list is added to the stack of saved lists under the
- * name @newname.  This is intended for use when a list is given
- * as the argument to a user-defined function: it is copied
- * under the name assigned by the function's parameter list.
+ * This is intended for use when a list is given as the 
+ * argument to a user-defined function: it is copied
+ * under the name assigned by the function's parameter 
+ * list.
  *
- * Returns: 0 on success, non-zero on error.
+ * The "level" of the returned list is 1 above the current
+ * level of function execution.
+ *
+ * Returns: the newly-saved list, or NULL on error.
  */
 
-int copy_named_list_as (const char *orig, const char *newname)
+int *copy_list_as (const int *list, const char *name)
 {
-    saved_list *sl;
-    int err = 0;
+    int err = real_remember_list(list, name, 1, NULL);
+    int *ret = NULL;
 
-    sl = get_saved_list_by_name(orig);
-    if (sl == NULL) {
-	err = 1;
-    } else {
-	err = real_remember_list(sl->list, newname, 1, NULL);
-	if (!err) {
-	    sl = list_stack[n_lists - 1];
-	    sl->level += 1;
-	}
+    if (!err) {
+	saved_list *sl = list_stack[n_lists - 1];
+
+	sl->level += 1;
+	ret = sl->list;
     }
 
-    return err;
-}
-
-/**
- * named_list_lower_level:
- * @name: the name of the list.
- *
- * If a saved list is found by the name @name, at the
- * current level of function execution, lower its level
- * by 1.  This is intended for use when a list is 
- * returned by a user-defined function: it is shifted to
- * the level of the caller.
- *
- * Returns: 0 on success, non-zero on error.
- */ 
-
-int named_list_lower_level (const char *name)
-{
-    saved_list *sl;
-    int err = 0;
-
-    sl = get_saved_list_by_name(name);
-    if (sl == NULL) {
-	err = E_DATA;
-    } else {
-	sl->level -= 1;
-    }
-
-    return err;
+    return ret;
 }
 
 /**
@@ -535,75 +518,74 @@ int named_list_lower_level (const char *name)
  * @name: the name to be given to the list.
  *
  * Creates an empty list under the given @name and adds it
- * to the stack of saved lists at the next level of
- * function execution.  This is intended for use when a null 
- * list is given as an argument to a user-defined function.
+ * to the stack of saved lists.  This is intended for use when 
+ * a null list argument is given to a user-defined function.
  *
- * Returns: 0 on success, non-zero on error.
+ * The "level" of the returned list is 1 above the current
+ * level of function execution.
+ *
+ * Returns: the newly-saved list, or NULL on error.
  */
 
-int create_named_null_list (const char *name)
+int *create_named_null_list (const char *name)
 {
-    saved_list *sl;
-    int *list;
-    int err;
+    int *list = gretl_null_list();
 
-    list = gretl_null_list();
-    if (list == NULL) {
-	return E_ALLOC;
-    }
+    if (list != NULL) {
+	saved_list *sl; 
+	int err;
 
-    err = real_remember_list(list, name, 1, NULL);
-    if (!err) {
-	sl = list_stack[n_lists - 1];
-	sl->level += 1;
-    } else {
+	err = real_remember_list(list, name, 1, NULL);
 	free(list);
+	list = NULL;
+
+	if (!err) {
+	    sl = list_stack[n_lists - 1];
+	    sl->level += 1;
+	    list = sl->list;
+	} 
     }
 
-    return err;
+    return list;
 }
 
 /**
- * create_named_signgleton_list:
+ * create_named_singleton_list:
  * @varnum: ID number of the series to use.
  * @name: the name to be given to the list.
  *
  * Creates a one-element list under the given @name and adds it
- * to the stack of saved lists at the next level of
- * function execution.  This is intended for use when a single 
- * series is given as an argument to a user-defined function in
- * a 'slot' where a list is required.
+ * to the stack of saved lists.  This is intended for use when a 
+ * single series is given as an argument to a user-defined function
+ * in a 'slot' where a list is required.
  *
- * Returns: 0 on success, non-zero on error.
+ * The "level" of the returned list is 1 above the current
+ * level of function execution.
+ *
+ * Returns: the newly saved list, or NULL on error.
  */
 
-int create_named_singleton_list (int varnum, const char *name)
+int *create_named_singleton_list (int varnum, const char *name)
 {
-    saved_list *sl = NULL;
-    int *list = NULL;
-    int err;
+    int *list = gretl_list_new(1);
+ 
+    if (list != NULL) {
+	saved_list *sl;
+	int err;
+	
+	list[1] = varnum;
+	err = real_remember_list(list, name, 1, NULL);
+	free(list);
+	list = NULL;
 
-    list = gretl_list_new(1);
-    if (list == NULL) {
-	return E_ALLOC;
+	if (!err) {
+	    sl = list_stack[n_lists - 1];
+	    sl->level += 1;
+	    list = sl->list;
+	}
     }
 
-    list[1] = varnum;
-
-    err = real_remember_list(list, name, 1, NULL);
-    if (!err) {
-	sl = list_stack[n_lists - 1];
-	sl->level += 1;
-    } 
-
-#if LDEBUG
-    fprintf(stderr, "create_named_singleton_list: sl at %p\n", (void *) sl);
-#endif
-
-    free(list);
-
-    return err;
+    return list;
 }
 
 /**
@@ -1085,14 +1067,13 @@ int *gretl_null_list (void)
 int *gretl_list_copy (const int *src)
 {
     int *targ = NULL;
-    int i;
 
     if (src != NULL) {
-	targ = malloc((src[0] + 1) * sizeof *targ);
+	int n = src[0] + 1;
+
+	targ = malloc(n * sizeof *targ);
 	if (targ != NULL) {
-	    for (i=0; i<=src[0]; i++) {
-		targ[i] = src[i];
-	    }
+	    memcpy(targ, src, n * sizeof *targ);
 	}
     }
 
@@ -1355,7 +1336,7 @@ char *gretl_list_to_string (const int *list)
 /**
  * gretl_list_get_names:
  * @list: array of integers.
- * @pdinfo: dataset information.
+ * @dset: dataset information.
  * @err: location to receive error code.
  * 
  * Prints the names of the members of @list of integers into 
@@ -1365,7 +1346,7 @@ char *gretl_list_to_string (const int *list)
  * or NULL on failure.
  */
 
-char *gretl_list_get_names (const int *list, const DATAINFO *pdinfo,
+char *gretl_list_get_names (const int *list, const DATASET *dset,
 			    int *err)
 {
     char *buf = NULL;
@@ -1383,10 +1364,10 @@ char *gretl_list_get_names (const int *list, const DATAINFO *pdinfo,
 
     for (i=1; i<=list[0]; i++) {
 	vi = list[i];
-	if (vi < 0 || vi >= pdinfo->v) {
+	if (vi < 0 || vi >= dset->v) {
 	    len += strlen("unknown") + 1;
 	} else {
-	    len += strlen(pdinfo->varname[vi]) + 1;
+	    len += strlen(dset->varname[vi]) + 1;
 	}
     }
 
@@ -1400,10 +1381,10 @@ char *gretl_list_get_names (const int *list, const DATAINFO *pdinfo,
 
     for (i=1; i<=list[0]; i++) {
 	vi = list[i];
-	if (vi < 0 || vi >= pdinfo->v) {
+	if (vi < 0 || vi >= dset->v) {
 	    strncat(buf, "unknown", strlen("unknown"));
 	} else {
-	    strncat(buf, pdinfo->varname[vi], strlen(pdinfo->varname[vi]));
+	    strncat(buf, dset->varname[vi], strlen(dset->varname[vi]));
 	}
 	if (i < list[0]) {
 	    strncat(buf, ",", 1);
@@ -1512,8 +1493,7 @@ static void reglist_move_const (int *list, int k)
  * reglist_check_for_const:
  * @list: regression list suitable for use with a gretl
  * model (should not contain #LISTSEP).
- * @Z: data array.
- * @pdinfo: dataset information.
+ * @dset: dataset struct.
  *
  * Checks @list for an intercept term (a variable all of
  * whose valid values in sample are 1).  If such a variable
@@ -1522,10 +1502,9 @@ static void reglist_move_const (int *list, int k)
  * Returns: 1 if the list contains an intercept, else 0.
  */
 
-int reglist_check_for_const (int *list, const double **Z,
-			     const DATAINFO *pdinfo)
+int reglist_check_for_const (int *list, const DATASET *dset)
 {
-    int cpos = gretl_list_const_pos(list, 2, Z, pdinfo);
+    int cpos = gretl_list_const_pos(list, 2, dset);
     int ret = 0;
 
     if (cpos > 1) {
@@ -1573,8 +1552,7 @@ int gretl_list_delete_at_pos (int *list, int pos)
 /**
  * gretl_list_purge_const:
  * @list: list of variable ID numbers.
- * @Z: data array.
- * @pdinfo: dataset information.
+ * @dset: dataset struct.
  *
  * Checks @list from position 1 onward for the presence of a 
  * variable whose valid values in sample all equal 1.0.  If 
@@ -1585,8 +1563,7 @@ int gretl_list_delete_at_pos (int *list, int pos)
  * Returns: 1 if a constant was found and deleted, else 0.
  */
 
-int gretl_list_purge_const (int *list, const double **Z,
-			    const DATAINFO *pdinfo)
+int gretl_list_purge_const (int *list, const DATASET *dset)
 {
     int i, gotc = 0;
     int l0 = list[0];
@@ -1595,7 +1572,7 @@ int gretl_list_purge_const (int *list, const double **Z,
        the only element behind the list separator, remove both
        the constant and the separator */
 
-    if (list[l0] == 0 || true_const(list[l0], Z, pdinfo)) {
+    if (list[l0] == 0 || true_const(list[l0], dset)) {
 	gotc = 1;
 	list[0] -= 1;
 	if (list[l0 - 1] == LISTSEP) {
@@ -1604,7 +1581,7 @@ int gretl_list_purge_const (int *list, const double **Z,
 	}
     } else {
 	for (i=1; i<l0; i++) {
-	    if (list[i] == 0 || true_const(list[i], Z, pdinfo)) {
+	    if (list[i] == 0 || true_const(list[i], dset)) {
 		for ( ; i<l0; i++) {
 		    list[i] = list[i+1];
 		}
@@ -1656,58 +1633,6 @@ int gretl_list_min_max (const int *list, int *lmin, int *lmax)
     }
 }
 
-static int *real_gretl_list_union (const int *orig, const int *add, 
-				   int *err, int duperr)
-{
-    int i, j, k;
-    int *big;
-    const int norig = orig[0];
-    const int nadd = add[0];
-
-    *err = 0;
-
-    big = gretl_list_new(norig + nadd);
-    if (big == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
-    for (i=0; i<=norig; i++) {
-	big[i] = orig[i];
-    }
-
-    k = orig[0];
-
-    for (i=1; i<=nadd; i++) {
-	int match = 0;
-
-	for (j=1; j<=norig; j++) {
-	    if (add[i] == orig[j]) {
-		/* a "new" var was already present */
-		if (duperr) {
-		    free(big);
-		    *err = E_ADDDUP;
-		    return NULL;
-		} else {
-		    match = 1;
-		}
-	    }
-	}
-	if (!match) {
-	    big[0] += 1;
-	    big[++k] = add[i];
-	}
-    }
-
-    if (duperr && big[0] == norig) {
-	free(big);
-	*err = E_NOADD;
-	return NULL;
-    }
-
-    return big;
-}
-
 /**
  * gretl_list_add:
  * @orig: an array of integers, the first element of which holds
@@ -1725,7 +1650,45 @@ static int *real_gretl_list_union (const int *orig, const int *add,
 
 int *gretl_list_add (const int *orig, const int *add, int *err)
 {
-    return real_gretl_list_union(orig, add, err, 1);
+    int n_orig = orig[0];
+    int n_add = add[0];
+    int i, j, k;
+    int *big;
+
+    *err = 0;
+
+    big = gretl_list_new(n_orig + n_add);
+    if (big == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (i=0; i<=n_orig; i++) {
+	big[i] = orig[i];
+    }
+
+    k = orig[0];
+
+    for (i=1; i<=n_add; i++) {
+	for (j=1; j<=n_orig; j++) {
+	    if (add[i] == orig[j]) {
+		/* a "new" var was already present */
+		free(big);
+		*err = E_ADDDUP;
+		return NULL;
+	    }
+	}
+	big[0] += 1;
+	big[++k] = add[i];
+    }
+
+    if (big[0] == n_orig) {
+	free(big);
+	big = NULL;
+	*err = E_NOADD;
+    }
+
+    return big;
 }
 
 /**
@@ -1741,7 +1704,67 @@ int *gretl_list_add (const int *orig, const int *add, int *err)
 
 int *gretl_list_union (const int *l1, const int *l2, int *err)
 {
-    return real_gretl_list_union(l1, l2, err, 0);
+    int *ret, *lcopy;
+    int n_orig = l1[0];
+    int n_add = l2[0];
+    int i, j, k;
+
+    *err = 0;
+
+    lcopy = gretl_list_copy(l2);
+
+    if (lcopy == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    } else {
+	/* see how many terms we're actually adding, if any */
+	for (i=1; i<=l2[0]; i++) {
+	    if (lcopy[i] == -1) {
+		continue;
+	    }
+	    k = in_gretl_list(l1, lcopy[i]);
+	    if (k > 0) {
+		/* element already present in l1 */
+		n_add--;
+		lcopy[i] = -1;
+	    } else {
+		/* not present in l1, but check for duplicates of 
+		   this element in l2 */
+		for (j=1; j<=l2[0]; j++) {
+		    if (j != i && l2[j] == l2[i]) {
+			n_add--;
+			lcopy[j] = -1;
+		    }
+		}
+	    }
+	}
+    }
+
+    if (n_add == 0) {
+	ret = gretl_list_copy(l1);
+    } else {
+	ret = gretl_list_new(n_orig + n_add);
+    }
+
+    if (ret == NULL) {
+	*err = E_ALLOC;
+    } else if (n_add > 0) {
+	for (i=1; i<=n_orig; i++) {
+	    ret[i] = l1[i];
+	}
+
+	k = l1[0];
+
+	for (i=1; i<=lcopy[0]; i++) {
+	    if (lcopy[i] != -1) {
+		ret[++k] = lcopy[i];
+	    }
+	}
+    }
+
+    free(lcopy);
+
+    return ret;
 }
 
 /**
@@ -1800,7 +1823,9 @@ int *gretl_list_intersection (const int *l1, const int *l2, int *err)
  * a count of the number of elements following.
  * @err: location to receive error code.
  *
- * Creates a list containing all but the last element of @orig.
+ * Creates a list containing all but the last element of @orig,
+ * which must not contain #LISTSEP and must contain at least
+ * two members.
  *
  * Returns: new list on success, NULL on error.
  */
@@ -1814,29 +1839,24 @@ int *gretl_list_omit_last (const int *orig, int *err)
 
     if (orig[0] < 2) {
 	*err = E_NOVARS;
-    }
-
-    /* can't handle compound lists */
-    if (*err == 0) {
+    } else {
 	for (i=1; i<=orig[0]; i++) {
 	    if (orig[i] == LISTSEP) {
+		/* can't handle compound lists */
 		*err = 1;
 		break;
 	    }
 	}
     }
 
-    if (*err == 0) {
-	list = malloc(orig[0] * sizeof *list);
+    if (!*err) {
+	list = gretl_list_new(orig[0] - 1);
 	if (list == NULL) {
-	   *err = E_ALLOC;
-	} 
-    }
-    
-    if (list != NULL) {
-	list[0] = orig[0] - 1;
-	for (i=1; i<orig[0]; i++) {
-	    list[i] = orig[i];
+	    *err = E_ALLOC;
+	} else {
+	    for (i=1; i<orig[0]; i++) {
+		list[i] = orig[i];
+	    }
 	}
     }
 
@@ -1858,100 +1878,87 @@ static int list_count (const int *list)
     return k;
 }
 
-static int *real_gretl_list_omit (const int *orig, const int *omit, 
-				  int minpos, int mode, int *err)
-{
-    const int nomit = omit[0];
-    const int norig = list_count(orig);
-    int real_nomit = nomit;
-    int *smal = NULL;
-    int i, j, k;
-
-    *err = 0;
-
-    /* check how many terms we're omitting */
-    for (i=1; i<=nomit; i++) {
-	int pos = in_gretl_list(orig, omit[i]);
-
-	if (pos < minpos) {
-	    if (mode == LIST_OMIT_TIGHT) {
-		/* "omission" is spurious */
-		gretl_errmsg_sprintf(_("Variable %d was not in the original list"),
-				     omit[i]);
-		*err = 1;
-		return NULL;
-	    } else {
-		real_nomit--;
-	    }
-	}
-    }
-
-    if (minpos > 1) {
-	/* it's an error to attempt to omit all vars */
-	if (nomit == norig - 1) {
-	    *err = E_NOVARS;
-	    return NULL;
-	}
-    }
-
-    if (real_nomit == norig) {
-	/* omitting all */
-	smal = gretl_null_list();
-	if (smal == NULL) {
-	    *err = E_ALLOC;
-	}
-	return smal;
-    }
-
-    smal = gretl_list_new(norig - real_nomit);
-    if (smal == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
-    for (i=1; i<minpos; i++) {
-	/* copy unaltered portion */
-	smal[i] = orig[i];
-    }
-
-    k = minpos;
-    for (i=minpos; i<=norig; i++) {
-        int match = 0;
-
-        for (j=1; j<=nomit; j++) {
-            if (orig[i] == omit[j]) {
-                match = 1; /* matching var: omit it */
-		break;
-            }
-        }
-        if (!match) { /* var is not in omit list: keep it */
-            smal[k++] = orig[i];
-        }
-    }
-
-    return smal;
-}
-
 /**
  * gretl_list_omit:
  * @orig: an array of integers, the first element of which holds
  * a count of the number of elements following.
  * @omit: list of variables to drop.
- * @minpos: minimum position to check.  This should be 2 for a regular
+ * @minpos: minimum position to check. This should be 2 for a regular
  * regression list, to skip the dependent var in position 1; but in
  * other contexts it may be 1 to start from the first element of @orig.
  * @err: pointer to receive error code.
  *
  * Creates a list containing the elements of @orig that are not
  * present in @omit.  It is an error if the @omit list contains
- * members that are not present in @orig.
+ * members that are not present in @orig, and also if the @omit
+ * list contains duplicated elements.
  *
  * Returns: new list on success, NULL on error.
  */
 
 int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
 {
-    return real_gretl_list_omit(orig, omit, minpos, LIST_OMIT_TIGHT, err);
+    int n_omit = omit[0];
+    int n_orig = list_count(orig);
+    int *ret = NULL;
+    int i, j, k;
+
+    if (n_omit > n_orig) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    *err = 0;
+
+    /* check for spurious "omissions" */
+    for (i=1; i<=omit[0]; i++) {
+	k = in_gretl_list(orig, omit[i]);
+	if (k < minpos) {
+	    gretl_errmsg_sprintf(_("Variable %d was not in the original list"),
+				 omit[i]);
+	    *err = 1;
+	    return NULL;
+	}
+    }
+
+    if (minpos > 1) {
+	/* regression list: it's an error to attempt to omit 
+	   all regressors */
+	if (n_omit == n_orig - 1) {
+	    *err = E_NOVARS;
+	    return NULL;
+	}
+    }
+
+    ret = gretl_list_new(n_orig - n_omit);
+
+    if (ret == NULL) {
+	*err = E_ALLOC;
+    } else if (n_omit < n_orig) {
+	int match;
+
+	k = 1;
+	for (i=1; i<=n_orig; i++) {
+	    if (i < minpos) {
+		ret[k++] = orig[i];
+	    } else {
+		match = 0;
+		for (j=1; j<=n_omit; j++) {
+		    if (orig[i] == omit[j]) {
+			/* matching var: omit it */
+			match = 1;
+			break;
+		    }
+		}
+		if (!match) { 
+		    /* var is not in omit list: keep it */
+		    ret[k++] = orig[i];
+		}
+	    }
+	}
+    }		
+
+    return ret;
 }
 
 /**
@@ -1964,14 +1971,55 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
  * Creates a list containing the elements of @orig that are not
  * present in @drop.  Unlike gretl_list_omit(), processing always
  * starts from position 1 in @orig, and it is not an error if
- * some members of @drop are not present in @orig.  
+ * some members of @drop are not present in @orig, or if some
+ * members of @drop are duplicated. 
  *
  * Returns: new list on success, NULL on error.
  */
 
 int *gretl_list_drop (const int *orig, const int *drop, int *err)
 {
-    return real_gretl_list_omit(orig, drop, 1, LIST_OMIT_SLOPPY, err);
+    int *lcopy = NULL;
+    int *ret = NULL;
+    int n_omit = 0;
+    int i, k;
+
+    *err = 0;
+
+    lcopy = gretl_list_copy(orig);
+
+    if (lcopy == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    } else {
+	/* see how many terms we're omitting */
+	for (i=1; i<=drop[0]; i++) {
+	    k = in_gretl_list(lcopy, drop[i]);
+	    if (k > 0) {
+		n_omit++;
+		lcopy[k] = -1;
+	    }
+	}
+    }
+
+    if (n_omit == 0) {
+	ret = lcopy;
+    } else {
+	ret = gretl_list_new(orig[0] - n_omit);
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	} else if (n_omit < orig[0]) {
+	    k = 1;
+	    for (i=1; i<=orig[0]; i++) {
+		if (lcopy[i] >= 0) {
+		    ret[k++] = orig[i];
+		}
+	    }
+	}
+	free(lcopy);
+    }
+
+    return ret;
 }
 
 /**
@@ -2100,9 +2148,8 @@ int *gretl_list_diff_new (const int *biglist, const int *sublist,
 int gretl_list_add_list (int **targ, const int *src)
 {
     int *big;
-    int n1 = (*targ)[0];
-    int n2 = src[0];
-    int i, err = 0;
+    int i, n1, n2;
+    int err = 0;
 
     if (targ == NULL || *targ == NULL) {
 	return E_DATA;
@@ -2113,6 +2160,9 @@ int gretl_list_add_list (int **targ, const int *src)
 	return 0;
     }
 
+    n1 = (*targ)[0];
+    n2 = src[0];
+
     big = realloc(*targ, (n1 + n2 + 1) * sizeof *big);
 
     if (big == NULL) {
@@ -2153,6 +2203,7 @@ int gretl_list_insert_list (int **targ, const int *src, int pos)
     }
 
     big = realloc(*targ, (bign + 1) * sizeof *big);
+
     if (big == NULL) {
 	err = E_ALLOC;
     } else {
@@ -2216,7 +2267,7 @@ int gretl_list_insert_list_minus (int **targ, const int *src, int pos)
  * list_members_replaced:
  * @list: an array of integer variable ID numbers, the first element
  * of which holds a count of the number of elements following.
- * @pdinfo: dataset information.
+ * @dset: dataset information.
  * @ref_id: ID number of reference #MODEL.
  *
  * Checks whether any variable in @list has been redefined via
@@ -2226,7 +2277,7 @@ int gretl_list_insert_list_minus (int **targ, const int *src, int pos)
  * Returns: 1 if any variables have been replaced, 0 otherwise.
  */
 
-int list_members_replaced (const int *list, const DATAINFO *pdinfo,
+int list_members_replaced (const int *list, const DATASET *dset,
 			   int ref_id)
 {
     const char *errmsg = N_("Can't do this: some vars in original "
@@ -2245,11 +2296,11 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 	if (list[j] == LISTSEP) {
 	    continue;
 	}
-	if (list[j] >= pdinfo->v) {
+	if (list[j] >= dset->v) {
 	    gretl_errmsg_set(_(errmsg));
 	    return E_DATA;
 	}
-	label = VARLABEL(pdinfo, list[j]);
+	label = VARLABEL(dset, list[j]);
 	*rword = '\0';
 	sscanf(label, "%15s", rword);
 	if (!strcmp(rword, _("Replaced"))) {
@@ -2270,8 +2321,7 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
  * @list: an array of integer variable ID numbers, the first element
  * of which holds a count of the number of elements following.
  * @minpos: position in @list at which to start the search (>= 1).
- * @Z: data array.
- * @pdinfo: dataset information.
+ * @dset: dataset struct.
  *
  * Checks @list for the presence, in position @minpos or higher, of
  * a variable whose valid values in sample all equal 1.  This usually
@@ -2282,8 +2332,8 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
  * found.
  */
 
-int gretl_list_const_pos (const int *list, int minpos, const double **Z, 
-			  const DATAINFO *pdinfo)
+int gretl_list_const_pos (const int *list, int minpos,  
+			  const DATASET *dset)
 {
     int i;
 
@@ -2300,7 +2350,7 @@ int gretl_list_const_pos (const int *list, int minpos, const double **Z,
 
     /* ... but if it's not found */
     for (i=minpos; i<=list[0]; i++) {
-        if (true_const(list[i], Z, pdinfo)) {
+        if (true_const(list[i], dset)) {
 	    return i;
 	}
     }
@@ -2645,7 +2695,7 @@ int gretl_list_n_distinct_members (const int *list)
 
 /**
  * full_var_list:
- * @pdinfo: dataset information.
+ * @dset: dataset information.
  * @nvars: location for return of number of elements in full list.
  *
  * Creates a newly allocated list including all series in the
@@ -2661,15 +2711,15 @@ int gretl_list_n_distinct_members (const int *list)
  * Returns: the allocated list, or NULL.
  */
 
-int *full_var_list (const DATAINFO *pdinfo, int *nvars)
+int *full_var_list (const DATASET *dset, int *nvars)
 {
     int fsd = gretl_function_depth();
     int i, j, nv = 0;
     int *list = NULL;
 
-    for (i=1; i<pdinfo->v; i++) {
-	if (!var_is_hidden(pdinfo, i) &&
-	    STACK_LEVEL(pdinfo, i) == fsd) {
+    for (i=1; i<dset->v; i++) {
+	if (!var_is_hidden(dset, i) &&
+	    STACK_LEVEL(dset, i) == fsd) {
 	    nv++;
 	}
     }
@@ -2684,9 +2734,9 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
 
     if (list != NULL) {
 	j = 1;
-	for (i=1; i<pdinfo->v; i++) {
-	    if (!var_is_hidden(pdinfo, i) &&
-		STACK_LEVEL(pdinfo, i) == fsd) {
+	for (i=1; i<dset->v; i++) {
+	    if (!var_is_hidden(dset, i) &&
+		STACK_LEVEL(dset, i) == fsd) {
 		list[j++] = i;
 	    }
 	}
@@ -2695,38 +2745,6 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
     return list;
 }
 
-/**
- * gretl_list_position:
- * @v: ID number of variable to find.
- * @list: list to search.
- *
- * Scans @list, from position 1 onward, for variable @v.
- * In case the list contains a list separator, #LISTSEP,
- * only the portion prior to the separator is considered.
- *
- * Returns: the 0-based position, or 0 if the variable is 
- * not found.
- */
-
-int gretl_list_position (int v, const int *list)
-{
-    int i, lmax = list[0];
-
-    /* handle lists with separator: search only first part */
-    for (i=1; i<=list[0]; i++) {
-	if (list[i] == LISTSEP) {
-	    lmax = i - 1;
-	    break;
-	}
-    }
-	    
-    for (i=lmax; i>=1; i--) {
-	if (v == list[i]) return i;
-    }
-
-    return 0;
-}
-
 /**
  * gretl_list_is_consecutive:
  * @list: list to check.
@@ -2752,7 +2770,7 @@ int gretl_list_is_consecutive (const int *list)
 /**
  * gretl_list_build:
  * @s: string list specification.
- * @pdinfo: dataset information.
+ * @dset: dataset information.
  * @err: location to receive error code
  *
  * Builds a list based on the specification in @s, which may include
@@ -2762,7 +2780,7 @@ int gretl_list_is_consecutive (const int *list)
  * Returns: the constructed list, or NULL on failure.
  */
 
-int *gretl_list_build (const char *s, const DATAINFO *pdinfo, int *err)
+int *gretl_list_build (const char *s, const DATASET *dset, int *err)
 {
     char test[32];
     int *list = NULL;
@@ -2796,8 +2814,8 @@ int *gretl_list_build (const char *s, const DATAINFO *pdinfo, int *err)
 		    *err = E_PARSE;
 		}
 	    } else {
-		v = series_index(pdinfo, test);
-		if (v < pdinfo->v) {
+		v = series_index(dset, test);
+		if (v < dset->v) {
 		    list = gretl_list_append_term(&list, v);
 		} else {
 		    nlist = get_list_by_name(test);
@@ -2830,8 +2848,6 @@ int load_user_lists_file (const char *fname)
     xmlNodePtr node = NULL;
     int i, nl, err = 0;
 
-    xmlKeepBlanksDefault(0);
-
     err = gretl_xml_open_doc_root(fname, "gretl-lists", &doc, &node);
     if (err) {
 	return err;
@@ -2892,7 +2908,6 @@ int load_user_lists_file (const char *fname)
 
     if (doc != NULL) {
 	xmlFreeDoc(doc);
-	xmlCleanupParser();
     }
 
     return err;
@@ -2943,13 +2958,13 @@ int gretl_serialize_lists (const char *fname)
 /**
  * gretl_list_print:
  * @lname: name of list.
- * @pdinfo: dataset information.
+ * @dset: dataset information.
  * @prn: gretl printing struct.
  * 
  * Prints to @prn the given @list of variables, by name.
  */
 
-void gretl_list_print (const char *lname, const DATAINFO *pdinfo,
+void gretl_list_print (const char *lname, const DATASET *dset,
 		       PRN *prn)
 {
     const int *list = get_list_by_name(lname);
@@ -2966,10 +2981,10 @@ void gretl_list_print (const char *lname, const DATAINFO *pdinfo,
 	    li = list[i];
 	    if (li == LISTSEP) {
 		len += pputs(prn, "; ");
-	    } else if (li < 0 || li >= pdinfo->v) {
+	    } else if (li < 0 || li >= dset->v) {
 		len += pputs(prn, "?? ");
 	    } else {
-		len += pprintf(prn, "%s ", pdinfo->varname[li]);
+		len += pprintf(prn, "%s ", dset->varname[li]);
 		if (len > testlen && i < list[0]) {
 		    pputs(prn, "\\\n "); 
 		    len = 1;
@@ -2982,20 +2997,22 @@ void gretl_list_print (const char *lname, const DATAINFO *pdinfo,
 
 /**
  * varname_match_list:
- * @pdinfo: pointer to dataset information.
+ * @dset: pointer to dataset information.
  * @pattern: pattern to be matched.
+ * @err: location to receive error code.
  *
  * Returns: a list of ID numbers of variables whose names
  * match @pattern, or NULL if there are no matches.
  */
 
-int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
+int *varname_match_list (const DATASET *dset, const char *pattern,
+			 int *err)
 {
     GPatternSpec *pspec;
     int *list = NULL;
     int i, fd, n = 0;
 
-    if (pdinfo == NULL || pdinfo->v == 0) {
+    if (dset == NULL || dset->v == 0) {
 	return NULL;
     }
 
@@ -3003,9 +3020,9 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 
     pspec = g_pattern_spec_new(pattern);
 
-    for (i=1; i<pdinfo->v; i++) { 
-	if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
-	    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+    for (i=1; i<dset->v; i++) { 
+	if (fd == 0 || fd == STACK_LEVEL(dset, i)) {
+	    if (g_pattern_match_string(pspec, dset->varname[i])) {
 		n++;
 	    }
 	}
@@ -3013,12 +3030,14 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 
     if (n > 0) {
 	list = gretl_list_new(n);
-	if (list != NULL) {
+	if (list == NULL) {
+	    *err = E_ALLOC;
+	} else {
 	    int j = 1;
 
-	    for (i=1; i<pdinfo->v; i++) { 
-		if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
-		    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+	    for (i=1; i<dset->v; i++) { 
+		if (fd == 0 || fd == STACK_LEVEL(dset, i)) {
+		    if (g_pattern_match_string(pspec, dset->varname[i])) {
 			list[j++] = i;
 		    }
 		}
@@ -3031,16 +3050,62 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
     return list;
 }
 
+/**
+ * ellipsis_list:
+ * @dset: pointer to dataset information.
+ * @v1: index of first variable.
+ * @v2: index of last variable.
+ * @err: location to receive error code.
+ *
+ * Returns: a list of ID numbers of variables running
+ * from @v1 to @v2.
+ */
+
+int *ellipsis_list (const DATASET *dset, int v1, int v2, int *err)
+{
+    int *list = NULL;
+    int i, fd, n = 0;
+
+    if (dset == NULL || dset->v == 0) {
+	return NULL;
+    }
+
+    fd = gretl_function_depth();
+
+    for (i=v1; i<=v2; i++) { 
+	if (fd == 0 || fd == STACK_LEVEL(dset, i)) {
+	    n++;
+	}
+    }
+
+    if (n > 0) {
+	list = gretl_list_new(n);
+	if (list == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    int j = 1;
+
+	    for (i=v1; i<=v2; i++) { 
+		if (fd == 0 || fd == STACK_LEVEL(dset, i)) {
+		    list[j++] = i;
+		}
+	    }
+	}
+    }
+
+    return list;
+}
+
 /**
  * varname_match_any:
- * @pdinfo: pointer to dataset information.
+ * @dset: pointer to dataset information.
  * @pattern: pattern to be matched.
  *
  * Returns: 1 if at least one variable in the dataset has a
  * name that matches @pattern, otherwise 0.
  */
 
-int varname_match_any (const DATAINFO *pdinfo, const char *pattern)
+int varname_match_any (const DATASET *dset, const char *pattern)
 {
     GPatternSpec *pspec;
     int i, fd, ret = 0;
@@ -3049,9 +3114,9 @@ int varname_match_any (const DATAINFO *pdinfo, const char *pattern)
 
     pspec = g_pattern_spec_new(pattern);
 
-    for (i=1; i<pdinfo->v; i++) { 
-	if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
-	    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+    for (i=1; i<dset->v; i++) { 
+	if (fd == 0 || fd == STACK_LEVEL(dset, i)) {
+	    if (g_pattern_match_string(pspec, dset->varname[i])) {
 		ret = 1;
 		break;
 	    }
