@@ -387,6 +387,16 @@ gretl_matrix_kronecker_product_new (const gretl_matrix *A,
 				    const gretl_matrix *B,
 				    int *err);
 
+int gretl_matrix_hdproduct (const gretl_matrix *A, 
+			    const gretl_matrix *B,
+			    gretl_matrix *C);
+
+gretl_matrix *
+gretl_matrix_hdproduct_new (const gretl_matrix *A, 
+			    const gretl_matrix *B,
+			    int *err);
+
+
 int
 gretl_matrix_I_kronecker (int p, const gretl_matrix *B,
 			  gretl_matrix *K);
@@ -676,7 +686,7 @@ gretl_matrix_array_new_with_size (int n, int rows, int cols);
 void gretl_matrix_array_free (gretl_matrix **A, int n);
 
 gretl_matrix *gretl_matrix_values (const double *x, int n,
-				   int *err);
+				   gretlopt opt, int *err);
 
 gretl_matrix *gretl_matrix_shape (const gretl_matrix *A, 
 				  int r, int c);
