@@ -40,10 +40,7 @@
 #include <fcntl.h> /* for 'open' */
 
 #include <glib.h>
-
-#if (GLIB_MAJOR_VERSION > 2 || GLIB_MINOR_VERSION >= 6)
-# include <glib/gstdio.h>
-#endif
+#include <glib/gstdio.h>
 
 #ifndef WIN32
 # ifdef USE_GTK3
@@ -258,7 +255,7 @@ FILE *gretl_fopen (const char *fname, const char *mode)
  * @fname: name of file to be opened.
  * @mode: mode in which to open the file.
  *
- * basically the same as gretl_fopen(), except that no
+ * Basically the same as gretl_fopen(), except that no
  * error message is logged if @fname cannot be opened.
  *
  * Returns: file pointer, or %NULL on failure.
@@ -588,7 +585,7 @@ int gretl_mkdir (const char *path)
     }
 
     if (pconv != NULL) {
-	test = win32_opendir(pconv);
+	test = gretl_opendir(pconv);
 	if (test != NULL) {
 	    closedir(test);
 	    done = 1;
@@ -597,7 +594,7 @@ int gretl_mkdir (const char *path)
 	}
 	g_free(pconv);
     } else {
-	test = win32_opendir(path);
+	test = gretl_opendir(path);
 	if (test != NULL) {
 	    closedir(test);
 	    done = 1;
@@ -663,20 +660,7 @@ int gretl_mkdir (const char *path)
 
     errno = 0;
 
-#if (GLIB_MAJOR_VERSION > 2 || GLIB_MINOR_VERSION >= 8)
     err = g_mkdir_with_parents(path, 0755);
-#else
-    err = mkdir(path, 0755);
-    if (err && errno == EEXIST) {
-	/* OK if we have an existing directory */
-	DIR *dir = opendir(path);
-
-	if (dir != NULL) {
-	    closedir(dir);
-	    err = 0;
-	}
-    }
-#endif
 
     if (err) {
 	fprintf(stderr, "%s: %s\n", path, strerror(errno));
@@ -748,6 +732,38 @@ int gretl_deltree (const char *path)
 
 #endif /* WIN32 or not */
 
+DIR *gretl_opendir (const char *name)
+{
+#ifdef WIN32
+    int n = strlen(name);
+    int fixit = 0;
+
+    if (n > 0 && name[n-1] == ':') {
+	/* opendir doesn't work on e.g. "f:" */
+	fixit = 1;
+    } else if (n > 3 && name[n-1] == '\\') {
+	/* and neither does it work on e.g. "c:\foo\" */
+	fixit = 2;
+    }
+
+    if (fixit) {
+	char tmp[MAXLEN];
+
+	*tmp = '\0';
+	strncat(tmp, name, MAXLEN - 2);
+	if (fixit == 1) {
+	    /* append backslash */
+	    strcat(tmp, "\\");
+	} else {
+	    /* chop backslash */
+	    tmp[strlen(tmp)-1] = '\0';
+	} 
+	return opendir(tmp);
+    }
+#endif
+    return opendir(name);
+}
+
 /**
  * gretl_setenv:
  * @name: name of variable to be set.
@@ -761,14 +777,12 @@ int gretl_deltree (const char *path)
 int gretl_setenv (const char *name, const char *value)
 {
 #ifdef WIN32
-    char estr[1024];
+    gchar *estr;
     int ok;
 
     /* belt and braces */
-    if (strlen(name) + strlen(value) + 1 < 1024) {
-	sprintf(estr, "%s=%s", name, value);
-	putenv(estr);
-    }
+    estr = g_strdup_printf("%s=%s", name, value);
+    putenv(estr);
 
     ok = SetEnvironmentVariable(name, value);
 
@@ -1066,27 +1080,27 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
 
 static char *search_dir (char *fname, const char *topdir, int code)
 {
-    FILE *test;
+    FILE *fp;
     char orig[MAXLEN];
 
     strcpy(orig, fname);
 
     if (gretl_path_prepend(fname, topdir) == 0) {
-	test = gretl_try_fopen(fname, "r");
-	if (test != NULL) {
-	    fclose(test);
+	fp = gretl_try_fopen(fname, "r");
+	if (fp != NULL) {
+	    fclose(fp);
 	    return fname;
 	}
 	if (code == DATA_SEARCH && add_suffix(fname, ".gdt")) {
-	    test = gretl_try_fopen(fname, "r");
-	    if (test != NULL) {
-		fclose(test);
+	    fp = gretl_try_fopen(fname, "r");
+	    if (fp != NULL) {
+		fclose(fp);
 		return fname;
 	    }
 	} else if (code == FUNCS_SEARCH && add_suffix(fname, ".gfn")) {
-	    test = gretl_try_fopen(fname, "r");
-	    if (test != NULL) {
-		fclose(test);
+	    fp = gretl_try_fopen(fname, "r");
+	    if (fp != NULL) {
+		fclose(fp);
 		return fname;
 	    }
 	}	    
@@ -1190,52 +1204,77 @@ static int shelldir_open_dotfile (char *fname, char *orig)
 }
 
 /**
- * get_plausible_functions_dir:
- * @fndir: location to receive directory name; should be of
- * length FILENAME_MAX or more.
- * @i: call index; should be zero on the first call.
+ * get_plausible_search_dirs:
+ * @type: should be DATA_SEARCH for data file packages,
+ * DB_SEARCH for gretl databases, or FUNCS_SEARCH for
+ * function packages.
+ * @n_dirs: location to receive the number of directories.
  * 
- * This function can be called repeatedly, with the call index
- * incremented each time -- so long as the return value is non-zero.
- * The @fndir argument will be filled out with the name of the "next"
- * directory that may plausibly contain gretl gfn files. If the call
- * index is out of bounds 0 is returned and @fndir is set to an
- * empty string.
- *
- * Returns: 1 if there are any more directories available, otherwise
- * zero.
+ * Returns: an array of plausible search paths, depending on the 
+ * @type of search. The array should be freed when you are done 
+ * with it, using free_strings_array().
  */
 
-int get_plausible_functions_dir (char *fndir, int i)
+char **get_plausible_search_dirs (int type, int *n_dirs)
 {
-    int ret = 1;
+    char **dirs = NULL;
+    const char *subdir;
+    char dirname[MAXLEN];
+    int err = 0;
 
-    *fndir = '\0';
-    
-    if (i == 0) {
-	/* pick up any function files in system dir */
-	build_path(fndir, gretl_home(), "functions", NULL);
-    } else if (i == 1) {
-	/* or any function files in the user's working dir... */
-	strcpy(fndir, gretl_workdir());
-    } else if (i == 2) {
-	/* or in any "functions" subdir thereof */
-	build_path(fndir, gretl_workdir(), "functions", NULL);
-    } else if (i == 3) {
-	/* or any in the user's dotdir */
-	build_path(fndir, gretl_dotdir(), "functions", NULL);
-    } else if (i == 4) {
-	/* or any in the default working dir, if not already searched */
-	const char *wdir = gretl_default_workdir();
+    *n_dirs = 0;
 
-	if (wdir != NULL) {
-	    build_path(fndir, wdir, "functions", NULL);
-	} 
+    if (type == DATA_SEARCH) {
+	subdir = "data";
+    } else if (type == DB_SEARCH) {
+	subdir = "db";
+    } else if (type == FUNCS_SEARCH) {
+	subdir = "functions";
+    } else if (type == SCRIPT_SEARCH) {
+	subdir = "scripts";
     } else {
-	ret = 0;
+	fprintf(stderr, "get_plausible_search_dir: no type specified\n");
+	return NULL;
     }
 
-    return ret;
+    /* system dir first */
+    build_path(dirname, gretl_home(), subdir, NULL);
+    err = strings_array_add(&dirs, n_dirs, dirname);
+
+#ifdef OSX_BUILD
+    if (!err) {
+	/* the user's ~/Library */
+	build_path(dirname, gretl_app_support_dir(), subdir, NULL);
+	err = strings_array_add(&dirs, n_dirs, dirname);
+    }
+#endif
+
+    if (!err) {
+	/* the user's working dir */
+	build_path(dirname, gretl_workdir(), subdir, NULL);
+	err = strings_array_add(&dirs, n_dirs, dirname);
+    }
+
+    if (!err) {
+	/* working dir, no subdir */
+	strcpy(dirname, gretl_workdir());
+	err = strings_array_add(&dirs, n_dirs, dirname);
+    }
+
+    if (!err) {
+	const char *wd = maybe_get_default_workdir();
+
+	if (wd != NULL) {
+	    build_path(dirname, wd, subdir, NULL);
+	    err = strings_array_add(&dirs, n_dirs, dirname);
+	    if (!err && type != FUNCS_SEARCH) {
+		strcpy(dirname, wd);
+		err = strings_array_add(&dirs, n_dirs, dirname);
+	    }
+	}
+    }
+
+    return dirs;
 }
 
 /* it's a dirent thing */
@@ -1246,9 +1285,10 @@ int get_plausible_functions_dir (char *fndir, int i)
 /**
  * gretl_function_package_get_path:
  * @name: the name of the package to find, without the .gfn extension.
- * @type: %PKG_REGULAR for a standard gfn package or %PKG_ADDON for
- * a reognized gretl "addon".
- * 
+ * @type: %PKG_SUBDIR for a recognized gretl "addon", %PKG_TOPLEV
+ * for a package file not in a subdirectory, or %PKG_ALL for a
+ * package that may be at either level.
+ *
  * Searches a list of directories in which we might expect to find
  * function packages, and, if the package in question is found,
  * returns a newly allocated string holding the full path to
@@ -1262,44 +1302,48 @@ char *gretl_function_package_get_path (const char *name,
 				       PkgType type)
 {
     char *ret = NULL;
-    char fndir[FILENAME_MAX];
     char path[FILENAME_MAX];
+    char **dirs;
     int found = 0;
-    int i = 0;
+    int i, n_dirs;
 
     *path = '\0';
 
-    while (get_plausible_functions_dir(fndir, i++) && !found) {
+    dirs = get_plausible_search_dirs(FUNCS_SEARCH, &n_dirs);
+
+    for (i=0; i<n_dirs && !found; i++) {
+	const char *fndir = dirs[i];
 	struct dirent *dirent;
 	const char *dname;
 	char *p, test[NAME_MAX+1];
 	DIR *dir;
 	
-	if (*fndir == '\0' || (dir = opendir(fndir)) == NULL) {
+	if ((dir = gretl_opendir(fndir)) == NULL) {
 	    continue;
 	}
 
-	/* look preferentially for .gfn files in their own
-	   subdirectories */
+	if (type != PKG_TOPLEV) {
+	    /* look preferentially for .gfn files in their own
+	       subdirectories */
+	    while ((dirent = readdir(dir)) != NULL && !found) {
+		dname = dirent->d_name;
+		if (!strcmp(dname, name)) {
+		    FILE *fp;
 
-	while ((dirent = readdir(dir)) != NULL && !found) {
-	    dname = dirent->d_name;
-	    if (!strcmp(dname, name)) {
-		FILE *fp;
-
-		sprintf(path, "%s%c%s%c%s.gfn", fndir, SLASH, 
-			dname, SLASH, dname);
-		fp = gretl_try_fopen(path, "r");
-		if (fp != NULL) {
-		    fclose(fp);
-		    found = 1;
-		} else {
-		    *path = '\0';
+		    sprintf(path, "%s%c%s%c%s.gfn", fndir, SLASH, 
+			    dname, SLASH, dname);
+		    fp = gretl_try_fopen(path, "r");
+		    if (fp != NULL) {
+			fclose(fp);
+			found = 1;
+		    } else {
+			*path = '\0';
+		    }
 		}
 	    }
 	}
 
-	if (!found && type != PKG_ADDON) {
+	if (!found && type != PKG_SUBDIR) {
 	    /* look for .gfn files in the top-level functions
 	       directory */
 	    rewinddir(dir);
@@ -1320,6 +1364,8 @@ char *gretl_function_package_get_path (const char *name,
 	closedir(dir);
     }
 
+    free_strings_array(dirs, n_dirs);
+
     if (*path != '\0') {
 	ret = gretl_strdup(path);
     }
@@ -1416,7 +1462,7 @@ char *gretl_addpath (char *fname, int script)
 
     /* try looking in default workdir? */
     if (1) {
-	const char *dwork = gretl_default_workdir();
+	const char *dwork = maybe_get_default_workdir();
 
 	if (dwork != NULL) {
 	    int ok = 0;
@@ -1785,65 +1831,59 @@ const char *gretl_workdir (void)
     return paths.workdir;
 }
 
-static char default_workdir[MAXLEN];
-
 #ifdef WIN32
 
-static const char *win32_default_workdir (void)
+static const char *win32_default_workdir (int force)
 {
-    const char *ret = NULL;
+    static char default_workdir[MAXLEN];
     char *base = mydocs_path();
     int ok = 0;
 
     if (base != NULL) {
 	sprintf(default_workdir, "%s\\gretl\\", base);
-	if (!strcmp(default_workdir, paths.workdir)) {
-	    *default_workdir = '\0';
-	} else {
-	    DIR *dir = win32_opendir(default_workdir);
+	if (force || strcmp(default_workdir, paths.workdir)) {
+	    if (force) {
+		ok = (gretl_mkdir(default_workdir) == 0);
+	    } else {
+		DIR *dir = gretl_opendir(default_workdir);
 
-	    if (dir != NULL) {
-		closedir(dir);
-		ok = 1;
+		if (dir != NULL) {
+		    closedir(dir);
+		    ok = 1;
+		}
 	    }
 	}
 	free(base);
     }
 
-    if (ok) {
-	ret = default_workdir;
-    }
-
-    return ret;
+    return (ok)? default_workdir : NULL;
 }
 
 #else /* !WIN32 */
 
-static const char *regular_default_workdir (void)
+static const char *regular_default_workdir (int force)
 {
-    const char *ret = NULL;
+    static char default_workdir[MAXLEN];
     char *home = getenv("HOME");
     int ok = 0;
 
     if (home != NULL) {
 	sprintf(default_workdir, "%s/gretl/", home);
-	if (!strcmp(default_workdir, paths.workdir)) {
-	    *default_workdir = '\0';
-	} else {
-	    DIR *dir = opendir(default_workdir);
+	if (force || strcmp(default_workdir, paths.workdir)) {
+	    if (force) {
+		ok = (gretl_mkdir(default_workdir) == 0);
+	    } else {
+		DIR *dir = opendir(default_workdir);
 
-	    if (dir != NULL) {
-		closedir(dir);
-		ok = 1;
+		if (dir != NULL) {
+		    closedir(dir);
+		    ok = 1;
+		} 
 	    }
 	}
     }
 
-    if (ok) {
-	ret = default_workdir;
-    }
-
-    return ret;
+    return (ok)? default_workdir : NULL;
 }
 
 #endif /* WIN32 or not */
@@ -1852,17 +1892,37 @@ static const char *regular_default_workdir (void)
  * gretl_default_workdir:
  *
  * Returns: the full path to the default value of the
- * user's gretl working directory if this differs from
- * the user's currently-defined gretl working directory,
- * or NULL if the two are the same.
+ * user's gretl working directory, or NULL if it
+ * happens that this path cannot be determined,
+ * or is not writable by the current user.
  */
 
 const char *gretl_default_workdir (void)
 {
 #ifdef WIN32
-    return win32_default_workdir();
+    return win32_default_workdir(1);
 #else
-    return regular_default_workdir();
+    return regular_default_workdir(1);
+#endif
+}
+
+/**
+ * maybe_get_default_workdir:
+ *
+ * Acts like gretl_default_workdir(), except that this
+ * function returns NULL if the default working
+ * directory is the same as the current gretl
+ * working directory, as returned by gretl_workdir().
+ *
+ * Returns: a path, or NULL.
+ */
+
+const char *maybe_get_default_workdir (void)
+{
+#ifdef WIN32
+    return win32_default_workdir(0);
+#else
+    return regular_default_workdir(0);
 #endif
 }
 
@@ -1912,11 +1972,7 @@ int set_gretl_work_dir (const char *path)
 
     errno = 0;
 
-#ifdef WIN32
-    test = win32_opendir(path);
-#else
-    test = opendir(path);
-#endif
+    test = gretl_opendir(path);
 
     if (test == NULL) {
 	gretl_errmsg_set_from_errno(path);
@@ -2621,22 +2677,6 @@ static void copy_paths_with_fallback (ConfigPaths *cpaths)
     path_init(paths.pngfont, cpaths->pngfont, 0);
 }
 
-static void set_up_sourceview_path (void)
-{
-    char envstr[MAXLEN];
-
-#ifdef WIN32
-    sprintf(envstr, "%sshare\\gtksourceview-1.0\\language-specs", paths.gretldir);
-    gretl_setenv("GTKSOURCEVIEW_LANGUAGE_DIR", envstr);
-#else
-    if (getenv("GTKSOURCEVIEW_LANGUAGE_DIR") == NULL) {
-	/* for the benefit of the bundled gtksourceview library */
-	sprintf(envstr, "%sgtksourceview", paths.gretldir);
-	gretl_setenv("GTKSOURCEVIEW_LANGUAGE_DIR", envstr);
-    }
-#endif
-}
-
 /* This is called after reading the gretl config file (or reading from
    the registry on Windows) at startup (and only then).  Subsequent
    updates to paths via the GUI (if any) are handled by the function
@@ -2665,7 +2705,7 @@ int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
 	err1 = validate_writedir(paths.workdir);
 	if (err1) {
 	    /* try falling back on the default working dir */
-	    const char *defpath = gretl_default_workdir();
+	    const char *defpath = maybe_get_default_workdir();
 
 	    if (defpath != NULL && *defpath != '\0' && 
 		strcmp(defpath, paths.workdir)) {
@@ -2677,8 +2717,6 @@ int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
 	}
     }
 
-    set_up_sourceview_path();
-
 #if defined(WIN32) || defined(OSX_BUILD) 
     shelldir_init(paths.workdir);
 #else
@@ -2944,15 +2982,17 @@ static int rc_bool (const char *s)
 
 void get_gretl_rc_path (char *rcfile)
 {
-    char *custprof = getenv("GRETL_PROFILE");
+    char *home = getenv("HOME");
 
-    if (custprof != NULL) {
-	strcpy(rcfile, custprof);
+    if (home != NULL) {
+	sprintf(rcfile, "%s/.gretl2rc", home);
     } else {
-	sprintf(rcfile, "%s/.gretl2rc", getenv("HOME"));
+	strcpy(rcfile, ".gretl2rc");
     }
 }
 
+#define PROXLEN 64
+
 /* non-Windows read of the gretl configuration file on behalf
    of the CLI program, gretlcli
 */
@@ -2963,7 +3003,7 @@ int cli_read_rc (void)
     FILE *fp = NULL;
     char rcfile[FILENAME_MAX];
     char line[MAXLEN], key[32], val[MAXLEN];
-    char dbproxy[21] = {0};
+    char dbproxy[PROXLEN] = {0};
     int usecwd = 0;
     int use_proxy = 0;
     int err = 0;
@@ -3005,7 +3045,7 @@ int cli_read_rc (void)
 	    } else if (!strcmp(key, "dbhost")) {
 		strncat(cpaths.dbhost, val, 32 - 1);
 	    } else if (!strcmp(key, "dbproxy")) {
-		strncat(dbproxy, val, 21 - 1);
+		strncat(dbproxy, val, PROXLEN - 1);
 	    } else if (!strcmp(key, "useproxy")) {
 		use_proxy = rc_bool(val);
 	    } else if (!strcmp(key, "x12a")) {
@@ -3024,7 +3064,15 @@ int cli_read_rc (void)
 		strncat(cpaths.pngfont, val, 128 - 1);
 	    } else if (!strcmp(key, "Gp_colors")) {
 		rc_set_gp_colors(val);
-	    } 
+	    } else if (!strcmp(key, "HC_xsect")) {
+		set_xsect_hccme(val);
+	    } else if (!strcmp(key, "HC_tseri")) {
+		set_tseries_hccme(val);
+	    } else if (!strcmp(key, "HC_panel")) {
+		set_panel_hccme(val);
+	    } else if (!strcmp(key, "HC_garch")) {
+		set_garch_robust_vcv(val);
+	    }
 	}
     }
 
@@ -3054,4 +3102,48 @@ int cli_read_rc (void)
     return err;
 }
 
+#endif /* !WIN32 */
+
+#ifdef OSX_BUILD
+
+const char *gretl_app_support_dir (void)
+{
+    static char suppdir[FILENAME_MAX];
+
+    if (*suppdir == '\0') {
+	/* try to ensure that we have a per-user Application
+	   Support dir, with appropriate subdirectories
+	*/
+	const char *home = getenv("HOME");
+
+	if (home == NULL) {
+	    fprintf(stderr, "problem: HOME is not defined\n");
+	} else {
+	    char *p;
+	    int err;
+
+	    sprintf(suppdir, "%s/Library/Application Support/gretl/functions", 
+		    home);
+	    p = strrchr(suppdir, '/') + 1;
+	    err = gretl_mkdir(suppdir);
+	    if (!err) {
+		strcpy(p, "data");
+		err = gretl_mkdir(suppdir);
+	    }
+	    if (!err) {
+		strcpy(p, "db");
+		err = gretl_mkdir(suppdir);
+	    }
+	    if (err) {
+		*suppdir = '\0';
+	    } else {
+		/* chop off subdir from name */
+		*p = '\0';
+	    }
+	}
+    }
+
+    return suppdir;
+}
+
 #endif
