@@ -21,10 +21,12 @@
 #define GRETL_PATHS_H
 
 #include <sys/stat.h>
+#include <dirent.h>
 
 typedef enum {
     CURRENT_DIR,
     DATA_SEARCH,
+    DB_SEARCH,
     SCRIPT_SEARCH,
     FUNCS_SEARCH,
     USER_SEARCH
@@ -38,7 +40,8 @@ typedef enum {
 
 typedef enum {
     PKG_ALL,
-    PKG_ADDON
+    PKG_SUBDIR,
+    PKG_TOPLEV
 } PkgType;
 
 typedef struct ConfigPaths_ ConfigPaths;
@@ -100,6 +103,8 @@ int gretl_mkdir (const char *path);
 
 int gretl_chdir (const char *path);
 
+DIR *gretl_opendir (const char *name);
+
 int gretl_deltree (const char *path);
 
 int gretl_setenv (const char *name, const char *value);
@@ -124,7 +129,7 @@ int gretl_set_paths (ConfigPaths *paths, gretlopt opt);
 
 int gretl_update_paths (ConfigPaths *cpaths, gretlopt opt);
 
-int get_plausible_functions_dir (char *fndir, int i);
+char **get_plausible_search_dirs (int type, int *n_dirs);
 
 char *gretl_function_package_get_path (const char *name,
 				       PkgType type);
@@ -141,6 +146,8 @@ const char *gretl_workdir (void);
 
 const char *gretl_default_workdir (void);
 
+const char *maybe_get_default_workdir (void);
+
 int set_gretl_work_dir (const char *path);
 
 const char *gretl_maybe_switch_dir (const char *fname);
@@ -197,4 +204,10 @@ int cli_read_rc (void);
 
 #endif
 
+#ifdef OSX_BUILD
+
+const char *gretl_app_support_dir (void);
+
+#endif
+
 #endif /* GRETL_PATHS_H */
