@@ -31,9 +31,10 @@
 # include "libset.h"
 #endif
 
+#include "version.h"
+
 #define WBUFSIZE 8192
 
-#include "version.h"
 #include "gretl_www.h"
 
 #include <stdio.h>
@@ -59,11 +60,12 @@
 #define DBHLEN 64
 
 static int wproxy;
-static char dbhost[DBHLEN]      = "ricardo.ecn.wfu.edu";
-static char gretlhost[DBHLEN]   = "ricardo.ecn.wfu.edu";
-static char datacgi[DBHLEN]     = "/gretl/cgi-bin/gretldata.cgi";
-static char updatecgi[DBHLEN]   = "/gretl/cgi-bin/gretl_update.cgi";
-static char manual_path[DBHLEN] = "/project/gretl/manual/";
+static char dbhost[DBHLEN]       = "ricardo.ecn.wfu.edu";
+static char gretlhost[DBHLEN]    = "ricardo.ecn.wfu.edu";
+static char datacgi[DBHLEN]      = "/gretl/cgi-bin/gretldata.cgi";
+static char updatecgi[DBHLEN]    = "/gretl/cgi-bin/gretl_update.cgi";
+static char manual_path[DBHLEN]  = "/project/gretl/manual/";
+static char dataset_path[DBHLEN] = "/project/gretl/datafiles/";
 
 #ifndef STANDALONE
 static char sffiles[DBHLEN] = "downloads.sourceforge.net";
@@ -264,14 +266,14 @@ struct urlinfo_ {
 };
 
 struct http_stat {
-    long len;			/* received length */
-    long contlen;		/* expected length */
-    int res;			/* the result of last read */
-    char *newloc;		/* new location (redirection) */
-    char *remote_time;		/* remote time-stamp string */
-    char *error;		/* textual HTTP error */
-    int statcode;		/* status code */
-    long dltime;		/* time of the download */
+    long len;		     /* received length */
+    long contlen;	     /* expected length */
+    int res;		     /* the result of last read */
+    char *newloc;	     /* new location (redirection) */
+    char *remote_time;	     /* remote time-stamp string */
+    char *error;	     /* textual HTTP error */
+    int statcode;	     /* status code */
+    long dltime;	     /* time of the download */
 };
 
 struct rbuf {
@@ -785,6 +787,10 @@ static int get_contents (int fd, FILE *fp, char **getbuf, long *len,
     }
 #endif
 
+#if WDEBUG
+    fprintf(stderr, "get_contents: expected = %d\n", (int) expected);
+#endif
+
     if (show) {
 	sp_ret = show_progress(res, expected, SP_LOAD_INIT);
     }
@@ -1542,8 +1548,6 @@ static char *print_option (int opt)
 	return "CHECK_DB";
     case LIST_PKGS:
 	return "LIST_PKGS";
-    case GRAB_PKG:
-	return "GRAB_PKG";
     default:
 	break;
     }
@@ -1647,8 +1651,6 @@ urlinfo_set_params (urlinfo *u, CGIOpt opt, const char *fname,
     if (fname != NULL) {
 	if (opt == GRAB_FILE || opt == GRAB_FUNC) {
 	    strcat(u->params, "&fname=");
-	} else if (opt == GRAB_PKG) {
-	    strcat(u->params, "&pkg=");
 	} else {
 	    strcat(u->params, "&dbase=");
 	}
@@ -1728,10 +1730,11 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
     }
 
     if (opt == GRAB_FOREIGN || opt == QUERY_SF) {
-	/* note: not using the gretl server */
 	urlinfo_set_dl_path(u, NULL, fname);
     } else if (opt == GRAB_PDF) {
 	urlinfo_set_dl_path(u, manual_path, fname);
+    } else if (opt == GRAB_PKG) {
+	urlinfo_set_dl_path(u, dataset_path, fname);
     } else if (opt == GRAB_FILE) {
 	urlinfo_set_cgi_path(u, updatecgi);
     } else {
@@ -1743,7 +1746,9 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
 	return 1;
     }
 
-    if (opt != GRAB_PDF && opt != GRAB_FOREIGN && opt != QUERY_SF) {
+    if (opt != GRAB_PDF && opt != GRAB_FOREIGN &&
+	opt != GRAB_PKG && opt != QUERY_SF) {
+	/* a gretl-server download */
 	urlinfo_set_params(u, opt, fname, dbseries);
 	if (u->params == NULL) {
 	    urlinfo_destroy(u, 0);
@@ -2015,40 +2020,40 @@ int upload_function_package (const char *login, const char *pass,
 
 int list_remote_dbs (char **getbuf)
 {
-    return retrieve_url (dbhost, LIST_DBS, NULL, NULL, SAVE_TO_BUFFER, 
-			 NULL, getbuf);
+    return retrieve_url(dbhost, LIST_DBS, NULL, NULL, 
+			SAVE_TO_BUFFER, NULL, getbuf);
 }
 
 int list_remote_function_packages (char **getbuf)
 {
-    return retrieve_url (gretlhost, LIST_FUNCS, NULL, NULL, SAVE_TO_BUFFER, 
-			 NULL, getbuf);
+    return retrieve_url(gretlhost, LIST_FUNCS, NULL, NULL, 
+			SAVE_TO_BUFFER, NULL, getbuf);
 }
 
 int query_sourceforge (const char *query, char **getbuf)
 {
-    return retrieve_url (sfweb, QUERY_SF, query, NULL, SAVE_TO_BUFFER, 
-			 NULL, getbuf);
+    return retrieve_url(sfweb, QUERY_SF, query, NULL, 
+			SAVE_TO_BUFFER, NULL, getbuf);
 }
 
 int list_remote_data_packages (char **getbuf)
 {
-    return retrieve_url (gretlhost, LIST_PKGS, NULL, NULL, SAVE_TO_BUFFER, 
-			 NULL, getbuf);
+    return retrieve_url(gretlhost, LIST_PKGS, NULL, NULL, 
+			SAVE_TO_BUFFER, NULL, getbuf);
 }
 
 int retrieve_remote_db_index (const char *dbname, char **getbuf) 
 {
-    return retrieve_url (dbhost, GRAB_IDX, dbname, NULL, SAVE_TO_BUFFER, 
-			 NULL, getbuf);
+    return retrieve_url(dbhost, GRAB_IDX, dbname, NULL, 
+			SAVE_TO_BUFFER, NULL, getbuf);
 }
 
 int retrieve_remote_db (const char *dbname, 
 			const char *localname,
 			int opt)
 {
-    return retrieve_url (dbhost, opt, dbname, NULL, SAVE_TO_FILE, 
-			 localname, NULL);
+    return retrieve_url(dbhost, opt, dbname, NULL, 
+			SAVE_TO_FILE, localname, NULL);
 }
 
 int check_remote_db (const char *dbname)
@@ -2056,8 +2061,8 @@ int check_remote_db (const char *dbname)
     char *getbuf = NULL;
     int err;
 
-    err = retrieve_url (dbhost, CHECK_DB, dbname, NULL, SAVE_TO_BUFFER, 
-			NULL, &getbuf);
+    err = retrieve_url(dbhost, CHECK_DB, dbname, NULL, 
+		       SAVE_TO_BUFFER, NULL, &getbuf);
 
     if (!err && getbuf != NULL) {
 	err = strncmp(getbuf, "OK", 2) != 0;
@@ -2086,8 +2091,8 @@ int check_remote_db (const char *dbname)
 int retrieve_remote_function_package (const char *pkgname, 
 				      const char *localname)
 {
-    return retrieve_url(gretlhost, GRAB_FUNC, pkgname, NULL, SAVE_TO_FILE, 
-			localname, NULL);
+    return retrieve_url(gretlhost, GRAB_FUNC, pkgname, NULL, 
+			SAVE_TO_FILE, localname, NULL);
 }
 
 /**
@@ -2097,7 +2102,7 @@ int retrieve_remote_function_package (const char *pkgname,
  * @localname: full path to which the package file should be
  * written on the local machine.
  *
- * Retrieves the specified file from the gretl data server.
+ * Retrieves the specified package from sourceforge.
  *
  * Returns: 0 on success, non-zero on failure.
  */
@@ -2105,8 +2110,8 @@ int retrieve_remote_function_package (const char *pkgname,
 int retrieve_remote_datafiles_package (const char *pkgname, 
 				       const char *localname)
 {
-    return retrieve_url(gretlhost, GRAB_PKG, pkgname, NULL, SAVE_TO_FILE, 
-			localname, NULL);
+    return retrieve_url(sffiles, GRAB_PKG, pkgname, NULL, 
+			SAVE_TO_FILE, localname, NULL);
 }
 
 /**
@@ -2128,8 +2133,8 @@ int retrieve_remote_db_data (const char *dbname,
 			     char **getbuf,
 			     int opt)
 {
-    return retrieve_url(dbhost, opt, dbname, varname, SAVE_TO_BUFFER, 
-			NULL, getbuf);
+    return retrieve_url(dbhost, opt, dbname, varname, 
+			SAVE_TO_BUFFER, NULL, getbuf);
 }
 
 /**
@@ -2138,16 +2143,16 @@ int retrieve_remote_db_data (const char *dbname,
  * @localname: full path to which the file should be written
  * on the local machine.
  *
- * Retrieves the specified manual file in PDF format from the 
- * gretl server.
+ * Retrieves the specified manual file in PDF format from 
+ * sourceforge.
  *
  * Returns: 0 on success, non-zero on failure.
  */
 
 int retrieve_manfile (const char *fname, const char *localname)
 {
-    return retrieve_url(sffiles, GRAB_PDF, fname, NULL, SAVE_TO_FILE,
-			localname, NULL);
+    return retrieve_url(sffiles, GRAB_PDF, fname, NULL, 
+			SAVE_TO_FILE, localname, NULL);
 }
 
 /**
