@@ -105,6 +105,7 @@ struct set_vars_ {
     struct robust_opts ropts;   /* robust standard error options */
     char shelldir[MAXLEN];      /* working dir for shell commands */
     char csv_na[8];             /* representation of NA in CSV output */
+    double nadarwat_trim;       /* multiple of h to use in nadarwat() for trimming */
 };
 
 #define ECHO "echo"
@@ -137,7 +138,8 @@ struct set_vars_ {
 			  !strcmp(s, BFGS_MAXGRAD) || \
 			  !strcmp(s, BHHH_TOLER) || \
 			  !strcmp(s, NLS_TOLER) || \
-			  !strcmp(s, QS_BANDWIDTH))
+			  !strcmp(s, QS_BANDWIDTH) || \
+			  !strcmp(s, NADARWAT_TRIM))
 
 #define libset_int(s) (!strcmp(s, BFGS_MAXITER) || \
 		       !strcmp(s, BFGS_VERBSKIP) || \
@@ -161,11 +163,13 @@ static int gretl_debug;
 static int user_mp_bits;
 static int R_functions;
 static int R_lib = 1;
+static int csv_digits;
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
 static int real_libset_read_script (const char *fname,
 				    PRN *prn);
+static void set_csv_digits (const char *s);
 
 static void robust_opts_init (struct robust_opts *r)
 {
@@ -388,6 +392,7 @@ static void state_vars_copy (set_vars *sv)
     sv->lbfgs_mem = state->lbfgs_mem;
     sv->garch_vcv = state->garch_vcv;
     sv->garch_robust_vcv = state->garch_robust_vcv;
+    sv->nadarwat_trim = state->nadarwat_trim;
 
     sv->initvals = gretl_matrix_copy(state->initvals);
     sv->matmask = gretl_matrix_copy(state->matmask);
@@ -427,6 +432,7 @@ static void state_vars_init (set_vars *sv)
     sv->garch_vcv = VCV_UNSET;
     sv->arma_vcv = VCV_HESSIAN;
     sv->garch_robust_vcv = VCV_UNSET;
+    sv->nadarwat_trim = 4.0;
 
     *sv->shelldir = '\0';
     *sv->csv_na = '\0';
@@ -434,26 +440,26 @@ static void state_vars_init (set_vars *sv)
     robust_opts_init(&sv->ropts);
 }
 
-int get_bkbp_k (const DATAINFO *pdinfo)
+int get_bkbp_k (const DATASET *dset)
 {
-    if (pdinfo->pd == 1) {
+    if (dset->pd == 1) {
 	return 3;
-    } else if (pdinfo->pd == 4) {
+    } else if (dset->pd == 4) {
 	return 12;
-    } else if (pdinfo->pd == 12) {
+    } else if (dset->pd == 12) {
 	return 36;
     } else {
 	return 3;
     }
 }
 
-void get_bkbp_periods (const DATAINFO *pdinfo, int *l, int *u)
+void get_bkbp_periods (const DATASET *dset, int *l, int *u)
 {
-    *l = (pdinfo->pd == 4)? 6 :
-	(pdinfo->pd == 12)? 18 : 2;
+    *l = (dset->pd == 4)? 6 :
+	(dset->pd == 12)? 18 : 2;
 
-    *u = (pdinfo->pd == 4)? 32 :
-	(pdinfo->pd == 12)? 96 : 8;
+    *u = (dset->pd == 4)? 32 :
+	(dset->pd == 12)? 96 : 8;
 }
 
 void set_gretl_echo (int e)
@@ -529,13 +535,13 @@ int get_mp_bits (void)
     }
 }
 
-char get_csv_delim (const DATAINFO *pdinfo)
+char get_csv_delim (const DATASET *dset)
 {
     check_for_state();
     if (state->delim > 0) {
 	return state->delim;
     } else {
-	return pdinfo->delim;
+	return dset->delim;
     }
 }
 
@@ -928,11 +934,11 @@ void set_garch_robust_vcv (const char *s)
 }
 
 static int set_line_width (const char *s0, const char *s1,
-			   DATAINFO *pdinfo, PRN *prn)
+			   DATASET *dset, PRN *prn)
 {
     int v, w, err = 0;
 
-    if (pdinfo == NULL) {
+    if (dset == NULL) {
 	/* treat as no-op */
 	return 0;
     }
@@ -944,7 +950,7 @@ static int set_line_width (const char *s0, const char *s1,
     if (isdigit((unsigned char) *s0)) {
 	v = atoi(s0);
     } else {
-	v = current_series_index(pdinfo, s0);
+	v = current_series_index(dset, s0);
     }
 
     if (v < 0) {
@@ -956,9 +962,9 @@ static int set_line_width (const char *s0, const char *s1,
     if (w < 0 || w > 32) {
 	err = E_DATA;
     } else {
-	var_set_linewidth(pdinfo, v, w);
+	var_set_linewidth(dset, v, w);
 	pprintf(prn, _("Line width for %s = %d\n"), 
-		pdinfo->varname[v], w);
+		dset->varname[v], w);
     }
 
     return err;
@@ -996,8 +1002,7 @@ static int set_initvals (const char *s, PRN *prn)
     return err;
 }
 
-static int set_matmask (const char *s, const double **Z,
-			const DATAINFO *pdinfo,
+static int set_matmask (const char *s, const DATASET *dset,
 			PRN *prn)
 {
     char vname[VNAMELEN];
@@ -1010,7 +1015,7 @@ static int set_matmask (const char *s, const double **Z,
 	gretl_matrix_free(state->matmask);
 	state->matmask = NULL;
     } else {
-	int t, v = current_series_index(pdinfo, vname);
+	int t, v = current_series_index(dset, vname);
 
 	if (v < 0) {
 	    err = E_UNKVAR;
@@ -1018,12 +1023,12 @@ static int set_matmask (const char *s, const double **Z,
 	    if (state->matmask != NULL) {
 		gretl_matrix_free(state->matmask);
 	    }
-	    state->matmask = gretl_column_vector_alloc(pdinfo->n);
+	    state->matmask = gretl_column_vector_alloc(dset->n);
 	    if (state->matmask == NULL) {
 		err = E_ALLOC;
 	    } else {
-		for (t=0; t<pdinfo->n; t++) {
-		    state->matmask->val[t] = Z[v][t];
+		for (t=0; t<dset->n; t++) {
+		    state->matmask->val[t] = dset->Z[v][t];
 		}
 	    }
 	}
@@ -1343,6 +1348,7 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_print_bool(USE_SVD, prn, opt);
     libset_print_bool(USE_FCP, prn, opt);
     libset_print_bool(DPDSTYLE, prn, opt);
+    libset_print_double(NADARWAT_TRIM, prn, opt);
 
     libset_header(N_("Random number generation"), prn, opt);
 
@@ -1483,9 +1489,8 @@ static int write_or_read_settings (gretlopt opt, PRN *prn)
     return err;
 }
 
-int execute_set_line (const char *line, const double **Z,
-		      DATAINFO *pdinfo, gretlopt opt, 
-		      PRN *prn)
+int execute_set_line (const char *line, DATASET *dset, 
+		      gretlopt opt, PRN *prn)
 {
     char setobj[32], setarg[32], setarg2[32];
     int k, nw, err = E_PARSE;
@@ -1510,7 +1515,7 @@ int execute_set_line (const char *line, const double **Z,
 	if (!strcmp(setobj, "initvals")) {
 	    return set_initvals(line, prn);
 	} else if (!strcmp(setobj, "matrix_mask")) {
-	    return set_matmask(line, Z, pdinfo, prn);
+	    return set_matmask(line, dset, prn);
 	} else if (!strcmp(setobj, "shelldir")) {
 	    return set_shelldir(line);
 	} else if (!strcmp(setobj, "codevars")) {
@@ -1534,7 +1539,10 @@ int execute_set_line (const char *line, const double **Z,
 	if (!strcmp(setobj, "csv_na")) {
 	    set_csv_na_string(setarg);
 	    return 0;
-	} 	    
+	} else if (!strcmp(setobj, "csv_digits")) {
+	    set_csv_digits(setarg);
+	    return 0;
+	}	    
 
 	lower(setarg);
 
@@ -1599,7 +1607,7 @@ int execute_set_line (const char *line, const double **Z,
 	}
     } else if (nw == 3) {
 	if (!strcmp(setobj, "linewidth")) {
-	    err = set_line_width(setarg, setarg2, pdinfo, prn);
+	    err = set_line_width(setarg, setarg2, dset, prn);
 	} else {
 	    err = E_UNKVAR;
 	}
@@ -1641,6 +1649,12 @@ double libset_get_double (const char *key)
 	}
     } else if (!strcmp(key, BFGS_MAXGRAD)) {
 	return state->bfgs_maxgrad;
+    } else if (!strcmp(key, NADARWAT_TRIM)) {
+	if (na(state->nadarwat_trim)) {
+	    return 4.0;
+	} else {
+	    return state->nadarwat_trim;
+	}
     } else {
 	fprintf(stderr, "libset_get_double: unrecognized "
 		"variable '%s'\n", key);	
@@ -1686,6 +1700,8 @@ int libset_set_double (const char *key, double val)
 	state->bfgs_toler = val;
     } else if (!strcmp(key, BFGS_MAXGRAD)) {
 	state->bfgs_maxgrad = val;
+    } else if (!strcmp(key, NADARWAT_TRIM)) {
+	state->nadarwat_trim = val;
     } else {
 	fprintf(stderr, "libset_set_double: unrecognized "
 		"variable '%s'\n", key);	
@@ -1737,6 +1753,8 @@ int libset_get_int (const char *key)
 	return get_blas_nmk_min();
     } else if (!strcmp(key, BFGS_VERBSKIP)) {
 	return state->bfgs_verbskip;
+    } else if (!strcmp(key, CSV_DIGITS)) {
+	return csv_digits;
     } else {
 	fprintf(stderr, "libset_get_int: unrecognized "
 		"variable '%s'\n", key);	
@@ -2301,11 +2319,6 @@ static DEBUG_READLINE dbg_readline;
 
 void set_debug_read_func (DEBUG_READLINE dfunc) 
 {
-
-    if (check_for_state()) {
-	return;
-    }
-
     dbg_readline = dfunc;
 }
 
@@ -2318,11 +2331,6 @@ static DEBUG_OUTPUT dbg_output;
 
 void set_debug_output_func (DEBUG_OUTPUT dout) 
 {
-
-    if (check_for_state()) {
-	return;
-    }
-
     dbg_output = dout;
 }
 
@@ -2331,6 +2339,24 @@ DEBUG_OUTPUT get_debug_output_func (void)
     return dbg_output;
 }
 
+/* support for GUI Stop button */
+
+static QUERY_STOP query_stop;
+
+void set_query_stop_func (QUERY_STOP query) 
+{
+    query_stop = query;
+}
+
+int check_for_stop (void)
+{
+    if (query_stop != NULL) {
+	return (*query_stop)();
+    } else {
+	return 0;
+    }
+}
+
 /* for setting what we print for NAs on CSV output */
 
 const char *get_csv_na_string (void)
@@ -2352,6 +2378,15 @@ void set_csv_na_string (const char *s)
     strncat(state->csv_na, s, 7);
 }
 
+static void set_csv_digits (const char *s)
+{
+    int k = atoi(s);
+
+    if (k >= 0 && k < 26) {
+	csv_digits = k;
+    }
+}
+
 int libset_write_script (const char *fname)
 {
     PRN *prn;
@@ -2400,7 +2435,7 @@ static int real_libset_read_script (const char *fname,
 		continue;
 	    }
 	    tailstrip(line);
-	    err = execute_set_line(line, NULL, NULL, OPT_NONE, prn);
+	    err = execute_set_line(line, NULL, OPT_NONE, prn);
 	    if (err && prn != NULL) {
 		break;
 	    }
