@@ -77,17 +77,20 @@ typedef enum {
 #define RNG              "RNG"
 #define DPDSTYLE         "dpdstyle"
 #define GMM_MAXITER      "gmm_maxiter"
+#define CSV_DIGITS       "csv_digits"
+#define NADARWAT_TRIM    "nadarwat_trim"
 
 typedef int (*ITER_PRINT_FUNC) (int, PRN *);
 typedef void (*SHOW_ACTIVITY_FUNC) (void);
 typedef int (*DEBUG_READLINE) (void *);
 typedef int (*DEBUG_OUTPUT) (void *);
+typedef int (*QUERY_STOP) (void);
 
 #define set_nls_toler(x) (libset_set_double(NLS_TOLER, x))
 
 int libset_init (void);
 void libset_cleanup (void);
-int libset_restore_state_zero (DATAINFO *pdinfo);
+int libset_restore_state_zero (DATASET *dset);
 
 int push_program_state (void);
 int pop_program_state (void);
@@ -114,8 +117,8 @@ void set_garch_robust_vcv (const char *s);
 int get_hac_lag (int T);
 int data_based_hac_bandwidth (void);
 
-int get_bkbp_k (const DATAINFO *pdinfo);
-void get_bkbp_periods (const DATAINFO *pdinfo, int *l, int *u);
+int get_bkbp_k (const DATASET *dset);
+void get_bkbp_periods (const DATASET *dset, int *l, int *u);
 
 void set_mp_bits (int b);
 int get_mp_bits (void);
@@ -153,14 +156,13 @@ int gretl_debugging_on (void);
 void shelldir_init (const char *s);
 char *get_shelldir (void);
 
-char get_csv_delim (const DATAINFO *pdinfo);
+char get_csv_delim (const DATASET *dset);
 
 const char *get_csv_na_string (void);
 void set_csv_na_string (const char *s);
 
-int execute_set_line (const char *line, const double **Z,
-		      DATAINFO *pdinfo, gretlopt opt, 
-		      PRN *prn);
+int execute_set_line (const char *line, DATASET *dset, 
+		      gretlopt opt, PRN *prn);
 
 void set_iter_print_func (ITER_PRINT_FUNC func);
 int iter_print_callback (int i, PRN *prn);
@@ -176,6 +178,9 @@ DEBUG_READLINE get_debug_read_func (void);
 void set_debug_output_func (DEBUG_OUTPUT dout);
 DEBUG_OUTPUT get_debug_output_func (void);
 
+void set_query_stop_func (QUERY_STOP query);
+int check_for_stop (void);
+
 void set_workdir_callback (int (*callback)());
 
 int libset_write_script (const char *fname);
