@@ -340,7 +340,7 @@ void gretl_object_unref (void *ptr, GretlObjType type)
 
 	if (pmod != NULL) {
 	    if (model_is_protected(pmod)) {
-		return; /* note */
+		return; /* note! */
 	    }
 	    rc = &pmod->refcount;
 	}
@@ -934,8 +934,7 @@ MODEL *maybe_stack_model (MODEL *pmod, CMD *cmd, PRN *prn, int *err)
    the form of a simple scalar */
 
 static double real_get_obj_scalar (void *p, GretlObjType type, 
-				   double ***pZ, DATAINFO *pdinfo,
-				   int idx)
+				   DATASET *dset, int idx)
 {
     double x = INVALID_STAT;
     int err = 0;
@@ -947,7 +946,7 @@ static double real_get_obj_scalar (void *p, GretlObjType type,
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = (MODEL *) p;
 
-	x = gretl_model_get_scalar(pmod, idx, pZ, pdinfo, &err);
+	x = gretl_model_get_scalar(pmod, idx, dset, &err);
 	if (err) {
 	    x = INVALID_STAT;
 	}
@@ -988,7 +987,7 @@ static double real_get_obj_scalar (void *p, GretlObjType type,
 
 static double *
 real_get_obj_series (void *p, GretlObjType type, int idx,
-		     const DATAINFO *pdinfo, int *err)
+		     const DATASET *dset, int *err)
 {
     double *x = NULL;
 
@@ -1000,7 +999,7 @@ real_get_obj_series (void *p, GretlObjType type, int idx,
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = (MODEL *) p;
 
-	x = gretl_model_get_series(pmod, pdinfo, idx, err);
+	x = gretl_model_get_series(pmod, dset, idx, err);
     } 
 
     return x;
@@ -1060,7 +1059,7 @@ real_get_obj_list (void *p, GretlObjType type, int idx, int *err)
 
 static char *
 real_get_obj_string (void *p, GretlObjType type, int idx, 
-		     const DATAINFO *pdinfo, int *err)
+		     const DATASET *dset, int *err)
 {
     char *str = NULL;
 
@@ -1075,7 +1074,7 @@ real_get_obj_string (void *p, GretlObjType type, int idx,
 	str = gretl_strdup(gretl_command_word(pmod->ci));
     } else if (type == GRETL_OBJ_EQN && idx == M_DEPVAR) {
 	const char *s = gretl_model_get_depvar_name((MODEL *) p, 
-						    pdinfo);
+						    dset);
 
 	str = gretl_strdup(s);
     } else {
@@ -1176,7 +1175,7 @@ int *saved_object_get_list (const char *oname, int idx, int *err)
 }
 
 char *saved_object_get_string (const char *oname, int idx, 
-			       const DATAINFO *pdinfo, int *err)
+			       const DATASET *dset, int *err)
 {
     char *ret = NULL;
     stacker *smatch;
@@ -1185,15 +1184,14 @@ char *saved_object_get_string (const char *oname, int idx,
 
     if (smatch != NULL) {
 	ret = real_get_obj_string(smatch->ptr, smatch->type, idx, 
-				  pdinfo, err);
+				  dset, err);
     }
 
     return ret;
 }
 
 double saved_object_get_scalar (const char *oname, int idx, 
-				double ***pZ, DATAINFO *pdinfo,
-				int *err)
+				DATASET *dset, int *err)
 {
     double ret = INVALID_STAT;
     stacker *smatch;
@@ -1202,7 +1200,7 @@ double saved_object_get_scalar (const char *oname, int idx,
 
     if (smatch != NULL) {
 	ret = real_get_obj_scalar(smatch->ptr, smatch->type, 
-				  pZ, pdinfo, idx);
+				  dset, idx);
     }
 
     if (ret == INVALID_STAT) {
@@ -1213,7 +1211,7 @@ double saved_object_get_scalar (const char *oname, int idx,
 }
 
 double *saved_object_get_series (const char *oname, int idx,
-				 const DATAINFO *pdinfo, 
+				 const DATASET *dset, 
 				 int *err)
 {
     double *x = NULL;
@@ -1223,7 +1221,7 @@ double *saved_object_get_series (const char *oname, int idx,
 
     if (smatch != NULL) {
 	x = real_get_obj_series(smatch->ptr, smatch->type, idx, 
-				pdinfo, err);
+				dset, err);
     }
 
     if (x == NULL && !*err) {
@@ -1270,7 +1268,7 @@ saved_object_get_matrix (const char *oname, int idx, int *err)
 
 gretl_matrix *
 saved_object_build_matrix (const char *oname, int idx, 
-			   const double **Z, const DATAINFO *pdinfo,
+			   const DATASET *dset,
 			   int *err)
 {
     stacker *smatch = find_smatch(oname);
@@ -1279,13 +1277,13 @@ saved_object_build_matrix (const char *oname, int idx,
     if (smatch == NULL) {
 	*err = E_DATA;
     } else if (idx == M_MNLPROBS && smatch->type == GRETL_OBJ_EQN) {
-	M = mn_logit_probabilities(smatch->ptr, Z, pdinfo, err);
+	M = mn_logit_probabilities(smatch->ptr, dset, err);
     } else if (idx == M_EC && smatch->type == GRETL_OBJ_VAR) {
-	M = VECM_get_EC_matrix(smatch->ptr, Z, pdinfo, err);
+	M = VECM_get_EC_matrix(smatch->ptr, dset, err);
     } else if (idx == M_VMA && smatch->type == GRETL_OBJ_VAR) {
-	M = gretl_VAR_get_vma_matrix(smatch->ptr, pdinfo, err);
+	M = gretl_VAR_get_vma_matrix(smatch->ptr, dset, err);
     } else if (idx == M_FEVD && smatch->type == GRETL_OBJ_VAR) {
-	M = gretl_VAR_get_full_FEVD_matrix(smatch->ptr, pdinfo, err);
+	M = gretl_VAR_get_full_FEVD_matrix(smatch->ptr, dset, err);
     } else {
 	*err = E_BADSTAT;
     }
@@ -1295,8 +1293,7 @@ saved_object_build_matrix (const char *oname, int idx,
 
 gretl_matrix *
 last_model_get_irf_matrix (int targ, int shock, double alpha, 
-			   const double **Z, const DATAINFO *pdinfo,
-			   int *err)
+			   const DATASET *dset, int *err)
 {
     stacker *smatch = find_smatch(NULL);
     gretl_matrix *M = NULL;
@@ -1305,12 +1302,39 @@ last_model_get_irf_matrix (int targ, int shock, double alpha,
 	*err = E_BADSTAT;
     } else {
 	M = gretl_VAR_get_impulse_response(smatch->ptr, targ, shock, 0,
-					   alpha, Z, pdinfo, err);
+					   alpha, dset, err);
     }
 
     return M;
 }
 
+void *last_model_get_data (const char *key, GretlType *type, 
+			   int *size, int *err)
+{
+    stacker *smatch = find_smatch(NULL);
+    void *ret = NULL;
+
+    if (smatch == NULL || smatch->type != GRETL_OBJ_EQN) {
+	*err = E_DATA;
+    } else {
+	const MODEL *pmod = smatch->ptr;
+	size_t sz = 0;
+
+	ret = gretl_model_get_data_full(pmod, key, type, &sz);
+	if (ret == NULL) {
+	    *err = E_DATA;
+	} else {
+	    *size = sz;
+	}
+    }
+
+    if (*err) {
+	gretl_errmsg_sprintf("\"%s\": %s", key, _("no such item"));
+    }
+
+    return ret;
+}
+
 static int namechar_spn_with_space (const char *s)
 {
     const char *ok = "abcdefghijklmnopqrstuvwxyz"
@@ -1332,8 +1356,7 @@ static int namechar_spn_with_space (const char *s)
 
 int parse_object_command (const char *s, char *name, char **cmd)
 {
-    int len, start = 0;
-    int quoted = 0;
+    int len, quoted = 0;
     int err = 0;
 
     *name = 0;
@@ -1341,7 +1364,7 @@ int parse_object_command (const char *s, char *name, char **cmd)
 
     /* skip any leading whitespace */
     while (*s && isspace(*s)) {
-	s++; start++;
+	s++;
     }
 
     /* skip an opening quote */
@@ -1393,48 +1416,23 @@ int parse_object_command (const char *s, char *name, char **cmd)
     return err;
 }
 
-int match_object_command (const char *s, GretlObjType type)
+int match_object_command (const char *s)
 {
-    if (type == GRETL_OBJ_EQN) {
-	if (*s == 0) return OBJ_ACTION_SHOW; /* default */
-	if (strcmp(s, "show") == 0) return OBJ_ACTION_SHOW;
-	if (strncmp(s, "add", 3) == 0) return OBJ_ACTION_ADD;
-	if (strncmp(s, "omit", 4) == 0) return OBJ_ACTION_OMIT;
-	if (strcmp(s, "free") == 0) return OBJ_ACTION_FREE;
-	if (*s == '$') return OBJ_ACTION_SHOW_STAT;
-    } 
-
-    if (type == GRETL_OBJ_VAR) {
-	if (*s == 0) return OBJ_ACTION_SHOW;
-	if (strcmp(s, "show") == 0) return OBJ_ACTION_SHOW;
-	if (strcmp(s, "irf") == 0)  return OBJ_ACTION_IRF;
-	if (strncmp(s, "omit", 4) == 0) return OBJ_ACTION_OMIT;
-	if (strcmp(s, "free") == 0) return OBJ_ACTION_FREE; 
-	if (*s == '$') return OBJ_ACTION_SHOW_STAT;
+    if (strcmp(s, "show") == 0) {
+	return OBJ_ACTION_SHOW;
+    } else if (strcmp(s, "free") == 0) {
+	return OBJ_ACTION_FREE;
+    } else {
+	return OBJ_ACTION_INVALID;
     }
-
-    if (type == GRETL_OBJ_SYS) {
-	if (*s == 0) return OBJ_ACTION_SHOW;
-	if (strcmp(s, "show") == 0) return OBJ_ACTION_SHOW;
-	if (strcmp(s, "free") == 0) return OBJ_ACTION_FREE; 
-	if (*s == '$') return OBJ_ACTION_SHOW_STAT;
-    } 
-
-    if (type == GRETL_OBJ_GRAPH) {
-	if (*s == 0) return OBJ_ACTION_SHOW;
-	if (strcmp(s, "show") == 0) return OBJ_ACTION_SHOW;
-    } 
-
-    if (type == GRETL_OBJ_TEXT) {
-	if (*s == 0) return OBJ_ACTION_SHOW;
-	if (strcmp(s, "show") == 0) return OBJ_ACTION_SHOW;
-    }  
-
-    return OBJ_ACTION_INVALID;
 }
 
 static void saved_object_free (stacker *s)
 {
+    if (s == NULL) {
+	return;
+    }
+
     if (s->type == GRETL_OBJ_EQN) {
 	if (!model_is_protected(s->ptr)) {
 	    gretl_model_free(s->ptr);
@@ -1448,7 +1446,7 @@ static void saved_object_free (stacker *s)
 
 #define sys_modtest_opt_ok(o) (o & (OPT_A | OPT_H | OPT_N))
 
-int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo, 
+int last_model_test_ok (int ci, gretlopt opt, const DATASET *dset, 
 			PRN *prn)
 {
     GretlObjType type;
@@ -1464,10 +1462,10 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = (MODEL *) ptr;
   
-	if (!model_test_ok(ci, opt, pmod, pdinfo)) {
+	if (!model_test_ok(ci, opt, pmod, dset)) {
 	    err = E_NOTIMP;
 	}
-	if (model_sample_problem(pmod, pdinfo)) {
+	if (model_sample_problem(pmod, dset)) {
 	    pputs(prn, _("Can't do: the current data set is different from "
 			 "the one on which\nthe reference model was estimated\n"));
 	    err = E_DATA;
@@ -1497,8 +1495,7 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
     return err;
 }
 
-int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, 
-			  gretlopt opt, PRN *prn)
+int last_model_test_uhat (DATASET *dset, gretlopt opt, PRN *prn)
 {
     GretlObjType type;
     void *ptr;
@@ -1510,7 +1507,7 @@ int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo,
     }
 
     if (type == GRETL_OBJ_EQN) {
-	err = model_error_dist(ptr, pZ, pdinfo, opt, prn);
+	err = model_error_dist(ptr, dset, opt, prn);
     } else if (type == GRETL_OBJ_SYS) {
 	err = system_normality_test(ptr, prn);
     } else if (type == GRETL_OBJ_VAR) {
@@ -1522,7 +1519,7 @@ int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
-int highest_numbered_var_in_saved_object (const DATAINFO *pdinfo)
+int highest_numbered_var_in_saved_object (const DATASET *dset)
 {
     GretlObjType type;
     void *ptr;
@@ -1539,7 +1536,7 @@ int highest_numbered_var_in_saved_object (const DATAINFO *pdinfo)
 	    continue;
 	}
 	if (type == GRETL_OBJ_EQN) {
-	    mvm = highest_numbered_var_in_model((MODEL *) ptr, pdinfo);
+	    mvm = highest_numbered_var_in_model((MODEL *) ptr, dset);
 	    if (mvm > vmax) {
 		vmax = mvm;
 	    }
@@ -1549,7 +1546,7 @@ int highest_numbered_var_in_saved_object (const DATAINFO *pdinfo)
 		vmax = mvm;
 	    }
 	} else if (type == GRETL_OBJ_SYS) {
-	    mvm = highest_numbered_var_in_system((equation_system *) ptr, pdinfo);
+	    mvm = highest_numbered_var_in_system((equation_system *) ptr, dset);
 	    if (mvm > vmax) {
 		vmax = mvm;
 	    }
@@ -1559,12 +1556,12 @@ int highest_numbered_var_in_saved_object (const DATAINFO *pdinfo)
     return vmax;
 }
 
-int check_variable_deletion_list (int *list, const DATAINFO *pdinfo)
+int check_variable_deletion_list (int *list, const DATASET *dset)
 {
     int pruned = 0;
     int i, vsave;
 
-    vsave = highest_numbered_var_in_saved_object(pdinfo);
+    vsave = highest_numbered_var_in_saved_object(dset);
 
     for (i=list[0]; i>0; i--) {
 	if (list[i] <= vsave) {
@@ -1588,7 +1585,11 @@ void gretl_saved_objects_cleanup (void)
 
     for (i=0; i<n_obj; i++) {
 	if (ostack[i].ptr == lmp) {
-	    /* don't double-free! */
+	    /* stacked object i also occupies the "last model" place;
+	       so we drop the associated refcount and nullify the
+	       last model pointer to guard against double-freeing
+	    */
+	    gretl_object_unref(lmp, lmt);
 	    lmp = NULL;
 	}
 #if ODEBUG
