@@ -40,11 +40,13 @@
 			 c == ESTIMATE ||	\
 			 c == FCAST ||		\
 			 c == FOREIGN ||	\
+                         c == FRACTINT ||       \
 			 c == FREQ ||		\
 			 c == KPSS ||		\
 			 c == MODTEST ||	\
                          c == LEVINLIN ||       \
 			 c == LOOP ||		\
+                         c == MAHAL ||          \
 			 c == NORMTEST ||	\
 			 c == OLS ||		\
 			 c == OMIT ||		\
@@ -82,7 +84,7 @@ struct flag_match {
 struct gretl_option gretl_opts[] = {
     { ADD,      OPT_B, "both", 0 },
     { ADD,      OPT_I, "silent", 0 },
-    { ADD,      OPT_T, "inst", 0 },
+    { ADD,      OPT_L, "lm", 0 },
     { ADF,      OPT_N, "nc", 0 }, 
     { ADF,      OPT_C, "c", 0 }, 
     { ADF,      OPT_D, "seasonals", 0 },
@@ -110,12 +112,14 @@ struct gretl_option gretl_opts[] = {
     { ARMA,     OPT_X, "x-12-arima", 0 },
     { ARMA,     OPT_Y, "y-diff-only", 0 },
     { BIPROBIT, OPT_G, "opg", 0 },
-    { BIPROBIT, OPT_Q, "quiet", 0 },
     { BIPROBIT, OPT_R, "robust", 0 },
     { BIPROBIT, OPT_V, "verbose", 0 },
     { BIPROBIT, OPT_X, "save-xbeta", 0 },
     { BXPLOT,   OPT_O, "notches", 0 },
+    { BXPLOT,   OPT_L, "outliers", 2 },
+    { BXPLOT,   OPT_P, "panel", 0 },
     { BXPLOT,   OPT_U, "output", 2 },
+    { BXPLOT,   OPT_Z, "factorized", 0 },
     { CHOW,     OPT_D, "dummy", 0 },
     { CLEAR,    OPT_D, "dataset", 0 },
     { CLEAR,    OPT_O, "other", 0 },
@@ -170,6 +174,7 @@ struct gretl_option gretl_opts[] = {
     { EQNPRINT, OPT_T, "t-ratios", 0 },
     { TABPRINT, OPT_O, "complete", 0 },
     { TABPRINT, OPT_R, "rtf", 0 },
+    { TABPRINT, OPT_T, "format", 2 },
 #if 0 /* not yet */
     { TABPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
 #endif
@@ -190,10 +195,13 @@ struct gretl_option gretl_opts[] = {
     { FOREIGN,  OPT_V, "verbose", 0 },
     { FRACTINT, OPT_G, "gph", 0 },
     { FRACTINT, OPT_A, "all", 0 },
-    { FRACTINT, OPT_Q, "quiet", 0 },
+    { FREQ,     OPT_G, "show-plot", 0 },
     { FREQ,     OPT_O, "gamma", 0 },
     { FREQ,     OPT_S, "silent", 0 },
     { FREQ,     OPT_Z, "normal", 0 },
+    { FREQ,     OPT_N, "nbins", 2 },
+    { FREQ,     OPT_M, "min", 2 },
+    { FREQ,     OPT_W, "binwidth", 2 },
     { FUNDEBUG, OPT_C, "continue", 0 },
     { FUNDEBUG, OPT_N, "next", 0 },
     { FUNDEBUG, OPT_Q, "quit", 0 },
@@ -236,14 +244,18 @@ struct gretl_option gretl_opts[] = {
     { IVREG,    OPT_S, "save", 0 },
     { IVREG,    OPT_T, "two-step", 0 },
     { IVREG,    OPT_W, "weights", 2 },
+    { IVREG,    OPT_X, "no-tests", 0 },
     { KALMAN,   OPT_C, "cross", 0 },
     { KALMAN,   OPT_D, "diffuse", 0 },
     { KPSS,     OPT_T, "trend", 0 },
+    { KPSS,     OPT_D, "seasonals", 0 },
     { KPSS,     OPT_V, "verbose", 0 },
     { KPSS,     OPT_F, "difference", 0 },
     { LEVERAGE, OPT_S, "save", 0 },
     { LEVINLIN, OPT_N, "nc", 0 },
     { LEVINLIN, OPT_T, "ct", 0 },
+    { MAKEPKG,  OPT_I, "index", 0 },
+    { MAKEPKG,  OPT_T, "translations", 0 },
     { MODTEST,  OPT_A, "autocorr", 0 },
     { MODTEST,  OPT_B, "breusch-pagan", 0 },
     { MODTEST,  OPT_C, "comfac", 0 },
@@ -263,7 +275,6 @@ struct gretl_option gretl_opts[] = {
     { LOGIT,    OPT_V, "verbose", 0 },
     { LOOP,     OPT_P, "progressive", 0 },
     { LOOP,     OPT_V, "verbose", 0 },
-    { MAHAL,    OPT_Q, "quiet", 0 },
     { MAHAL,    OPT_S, "save", 0 },
     { MAHAL,    OPT_V, "vcv", 0 },
     { MEANTEST, OPT_O, "unequal-vars", 0 },
@@ -305,9 +316,9 @@ struct gretl_option gretl_opts[] = {
     { OLS,      OPT_V, "anova", 0 },
     { OMIT,     OPT_A, "auto", 1 },
     { OMIT,     OPT_B, "both", 0 },
+    { OMIT,     OPT_X, "chi-square", 0 },
     { OMIT,     OPT_I, "silent", 0 },
-    { OMIT,     OPT_T, "inst", 0 },
-    { OMIT,     OPT_W, "wald", 0 },
+    { OMIT,     OPT_W, "test-only", 0 },
     { OPEN,     OPT_B, "progress-bar", 0 },
     { OPEN,     OPT_C, "coded", 0 },
     { OPEN,     OPT_D, "drop-empty", 0 },
@@ -416,6 +427,7 @@ struct gretl_option gretl_opts[] = {
     { TOBIT,    OPT_V, "verbose", 0 },
     { VAR,      OPT_D, "seasonals", 0 },
     { VAR,      OPT_F, "variance-decomp", 0 },
+    { VAR,      OPT_H, "robust-hac", 0 },
     { VAR,      OPT_I, "impulse-responses", 0 },
     { VAR,      OPT_L, "lagselect", 0 },
     { VAR,      OPT_N, "nc", 0 },
@@ -928,6 +940,12 @@ gretlopt valid_long_opt (int ci, const char *lopt, OptStatus *status)
 	} 
     }  
 
+    /* backward compatibility */
+    if (opt == OPT_NONE && !strcmp(lopt, "wald")) {
+	opt = OPT_W;
+	*status = 0;
+    }
+
     return opt;
 }
 
@@ -1185,8 +1203,13 @@ gretlopt get_gretl_options (char *line, int *err)
 	return oflags;
     }
 
-    if (ci != SET && ci != SETINFO && ci != TABPRINT && ci != EQNPRINT) {
-	/* try for short-form options (e.g. "-o") */
+    if (ci != SET && ci != SETINFO && ci != SMPL &&
+	ci != TABPRINT && ci != EQNPRINT) {
+	/* try for short-form options (e.g. "-o"): but note that
+	   with some commands there's the possibility of collision
+	   between short options and other syntactical elements,
+	   so we only recognize long-form options.
+	*/
 	opt = get_short_opts(line, ci, (ci == SMPL)? NULL : &myerr);
 	if (!myerr && opt) {
 	    oflags |= opt;
@@ -1211,6 +1234,18 @@ gretlopt get_gretl_options (char *line, int *err)
     return oflags;
 }
 
+static int quote_option_parm (int ci, gretlopt opt, 
+			      const char *s)
+{
+    if (ci == TABPRINT && opt == OPT_T) {
+	if (strchr(s, '%')) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 static PRN *flagprn;
 
 /**
@@ -1269,7 +1304,11 @@ const char *print_flags (gretlopt oflags, int ci)
 	    if (gretl_opts[i].parminfo) {
 		parm = get_optval_string(ci, opt);
 		if (parm != NULL && *parm != '\0') {
-		    pprintf(flagprn, "=%s", parm);
+		    if (quote_option_parm(ci, opt, parm)) {
+			pprintf(flagprn, "=\"%s\"", parm);
+		    } else {
+			pprintf(flagprn, "=%s", parm);
+		    }
 		}
 	    }
 	}
