@@ -40,16 +40,17 @@
  * Note that if you wish to make use of gretl plugins in your own
  * program, you will have to build and install the plugins (of
  * course) and you will also have to tell libgretl where to
- * find them. This can be done via the  environment variable
+ * find them. This can be done via the environment variable
  * GRETL_PLUGIN_PATH.  If gretl has been installed under
  * /usr/local then in a C program you could do:
  *
- * putenv("GRETL_PLUGIN_PATH=/usr/local/lib/gretl-gtk2");
+ * gretl_setenv("GRETL_PLUGIN_PATH", "/usr/local/lib/gretl-gtk2/");
  *
  */
 
 enum {
-    P_EXCEL_IMPORT = 1,
+    P_XLS_IMPORT = 1,
+    P_XLSX_IMPORT,
     P_GNUMERIC_IMPORT,
     P_ODS_IMPORT,
     P_JOHANSEN,
@@ -102,7 +103,8 @@ struct plugin_function {
 
 struct plugin_info plugins[] = {
     { 0,                 NULL },
-    { P_EXCEL_IMPORT,    "excel_import" },
+    { P_XLS_IMPORT,      "excel_import" },
+    { P_XLSX_IMPORT,     "xlsx_import" },
     { P_GNUMERIC_IMPORT, "gnumeric_import" },
     { P_ODS_IMPORT,      "ods_import" },
     { P_JOHANSEN,        "johansen" },
@@ -145,7 +147,8 @@ struct plugin_info plugins[] = {
 
 struct plugin_function plugin_functions[] = { 
     /* data importers */
-    { "xls_get_data",      P_EXCEL_IMPORT },
+    { "xls_get_data",      P_XLS_IMPORT },
+    { "xlsx_get_data",     P_XLSX_IMPORT },
     { "gnumeric_get_data", P_GNUMERIC_IMPORT },
     { "ods_get_data",      P_ODS_IMPORT },
     { "wf1_get_data",      P_EVIEWS_IMPORT },
@@ -386,6 +389,7 @@ void *get_plugin_function (const char *funcname, void **handle)
 
     *handle = get_plugin_handle(plugname);
     if (*handle == NULL) {
+	fprintf(stderr, "handle == NULL for '%s'\n", plugname);
 	return NULL;
     }
 
@@ -406,6 +410,7 @@ void *get_plugin_function (const char *funcname, void **handle)
 
     if (funp == NULL) {
 	gretl_errmsg_set(_("Couldn't load plugin function"));
+	fprintf(stderr, "plugname = '%s' for function '%s'\n", plugname, funcname);
 	close_plugin(*handle);
 	*handle = NULL;
     }
