@@ -397,7 +397,7 @@ double normal_pvalue_1 (double x)
  * #NADBL on failure.
  */
 
-double student_cdf (int df, double x)
+double student_cdf (double df, double x)
 {
     double p = NADBL;
 
@@ -421,7 +421,7 @@ double student_cdf (int df, double x)
  * #NADBL on failure.
  */
 
-static double student_cdf_comp (int df, double x)
+static double student_cdf_comp (double df, double x)
 {
     double p = NADBL;
 
@@ -482,7 +482,7 @@ double normal_cdf_comp (double x)
  * or #NADBL on failure.
  */
 
-double student_pvalue_1 (int df, double x)
+double student_pvalue_1 (double df, double x)
 {
     double p = NADBL;
 
@@ -508,7 +508,7 @@ double student_pvalue_1 (int df, double x)
  * #NADBL on failure.
  */
 
-double student_pvalue_2 (int df, double x)
+double student_pvalue_2 (double df, double x)
 {
     double p = NADBL;
 
@@ -528,8 +528,6 @@ double student_pvalue_2 (int df, double x)
     return p;
 }
 
-#define df_ok(d) (floor(d) == d && d < (double) INT_MAX)
-
 /**
  * student_critval:
  * @df: degrees of freedom.
@@ -544,22 +542,14 @@ double student_critval (double df, double a)
 {
     double x;
 
-    if (df < 1) {
+    if (df < 0) {
 	return NADBL;
     }    
 
-    if (df_ok(df)) {
-	if (a > .10) {
-	    x = stdtri((int) df, 1 - a);
-	} else {
-	    x = -stdtri((int) df, a);
-	}
+    if (a > .10) {
+	x = stdtri(df, 1 - a);
     } else {
-	if (a > .10) {
-	    x = ndtri(1 - a);
-	} else {
-	    x = -ndtri(a);
-	}
+	x = -stdtri(df, a);
     }
 
     if (get_cephes_errno()) {
@@ -583,15 +573,11 @@ double student_cdf_inverse (double df, double a)
 {
     double x;
 
-    if (df < 1) {
+    if (df < 0) {
 	return NADBL;
     }
 
-    if (df_ok(df)) {
-	x = stdtri((int) df, a);
-    } else {
-	x = ndtri(a);
-    }    
+    x = stdtri(df, a);
 
     if (get_cephes_errno()) {
 	x = NADBL;
@@ -1361,6 +1347,36 @@ double gamma_cdf_comp (double s1, double s2, double x, int control)
     return p;
 }
 
+/**
+ * gamma_cdf_inverse:
+ * @shape: shape.
+ * @scale: scale.
+ * @p: probability.
+ *
+ * Returns: the argument x such that the integral from zero to @x of
+ * the gamma density with given scale and shape parameters is equal to
+ * the given probability @p, or #NADBL on failure. Note that the
+ * alternate parametrization (mean, variance) is not supported.
+ */
+
+double gamma_cdf_inverse (double shape, double scale, double p)
+{
+    double x = NADBL;
+
+    if (p==0) {
+	return 0;
+    }
+
+    if (shape > 0 && scale > 0 && p > 0 && p < 1) {
+	x = igami(shape, 1-p) * scale;
+	if (get_cephes_errno()) {
+	    x = NADBL;
+	}
+    }
+
+    return x;
+}
+
 static int gamma_pdf_array (double shape, double scale, 
 			    double *x, int n)
 {
@@ -1761,6 +1777,44 @@ double GED_cdf_comp (double nu, double x)
     }
 }
 
+/**
+ * GED_cdf_inverse:
+ * @nu: shape parameter.
+ * @a: probability.
+ *
+ * Returns: the argument x such that the integral from minus infinity
+ * to @x of the standardized GED density with shape parameter @nu is
+ * equal to the given probability @a, or #NADBL on failure. We exploit
+ * the well-known relationship between the standardized GED and the
+ * Gamma variates.
+ */
+
+double GED_cdf_inverse (double nu, double a)
+{
+    if (nu > 0 && a < 1 && a > 0) {
+	double a2, p, lg1, lg3, sd, x;
+	int sgn;
+
+	if (a > 0.5) {
+	    a2 = 2*a - 1;
+	    sgn = 1;
+	} else {
+	    a2 = 1 - 2*a;
+	    sgn = -1;
+	}
+
+	p   = 1/nu;
+	lg1 = ln_gamma(p);
+	lg3 = ln_gamma(3*p);
+	sd  = pow(2.0, p) * exp(0.5*(lg3 - lg1));
+	x   = gamma_cdf_inverse(p, 2.0, a2);
+
+	return sgn * pow(x, p) / sd;
+    } else {
+	return NADBL;
+    }
+}
+
 /**
  * johansen_trace_pval:
  * @N: the number of potentially cointegrated variables
@@ -1912,12 +1966,16 @@ double gretl_get_cdf_inverse (char st, const double *parm,
 	y = student_cdf_inverse(parm[0], a);
     } else if (st == 'X') {
 	y = chisq_cdf_inverse((int) parm[0], a);
+    } else if (st == 'G') {
+	y = gamma_cdf_inverse(parm[0], parm[1], a);
     } else if (st == 'F') {
 	y = snedecor_cdf_inverse((int) parm[0], (int) parm[1], a);
     } else if (st == 'B') {
 	y = binomial_cdf_inverse((int) parm[0], (int) parm[1], a);
     } else if (st == 'P') {
 	y = poisson_cdf_inverse((int) parm[0], a);
+    } else if (st == 'E') {
+	y = GED_cdf_inverse(parm[0], a);
     } 
 
     return y;
@@ -1986,7 +2044,7 @@ double gretl_get_cdf (char st, const double *parm, double x)
     if (st == 'z') {
 	y = normal_cdf(x);
     } else if (st == 't') {
-	y = student_cdf((int) parm[0], x);
+	y = student_cdf(parm[0], x);
     } else if (st == 'X') {
 	y = chisq_cdf((int) parm[0], x);
     } else if (st == 'F') {
@@ -2119,7 +2177,7 @@ double gretl_get_pvalue (char st, const double *parm, double x)
     if (st == 'z') {
 	y = normal_cdf_comp(x);
     } else if (st == 't') {
-	y = student_cdf_comp((int) parm[0], x);
+	y = student_cdf_comp(parm[0], x);
     } else if (st == 'X') {
 	y = chisq_cdf_comp((int) parm[0], x);
     } else if (st == 'F') {
@@ -2144,16 +2202,170 @@ double gretl_get_pvalue (char st, const double *parm, double x)
     return y;
 }
 
+static int gretl_fill_random_array (double *x, int t1, int t2,
+				    char st, const double *parm,
+				    const double *vecp1, 
+				    const double *vecp2)
+{
+    int t, err = 0;
+
+    if (st == 'u') {
+	/* uniform, continuous */
+	double min = parm[0], max = parm[1];
+
+	if (vecp1 != NULL || vecp2 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		if (vecp1 != NULL) min = vecp1[t];
+		if (vecp2 != NULL) max = vecp2[t];
+		err = gretl_rand_uniform_minmax(x, t, t, min, max);
+	    }
+	} else {
+	    err = gretl_rand_uniform_minmax(x, t1, t2, min, max);
+	}
+    } else if (st == 'i') {
+	/* uniform, discrete */
+	int min = (int) parm[0], max = (int) parm[1];
+
+	if (vecp1 != NULL || vecp2 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		if (vecp1 != NULL) min = (int) vecp1[t];
+		if (vecp2 != NULL) max = (int) vecp2[t];
+		err = gretl_rand_uniform_int_minmax(x, t, t, min, max,
+						    OPT_NONE);
+	    }
+	} else {
+	    err = gretl_rand_uniform_int_minmax(x, t1, t2, min, max,
+						OPT_NONE);
+	}	
+    } else if (st == 'z') {
+	/* normal */
+	double mu = parm[0], sd = parm[1];
+
+	if (vecp1 != NULL || vecp2 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		if (vecp1 != NULL) mu = vecp1[t];
+		if (vecp2 != NULL) sd = vecp2[t];
+		err = gretl_rand_normal_full(x, t, t, mu, sd);
+	    }
+	} else {
+	    err = gretl_rand_normal_full(x, t1, t2, mu, sd);
+	}
+    } else if (st == 't') {
+	/* Student's t */
+	double v = parm[0];
+
+	if (vecp1 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		v = vecp1[t];
+		err = gretl_rand_student(x, t, t, v);
+	    }
+	} else {	
+	    err = gretl_rand_student(x, t1, t2, v);
+	}
+    } else if (st == 'X') {
+	/* chi-square */
+	int v = parm[0];
+
+	if (vecp1 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		v = vecp1[t];
+		err = gretl_rand_chisq(x, t, t, v);
+	    }
+	} else {
+	    err = gretl_rand_chisq(x, t1, t2, v);
+	}
+    } else if (st == 'F') {
+	/* Snedecor F */
+	int v1 = parm[0], v2 = parm[1];
+
+	if (vecp1 != NULL || vecp2 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		if (vecp1 != NULL) v1 = vecp1[t];
+		if (vecp2 != NULL) v2 = vecp2[t];
+		err = gretl_rand_F(x, t, t, v1, v2);
+	    }
+	} else {
+	    err = gretl_rand_F(x, t1, t2, v1, v2);
+	}
+    } else if (st == 'G') {
+	/* gamma */
+	double shape = parm[0], scale = parm[1];
+
+	if (vecp1 != NULL || vecp2 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		if (vecp1 != NULL) shape = vecp1[t];
+		if (vecp2 != NULL) scale = vecp2[t];
+		err = gretl_rand_gamma(x, t, t, shape, scale);
+	    }
+	} else {
+	    err = gretl_rand_gamma(x, t1, t2, shape, scale);
+	}
+    } else if (st == 'B') {
+	/* binomial */
+	double pr = parm[0];
+	int n = parm[1];
+
+	if (vecp1 != NULL || vecp2 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		if (vecp1 != NULL) pr = vecp1[t];
+		if (vecp2 != NULL) n = vecp2[1];
+		err = gretl_rand_binomial(x, t, t, n, pr);
+	    }
+	} else {
+	    err = gretl_rand_binomial(x, t1, t2, n, pr);
+	}
+    } else if (st == 'P') {
+	/* Poisson */
+	double m = parm[0];
+
+	if (vecp1 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		m = vecp1[t];
+		err = gretl_rand_poisson(x, t, t, &m, 0);
+	    }
+	} else {
+	    err = gretl_rand_poisson(x, t1, t2, &m, 0);
+	}
+    } else if (st == 'W') {
+	/* Weibull */ 
+	double shape = parm[0], scale = parm[1];
+
+	if (vecp1 != NULL || vecp2 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		if (vecp1 != NULL) shape = vecp1[t];
+		if (vecp2 != NULL) scale = vecp2[t];
+		err = gretl_rand_weibull(x, t, t, shape, scale);
+	    }
+	} else {
+	    err = gretl_rand_weibull(x, t1, t2, shape, scale);
+	}
+    } else if (st == 'E') {
+	/* GED */ 
+	double nu = parm[0];
+
+	if (vecp1 != NULL) {
+	    for (t=t1; t<=t2 && !err; t++) {
+		if (vecp1 != NULL) nu = vecp1[t];
+		err = gretl_rand_GED(x, t, t, nu);
+	    }
+	} else {
+	    err = gretl_rand_GED(x, t1, t2, nu);
+	}
+    }	
+
+    return err;
+}
+
 /**
  * gretl_get_random_series:
  * @st: distribution code.
  * @parm: array holding either one or two scalar 
  * parameter values, depending on the distribution.
- * @serp1: series containing values for first param,
+ * @vecp1: series containing values for first param,
  * or %NULL.
- * @serp2: series containing values for second param,
+ * @vecp2: series containing values for second param,
  * or %NULL.
- * @pdinfo: dataset information.
+ * @dset: dataset information.
  * @err: location to receive error code.
  *
  * Produces a random series conforming to the distribution
@@ -2168,157 +2380,52 @@ double gretl_get_pvalue (char st, const double *parm, double x)
  */
 
 double *gretl_get_random_series (char st, const double *parm,
-				 const double *serp1, 
-				 const double *serp2,
-				 const DATAINFO *pdinfo,
+				 const double *vecp1, 
+				 const double *vecp2,
+				 const DATASET *dset,
 				 int *err)
 {
-    double *x = malloc(pdinfo->n * sizeof *x);
-    int t1 = pdinfo->t1;
-    int t2 = pdinfo->t2;
-    int t;
+    double *x = malloc(dset->n * sizeof *x);
 
     if (x == NULL) {
 	*err = E_ALLOC;
-	return NULL;
+    } else {
+	int t;
+
+	for (t=0; t<dset->n; t++) {
+	    x[t] = NADBL;
+	}
+
+	*err = gretl_fill_random_array(x, dset->t1, dset->t2, st,
+				       parm, vecp1, vecp2);
     }
 
-    for (t=0; t<pdinfo->n; t++) {
-	x[t] = NADBL;
-    }
-
-    if (st == 'u') {
-	/* uniform */
-	double min = parm[0], max = parm[1];
-
-	if (serp1 != NULL || serp2 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		if (serp1 != NULL) min = serp1[t];
-		if (serp2 != NULL) max = serp2[t];
-		*err = gretl_rand_uniform_minmax(x, t, t, min, max);
-	    }
-	} else {
-	    *err = gretl_rand_uniform_minmax(x, t1, t2, min, max);
-	}
-    } else if (st == 'z') {
-	/* normal */
-	double mu = parm[0], sd = parm[1];
-
-	if (serp1 != NULL || serp2 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		if (serp1 != NULL) mu = serp1[t];
-		if (serp2 != NULL) sd = serp2[t];
-		*err = gretl_rand_normal_full(x, t, t, mu, sd);
-	    }
-	} else {
-	    *err = gretl_rand_normal_full(x, t1, t2, mu, sd);
-	}
-    } else if (st == 't') {
-	/* Student's t */
-	int v = parm[0];
-
-	if (serp1 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		v = serp1[t];
-		*err = gretl_rand_student(x, t, t, v);
-	    }
-	} else {	
-	    *err = gretl_rand_student(x, t1, t2, v);
-	}
-    } else if (st == 'X') {
-	/* chi-square */
-	int v = parm[0];
-
-	if (serp1 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		v = serp1[t];
-		*err = gretl_rand_chisq(x, t, t, v);
-	    }
-	} else {
-	    *err = gretl_rand_chisq(x, t1, t2, v);
-	}
-    } else if (st == 'F') {
-	/* Snedecor F */
-	int v1 = parm[0], v2 = parm[1];
-
-	if (serp1 != NULL || serp2 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		if (serp1 != NULL) v1 = serp1[t];
-		if (serp2 != NULL) v2 = serp2[t];
-		*err = gretl_rand_F(x, t, t, v1, v2);
-	    }
-	} else {
-	    *err = gretl_rand_F(x, t1, t2, v1, v2);
-	}
-    } else if (st == 'G') {
-	/* gamma */
-	double shape = parm[0], scale = parm[1];
-
-	if (serp1 != NULL || serp2 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		if (serp1 != NULL) shape = serp1[t];
-		if (serp2 != NULL) scale = serp2[t];
-		*err = gretl_rand_gamma(x, t, t, shape, scale);
-	    }
-	} else {
-	    *err = gretl_rand_gamma(x, t1, t2, shape, scale);
-	}
-    } else if (st == 'B') {
-	/* binomial */
-	double pr = parm[0];
-	int n = parm[1];
-
-	if (serp1 != NULL || serp2 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		if (serp1 != NULL) pr = serp1[t];
-		if (serp2 != NULL) n = serp2[1];
-		*err = gretl_rand_binomial(x, t, t, n, pr);
-	    }
-	} else {
-	    *err = gretl_rand_binomial(x, t1, t2, n, pr);
-	}
-    } else if (st == 'P') {
-	/* Poisson */
-	double m = parm[0];
-
-	if (serp1 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		m = serp1[t];
-		*err = gretl_rand_poisson(x, t, t, &m, 0);
-	    }
-	} else {
-	    *err = gretl_rand_poisson(x, t1, t2, &m, 0);
-	}
-    } else if (st == 'W') {
-	/* Weibull */ 
-	double shape = parm[0], scale = parm[1];
-
-	if (serp1 != NULL || serp2 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		if (serp1 != NULL) shape = serp1[t];
-		if (serp2 != NULL) scale = serp2[t];
-		*err = gretl_rand_weibull(x, t, t, shape, scale);
-	    }
-	} else {
-	    *err = gretl_rand_weibull(x, t1, t2, shape, scale);
-	}
-    } else if (st == 'E') {
-	/* GED */ 
-	double nu = parm[0];
-
-	if (serp1 != NULL) {
-	    for (t=t1; t<=t2 && !*err; t++) {
-		if (serp1 != NULL) nu = serp1[t];
-		*err = gretl_rand_GED(x, t, t, nu);
-	    }
-	} else {
-	    *err = gretl_rand_GED(x, t1, t2, nu);
-	}
-    }	
-
     return x;
 }
 
+gretl_matrix *gretl_get_random_matrix (char st, const double *parm,
+				       int rows, int cols,
+				       int *err)
+{
+    gretl_matrix *m = NULL;
+    int n = rows * cols;
+
+    if (n <= 0) {
+	*err = E_INVARG;
+    } else {
+	m = gretl_matrix_alloc(rows, cols);
+	if (m == NULL) {
+	    *err = E_ALLOC;
+	    return NULL;
+	} else {
+	    *err = gretl_fill_random_array(m->val, 0, n - 1, st,
+					   parm, NULL, NULL);
+	}
+    }
+
+    return m;
+}
+
 static int 
 print_pv_string (double x, double p, PRN *prn)
 {
@@ -2386,7 +2493,7 @@ void print_pvalue (char st, const double *parm, double x,
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pv, 1 - 2 * pv);
 	} else {
-	    pc = student_cdf((int) parm[0], x);
+	    pc = student_cdf(parm[0], x);
 	    pprintf(prn, _("(to the left: %g)\n"), pc);
 	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		    2 * pc, 1 - 2 * pc);
@@ -2491,8 +2598,7 @@ void print_pvalue (char st, const double *parm, double x,
  * pvalue B prob n x (Binomial distribution).
  * pvalue P mean k (Poisson distribution).
  * pvalue W shape scale x (Weibull distribution).
- * @pZ: pointer to the data array.
- * @pdinfo: data information struct.
+ * @dset: dataset struct.
  * @prn: gretl printing struct.
  *
  * Calculates and prints the probability that a random variable 
@@ -2502,9 +2608,7 @@ void print_pvalue (char st, const double *parm, double x,
  * Returns: 0 on success, non-zero code on error.
  */
 
-int batch_pvalue (const char *str, 
-		  double ***pZ, DATAINFO *pdinfo, 
-		  PRN *prn)
+int batch_pvalue (const char *str, DATASET *dset, PRN *prn)
 {
     double pv = NADBL;
     char line[MAXLEN];
@@ -2539,7 +2643,7 @@ int batch_pvalue (const char *str,
     free_strings_array(S, n);
 
     if (!err) {
-	pv = generate_scalar(line, pZ, pdinfo, &err);
+	pv = generate_scalar(line, dset, &err);
     }
 
     if (!err) {
