@@ -28,7 +28,7 @@
 #include "gretl_f2c.h"
 #include "clapack_double.h"
 
-#define ESSZERO      1e-22 /* SSR less than this counts as zero */
+#define ESSZERO 1e-22 /* SSR less than this counts as zero */
 
 enum {
     VCV_SIMPLE,
@@ -206,7 +206,7 @@ static int qr_make_vcv (MODEL *pmod, gretl_matrix *v, int flag)
     return 0;
 }
 
-static void get_resids_and_SSR (MODEL *pmod, const double **Z,
+static void get_resids_and_SSR (MODEL *pmod, const DATASET *dset,
 				gretl_matrix *yhat, int fulln)
 {
     int dwt = gretl_model_get_int(pmod, "wt_dummy");
@@ -228,11 +228,11 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 	    if (t < pmod->t1 || t > pmod->t2) {
 		pmod->yhat[t] = u[t] = NADBL;
 	    } else {
-		y = Z[yvar][t];
+		y = dset->Z[yvar][t];
 		if (t == pmod->t1 && pwe) {
 		    y *= sqrt(1.0 - pmod->rho * pmod->rho);
 		} else {
-		    y -= pmod->rho * Z[yvar][t-1];
+		    y -= pmod->rho * dset->Z[yvar][t-1];
 		}
 		pmod->yhat[t] = yhat->val[i];
 		u[t] = y - yhat->val[i];
@@ -245,12 +245,12 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 	    if (t < pmod->t1 || t > pmod->t2 || model_missing(pmod, t)) {
 		pmod->yhat[t] = u[t] = NADBL;
 	    } else {
-		y = Z[yvar][t];
-		if (dwt && Z[dwt][t] == 0.0) {
+		y = dset->Z[yvar][t];
+		if (dwt && dset->Z[dwt][t] == 0.0) {
 		    pmod->yhat[t] = NADBL;
 		} else {
 		    if (!dwt) {
-			y *= sqrt(Z[pmod->nwt][t]);
+			y *= sqrt(dset->Z[pmod->nwt][t]);
 		    }
 		    pmod->yhat[t] = yhat->val[i];
 		    u[t] = y - yhat->val[i];
@@ -265,7 +265,7 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 		pmod->yhat[t] = u[t] = NADBL;
 	    } else {
 		pmod->yhat[t] = yhat->val[i];
-		u[t] = Z[yvar][t] - yhat->val[i];
+		u[t] = dset->Z[yvar][t] - yhat->val[i];
 		pmod->ess += u[t] * u[t];
 		i++;
 	    }
@@ -279,7 +279,7 @@ static void get_resids_and_SSR (MODEL *pmod, const double **Z,
 }
 
 static void 
-get_data_X (gretl_matrix *X, const MODEL *pmod, const double **Z)
+get_data_X (gretl_matrix *X, const MODEL *pmod, const DATASET *dset)
 {
     int wt = pmod->nwt;
     int i, j, t, vi;
@@ -291,22 +291,23 @@ get_data_X (gretl_matrix *X, const MODEL *pmod, const double **Z)
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    if (!model_missing(pmod, t)) {
 		if (wt) {
-		    X->val[j++] = sqrt(Z[wt][t]) * Z[vi][t];
+		    X->val[j++] = sqrt(dset->Z[wt][t]) * dset->Z[vi][t];
 		} else {
-		    X->val[j++] = Z[vi][t];
+		    X->val[j++] = dset->Z[vi][t];
 		}
 	    }
 	}
     }
 }
 
-static gretl_matrix *make_data_X (const MODEL *pmod, const double **Z)
+static gretl_matrix *make_data_X (const MODEL *pmod, 
+				  const DATASET *dset)
 {
     gretl_matrix *X;
 
     X = gretl_matrix_alloc(pmod->nobs, pmod->ncoeff);
     if (X != NULL) {
-	get_data_X(X, pmod, Z);
+	get_data_X(X, pmod, dset);
     }
 
     return X;
@@ -314,7 +315,7 @@ static gretl_matrix *make_data_X (const MODEL *pmod, const double **Z)
 
 /* Calculate W(t)-transpose * W(t-lag) */
 
-static void wtw (gretl_matrix *wt, gretl_matrix *X, 
+static void wtw (gretl_matrix *wt, const gretl_matrix *X, 
 		 int n, int t, int lag)
 {
     int i, j;
@@ -444,15 +445,15 @@ int newey_west_bandwidth (const gretl_matrix *f, int kern, int *h, double *bt)
     return err;
 }
 
-static int prewhiten_uhat (double **pu, int T, double *pa)
+static double *prewhiten_uhat (const double *u, int T, double *pa)
 {
     double a, num = 0.0, den = 0.0;
-    double *uw, *u = *pu;
+    double *uw;
     int sgn, t;
 
     uw = malloc(T * sizeof *uw);
     if (uw == NULL) {
-	return E_ALLOC;
+	return NULL;
     }
 
     for (t=1; t<T; t++) {
@@ -471,9 +472,121 @@ static int prewhiten_uhat (double **pu, int T, double *pa)
     } 
 
     *pa = a;
-    *pu = uw;
 
-    return 0;
+    return uw;
+}
+
+gretl_matrix *HAC_XOX (const gretl_matrix *uhat, const gretl_matrix *X,
+		       VCVInfo *vi, int *err)
+{
+    gretl_matrix *XOX = NULL, *Wtj = NULL, *Gj = NULL;
+    int prewhiten = libset_get_bool(PREWHITEN);
+    int kern = libset_get_int(HAC_KERNEL);
+    int T = X->rows;
+    int k = X->cols;
+    int p, j, t;
+    double wj, uu;
+    double a = 0, bt = 0;
+    double *u = NULL;
+
+    if (prewhiten) {
+	u = prewhiten_uhat(uhat->val, T, &a);
+	if (u == NULL) {
+	    *err = E_ALLOC;
+	    return NULL;
+	}
+    } else {
+	u = uhat->val;
+    }
+
+    XOX = gretl_zero_matrix_new(k, k);
+    Wtj = gretl_matrix_alloc(k, k);
+    Gj = gretl_matrix_alloc(k, k);
+
+    if (XOX == NULL || Wtj == NULL || Gj == NULL) {
+	*err = E_ALLOC;
+	goto bailout;
+    }
+
+    /* determine the bandwidth setting */
+
+    if (data_based_hac_bandwidth()) {
+	gretl_matrix umat;
+
+	umat.rows = T;
+	umat.cols = 1;
+	umat.val = u;
+	umat.t1 = umat.t2 = 0;
+	*err = newey_west_bandwidth(&umat, kern, &p, &bt);
+	if (*err) {
+	    goto bailout;
+	}
+    } else if (kern == KERNEL_QS) {
+	bt = libset_get_double(QS_BANDWIDTH);
+	p = T - 1;
+    } else {
+	p = get_hac_lag(T);
+    }
+
+    for (j=0; j<=p; j++) {
+	/* cumulate running sum of Gamma-hat terms */
+	gretl_matrix_zero(Gj);
+	for (t=j; t<T; t++) {
+	    /* W(t)-transpose * W(t-j) */
+	    wtw(Wtj, X, k, t, j);
+	    uu = u[t] * u[t-j];
+	    gretl_matrix_multiply_by_scalar(Wtj, uu);
+	    /* DM equation (9.36), p. 363 */
+	    gretl_matrix_add_to(Gj, Wtj);
+	}
+
+	if (j > 0) {
+	    /* Gamma(j) = Gamma(j) + Gamma(j)-transpose */
+	    gretl_matrix_add_self_transpose(Gj);
+	    if (kern == KERNEL_QS) {
+		wj = qs_hac_weight(bt, j);
+	    } else {
+		wj = hac_weight(kern, p, j);
+	    }
+	    gretl_matrix_multiply_by_scalar(Gj, wj);
+	}
+
+	/* DM equation (9.38), p. 364 */
+	gretl_matrix_add_to(XOX, Gj);
+    }
+
+    if (prewhiten) {
+	/* re-color */
+	gretl_matrix_divide_by_scalar(XOX, (1-a) * (1-a));
+    }
+
+    vi->vmaj = VCV_HAC;
+    vi->vmin = kern;
+    vi->flags = prewhiten;
+
+    if (kern == KERNEL_QS) {
+	vi->order = 0;
+	vi->bw = bt;
+    } else {
+	vi->order = p;
+	vi->bw = NADBL;
+    }
+
+ bailout:
+
+    gretl_matrix_free(Wtj);
+    gretl_matrix_free(Gj);
+
+    if (u != uhat->val) {
+	free(u);
+    }
+
+    if (*err && XOX != NULL) {
+	gretl_matrix_free(XOX);
+	XOX = NULL;
+    }
+
+    return XOX;
 }
 
 /* Calculate HAC covariance matrix.  Algorithm and (basically)
@@ -481,125 +594,54 @@ static int prewhiten_uhat (double **pu, int T, double *pa)
    and Methods, chapter 9.
 */
 
-static int qr_make_hac (MODEL *pmod, const double **Z, gretl_matrix *xpxinv)
+static int qr_make_hac (MODEL *pmod, const DATASET *dset, 
+			gretl_matrix *XTXi)
 {
-    gretl_matrix *vcv = NULL, *wtj = NULL, *gammaj = NULL;
-    gretl_matrix *X;
-    int prewhiten = libset_get_bool(PREWHITEN);
-    int kern = libset_get_int(HAC_KERNEL);
+    gretl_matrix *X, *XOX, *V = NULL;
+    gretl_matrix umat;
+    VCVInfo vi;
     int T = pmod->nobs;
-    int k = pmod->ncoeff;
-    int free_uhat = 0;
-    int p, j, t;
-    double wj, uu;
-    double a = 0, bt = 0;
-    double *uhat;
     int err = 0;
 
-    X = make_data_X(pmod, Z);
-    if (X == NULL) return 1;
+    X = make_data_X(pmod, dset);
+    if (X == NULL) {
+	return E_ALLOC;
+    }
 
     /* pmod->uhat is a full-length series: we must take an offset
        into it, equal to the offset of the data on which the model
        is actually estimated.
     */
-    uhat = pmod->uhat + pmod->t1;
+    umat.rows = T;
+    umat.cols = 1;
+    umat.t1 = umat.t2 = 0;
+    umat.val = pmod->uhat + pmod->t1;
 
-    if (prewhiten) {
-	err = prewhiten_uhat(&uhat, T, &a);
-	if (err) {
-	    return err;
+    XOX = HAC_XOX(&umat, X, &vi, &err);
+
+    if (!err) {
+	V = gretl_matrix_alloc(XOX->rows, XOX->rows);
+	if (V == NULL) {
+	    err = E_ALLOC;
 	}
-	free_uhat = 1;
     }
 
-    vcv = gretl_matrix_alloc(k, k);
-    wtj = gretl_matrix_alloc(k, k);
-    gammaj = gretl_matrix_alloc(k, k);
-
-    if (vcv == NULL || wtj == NULL || gammaj == NULL) {
-	err = 1;
-	goto bailout;
+    if (!err) {
+	gretl_matrix_qform(XTXi, GRETL_MOD_TRANSPOSE, XOX,
+			   V, GRETL_MOD_NONE);
+	/* Transcribe vcv into triangular representation */
+	err = qr_make_vcv(pmod, V, VCV_ROBUST);
     }
 
-    /* determine the bandwidth setting */
+    if (!err) {
+	gretl_model_set_full_vcv_info(pmod, vi.vmaj, vi.vmin,
+				      vi.order, vi.flags,
+				      vi.bw);
+    }	
 
-    if (data_based_hac_bandwidth()) {
-	gretl_matrix u;
-
-	u.rows = T;
-	u.cols = 1;
-	u.val = uhat;
-	err = newey_west_bandwidth(&u, kern, &p, &bt);
-	if (err) {
-	    goto bailout;
-	}
-    } else if (kern == KERNEL_QS) {
-	bt = libset_get_double(QS_BANDWIDTH);
-	p = pmod->nobs - 1;
-    } else {
-	p = get_hac_lag(T);
-    }
-
-    if (kern == KERNEL_QS) {
-	gretl_model_set_full_vcv_info(pmod, VCV_HAC, kern, 0, 
-				      prewhiten, bt);
-    } else {
-	gretl_model_set_full_vcv_info(pmod, VCV_HAC, kern, p, 
-				      prewhiten, NADBL);
-    }
-
-    gretl_matrix_zero(vcv);
-
-    for (j=0; j<=p; j++) {
-	/* cumulate running sum of Gamma-hat terms */
-	gretl_matrix_zero(gammaj);
-	for (t=j; t<T; t++) {
-	    /* W(t)-transpose * W(t-j) */
-	    wtw(wtj, X, k, t, j);
-	    uu = uhat[t] * uhat[t-j];
-	    gretl_matrix_multiply_by_scalar(wtj, uu);
-	    /* DM equation (9.36), p. 363 */
-	    gretl_matrix_add_to(gammaj, wtj);
-	}
-
-	if (j > 0) {
-	    /* Gamma(j) = Gamma(j) + Gamma(j)-transpose */
-	    gretl_matrix_add_self_transpose(gammaj);
-	    if (kern == KERNEL_QS) {
-		wj = qs_hac_weight(bt, j);
-	    } else {
-		wj = hac_weight(kern, p, j);
-	    }
-	    gretl_matrix_multiply_by_scalar(gammaj, wj);
-	}
-
-	/* DM equation (9.38), p. 364 */
-	gretl_matrix_add_to(vcv, gammaj);
-    }
-
-    if (prewhiten) {
-	/* re-color */
-	gretl_matrix_divide_by_scalar(vcv, (1-a) * (1-a));
-    }
-
-    gretl_matrix_copy_values(wtj, vcv);
-    gretl_matrix_qform(xpxinv, GRETL_MOD_TRANSPOSE, wtj,
-		       vcv, GRETL_MOD_NONE);
-
-    /* Transcribe vcv into triangular representation */
-    err = qr_make_vcv(pmod, vcv, VCV_ROBUST);
-
- bailout:
-
-    gretl_matrix_free(wtj);
-    gretl_matrix_free(gammaj);
-    gretl_matrix_free(vcv);
     gretl_matrix_free(X);
-
-    if (free_uhat) {
-	free(uhat);
-    }
+    gretl_matrix_free(XOX);
+    gretl_matrix_free(V);
 
     return err;
 }
@@ -629,8 +671,8 @@ static void do_X_prime_diag (const gretl_matrix *X,
    page 200.  Implements HC0, HC1, HC2 and HC3.
 */
 
-static int qr_make_hccme (MODEL *pmod, const double **Z, 
-			  gretl_matrix *Q, gretl_matrix *xpxinv)
+static int qr_make_hccme (MODEL *pmod, const DATASET *dset,
+			  gretl_matrix *Q, gretl_matrix *XTXi)
 {
     gretl_matrix *X;
     gretl_matrix *diag = NULL;
@@ -641,7 +683,7 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
     int i, t;
     int err = 0;
 
-    X = make_data_X(pmod, Z);
+    X = make_data_X(pmod, dset);
     if (X == NULL) return 1;
 
     diag = gretl_vector_from_array(pmod->uhat + pmod->t1, T,
@@ -685,7 +727,7 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
 
     do_X_prime_diag(X, diag, tmp1);
     gretl_matrix_multiply(tmp1, X, tmp2);
-    gretl_matrix_qform(xpxinv, GRETL_MOD_NONE, tmp2,
+    gretl_matrix_qform(XTXi, GRETL_MOD_NONE, tmp2,
 		       vcv, GRETL_MOD_NONE);
 
     /* Transcribe vcv into triangular representation */
@@ -702,13 +744,13 @@ static int qr_make_hccme (MODEL *pmod, const double **Z,
     return err;
 }
 
-static int qr_dw_stats (MODEL *pmod, const double **Z,
+static int qr_dw_stats (MODEL *pmod, const DATASET *dset,
 			gretl_matrix *X, gretl_matrix *u)
 {
     double DW, pv;
     int t, s, err = 0;
 
-    get_data_X(X, pmod, Z);
+    get_data_X(X, pmod, dset);
 
     for (s=0, t=pmod->t1; t<=pmod->t2; s++, t++) {
 	gretl_vector_set(u, s, pmod->uhat[t]);
@@ -732,7 +774,7 @@ static int qr_make_regular_vcv (MODEL *pmod, gretl_matrix *v,
     return qr_make_vcv(pmod, v, flag);
 }
 
-static void get_model_data (MODEL *pmod, const double **Z, 
+static void get_model_data (MODEL *pmod, const DATASET *dset,
 			    gretl_matrix *Q, gretl_matrix *y)
 {
     int dwt = gretl_model_get_int(pmod, "wt_dummy");
@@ -756,16 +798,16 @@ static void get_model_data (MODEL *pmod, const double **Z,
 	    if (model_missing(pmod, t)) {
 		continue;
 	    }
-	    x = Z[pmod->list[i]][t];
+	    x = dset->Z[pmod->list[i]][t];
 	    if (dwt) {
-		if (Z[dwt][t] == 0.0) continue;
+		if (dset->Z[dwt][t] == 0.0) continue;
 	    } else if (pmod->nwt) {
-		x *= sqrt(Z[pmod->nwt][t]);
+		x *= sqrt(dset->Z[pmod->nwt][t]);
 	    } else if (qdiff) {
 		if (pwe && t == pmod->t1) {
 		    x *= pw1;
 		} else {
-		    x -= pmod->rho * Z[pmod->list[i]][t-1];
+		    x -= pmod->rho * dset->Z[pmod->list[i]][t-1];
 		}
 	    }
 	    Q->val[j++] = x;
@@ -779,16 +821,16 @@ static void get_model_data (MODEL *pmod, const double **Z,
 	    if (model_missing(pmod, t)) {
 		continue;
 	    }		
-	    x = Z[pmod->list[1]][t];
+	    x = dset->Z[pmod->list[1]][t];
 	    if (dwt) {
-		if (Z[dwt][t] == 0.0) continue;
+		if (dset->Z[dwt][t] == 0.0) continue;
 	    } else if (pmod->nwt) {
-		x *= sqrt(Z[pmod->nwt][t]);
+		x *= sqrt(dset->Z[pmod->nwt][t]);
 	    } else if (qdiff) {
 		if (pwe && t == pmod->t1) {
 		    x *= pw1;
 		} else {
-		    x -= pmod->rho * Z[pmod->list[1]][t-1];
+		    x -= pmod->rho * dset->Z[pmod->list[1]][t-1];
 		}
 	    }
 	    y->val[j++] = x;
@@ -873,7 +915,8 @@ static int QR_decomp_plus (gretl_matrix *Q, gretl_matrix *R, int *rank,
 #define REDEBUG 0
 
 static void
-drop_redundant_vars (MODEL *pmod, gretl_matrix *R, int rank, gretlopt opt)
+drop_redundant_vars (MODEL *pmod, DATASET *dset, gretl_matrix *R, 
+		     int rank, gretlopt opt)
 {
     int *dlist = NULL;
     double d;
@@ -901,6 +944,8 @@ drop_redundant_vars (MODEL *pmod, gretl_matrix *R, int rank, gretlopt opt)
 		dlist[0] += 1;
 		dlist[dlist[0]] = pmod->list[j];
 	    }
+	    fprintf(stderr, "drop redundant variable %d (%s)\n",
+		    pmod->list[j], dset->varname[pmod->list[j]]);
 	    gretl_list_delete_at_pos(pmod->list, j--);
 	    nd++;
 	}
@@ -916,8 +961,7 @@ drop_redundant_vars (MODEL *pmod, gretl_matrix *R, int rank, gretlopt opt)
     }   
 }
 
-int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
-		      gretlopt opt)
+int gretl_qr_regress (MODEL *pmod, DATASET *dset, gretlopt opt)
 {
     integer T, k;
     gretl_matrix *Q = NULL, *y = NULL;
@@ -941,20 +985,20 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
 	goto qr_cleanup;
     }
 
-    get_model_data(pmod, Z, Q, y);
+    get_model_data(pmod, dset, Q, y);
     err = QR_decomp_plus(Q, R, &rank, &warn);
 
     /* handling of (near-)perfect collinearity */
     if (err == E_SINGULAR && !(opt & OPT_Z)) {
-	drop_redundant_vars(pmod, R, rank, opt);
+	drop_redundant_vars(pmod, dset, R, rank, opt);
 	k = pmod->list[0] - 1;
 	gretl_matrix_reuse(Q, T, k);
 	gretl_matrix_reuse(R, k, k);
 	gretl_matrix_reuse(V, k, k);
-	get_model_data(pmod, Z, Q, y);
+	get_model_data(pmod, dset, Q, y);
 	err = QR_decomp_plus(Q, R, NULL, &warn);
 	if (!err) {
-	    maybe_shift_ldepvar(pmod, Z, pdinfo);
+	    maybe_shift_ldepvar(pmod, dset);
 	}
     }
 
@@ -970,7 +1014,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
 	goto qr_cleanup;
     }
 
-    if (allocate_model_arrays(pmod, k, pdinfo->n)) {
+    if (allocate_model_arrays(pmod, k, dset->n)) {
 	err = E_ALLOC;
 	goto qr_cleanup;
     }
@@ -988,7 +1032,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
     gretl_matrix_multiply(Q, g, y);    
 
     /* get vector of residuals and SSR */
-    get_resids_and_SSR(pmod, Z, y, pdinfo->n);
+    get_resids_and_SSR(pmod, dset, y, dset->n);
 
     /* standard error of regression */
     if (T - k > 0) {
@@ -1011,20 +1055,20 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
     if (opt & OPT_R) { 
 	pmod->opt |= OPT_R;
 	if ((opt & OPT_T) && !libset_get_bool(FORCE_HC)) {
-	    qr_make_hac(pmod, Z, V);
+	    qr_make_hac(pmod, dset, V);
 	} else {
-	    qr_make_hccme(pmod, Z, Q, V);
+	    qr_make_hccme(pmod, dset, Q, V);
 	}
     } else {
 	qr_make_regular_vcv(pmod, V, opt);
     }
 
     /* get R^2, F */
-    qr_compute_stats(pmod, Z[pmod->list[1]], T, opt);
+    qr_compute_stats(pmod, dset->Z[pmod->list[1]], T, opt);
 
     /* D-W stat and p-value */
     if ((opt & OPT_I) && pmod->missmask == NULL) {
-	qr_dw_stats(pmod, Z, Q, y);
+	qr_dw_stats(pmod, dset, Q, y);
     }
 
     /* near-singularity? */
@@ -1047,8 +1091,7 @@ int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
     return err;    
 }
 
-int qr_tsls_vcv (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
-		 gretlopt opt)
+int qr_tsls_vcv (MODEL *pmod, const DATASET *dset, gretlopt opt)
 {
     gretl_matrix *Q = NULL;
     gretl_matrix *R = NULL;
@@ -1057,7 +1100,7 @@ int qr_tsls_vcv (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
 
     k = pmod->list[0] - 1;        /* # of cols (variables) */
 
-    Q = make_data_X(pmod, Z);
+    Q = make_data_X(pmod, dset);
     R = gretl_matrix_alloc(k, k);
     V = gretl_matrix_alloc(k, k);
 
@@ -1078,18 +1121,18 @@ int qr_tsls_vcv (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
 
     /* VCV and standard errors */
     if (opt & OPT_R) {
-	if (dataset_is_panel(pdinfo)) {
+	if (dataset_is_panel(dset)) {
 	    err = qr_make_regular_vcv(pmod, V, OPT_X);
 	    if (!err) {
-		err = panel_tsls_robust_vcv(pmod, Z, pdinfo);
+		err = panel_tsls_robust_vcv(pmod, dset);
 	    }
-	} else if (dataset_is_time_series(pdinfo) && 
+	} else if (dataset_is_time_series(dset) && 
 		   !libset_get_bool(FORCE_HC)) {
 	    pmod->opt |= OPT_R;
-	    err = qr_make_hac(pmod, Z, V);
+	    err = qr_make_hac(pmod, dset, V);
 	} else {
 	    pmod->opt |= OPT_R;
-	    err = qr_make_hccme(pmod, Z, Q, V);
+	    err = qr_make_hccme(pmod, dset, Q, V);
 	}
     } else {
 	qr_make_regular_vcv(pmod, V, OPT_NONE);
