@@ -23,16 +23,15 @@
 #include <time.h>
 #include <glib.h>
 
-#ifndef NO_SFMT /* flag to cut out all SFMT code */
-# ifdef USE_SSE2
-#  define HAVE_SSE2
-# endif
-# include "../../rng/SFMT.c"
-# if defined(HAVE_POSIX_MEMALIGN) || defined(OSX_BUILD) || defined(WIN32)
-#  define USE_SFMT_ARRAYS 0 /* faster, but needs more testing */
-# else
-#  define USE_SFMT_ARRAYS 0
-# endif
+#ifdef USE_SSE2
+# define HAVE_SSE2
+#endif
+
+#include "../../rng/SFMT.c"
+#if defined(HAVE_POSIX_MEMALIGN) || defined(OSX_BUILD) || defined(WIN32)
+# define USE_SFMT_ARRAYS 0 /* potentially faster, but needs more testing */
+#else
+# define USE_SFMT_ARRAYS 0
 #endif
 
 /**
@@ -56,11 +55,7 @@
 static GRand *gretl_rand;
 static guint32 useed;
 
-#ifdef NO_SFMT
-static int use_sfmt = 0;
-#else
 static int use_sfmt = 1;
-#endif
 
 static guint32 gretl_rand_octet (guint32 *sign);
 
@@ -70,25 +65,25 @@ static void *gretl_memalign (size_t size, int *err)
 {
     void *mem = NULL;
 
-#if defined(HAVE_POSIX_MEMALIGN)
+# if defined(HAVE_POSIX_MEMALIGN)
     *err = posix_memalign((void **) &mem, 16, size);
     if (*err) {
 	fprintf(stderr, "posix_memalign: err = %d\n", *err);
 	return NULL;
     }
-#elif defined(WIN32)
+# elif defined(WIN32)
     mem = win32_memalign(size, 16);
     if (mem == NULL) {
 	fputs("win32_memalign: failed\n", stderr);
 	*err = E_ALLOC;
     }
-#else /* OS X: should be aligned OK by default */
+# else /* OS X: should be aligned OK by default */
     mem = malloc(size);
     if (mem == NULL) {
 	fputs("osx_malloc: failed\n", stderr);
 	*err = E_ALLOC;
     }
-#endif
+# endif
 
     return mem;
 }
@@ -117,11 +112,11 @@ static int sfmt_array_setup (void)
 
 static void sfmt_array_cleanup (void)
 {
-#if defined(WIN32) && !defined(HAVE_POSIX_MEMALIGN)
+# if defined(WIN32) && !defined(HAVE_POSIX_MEMALIGN)
     win32_aligned_free(rbuf);
-#else
+# else
     free(rbuf);
-#endif
+# endif
     rbuf = NULL;
 }
 
@@ -148,11 +143,11 @@ static int sfmt_fill_U01_array (double *a, int t1, int t2, int n)
 	a[t] = to_real2(buf[i++]);
     }
 
-#if defined(WIN32) && !defined(HAVE_POSIX_MEMALIGN)
+# if defined(WIN32) && !defined(HAVE_POSIX_MEMALIGN)
     win32_aligned_free(buf);
-#else
+# else
     free(buf);
-#endif
+# endif
 
     return err;
 }
@@ -167,9 +162,9 @@ static inline guint32 sfmt_rand32 (void)
     return rbuf[r_i++];
 }
 
-#else
+#else /* !USE_SFMT_ARRAYS */
 
-#define sfmt_rand32 gen_rand32
+# define sfmt_rand32 gen_rand32
 
 #endif /* USE_SFMT_ARRAYS */
 
@@ -183,11 +178,9 @@ void gretl_rand_init (void)
 {
     useed = time(NULL);
 
-#ifndef NO_SFMT
     init_gen_rand(useed);
-# if USE_SFMT_ARRAYS
+#if USE_SFMT_ARRAYS
     sfmt_array_setup();
-# endif
 #endif
 
     if (gretl_rand == NULL) {
@@ -241,11 +234,9 @@ unsigned int gretl_rand_get_seed (void)
 void gretl_rand_set_seed (unsigned int seed)
 {
     useed = (seed == 0)? time(NULL) : seed;
-#ifndef NO_SFMT
     init_gen_rand(useed); 
-# if USE_SFMT_ARRAYS
+#if USE_SFMT_ARRAYS
     sfmt_array_setup();
-# endif
 #endif
     g_rand_set_seed(gretl_rand, useed);
     gretl_rand_octet(NULL);
@@ -311,15 +302,11 @@ int gretl_rand_get_sfmt (void)
 
 double gretl_rand_01 (void)
 {
-#ifdef NO_SFMT
-    return g_rand_double(gretl_rand);
-#else
     if (use_sfmt) {
 	return to_real2(sfmt_rand32());
     } else {
 	return g_rand_double(gretl_rand);
     }
-#endif
 }
 
 /* Below: an implementation of the Marsaglia/Tsang Ziggurat method for
@@ -485,15 +472,11 @@ static guint32 gretl_rand_octet (guint32 *sign)
     }
 
     if (i == 0) {
-#ifdef NO_SFMT
-	wr.u = g_rand_int(gretl_rand);
-#else
 	if (use_sfmt) {
 	    wr.u = sfmt_rand32();
 	} else {
 	    wr.u = g_rand_int(gretl_rand);
 	}
-#endif
 	i = 4;
     }
 
@@ -508,11 +491,7 @@ static double ran_normal_ziggurat (void)
     double x, y;
 
     while (1) {
-#ifdef NO_SFMT
-	j = g_rand_int(gretl_rand);
-#else
 	j = use_sfmt ? sfmt_rand32() : g_rand_int(gretl_rand);
-#endif
 	i = gretl_rand_octet(&sign);
 	j = j >> 2;
 
@@ -711,8 +690,6 @@ int gretl_rand_normal_full (double *a, int t1, int t2,
     return 0;
 }
 
-#ifndef NO_SFMT
-
 static guint32 sfmt_int_range (guint32 begin, guint32 end)
 {
     guint32 dist = end - begin;
@@ -743,8 +720,6 @@ static guint32 sfmt_int_range (guint32 begin, guint32 end)
     return begin + rval;
 }
 
-#endif
-
 /**
  * gretl_rand_uniform_minmax:
  * @a: target array.
@@ -773,16 +748,12 @@ int gretl_rand_uniform_minmax (double *a, int t1, int t2,
     }
 
     for (t=t1; t<=t2; t++) {
-#ifdef NO_SFMT
-	a[t] = g_rand_double_range(gretl_rand, min, max);
-#else
 	if (use_sfmt) {
 	    /* FIXME use native array functionality? */
 	    a[t] = to_real2(sfmt_rand32()) * (max - min) + min;
 	} else {
 	    a[t] = g_rand_double_range(gretl_rand, min, max);
 	}
-#endif
     }
 
     return 0;
@@ -796,7 +767,7 @@ int gretl_rand_uniform_minmax (double *a, int t1, int t2,
  * @max: upper closed bound of range.
  *
  * Fill array @a of length @n with pseudo-random drawings
- * from the uniform distribution on @min to @max, using the
+ * from the uniform distribution on [@min, @max], using the
  * Mersenne Twister.
  *
  * Returns: 0 on success, 1 on invalid input.
@@ -804,29 +775,96 @@ int gretl_rand_uniform_minmax (double *a, int t1, int t2,
 
 int gretl_rand_int_minmax (int *a, int n, int min, int max) 
 {
-    int i;
+    int i, err = 0;
 
     if (max < min) {
-	return E_INVARG;
+	err = E_INVARG;
+    } else if (min == max) {
+	for (i=0; i<n; i++) {
+	    a[i] = min;
+	}
+    } else {
+	/* in g_rand_int_range() the upper bound is open */
+	for (i=0; i<n; i++) {
+	    if (use_sfmt) {
+		a[i] = sfmt_int_range(min, max + 1);
+	    } else {
+		a[i] = g_rand_int_range(gretl_rand, min, max + 1);
+	    } 
+	}
     }
 
-    /* note: in g_rand_int_range the upper bound is open */
+    return err;
+}
+
+static int already_selected (double *a, int n, double val)
+{
+    int i;
 
     for (i=0; i<n; i++) {
-#ifdef NO_SFMT
-	a[i] = g_rand_int_range(gretl_rand, min, max + 1);
-#else
-	if (use_sfmt) {
-	    a[i] = sfmt_int_range(min, max + 1);
-	} else {
-	    a[i] = g_rand_int_range(gretl_rand, min, max + 1);
-	} 
-#endif
+	if (a[i] == val) {
+	    return 1;
+	}
     }
 
     return 0;
 }
 
+/**
+ * gretl_rand_uniform_int_minmax:
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
+ * @min: lower closed bound of range.
+ * @max: upper closed bound of range.
+ * @opt: include OPT_O for sampling without replacement.
+ *
+ * Fill the selected subset of array @a with pseudo-random drawings
+ * from the uniform distribution on [@min, @max], using the
+ * Mersenne Twister.
+ *
+ * Returns: 0 on success, 1 on invalid input.
+ */
+
+int gretl_rand_uniform_int_minmax (double *a, int t1, int t2, 
+				   int min, int max,
+				   gretlopt opt) 
+{
+    int t, err = 0;
+
+    if (max < min) {
+	err = E_INVARG;
+    } else if (max == min) {
+	for (t=t1; t<=t2; t++) {
+	    a[t] = min;
+	}
+    } else {
+	double x;
+	int i = 0;
+
+	for (t=t1; t<=t2; t++) {
+	    if (use_sfmt) {
+		x = sfmt_int_range(min, max + 1);
+	    } else {
+		x = g_rand_int_range(gretl_rand, min, max + 1);
+	    } 
+	    if (opt & OPT_O) {
+		while (already_selected(a, i, x)) {
+		    if (use_sfmt) {
+			x = sfmt_int_range(min, max + 1);
+		    } else {
+			x = g_rand_int_range(gretl_rand, min, max + 1);
+		    } 
+		}
+	    }
+	    a[t] = x;
+	    i++;
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_rand_uniform:
  * @a: target array
@@ -842,12 +880,7 @@ void gretl_rand_uniform (double *a, int t1, int t2)
 {
     int t;
 
-#ifdef NO_SFMT
-    for (t=t1; t<=t2; t++) {
-	a[t] = g_rand_double(gretl_rand);
-    }
-#else 
-# if USE_SFMT_ARRAYS
+#if USE_SFMT_ARRAYS
     if (use_sfmt) {
 	int n = t2 - t1 + 1;
 	int err = 0;
@@ -859,7 +892,7 @@ void gretl_rand_uniform (double *a, int t1, int t2)
 	    }
 	}
     }
-# endif
+#endif
     if (use_sfmt) {
 	for (t=t1; t<=t2; t++) {
 	   a[t] = to_real2(sfmt_rand32());
@@ -869,7 +902,6 @@ void gretl_rand_uniform (double *a, int t1, int t2)
 	   a[t] = g_rand_double(gretl_rand);
 	}
     }
-#endif	
 }
 
 /**
@@ -1004,19 +1036,19 @@ int gretl_rand_chisq (double *a, int t1, int t2, int v)
  *
  * Fill the selected range of array @a with pseudo-random drawings
  * from the Student t distribution with @v degrees of freedom, 
- * using the Mersenne Twister for uniform input and the Box-Muller 
+ * using the Mersenne Twister for uniform input and the ziggurat 
  * method for converting to the normal distribution.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int gretl_rand_student (double *a, int t1, int t2, int v) 
+int gretl_rand_student (double *a, int t1, int t2, double v) 
 {
     double *X2 = NULL;
     int T = t2 - t1 + 1;
     int t;
 
-    if (v < 1) {
+    if (v <= 0) {
 	return E_INVARG;
     }
 
@@ -1026,7 +1058,7 @@ int gretl_rand_student (double *a, int t1, int t2, int v)
     }
 
     gretl_rand_normal(a, t1, t2);
-    gretl_rand_chisq(X2, 0, T-1, v);
+    gretl_rand_gamma(X2, 0, T-1, 0.5*v, 2);
 
     for (t=0; t<T; t++) {
 	a[t + t1] /= sqrt(X2[t] / v);
@@ -1277,15 +1309,11 @@ int gretl_rand_GED (double *a, int t1, int t2, double nu)
 
 unsigned int gretl_rand_int_max (unsigned int max)
 {
-#ifdef NO_SFMT
-    return g_rand_int_range(gretl_rand, 0, max);
-#else
     if (use_sfmt) {
 	return sfmt_int_range(0, max);
     } else {
 	return g_rand_int_range(gretl_rand, 0, max);
     }
-#endif
 }
 
 /**
@@ -1297,14 +1325,10 @@ unsigned int gretl_rand_int_max (unsigned int max)
 
 unsigned int gretl_rand_int (void)
 {
-#ifdef NO_SFMT
-    return g_rand_int(gretl_rand);
-#else
     if (use_sfmt) {
 	return sfmt_rand32();
     } else {
 	return g_rand_int(gretl_rand);
     }
-#endif
 }
 
