@@ -89,6 +89,7 @@ double dot_atof (const char *s)
 
 	atof_point = *lc->decimal_point;
     }
+
     if (atof_point == '.') {
 	x = atof(s);
     } else {
@@ -264,25 +265,6 @@ int charpos (char c, const char *s)
     return -1;
 }
 
-/**
- * lastchar:
- * @c: the character to look for.
- * @s: the string to examine.
- *
- * Returns: 1 if @c is the last character in @s, 0 otherwise
- */
-
-int lastchar (char c, const char *s)
-{
-    int ret = 0;
-
-    if (s != NULL && s[strlen(s) - 1] == c) {
-	ret = 1;
-    }
-
-    return ret;
-}
-
 /**
  * charsub:
  * @str: the string to operate on.
@@ -497,9 +479,11 @@ char *gretl_strdup (const char *src)
     char *targ = NULL;
 
     if (src != NULL) {
-	targ = malloc(strlen(src) + 1);
+	size_t n = strlen(src) + 1;
+
+	targ = malloc(n);
 	if (targ != NULL) {
-	    strcpy(targ, src);
+	    memcpy(targ, src, n);
 	}
     }
 
@@ -528,8 +512,8 @@ char *gretl_strndup (const char *src, size_t n)
 
 	targ = malloc(len + 1);
 	if (targ != NULL) {
-	    *targ = '\0';
-	    strncat(targ, src, len);
+	    memcpy(targ, src, len);
+	    targ[len] = '\0';
 	}
     }
 
@@ -1071,38 +1055,6 @@ char *switch_ext_new (const char *src, const char *ext)
     return ret;
 }
 
-/**
- * get_base:
- * @targ: the target or output string (must be pre-allocated).
- * @src: the source or input string.
- * @c: the "base marker" character.
- *
- * If @c is found in @src, puts into @targ the portion of @src up to and 
- * including the last occurrence of @c within @src.
- *
- * Returns: 1 if @c is found in @str, otherwise 0.
- */
-
-int get_base (char *targ, const char *src, char c)
-{
-    int ret = 0;
-
-    if (src != NULL && *src != '\0') {
-	int i, n = strlen(src);
-
-	for (i=n-1; i>=0; i--) {
-	    if (src[i] == c) {
-		*targ = '\0';
-		strncat(targ, src, i + 1);
-		ret = 1;
-		break;
-	    }
-	}
-    }
-
-    return ret;
-}
-
 /**
  * top_n_tail:
  * @str: the string to process.
@@ -1607,12 +1559,21 @@ static int compare_strings (const void *a, const void *b)
  * strings on exit. It is assumed that storage for the
  * strings array was obtained via strings_array_new() or
  * a similar libgretl function.
+ *
+ * Returns: 0 on success, non-zero on error.
  */
 
 int strings_array_sort (char ***pS, int *n, gretlopt opt)
 {
-    char **S = *pS;
-    int ns = *n;
+    char **S;
+    int ns;
+
+    if (pS == NULL || n == NULL) {
+	return E_DATA;
+    }
+
+    S = *pS;
+    ns = *n;
 
     qsort(S, ns, sizeof *S, compare_strings);
 
@@ -1635,7 +1596,7 @@ int strings_array_sort (char ***pS, int *n, gretlopt opt)
 
 	    if (tmp != NULL) {
 		*pS = tmp;
-	    }
+	    } 
 	    *n = m;
 	}
     }
@@ -1695,17 +1656,17 @@ void free_strings_array (char **strs, int nstrs)
  * get_obs_string:
  * @obs: char array big enough to hold the observation (#OBSLEN).
  * @t: zero-based observation number.
- * @pdinfo: pointer to dataset information.
+ * @dset: pointer to dataset information.
  *
  * Returns: the observation string corresponding to @t.
  */
 
-char *get_obs_string (char *obs, int t, const DATAINFO *pdinfo)
+char *get_obs_string (char *obs, int t, const DATASET *dset)
 {
-    if (dataset_has_markers(pdinfo)) { 
-	strcpy(obs, pdinfo->S[t]);
+    if (dataset_has_markers(dset)) { 
+	strcpy(obs, dset->S[t]);
     } else {
-	ntodate(obs, t, pdinfo);
+	ntodate(obs, t, dset);
     }
 
     return obs;
@@ -1782,25 +1743,25 @@ void modify_date_for_csv (char *s, int pd)
 /**
  * csv_obs_string_to_prn:
  * @t: 0-based observation number.
- * @pdinfo: data set information struct.
+ * @dset: data set information struct.
  * @prn: printing struct.
  *
  * Prints the observation string corresponding to obervation @t to
  * @prn, in a format suitable for a CSV file.
  */
 
-void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
+void csv_obs_to_prn (int t, const DATASET *dset, PRN *prn)
 {
-    if (pdinfo->S != NULL) {
-	pprintf(prn, "%s%c", pdinfo->S[t], pdinfo->delim);
-    } else if (pdinfo->structure != CROSS_SECTION) {
+    if (dset->S != NULL) {
+	pprintf(prn, "%s%c", dset->S[t], dset->delim);
+    } else if (dset->structure != CROSS_SECTION) {
 	char tmp[OBSLEN];
 
-	ntodate(tmp, t, pdinfo);
-	if (quarterly_or_monthly(pdinfo)) {
-	    modify_date_for_csv(tmp, pdinfo->pd);
+	ntodate(tmp, t, dset);
+	if (quarterly_or_monthly(dset)) {
+	    modify_date_for_csv(tmp, dset->pd);
 	}
-	pprintf(prn, "%s%c", tmp, pdinfo->delim);
+	pprintf(prn, "%s%c", tmp, dset->delim);
     }
 }
 	
@@ -2005,7 +1966,7 @@ void unescape_url (char *url)
  * make_varname_unique:
  * @vname: tentative name for variable.
  * @v: the ID number for the new variable.
- * @pdinfo: dataset information.
+ * @dset: dataset information.
  *
  * Given a tentative name for a new variable, check that it
  * is not a duplicate of an existing varname.  If it is,
@@ -2018,7 +1979,7 @@ void unescape_url (char *url)
  * Returns: the (possibly modified) variable name.
  */
 
-char *make_varname_unique (char *vname, int v, DATAINFO *pdinfo) 
+char *make_varname_unique (char *vname, int v, DATASET *dset) 
 {
     const char *add = "abcdefghijklmnopqrstuvwxyz";
     size_t n = strlen(vname);
@@ -2030,8 +1991,8 @@ char *make_varname_unique (char *vname, int v, DATAINFO *pdinfo)
 
     for (j=0; j<26; j++) {
 	conflict = 0;
-	for (i=1; i<pdinfo->v && !conflict; i++) {
-	    if (i != v && !strcmp(vname, pdinfo->varname[i])) {
+	for (i=1; i<dset->v && !conflict; i++) {
+	    if (i != v && !strcmp(vname, dset->varname[i])) {
 		conflict = 1;
 	    }
 	}
@@ -2046,17 +2007,17 @@ char *make_varname_unique (char *vname, int v, DATAINFO *pdinfo)
     return vname;
 }
 
-int fix_varname_duplicates (DATAINFO *pdinfo)
+int fix_varname_duplicates (DATASET *dset)
 {
     int dups = 0;
     int i, j;
 
-    for (i=1; i<pdinfo->v; i++) {
-	for (j=i+1; j<pdinfo->v; j++) {
-	    if (!strcmp(pdinfo->varname[i], pdinfo->varname[j])) {
-		fprintf(stderr, "'%s' duplicated variable name\n", pdinfo->varname[i]);
+    for (i=1; i<dset->v; i++) {
+	for (j=i+1; j<dset->v; j++) {
+	    if (!strcmp(dset->varname[i], dset->varname[j])) {
+		fprintf(stderr, "'%s' duplicated variable name\n", dset->varname[i]);
 		dups = 1;
-		make_varname_unique(pdinfo->varname[j], j, pdinfo);
+		make_varname_unique(dset->varname[j], j, dset);
 	    }
 	}
     }
