@@ -59,8 +59,6 @@ char *delchar (int c, char *str);
 
 int charpos (char c, const char *s);
 
-int lastchar (char c, const char *s);
-
 int ends_with_backslash (const char *s);
 
 int gretl_namechar_spn (const char *s);
@@ -103,8 +101,6 @@ char *switch_ext (char *targ, const char *src, const char *ext);
 
 char *switch_ext_new (const char *src, const char *ext);
 
-int get_base (char *targ, const char *src, char c);
-
 int equation_get_lhs_and_rhs (const char *s, char **plh, char **prh);
 
 int top_n_tail (char *str, size_t maxlen, int *err);
@@ -136,7 +132,7 @@ int strings_array_cmp (char **strs1, char **strs2, int n);
 
 void free_strings_array (char **strs, int nstrs);
 
-char *get_obs_string (char *obs, int t, const DATAINFO *pdinfo);
+char *get_obs_string (char *obs, int t, const DATASET *dset);
 
 double obs_str_to_double (const char *obs);
 
@@ -144,7 +140,7 @@ char *colonize_obs (char *obs);
 
 void modify_date_for_csv (char *s, int pd);
 
-void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn);
+void csv_obs_to_prn (int t, const DATASET *dset, PRN *prn);
 
 char *print_time (char *s);
 
@@ -156,9 +152,9 @@ int gretl_xml_encode_to_buf (char *targ, const char *src, int n);
 
 void unescape_url (char *url);
 
-char *make_varname_unique (char *vname, int v, DATAINFO *pdinfo);
+char *make_varname_unique (char *vname, int v, DATASET *dset);
 
-int fix_varname_duplicates (DATAINFO *pdinfo);
+int fix_varname_duplicates (DATASET *dset);
 
 char *append_dir (char *fname, const char *dir);
 
