@@ -29,13 +29,13 @@
 
 #define TDEBUG 0
 
-static void tsls_omitzero (int *list, const double **Z, int t1, int t2)
+static void tsls_omitzero (int *list, const DATASET *dset)
 {
     int i, v;
 
     for (i=list[0]; i>1; i--) {
         v = list[i];
-        if (gretl_iszero(t1, t2, Z[v])) {
+        if (gretl_iszero(dset->t1, dset->t2, dset->Z[v])) {
 	    gretl_list_delete_at_pos(list, i);
 	}
     }
@@ -72,7 +72,7 @@ void tsls_free_data (const MODEL *pmod)
 
 static int 
 tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
-		double **Z, DATAINFO *pdinfo)
+		DATASET *dset)
 {
     double **X = NULL;
     char *endog = NULL;
@@ -87,7 +87,7 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 
     /* re-use old pointers if applicable */
 
-    X = gretl_model_get_data_and_size(pmod, "tslsX", &xs_old);
+    X = gretl_model_get_data_full(pmod, "tslsX", NULL, &xs_old);
     if (X != NULL) {
 	if (Xsize == xs_old) {
 	    recycle_X = 1;
@@ -106,7 +106,7 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 	}
     }
 
-    endog = gretl_model_get_data_and_size(pmod, "endog", &es_old);
+    endog = gretl_model_get_data_full(pmod, "endog", NULL, &es_old);
     if (endog != NULL) {
 	if (esize == es_old) {
 	    recycle_e = 1;
@@ -128,8 +128,8 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
     /* steal the appropriate columns from Z */
     for (i=1; i<=hatlist[0]; i++) {
 	v = hatlist[i];
-	X[i-1] = Z[v];
-	Z[v] = NULL;
+	X[i-1] = dset->Z[v];
+	dset->Z[v] = NULL;
     }
 
     for (i=0; i<pmod->ncoeff; i++) {
@@ -403,7 +403,7 @@ int *tsls_make_endolist (const int *reglist, int **instlist,
 
 static int fill_E_matrix (gretl_matrix *E, MODEL *pmod, 
 			  const int *reglist, const int *instlist,
-			  double **Z, DATAINFO *pdinfo)
+			  DATASET *dset)
 {
     MODEL emod;
     int *elist;
@@ -436,7 +436,7 @@ static int fill_E_matrix (gretl_matrix *E, MODEL *pmod,
 	}
 
 	/* regress the given endogenous var on all the instruments */
-	emod = lsq(elist, Z, pdinfo, OLS, OPT_A);
+	emod = lsq(elist, dset, OLS, OPT_A);
 	if ((err = emod.errcode)) {
 	    clear_model(&emod);
 	    break;
@@ -477,8 +477,7 @@ static int tsls_loglik (MODEL *pmod,
 			int nendo,
 			const int *reglist,
 			const int *instlist,
-			double **Z, 
-			DATAINFO *pdinfo)
+			DATASET *dset)
 {
     gretl_matrix *E, *W;
     int T = pmod->nobs;
@@ -495,7 +494,7 @@ static int tsls_loglik (MODEL *pmod,
     }
 
     if (!err) {
-	err = fill_E_matrix(E, pmod, reglist, instlist, Z, pdinfo);
+	err = fill_E_matrix(E, pmod, reglist, instlist, dset);
     }
 
     if (!err) {
@@ -527,7 +526,7 @@ static int tsls_loglik (MODEL *pmod,
 
 static int 
 ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist, 
-		   double ***pZ, DATAINFO *pdinfo)
+		   DATASET *dset)
 {
     int t1 = pmod->t1;
     int t2 = pmod->t2;
@@ -536,7 +535,7 @@ ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist,
 
     MODEL smod;
     int *OT_list = NULL;
-    int nv = pdinfo->v;
+    int nv = dset->v;
 
     if (Orank == 0) {
 	return 0;
@@ -547,18 +546,18 @@ ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist,
 	return 0;
     }
 
-    err = dataset_add_series(1, pZ, pdinfo);
+    err = dataset_add_series(1, dset);
     if (err) {
 	return err;
     }
 
     for (t=t1; t<=t2; t++) {
-	(*pZ)[nv][t] = pmod->uhat[t];
+	dset->Z[nv][t] = pmod->uhat[t];
     }
 
     OT_list = gretl_list_new(ninst + 1);
     if (OT_list == NULL) {
-	dataset_drop_last_variables(1, pZ, pdinfo);
+	dataset_drop_last_variables(1, dset);
 	return E_ALLOC;
     }
 
@@ -567,7 +566,12 @@ ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist,
 	OT_list[i] = instlist[i-1];
     }
 
-    smod = lsq(OT_list, *pZ, pdinfo, OLS, OPT_A);
+#if TDEBUG
+    fprintf(stderr, "running regression for Sargan test\n");
+#endif
+
+    smod = lsq(OT_list, dset, OLS, OPT_A);
+
     if (smod.errcode) {
 	fprintf(stderr, "ivreg_sargan_test: smod.errcode = %d\n", smod.errcode);
 	err = smod.errcode;
@@ -585,7 +589,7 @@ ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist,
     }
 
     clear_model(&smod);
-    dataset_drop_last_variables(1, pZ, pdinfo);
+    dataset_drop_last_variables(1, dset);
     free(OT_list);
 
     return err;
@@ -597,13 +601,13 @@ ivreg_sargan_test (MODEL *pmod, int Orank, int *instlist,
 
 static int 
 tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
-		   double ***pZ, DATAINFO *pdinfo)
+		   DATASET *dset)
 {
     MODEL hmod;
     double URSS;
     int *HT_list = NULL;
     int t, df;
-    int nv = pdinfo->v;
+    int nv = dset->v;
     int ku = 0;
     int err = 0;
 
@@ -624,8 +628,12 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 	return err;
     } 
 
+#if TDEBUG
+    fprintf(stderr, "running regression for Hausman test, 1\n");
+#endif
+
     /* estimate the unrestricted model */
-    hmod = lsq(HT_list, *pZ, pdinfo, OLS, OPT_A);
+    hmod = lsq(HT_list, dset, OLS, OPT_A);
     if (hmod.errcode) {
 	fprintf(stderr, "tsls_hausman_test: hmod.errcode (U) = %d\n", hmod.errcode);
 	err = hmod.errcode;
@@ -634,7 +642,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 
 #if HTDBG
     pprintf(dbgprn, "Hausman: unrestricted model (df = %d)\n", hmod.dfd);
-    printmodel(&hmod, pdinfo, OPT_NONE, dbgprn);
+    printmodel(&hmod, dset, OPT_NONE, dbgprn);
     nobs1 = hmod.nobs;
 #endif
 
@@ -647,13 +655,13 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     ku = hmod.ncoeff;
 
     /* add fitted values from unrestricted model to dataset */
-    err = dataset_add_series(1, pZ, pdinfo);
+    err = dataset_add_series(1, dset);
     if (err) {
 	goto bailout;
     } 
 
     for (t=hmod.t1; t<=hmod.t2; t++) {
-	(*pZ)[nv][t] = hmod.yhat[t];
+	dset->Z[nv][t] = hmod.yhat[t];
     }
 
     clear_model(&hmod);
@@ -662,15 +670,19 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     HT_list = gretl_list_copy(reglist);
     HT_list[1] = nv;
 
+#if TDEBUG
+    fprintf(stderr, "running regression for Hausman test, 2\n");
+#endif
+
     /* regress the U fitted values on the regressors of the restricted 
        model (so the sum of squares equals RRSS-URSS)
     */
-    hmod = lsq(HT_list, *pZ, pdinfo, OLS, OPT_A);
+    hmod = lsq(HT_list, dset, OLS, OPT_A);
 
 #if HTDBG
-    strcpy(pdinfo->varname[pdinfo->v - 1], "Ufitvals");
+    strcpy(dset->varname[dset->v - 1], "Ufitvals");
     pprintf(dbgprn, "Hausman: aux regression\n", hmod.dfd);
-    printmodel(&hmod, pdinfo, OPT_NONE, dbgprn);
+    printmodel(&hmod, dset, OPT_NONE, dbgprn);
     pprintf(dbgprn, "smpl1 = %d, smpl2 = %d\n", nobs1, hmod.nobs);
     pprintf(dbgprn, "k1 = %d, k2 = %d\n", hmod.ncoeff, ku);
 #endif
@@ -707,7 +719,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     gretl_print_destroy(dbgprn);
 #endif
 
-    dataset_drop_last_variables(pdinfo->v - nv, pZ, pdinfo);
+    dataset_drop_last_variables(dset->v - nv, dset);
     clear_model(&hmod);
     free(HT_list);
 
@@ -717,9 +729,9 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 /* form matrix of instruments and perform QR decomposition
    of this matrix; return Q */
 
-static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
-			     const double **Z, const DATAINFO *pdinfo, 
-			     char *mask, gretlopt opt, int *err)
+static gretl_matrix *tsls_Q (int *instlist, int **pdlist,
+			     const DATASET *dset, char *mask, 
+			     int *err)
 {
     gretl_matrix *Q = NULL;
     gretl_matrix *R = NULL;
@@ -728,7 +740,8 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
     double test;
     int i, j, k;
 
-    Q = gretl_matrix_data_subset_masked(instlist, Z, pdinfo->t1, pdinfo->t2, 
+    Q = gretl_matrix_data_subset_masked(instlist, dset, 
+					dset->t1, dset->t2, 
 					mask, err);
     if (*err) {
 	return NULL;
@@ -787,7 +800,7 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 		    droplist[droplist[0]] = instlist[j];
 		}	    
 		fprintf(stderr, "tsls_Q: dropping redundant instrument %d (%s)\n", 
-			instlist[j], pdinfo->varname[instlist[j]]);
+			instlist[j], dset->varname[instlist[j]]);
 		gretl_list_delete_at_pos(instlist, j--);
 	    }
 	    j++;
@@ -795,7 +808,8 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 
 	k = instlist[0];
 	gretl_matrix_free(Q);
-	Q = gretl_matrix_data_subset_masked(instlist, Z, pdinfo->t1, pdinfo->t2, 
+	Q = gretl_matrix_data_subset_masked(instlist, dset, 
+					    dset->t1, dset->t2, 
 					    mask, err);
 	if (!*err) {
 	    R = gretl_matrix_reuse(R, k, k);
@@ -819,7 +833,7 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 }
 
 static int tsls_form_xhat (gretl_matrix *Q, double *x, double *xhat,
-			   DATAINFO *pdinfo, const char *mask)
+			   DATASET *dset, const char *mask)
 {
     double *r;
     int k = gretl_matrix_cols(Q);
@@ -830,17 +844,17 @@ static int tsls_form_xhat (gretl_matrix *Q, double *x, double *xhat,
 	return E_ALLOC;
     }
 
-#if TDEBUG
+#if TDEBUG > 1
     fprintf(stderr, "tsls_form_xhat: t1=%d, t2=%d, mask=%p\n",
-	    pdinfo->t1, pdinfo->t2, (void *) mask);
+	    dset->t1, dset->t2, (void *) mask);
 #endif
 
     /* form r = Q'y */
     for (i=0; i<k; i++) {
 	r[i] = 0.0;
 	s = 0;
-	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    if (mask != NULL && mask[t - pdinfo->t1]) {
+	for (t=dset->t1; t<=dset->t2; t++) {
+	    if (mask != NULL && mask[t - dset->t1]) {
 		continue;
 	    }
 	    r[i] += gretl_matrix_get(Q, s++, i) * x[t];
@@ -849,8 +863,8 @@ static int tsls_form_xhat (gretl_matrix *Q, double *x, double *xhat,
 
     /* form Qr = QQ'y */
     s = 0;
-    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	if (mask != NULL && mask[t - pdinfo->t1]) {
+    for (t=dset->t1; t<=dset->t2; t++) {
+	if (mask != NULL && mask[t - dset->t1]) {
 	    xhat[t] = NADBL;
 	    continue;
 	}
@@ -867,8 +881,8 @@ static int tsls_form_xhat (gretl_matrix *Q, double *x, double *xhat,
 }
 
 static void tsls_residuals (MODEL *pmod, const int *reglist, 
-			    const double **Z, const DATAINFO *pdinfo,
-			    int panel_N, gretlopt opt)
+			    const DATASET *dset,
+			    gretlopt opt)
 {
     int yno = reglist[1];
     double yh, sigma0 = pmod->sigma;
@@ -882,10 +896,10 @@ static void tsls_residuals (MODEL *pmod, const int *reglist,
 	}
 	yh = 0.0;
 	for (i=0; i<pmod->ncoeff; i++) {
-	    yh += pmod->coeff[i] * Z[reglist[i+2]][t];
+	    yh += pmod->coeff[i] * dset->Z[reglist[i+2]][t];
 	}
 	pmod->yhat[t] = yh; 
-	pmod->uhat[t] = Z[yno][t] - yh;
+	pmod->uhat[t] = dset->Z[yno][t] - yh;
 	pmod->ess += pmod->uhat[t] * pmod->uhat[t];
     }
 
@@ -909,13 +923,12 @@ static void tsls_residuals (MODEL *pmod, const int *reglist,
 #define TSLS_CORR_RSQ 1
 
 static void tsls_extra_stats (MODEL *pmod, int yno, int overid, 
-			      const double **Z,
-			      const DATAINFO *pdinfo)
+			      const DATASET *dset)
 {
     double r;
 
 #if TSLS_CORR_RSQ
-    pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, Z[yno], pmod->yhat);
+    pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, dset->Z[yno], pmod->yhat);
 #else
     pmod->rsq = 1 - pmod->ess / pmod->tss;
 #endif
@@ -940,10 +953,10 @@ static void tsls_extra_stats (MODEL *pmod, int yno, int overid,
     pmod->criterion[C_BIC] = NADBL;
     pmod->criterion[C_HQC] = NADBL;
     
-    if (dataset_is_time_series(pdinfo) && !model_has_missing_obs(pmod)) {
+    if (dataset_is_time_series(dset) && !model_has_missing_obs(pmod)) {
 	/* time series, no missing obs within sample range */
 	pmod->rho = rhohat(1, pmod->t1, pmod->t2, pmod->uhat);
-	pmod->dw = dwstat(1, pmod, Z);
+	pmod->dw = dwstat(1, pmod, (const double **) dset->Z);
     } else {
 	pmod->rho = pmod->dw = NADBL;
     }
@@ -1008,8 +1021,9 @@ reglist_remove_redundant_vars (const MODEL *tmod, int *s2list, int *reglist)
 static int 
 compute_stock_yogo (MODEL *pmod, const int *endolist, 
 		    const int *instlist, const int *hatlist,
-		    const double **dZ)
+		    const DATASET *dset)
 {
+    const double **dZ = (const double **) dset->Z;
     gretl_matrix_block *B = NULL;
     gretl_matrix_block *B2 = NULL;
     gretl_matrix *G, *S, *Y;
@@ -1153,14 +1167,22 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
 	if (!err) {
 	    gretl_matrix_xtr_symmetric(S);
 	    gretl_matrix_divide_by_scalar(S, T - K1 - K2);
+	    for (i=0; i<S->rows; i++) {
+		if (gretl_matrix_get(S, i, i) <= 0) {
+		    err = E_SINGULAR;
+		    break;
+		}
+	    }
 	}
     }
 
     if (!err) {
 	double rc = gretl_symmetric_matrix_rcond(S, &err);
 
-	if (!err && (na(rc) || rc < 1.0e-6)) {
+	if (!err && (na(rc) || rc < 1.0e-7)) { /* note: was 1.0e-6 */
+#if 0
 	    fprintf(stderr, "Stock-Yogo: rcond(S) = %g\n", rc);
+#endif
 	    err = E_SINGULAR;
 	}
     }
@@ -1170,7 +1192,7 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
 	err = gretl_invert_symmetric_matrix(S);
 	if (!err) {
 	    err = gretl_matrix_cholesky_decomp(S);
-	}
+	} 
     }
 
     if (!err) {
@@ -1205,9 +1227,11 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
     gretl_matrix_block_destroy(B);
     gretl_matrix_block_destroy(B2);
 
+#if 0
     if (err) {
 	fprintf(stderr, "compute_stock_yogo: err = %d\n", err);
     }
+#endif
 
     return err;
 }
@@ -1220,8 +1244,8 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
 
 static int 
 compute_first_stage_F (MODEL *pmod, int v, const int *reglist, 
-		       const int *instlist, double ***pZ, 
-		       DATAINFO *pdinfo, gretlopt opt)
+		       const int *instlist, DATASET *dset, 
+		       gretlopt opt)
 {
     MODEL mod1;
     int *list1 = NULL;
@@ -1261,8 +1285,11 @@ compute_first_stage_F (MODEL *pmod, int v, const int *reglist,
     if (!err) {
 	gretlopt myopt = (opt & OPT_R)? (OPT_A | OPT_R) : OPT_A;
 
-	mod1 = lsq(list1, *pZ, pdinfo, OLS, myopt);
+	mod1 = lsq(list1, dset, OLS, myopt);
 	err = mod1.errcode;
+	if (err) {
+	    fprintf(stderr, "compute_first_stage F: lsq failed\n");
+	}
     }
 
     if (!err) {
@@ -1289,12 +1316,10 @@ compute_first_stage_F (MODEL *pmod, int v, const int *reglist,
     return err;
 }
 
-static int tsls_adjust_sample (const int *list, 
-			       const double **Z, DATAINFO *pdinfo,
-			       gretlopt opt, char **pmask,
-			       int *panel_N)
+static int tsls_adjust_sample (const int *list, DATASET *dset,
+			       char **pmask)
 {
-    int i, t, t1min = pdinfo->t1, t2max = pdinfo->t2;
+    int i, t, t1min = dset->t1, t2max = dset->t2;
     char *mask = NULL;
     int T, vi, missobs;
     int err = 0;
@@ -1307,7 +1332,7 @@ static int tsls_adjust_sample (const int *list,
 	    if (vi == 0 || vi == LISTSEP) {
 		continue;
 	    }
-	    if (na(Z[vi][t])) {
+	    if (na(dset->Z[vi][t])) {
 		missobs = 1;
 		break;
 	    }
@@ -1327,7 +1352,7 @@ static int tsls_adjust_sample (const int *list,
 	    if (vi == 0 || vi == LISTSEP) {
 		continue;
 	    }
-	    if (na(Z[vi][t])) {
+	    if (na(dset->Z[vi][t])) {
 		missobs = 1;
 		break;
 	    }
@@ -1347,7 +1372,7 @@ static int tsls_adjust_sample (const int *list,
 	    if (vi == 0 || vi == LISTSEP) {
 		continue;
 	    }
-	    if (na(Z[vi][t])) {
+	    if (na(dset->Z[vi][t])) {
 		missobs++;
 		break;
 	    }
@@ -1369,7 +1394,7 @@ static int tsls_adjust_sample (const int *list,
 		    if (vi == 0 || vi == LISTSEP) {
 			continue;
 		    }
-		    if (na(Z[vi][t])) {
+		    if (na(dset->Z[vi][t])) {
 			mask[t - t1min] = 1;
 			break;
 		    }
@@ -1384,8 +1409,8 @@ static int tsls_adjust_sample (const int *list,
 #endif
 
     if (!err) {
-	pdinfo->t1 = t1min; 
-	pdinfo->t2 = t2max;
+	dset->t1 = t1min; 
+	dset->t2 = t2max;
     }
 
     *pmask = mask;
@@ -1455,12 +1480,12 @@ int ivreg_process_lists (const int *list, int **reglist, int **instlist)
 /**
  * tsls:
  * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
+ * @dset: dataset struct.
  * @opt: may contain %OPT_R for robust VCV, %OPT_N for no df correction, 
- * %OPT_A if this is an auxiliary reression, %OPT_E is we're
- * estimating one equation within a system; and %OPT_H to
- * add "hatlist" to model even under %OPT_E.
+ * %OPT_A if this is an auxiliary reression, %OPT_E if we're
+ * estimating one equation within a system, %OPT_H to
+ * add "hatlist" to model even under %OPT_E, %OPT_X to skip
+ * the set of tests that are usually calculated.
  *
  * Estimate the model given in @list by means of Two-Stage Least
  * Squares.  If %OPT_E is given, fitted values from the first-stage
@@ -1469,22 +1494,21 @@ int ivreg_process_lists (const int *list, int **reglist, int **instlist)
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
-	    gretlopt opt)
+MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
 {
     MODEL tsls;
     gretl_matrix *Q = NULL;
     char *missmask = NULL;
-    int panel_N = 0;
-    int orig_t1 = pdinfo->t1, orig_t2 = pdinfo->t2;
+    int orig_t1 = dset->t1, orig_t2 = dset->t2;
     int *reglist = NULL, *instlist = NULL;
     int *hatlist = NULL, *s2list = NULL;
     int *exolist = NULL, *endolist = NULL;
     int *droplist = NULL;
     int addconst = 0, addvars = 0;
     int ninst = 0, nreg = 0;
-    int orig_nvar = pdinfo->v;
+    int orig_nvar = dset->v;
     int sysest = (opt & OPT_E);
+    int no_tests = (opt & OPT_X);
     int nendo, ev = 0;
     int OverIdRank = 0;
     int i, err = 0;
@@ -1507,8 +1531,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* adjust sample range for missing observations */
-    err = tsls_adjust_sample(list, (const double **) *pZ, pdinfo, 
-			     opt, &missmask, &panel_N);
+    err = tsls_adjust_sample(list, dset, &missmask);
 
     if (!err) {
 	/* allocate second stage regression list */
@@ -1530,9 +1553,9 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     /* drop any vars that are all zero, and reshuffle the constant
        into first position among the independent vars 
     */
-    tsls_omitzero(reglist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
-    reglist_check_for_const(reglist, (const double **) *pZ, pdinfo);
-    tsls_omitzero(instlist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
+    tsls_omitzero(reglist, dset);
+    reglist_check_for_const(reglist, dset);
+    tsls_omitzero(instlist, dset);
 
     /* Determine the list of variables (endolist) for which we need to
        obtain fitted values in the first stage.  Note that we accept
@@ -1557,9 +1580,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	Q = tsls_Q(instlist, reglist, &droplist,
-		   (const double **) *pZ, pdinfo,
-		   missmask, opt, &err);
+	Q = tsls_Q(instlist, &droplist, dset, missmask, &err);
     }
 
     if (err) {
@@ -1582,7 +1603,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     /* allocate storage for fitted vars (etc.), if needed */
     addvars = nendo;
     if (addvars > 0) {
-	err = dataset_add_series(addvars, pZ, pdinfo);
+	err = dataset_add_series(addvars, dset);
 	if (err) {
 	    goto bailout;
 	}
@@ -1605,15 +1626,15 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 	int v1 = orig_nvar + i;
 
 	/* form xhat_i = QQ'x_i */
-	err = tsls_form_xhat(Q, (*pZ)[v0], (*pZ)[v1], pdinfo, 
+	err = tsls_form_xhat(Q, dset->Z[v0], dset->Z[v1], dset, 
 			     missmask);
 	if (err) {
 	    goto bailout;
 	}
 
 	/* name the fitted series according to the original */
-	strcpy(pdinfo->varname[v1], "h_");
-	strncat(pdinfo->varname[v1], pdinfo->varname[v0], VNAMELEN - 3);
+	strcpy(dset->varname[v1], "h_");
+	strncat(dset->varname[v1], dset->varname[v0], VNAMELEN - 3);
 
 	/* substitute v1 into the right place in the second-stage
 	   regression list */
@@ -1624,20 +1645,24 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* second-stage regression */
-    tsls = lsq(s2list, *pZ, pdinfo, OLS, (sysest)? OPT_Z : OPT_NONE);
+    tsls = lsq(s2list, dset, OLS, (sysest)? OPT_Z : OPT_NONE);
     if (tsls.errcode) {
 	fprintf(stderr, "tsls, stage 2: tsls.errcode = %d\n", tsls.errcode);
 	goto bailout;
     }
 
-    if (!sysest || (opt & OPT_H)) {
-	if (nendo == 1) {
-	    /* handles robust estimation, for single endogenous regressor */
-	    compute_first_stage_F(&tsls, ev, reglist, instlist, pZ, pdinfo, opt);
-	} else if (!(opt & OPT_R) && nendo > 0) {
-	    /* at present, only handles case of i.i.d. errors */
-	    compute_stock_yogo(&tsls, endolist, instlist, hatlist,
-			       (const double **) *pZ);
+    /* record the number of instruments used */
+    gretl_model_set_int(&tsls, "ninst", ninst);
+
+    if (!no_tests) {
+	if (!sysest || (opt & OPT_H)) {
+	    if (nendo == 1) {
+		/* handles robust estimation, for single endogenous regressor */
+		compute_first_stage_F(&tsls, ev, reglist, instlist, dset, opt);
+	    } else if (!(opt & OPT_R) && nendo > 0) {
+		/* at present, only handles case of i.i.d. errors */
+		compute_stock_yogo(&tsls, endolist, instlist, hatlist, dset);
+	    }
 	}
     } 
 
@@ -1645,18 +1670,20 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 	/* Were collinear regressors dropped? If so, adjustments are needed */
 	OverIdRank += s2list[0] - tsls.list[0];
 	reglist_remove_redundant_vars(&tsls, s2list, reglist);
+#if TDEBUG
+	fprintf(stderr, "tsls: dropped collinear vars\n");
+#endif
     }
 
     if (nendo > 0) {
 	/* special: we need to use the original RHS vars to compute
 	   residuals and associated statistics */
-	tsls_residuals(&tsls, reglist, (const double **) *pZ, 
-		       pdinfo, panel_N, opt);
+	tsls_residuals(&tsls, reglist, dset, opt);
     }
 
     if (opt & OPT_R) {
 	/* robust standard errors called for */
-	qr_tsls_vcv(&tsls, (const double **) *pZ, pdinfo, opt);
+	qr_tsls_vcv(&tsls, dset, opt);
 	if (tsls.errcode) {
 	    goto bailout;
 	}	
@@ -1664,18 +1691,17 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     if (nendo > 0) {
 	/* compute additional statistics (R^2, F, etc.) */
-	tsls_extra_stats(&tsls, reglist[1], OverIdRank, 
-			 (const double **) *pZ, pdinfo);
+	tsls_extra_stats(&tsls, reglist[1], OverIdRank, dset);
     }
 
-    if (!sysest && nendo > 0) {
+    if (!sysest && !no_tests && nendo > 0) {
 	if (hatlist != NULL) {
-	    tsls_hausman_test(&tsls, reglist, hatlist, pZ, pdinfo);
+	    tsls_hausman_test(&tsls, reglist, hatlist, dset);
 	}
 	if (OverIdRank > 0) {
-	    ivreg_sargan_test(&tsls, OverIdRank, instlist, pZ, pdinfo);
+	    ivreg_sargan_test(&tsls, OverIdRank, instlist, dset);
 	} else {
-	    tsls_loglik(&tsls, nendo, reglist, instlist, *pZ, pdinfo);
+	    tsls_loglik(&tsls, nendo, reglist, instlist, dset);
 	}
     }
 
@@ -1710,7 +1736,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     if (!tsls.errcode && endolist != NULL) {
 	if (sysest) {
 	    /* save first-stage fitted values */
-	    tsls_save_data(&tsls, hatlist, exolist, *pZ, pdinfo);
+	    tsls_save_data(&tsls, hatlist, exolist, dset);
 	}
 	if (!sysest || (opt & OPT_H)) {
 	    /* save list of endogenous regressors on model */
@@ -1720,7 +1746,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     } 
 
     /* delete any first-stage fitted values from dataset */
-    dataset_drop_last_variables(pdinfo->v - orig_nvar, pZ, pdinfo);
+    dataset_drop_last_variables(dset->v - orig_nvar, dset);
 
     free(reglist); 
     free(instlist);
@@ -1734,8 +1760,8 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* restore original sample range */
-    pdinfo->t1 = orig_t1;
-    pdinfo->t2 = orig_t2;
+    dset->t1 = orig_t1;
+    dset->t2 = orig_t2;
 
     return tsls;
 }
