@@ -104,7 +104,7 @@ static user_matrix *user_matrix_new (gretl_matrix *M, const char *name)
     return u;
 }
 
-static int matrix_is_user_matrix (const gretl_matrix *m)
+static int matrix_is_saved (const gretl_matrix *m)
 {
     int i;
 
@@ -114,6 +114,10 @@ static int matrix_is_user_matrix (const gretl_matrix *m)
 	}
     }
 
+    if (data_is_bundled((void *) m)) {
+	return 1;
+    }
+
     return 0;
 }
 
@@ -155,7 +159,7 @@ static user_matrix *real_user_matrix_add (gretl_matrix *M, const char *name,
 	matrices = tmp;
     }
 
-    if (matrix_is_user_matrix(M)) {
+    if (matrix_is_saved(M)) {
 	/* ensure uniqueness of matrix pointers */
 	gretl_matrix *Mcpy = gretl_matrix_copy(M);
 
@@ -316,6 +320,11 @@ int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M)
     return 0;
 }
 
+int user_matrix_get_level (user_matrix *u)
+{
+    return (u == NULL)? -1 : u->level;
+}
+
 int user_matrix_adjust_level (user_matrix *u, int adj)
 {
     if (u == NULL) {
@@ -622,10 +631,11 @@ static int vec_is_const (const gretl_matrix *m, int n)
     return 1;
 }
 
-/* convert matrix subspec into list of rows or columns */
+/* convert a matrix subspec component into list of rows 
+   or columns */
 
-static int *mspec_to_list (int type, union msel *sel, int n,
-			   int *err)
+static int *mspec_make_list (int type, union msel *sel, int n,
+			     int *err)
 {
     int *slice = NULL;
     int i, ns = 0;
@@ -634,22 +644,7 @@ static int *mspec_to_list (int type, union msel *sel, int n,
 	return NULL;
     }
 
-    if (type == SEL_RANGE) {
-	if (sel->range[1] == MSEL_MAX) {
-	    sel->range[1] = n;
-	}
-	if (msel_out_of_bounds(sel->range, n)) {
-	    *err = E_DATA;
-	} else {
-	    ns = sel->range[1] - sel->range[0] + 1;
-	    if (ns <= 0) {
-		gretl_errmsg_sprintf(_("Range %d to %d is non-positive!"),
-				     sel->range[0], sel->range[1]); 
-		*err = E_DATA;
-	    }
-	}
-    } else {
-	/* SEL_MATRIX */
+    if (type == SEL_MATRIX) {
 	if (sel->m == NULL) {
 	    gretl_errmsg_set(_("Range is non-positive!"));
 	    *err = E_DATA;
@@ -663,7 +658,22 @@ static int *mspec_to_list (int type, union msel *sel, int n,
 		ns = 1;
 	    }
 	}
-    }
+    } else {
+	/* range or element */
+	if (sel->range[1] == MSEL_MAX) {
+	    sel->range[1] = n;
+	}
+	if (msel_out_of_bounds(sel->range, n)) {
+	    *err = E_DATA;
+	} else {
+	    ns = sel->range[1] - sel->range[0] + 1;
+	    if (ns <= 0) {
+		gretl_errmsg_sprintf(_("Range %d to %d is non-positive!"),
+				     sel->range[0], sel->range[1]); 
+		*err = E_DATA;
+	    }
+	}
+    } 
 
     if (*err) {
 	return NULL;
@@ -676,11 +686,11 @@ static int *mspec_to_list (int type, union msel *sel, int n,
     }
 
     for (i=0; i<slice[0]; i++) {
-	if (type == SEL_RANGE) {
-	    slice[i+1] = sel->range[0] + i;
-	} else {
+	if (type == SEL_MATRIX) {
 	    slice[i+1] = sel->m->val[i];
-	}
+	} else {
+	    slice[i+1] = sel->range[0] + i;
+	} 
     }
 
     for (i=1; i<=slice[0] && !*err; i++) {
@@ -699,34 +709,61 @@ static int *mspec_to_list (int type, union msel *sel, int n,
     return slice;
 }
 
-static int get_slices (matrix_subspec *spec, 
-		       int **rslice, int **cslice,
-		       const gretl_matrix *M)
+/* catch the case of an implicit column or row specification for
+   a sub-matrix of an (n x 1) or (1 x m) matrix; also catch the
+   error of giving just one row/col spec for a matrix that has
+   more than one row and more than one column
+*/
+
+int check_matrix_subspec (matrix_subspec *spec, const gretl_matrix *m)
 {
     int err = 0;
 
     if (spec->type[1] == SEL_NULL) {
-	/* we were given, e.g., "M[2]" */
-	if (M->rows == 1) {
-	    /* treat the one selection as column selection */
-	    *rslice = NULL;
-	    *cslice = mspec_to_list(spec->type[0], &spec->sel[0], 
-				    M->cols, &err);
-	    return err;
-	} 
+	/* we got only one row/col spec */
+	if (m->cols == 1) {
+	    /* OK: implicitly col = 1 */
+	    spec->type[1] = SEL_RANGE;
+	    mspec_set_col_index(spec, 1);
+	} else if (m->rows == 1) {
+	    /* OK: implicitly row = 1, and transfer the single 
+	       given spec to the column dimension */
+	    spec->type[1] = spec->type[0];
+	    if (spec->type[1] == SEL_MATRIX) {
+		spec->sel[1].m = spec->sel[0].m;
+	    } else {
+		spec->sel[1].range[0] = spec->sel[0].range[0];
+		spec->sel[1].range[1] = spec->sel[0].range[1];
+	    }		
+	    spec->type[0] = SEL_RANGE;
+	    mspec_set_row_index(spec, 1);
+	} else {
+	    gretl_errmsg_set(_("Ambiguous matrix index"));
+	    err = E_DATA;
+	}	    
     }
 
-    *rslice = mspec_to_list(spec->type[0], &spec->sel[0], 
-			    M->rows, &err);
-    if (err) {
-	return err;
+    if (spec->type[0] == SEL_RANGE && spec->type[1] == SEL_RANGE) {
+	if (spec->sel[0].range[0] == spec->sel[0].range[1] &&
+	    spec->sel[1].range[0] == spec->sel[1].range[1]) {
+	    spec->type[0] = spec->type[1] = SEL_ELEMENT;
+	}
     }
 
-    *cslice = mspec_to_list(spec->type[1], &spec->sel[1], 
-			    M->cols, &err);
-    if (err) {
-	free(*rslice);
-	*rslice = NULL;
+    return err;
+}
+
+static int get_slices (matrix_subspec *spec, 
+		       const gretl_matrix *M)
+{
+    int err = 0;
+
+    spec->rslice = mspec_make_list(spec->type[0], &spec->sel[0], 
+				   M->rows, &err);
+
+    if (!err) {
+	spec->cslice = mspec_make_list(spec->type[1], &spec->sel[1], 
+				       M->cols, &err);
     }
 
     return err;
@@ -745,8 +782,6 @@ static int matrix_replace_submatrix (gretl_matrix *M,
     int mc = gretl_matrix_cols(M);
     int sr = gretl_matrix_rows(S);
     int sc = gretl_matrix_cols(S);
-    int *rslice = NULL;
-    int *cslice = NULL;
     int sscalar = 0;
     int err = 0;
 
@@ -766,30 +801,32 @@ static int matrix_replace_submatrix (gretl_matrix *M,
 	return matrix_insert_diagonal(M, S, mr, mc);
     }
 
-    /* parse mspec into lists of affected rows and columns */
-    err = get_slices(spec, &rslice, &cslice, M);
-    if (err) {
-	return err;
+    if (spec->rslice == NULL && spec->cslice == NULL) {
+	/* parse mspec into lists of affected rows and columns */
+	err = get_slices(spec, M);
+	if (err) {
+	    return err;
+	}
     }
 
 #if MDEBUG
-    printlist(rslice, "rslice (rows list)");
-    printlist(cslice, "cslice (cols list)");
+    printlist(spec->rslice, "rslice (rows list)");
+    printlist(spec->cslice, "cslice (cols list)");
     fprintf(stderr, "orig M = %d x %d, S = %d x %d\n", mr, mc, sr, sc);
 #endif
 
     if (sr == 1 && sc == 1) {
 	/* selection matrix is a scalar */
 	sscalar = 1;
-	sr = (rslice == NULL)? mr : rslice[0];
-	sc = (cslice == NULL)? mc : cslice[0];
-    } else if (rslice != NULL && rslice[0] != sr) {
+	sr = (spec->rslice == NULL)? mr : spec->rslice[0];
+	sc = (spec->cslice == NULL)? mc : spec->cslice[0];
+    } else if (spec->rslice != NULL && spec->rslice[0] != sr) {
 	fprintf(stderr, "mspec has %d rows but substitute matrix has %d\n", 
-		rslice[0], sr);
+		spec->rslice[0], sr);
 	err = E_NONCONF;
-    } else if (cslice != NULL && cslice[0] != sc) {
+    } else if (spec->cslice != NULL && spec->cslice[0] != sc) {
 	fprintf(stderr, "mspec has %d cols but substitute matrix has %d\n", 
-		cslice[0], sc);
+		spec->cslice[0], sc);
 	err = E_NONCONF;
     }
 
@@ -801,10 +838,10 @@ static int matrix_replace_submatrix (gretl_matrix *M,
 	x = (sscalar)? S->val[0] : 0.0;
 
 	for (i=0; i<sr; i++) {
-	    mi = (rslice == NULL)? k++ : rslice[i+1] - 1;
+	    mi = (spec->rslice == NULL)? k++ : spec->rslice[i+1] - 1;
 	    l = 0;
 	    for (j=0; j<sc; j++) {
-		mj = (cslice == NULL)? l++ : cslice[j+1] - 1;
+		mj = (spec->cslice == NULL)? l++ : spec->cslice[j+1] - 1;
 		if (!sscalar) {
 		    x = gretl_matrix_get(S, i, j);
 		}
@@ -813,19 +850,15 @@ static int matrix_replace_submatrix (gretl_matrix *M,
 	}
     }
 
-    free(rslice);
-    free(cslice);
-
     return err;
 }
 
 gretl_matrix *matrix_get_submatrix (const gretl_matrix *M, 
 				    matrix_subspec *spec,
+				    int prechecked, 
 				    int *err)
 {
     gretl_matrix *S;
-    int *rslice = NULL;
-    int *cslice = NULL;
     int r, c;
 
     if (gretl_is_null_matrix(M)) {
@@ -833,23 +866,38 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
 	return NULL;
     }
 
-    if (spec->type[0] == SEL_DIAG) {
-	return gretl_matrix_get_diagonal(M, err);
+    if (!prechecked) {
+	*err = check_matrix_subspec(spec, M);
+	if (*err) {
+	    return NULL;
+	}
     }
 
-    *err = get_slices(spec, &rslice, &cslice, M);
-    if (*err) {
-	return NULL;
+    if (spec->type[0] == SEL_DIAG) {
+	return gretl_matrix_get_diagonal(M, err);
+    } else if (spec->type[0] == SEL_ELEMENT) {
+	int i = mspec_get_row_index(spec);
+	int j = mspec_get_col_index(spec);
+	double x = matrix_get_element(M, i, j, err);
+
+	return (*err)? NULL : gretl_matrix_from_scalar(x);
+    }
+
+    if (spec->rslice == NULL && spec->cslice == NULL) {
+	*err = get_slices(spec, M);
+	if (*err) {
+	    return NULL;
+	}
     }
 
 #if MDEBUG
-    printlist(rslice, "rslice");
-    printlist(cslice, "cslice");
+    printlist(spec->rslice, "rslice");
+    printlist(spec->cslice, "cslice");
     fprintf(stderr, "M = %d x %d\n", M->rows, M->cols);
 #endif
 
-    r = (rslice == NULL)? M->rows : rslice[0];
-    c = (cslice == NULL)? M->cols : cslice[0];
+    r = (spec->rslice == NULL)? M->rows : spec->rslice[0];
+    c = (spec->cslice == NULL)? M->cols : spec->cslice[0];
 
     S = gretl_matrix_alloc(r, c);
     if (S == NULL) {
@@ -863,10 +911,10 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
 
 	k = 0;
 	for (i=0; i<r && !*err; i++) {
-	    mi = (rslice == NULL)? k++ : rslice[i+1] - 1;
+	    mi = (spec->rslice == NULL)? k++ : spec->rslice[i+1] - 1;
 	    l = 0;
 	    for (j=0; j<c && !*err; j++) {
-		mj = (cslice == NULL)? l++ : cslice[j+1] - 1;
+		mj = (spec->cslice == NULL)? l++ : spec->cslice[j+1] - 1;
 		x = gretl_matrix_get(M, mi, mj);
 		gretl_matrix_set(S, i, j, x);
 	    }
@@ -880,12 +928,33 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
 	}
     }
 
-    free(rslice);
-    free(cslice);
-
     return S;
 }
 
+double matrix_get_element (const gretl_matrix *M, int i, int j,
+			   int *err)
+{
+    double x = NADBL;
+
+    /* The incoming i and j are from userspace, and will
+       be 1-based.
+    */
+    i--;
+    j--;
+
+    if (M == NULL) {
+	*err = E_DATA;
+    } else if (i < 0 || i >= M->rows || j < 0 || j >= M->cols) {
+	gretl_errmsg_sprintf(_("Index value %d is out of bounds"), 
+			     (i < 0 || i >= M->rows)? (j+1) : (i+1));
+	*err = 1;
+    } else {
+	x = gretl_matrix_get(M, i, j);
+    }
+
+    return x;
+}
+
 gretl_matrix *user_matrix_get_submatrix (const char *name, 
 					 matrix_subspec *spec,
 					 int *err)
@@ -897,7 +966,7 @@ gretl_matrix *user_matrix_get_submatrix (const char *name,
     if (M == NULL) {
 	*err = E_UNKVAR;
     } else {
-	S = matrix_get_submatrix(M, spec, err);
+	S = matrix_get_submatrix(M, spec, 0, err);
     }
 
     return S;
@@ -1189,7 +1258,7 @@ int umatrix_set_names_from_string (const gretl_matrix *M,
 
 int umatrix_set_names_from_list (const gretl_matrix *M, 
 				 const int *list,
-				 const DATAINFO *pdinfo,
+				 const DATASET *dset,
 				 int byrow)
 {
     user_matrix *u = get_user_matrix_by_data(M);
@@ -1213,7 +1282,7 @@ int umatrix_set_names_from_list (const gretl_matrix *M,
 	}
 
 	for (i=0; i<n && !err; i++) {
-	    S[i] = gretl_strndup(pdinfo->varname[list[i+1]], 12);
+	    S[i] = gretl_strndup(dset->varname[list[i+1]], 12);
 	    if (S[i] == NULL) {
 		err = E_ALLOC;
 	    }
@@ -1278,10 +1347,10 @@ user_matrix_get_determinant (gretl_matrix *m, int f, int *err)
 
     if (gretl_is_null_matrix(m)) {
 	return d;
-    } else if (!matrix_is_user_matrix(m)) {
-	tmp = m;
-    } else {
+    } else if (matrix_is_saved(m)) {
 	tmp = gretl_matrix_copy(m);
+    } else {
+	tmp = m;
     }
 
     if (tmp != NULL) {
@@ -1305,14 +1374,16 @@ gretl_matrix *user_matrix_matrix_func (gretl_matrix *m, int f,
 
     if (gretl_is_null_matrix(m)) {
 	*err = E_DATA;
-    } else if (!matrix_is_user_matrix(m)) {
-	R = m;
-    } else {
+    } else if (matrix_is_saved(m)) {
+	/* don't mess with the original matrix! */
 	R = gretl_matrix_copy(m);
 	if (R == NULL) {
 	    *err = E_ALLOC;
-	}
-    }
+	}	
+    } else {
+	/* use (overwrite) the input matrix */
+	R = m;
+    } 
 
     if (R != NULL) {
 	if (f == F_CDEMEAN) {
