@@ -23,6 +23,7 @@
 #include "vartest.h"
 #include "matrix_extra.h"
 #include "libset.h"
+#include "qr_estimate.h"
 
 int gretl_VAR_normality_test (const GRETL_VAR *var, PRN *prn)
 {
@@ -38,48 +39,108 @@ int gretl_VAR_normality_test (const GRETL_VAR *var, PRN *prn)
 }
 
 int gretl_VAR_autocorrelation_test (GRETL_VAR *var, int order, 
-				    double ***pZ, DATAINFO *pdinfo,
+				    DATASET *dset, gretlopt opt,
 				    PRN *prn)
 {
     MODEL *pmod;
+    gretl_matrix *tests, *pvals;
     double lb;
-    int i, err = 0;
+    int i, quiet = (opt & OPT_Q);
+    int err = 0;
 
     if (order == 0) {
-	order = pdinfo->pd;
+	order = dset->pd;
+    }
+
+    tests = gretl_column_vector_alloc(var->neqns);
+    pvals = gretl_column_vector_alloc(var->neqns);
+
+    if (tests == NULL || pvals == NULL) {
+	err = E_ALLOC;
     }
 
     for (i=0; i<var->neqns && !err; i++) {
 	pmod = var->models[i];
-	pprintf(prn, "%s %d:\n", _("Equation"), i + 1);
+	if (!quiet) {
+	    pprintf(prn, "%s %d:\n", _("Equation"), i + 1);
+	}
 	if (pmod->list[0] == 1) {
-	    /* VECM */
+	    /* only the dependent variable is recorded */
 	    lb = ljung_box(order, pmod->t1, pmod->t2, pmod->uhat, &err);
 	    if (!err) {
-		pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
-			lb, _("with p-value"), _("Chi-square"), order,
-			lb, chisq_cdf_comp(order, lb));
-		pputc(prn, '\n');
+		tests->val[i] = lb;
+		pvals->val[i] = chisq_cdf_comp(order, lb);
+		if (!(opt & OPT_Q)) {
+		    pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
+			    lb, _("with p-value"), _("Chi-square"), order,
+			    lb, pvals->val[i]);
+		    pputc(prn, '\n');
+		}
 	    }
 	} else {
-	    err = autocorr_test(pmod, order, pZ, pdinfo,
-				OPT_Q | OPT_S, prn);
-	    gretl_model_test_print(var->models[i], 0, prn);
-	    gretl_model_destroy_tests(var->models[i]);
+	    if (quiet) {
+		err = autocorr_test(pmod, order, dset, OPT_Q, prn);
+	    } else {
+		err = autocorr_test(pmod, order, dset, OPT_Q | OPT_S, prn);
+	    }
+	    if (!err) {
+		tests->val[i] = get_last_test_statistic(NULL);
+		pvals->val[i] = get_last_pvalue(NULL);
+		if (!quiet) {
+		    gretl_model_test_print(var->models[i], 0, prn);
+		    gretl_model_destroy_tests(var->models[i]);
+		}
+	    }
 	}
     }
 
+    if (!err) {
+	record_matrix_test_result(tests, pvals);
+    } else {
+	gretl_matrix_free(tests);
+	gretl_matrix_free(pvals);
+    }
+
     return err;
 }
 
 int gretl_VAR_arch_test (GRETL_VAR *var, int order, 
-			 DATAINFO *pdinfo, PRN *prn)
+			 DATASET *dset, gretlopt opt,
+			 PRN *prn)
 {
+    gretl_matrix *tests, *pvals;
     int i, err = 0;
 
+    if (order == 0) {
+	order = dset->pd;
+    }
+
+    tests = gretl_column_vector_alloc(var->neqns);
+    pvals = gretl_column_vector_alloc(var->neqns);
+
+    if (tests == NULL || pvals == NULL) {
+	err = E_ALLOC;
+    } else {
+	pprintf(prn, "%s %d\n\n", _("Test for ARCH of order"), 
+		order);
+    }
+
     for (i=0; i<var->neqns && !err; i++) {
 	pprintf(prn, "%s %d:\n", _("Equation"), i + 1);
-	err = arch_test(var->models[i], order, pdinfo, OPT_NONE, prn);
+	/* add OPT_M for multi-equation output */
+	err = arch_test(var->models[i], order, dset, 
+			opt | OPT_M, prn);
+	if (!err) {
+	    tests->val[i] = get_last_test_statistic(NULL);
+	    pvals->val[i] = get_last_pvalue(NULL);
+	}
+    }
+
+    if (!err) {
+	record_matrix_test_result(tests, pvals);
+    } else {
+	gretl_matrix_free(tests);
+	gretl_matrix_free(pvals);
     }
 
     return err;
@@ -250,8 +311,7 @@ static void gretl_VAR_print_lagsel (gretl_matrix *lltab,
 
 /* apparatus for selecting the optimal lag length for a VAR */
 
-int VAR_do_lagsel (GRETL_VAR *var, const double **Z, 
-		   const DATAINFO *pdinfo, PRN *prn)
+int VAR_do_lagsel (GRETL_VAR *var, const DATASET *dset, PRN *prn)
 {
     gretl_matrix *crittab = NULL;
     gretl_matrix *lltab = NULL;
@@ -293,7 +353,7 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
     for (j=1; j<p && !err; j++) {
 	int jxcols = cols0 + j * n;
 
-	VAR_fill_X(var, j, Z, pdinfo);
+	VAR_fill_X(var, j, dset);
 
 	gretl_matrix_reuse(var->X, T, jxcols);
 	gretl_matrix_reuse(var->B, jxcols, n);
@@ -343,9 +403,11 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
 	gretl_matrix_set(crittab, m, 1, var->BIC);
 	gretl_matrix_set(crittab, m, 2, var->HQC);
 	gretl_VAR_print_lagsel(lltab, crittab, best_row, prn);
+	record_matrix_test_result(crittab, NULL);
+	crittab = NULL;
     }
 
-    bailout:
+ bailout:
 
     gretl_matrix_free(crittab);
     gretl_matrix_free(lltab);
@@ -393,17 +455,33 @@ static int VAR_robust_vcv (GRETL_VAR *var, gretl_matrix *V,
     gretl_matrix *XOX = NULL;
     gretl_matrix *hvec = NULL;
     double xij, xti, xtj, utk, u2, ht;
+    VCVInfo vi = {0};
     int T = var->T;
     int g = var->ncoeff;
     int i, j, t;
     int err = 0;
 
-    XOX = gretl_matrix_alloc(g, g);
+    if (var->robust == VAR_HAC) {
+	gretl_matrix *uk = gretl_column_vector_alloc(T);
+
+	if (uk != NULL) {
+	    for (t=0; t<T; t++) {
+		uk->val[t] = gretl_matrix_get(var->E, t, k);
+	    }
+	    XOX = HAC_XOX(uk, var->X, &vi, &err);
+	    gretl_matrix_free(uk);
+	}
+    } else {
+	XOX = gretl_matrix_alloc(g, g);
+    }
+
     if (XOX == NULL) {
 	return E_ALLOC;
     }
 
-    if (hcv > 1) {
+    if (var->robust == VAR_HAC) {
+	goto finish;
+    } else if (hcv > 1) {
 	hvec = VAR_get_hvec(var->X, var->XTX, &err);
 	if (err) {
 	    gretl_matrix_free(XOX);
@@ -439,10 +517,18 @@ static int VAR_robust_vcv (GRETL_VAR *var, gretl_matrix *V,
 	}
     }
 
+ finish:
+
     gretl_matrix_qform(var->XTX, GRETL_MOD_TRANSPOSE, XOX,
 		       V, GRETL_MOD_NONE);
 
-    gretl_model_set_vcv_info(pmod, VCV_HC, hcv);
+    if (var->robust == VAR_HAC) {
+	gretl_model_set_full_vcv_info(pmod, VCV_HAC, vi.vmin,
+				      vi.order, vi.flags,
+				      vi.bw);
+    } else {
+	gretl_model_set_vcv_info(pmod, VCV_HC, hcv);
+    }
 
     gretl_matrix_free(XOX);
     gretl_matrix_free(hvec);
@@ -599,7 +685,7 @@ static int *build_VAR_list (const GRETL_VAR *var, int *exolist, int *err)
 
 static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
 				     const GRETL_VAR *new,
-				     const DATAINFO *pdinfo,
+				     const DATASET *dset,
 				     PRN *prn)
 {
     int *omitlist;
@@ -629,7 +715,7 @@ static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
     pputs(prn, _("\n  Null hypothesis: the regression parameters are "
 		 "zero for the variables\n\n"));
     for (i=1; i<=omitlist[0]; i++) {
-	pprintf(prn, "    %s\n", pdinfo->varname[omitlist[i]]);	
+	pprintf(prn, "    %s\n", dset->varname[omitlist[i]]);	
     }
 
     pprintf(prn, "\n  %s: %s(%d) = %g, ", _("Test statistic"), 
@@ -645,8 +731,7 @@ static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
  * gretl_VAR_omit_test:
  * @omitvars: list of variables to omit from original model.
  * @orig: pointer to original VAR.
- * @Z: data array.
- * @pdinfo: information on the data set.
+ * @dset: datset struct.
  * @prn: gretl printing struct.
  * @err: location to receive error code.
  *
@@ -659,13 +744,12 @@ static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
  */
 
 GRETL_VAR *gretl_VAR_omit_test (const int *omitvars, const GRETL_VAR *orig, 
-				const double **Z, DATAINFO *pdinfo, 
-				PRN *prn, int *err)
+				DATASET *dset, PRN *prn, int *err)
 {
     GRETL_VAR *var = NULL;
     gretlopt opt = OPT_NONE;
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
+    int smpl_t1 = dset->t1;
+    int smpl_t2 = dset->t2;
     int *tmplist = NULL;
     int *varlist = NULL;
     int c1 = 0;
@@ -687,7 +771,7 @@ GRETL_VAR *gretl_VAR_omit_test (const int *omitvars, const GRETL_VAR *orig,
 #endif
 
     if (orig->ifc) {
-	c1 = !gretl_list_const_pos(omitvars, 1, Z, pdinfo);
+	c1 = !gretl_list_const_pos(omitvars, 1, dset);
     } 
 
     /* create reduced exogenous vars list for test VAR */
@@ -730,19 +814,19 @@ GRETL_VAR *gretl_VAR_omit_test (const int *omitvars, const GRETL_VAR *orig,
 
     /* impose as sample range the estimation range of the 
        original VAR */
-    pdinfo->t1 = orig->t1;
-    pdinfo->t2 = orig->t2;
+    dset->t1 = orig->t1;
+    dset->t2 = orig->t2;
 
-    var = gretl_VAR(orig->order, varlist, Z, pdinfo, opt, prn, err);
+    var = gretl_VAR(orig->order, varlist, dset, opt, prn, err);
 
     /* now, if var is non-NULL, do the actual test(s) */
     if (var != NULL) {
-	*err = gretl_VAR_real_omit_test(orig, var, pdinfo, prn);
+	*err = gretl_VAR_real_omit_test(orig, var, dset, prn);
     }
 
-    /* put back into pdinfo what was there on input */
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
+    /* put back into dset what was there on input */
+    dset->t1 = smpl_t1;
+    dset->t2 = smpl_t2;
 
  bailout:
 
