@@ -250,7 +250,7 @@ static int anova_make_value_vecs (struct anova *v)
 {
     int err = 0;
 
-    v->tvals = gretl_matrix_values(v->tvec, v->n, &err);
+    v->tvals = gretl_matrix_values(v->tvec, v->n, OPT_S, &err);
 
     if (!err && v->tvals->rows < 2) {
 	gretl_errmsg_set("Insufficient observations");
@@ -258,7 +258,7 @@ static int anova_make_value_vecs (struct anova *v)
     }
 
     if (!err && v->bvec != NULL) {
-	v->bvals = gretl_matrix_values(v->bvec, v->n, &err);
+	v->bvals = gretl_matrix_values(v->bvec, v->n, OPT_S, &err);
 	if (!err && v->bvals->rows < 2) {
 	    gretl_errmsg_set("Insufficient observations");
 	    err = E_DATA;
@@ -339,7 +339,7 @@ static void anova_add_F_stat (struct anova *v)
    @opt can contain OPT_Q to suppress printing.
 */
 
-int gretl_anova (const int *list, const double **Z, const DATAINFO *pdinfo, 
+int gretl_anova (const int *list, const DATASET *dset, 
 		 gretlopt opt, PRN *prn)
 {
     struct anova v;
@@ -354,31 +354,31 @@ int gretl_anova (const int *list, const double **Z, const DATAINFO *pdinfo,
 
     anova_init(&v);
 
-    t1 = pdinfo->t1;
-    t2 = pdinfo->t2;
+    t1 = dset->t1;
+    t2 = dset->t2;
 
-    list_adjust_sample(list, &t1, &t2, Z);
+    list_adjust_sample(list, &t1, &t2, dset);
 
     v.n = t2 - t1 + 1;
     if (v.n < 2) {
 	return E_TOOFEW;
     }
 
-    y = Z[list[1]];
-    xt = Z[list[2]];
-    xb = (list[0] == 3)? Z[list[3]] : NULL;
+    y = dset->Z[list[1]];
+    xt = dset->Z[list[2]];
+    xb = (list[0] == 3)? dset->Z[list[3]] : NULL;
 
     /* check that treatment (and block, if present) are discrete */
 
-    if (!var_is_discrete(pdinfo, list[2]) && 
+    if (!var_is_discrete(dset, list[2]) && 
 	!gretl_isdiscrete(t1, t2, xt)) {
-	gretl_errmsg_set("anova: the x variables must be discrete");
+	gretl_errmsg_set(_("anova: the treatment variable must be discrete"));
 	return E_DATA;
     }
 
-    if (xb != NULL && !var_is_discrete(pdinfo, list[3]) && 
+    if (xb != NULL && !var_is_discrete(dset, list[3]) && 
 	!gretl_isdiscrete(t1, t2, xb)) {
-	gretl_errmsg_set("anova: the x variables must be discrete");
+	gretl_errmsg_set(_("anova: the block variable must be discrete"));
 	return E_DATA;
     }
 
@@ -492,8 +492,8 @@ int gretl_anova (const int *list, const double **Z, const DATAINFO *pdinfo,
     anova_add_F_stat(&v);
 	
     if (!(opt & OPT_Q)) {
-	const char *yname = pdinfo->varname[list[1]];
-	const char *tname = pdinfo->varname[list[2]];
+	const char *yname = dset->varname[list[1]];
+	const char *tname = dset->varname[list[2]];
 
 	pputc(prn, '\n');
 	pprintf(prn, _("%s, response = %s, treatment = %s:"), 
