@@ -141,7 +141,7 @@ static bp_container *bp_container_new (const int *list)
 /* Initialization-related functions                                       */
 /* ---------------------------------------------------------------------- */
 
-static int bp_cont_fill_data (bp_container *bp, const double **Z)
+static int bp_cont_fill_data (bp_container *bp, DATASET *dset)
 {
     int nmissing = 0;
     int i, t, T, err = 0;
@@ -168,15 +168,17 @@ static int bp_cont_fill_data (bp_container *bp, const double **Z)
     i = 0;
     for (t=bp->t1; t<=bp->t2; t++) {
 	if (!bp->mask[t - bp->t1]) {
-	    bp->s1[i] = (Z[bp->depvar1][t] == 1.0);
-	    bp->s2[i] = (Z[bp->depvar2][t] == 1.0);
+	    bp->s1[i] = (dset->Z[bp->depvar1][t] == 1.0);
+	    bp->s2[i] = (dset->Z[bp->depvar2][t] == 1.0);
 	    i++;
 	}
     }	    
 
-    bp->reg1 = gretl_matrix_data_subset_masked(bp->X1list, Z, bp->t1, bp->t2, 
+    bp->reg1 = gretl_matrix_data_subset_masked(bp->X1list, dset,
+					       bp->t1, bp->t2, 
 					       bp->mask, &err);
-    bp->reg2 = gretl_matrix_data_subset_masked(bp->X2list, Z, bp->t1, bp->t2, 
+    bp->reg2 = gretl_matrix_data_subset_masked(bp->X2list, dset,
+					       bp->t1, bp->t2, 
 					       bp->mask, &err);
 
     return err;
@@ -185,12 +187,11 @@ static int bp_cont_fill_data (bp_container *bp, const double **Z)
 /*
   Records into the container a few items:
 
-  * sample beginning and end
   * the lists
   * the number of parameters
 */
 
-static int bp_base_info (bp_container *bp, const double **Z, DATAINFO *pdinfo)
+static int bp_base_info (bp_container *bp, DATASET *dset)
 {
     int nelem = bp->list[0];
     int i, seppos;
@@ -233,34 +234,6 @@ static int bp_base_info (bp_container *bp, const double **Z, DATAINFO *pdinfo)
     return err;
 }
 
-static double match_residuals (int T, double *u1, double *u2, char *mask)
-{
-    double v1 = 0, v2 = 0, cv = 0;
-    double u1t, u2t, rho;
-    int t;
-
-    for (t=0; t<T; t++) {
-	u1t = u1[t];
-	u2t = u2[t];
-	mask[t] = (na(u1t) || na(u2t));
-	if (!mask[t]) {
-	    v1 += u1t * u1t; 
-	    v2 += u2t * u2t; 
-	    cv += u1t * u2t; 
-	}
-    }
-
-    rho = cv / sqrt(v1 * v2);
-
-    if (rho > 0.99) {
-	rho = 0.99;
-    } else if (rho < -0.99) {
-	rho = -0.99;
-    }
-
-    return rho;
-}
-
 static int bp_make_lists (bp_container *bp, int **plist1, int **plist2)
 {
     int *list1, *list2;
@@ -321,8 +294,7 @@ static int bp_add_coeff_vecs_and_mask (bp_container *bp, int T)
 */
 
 static int biprobit_first_pass (bp_container *bp, MODEL *olsmod, 
-				double **Z, DATAINFO *pdinfo,
-				PRN *prn)
+				DATASET *dset, PRN *prn)
 {
     MODEL probmod;
     double *u2, *u1 = NULL;
@@ -361,14 +333,14 @@ static int biprobit_first_pass (bp_container *bp, MODEL *olsmod,
 
     set_reference_missmask_from_model(olsmod);
 
-    probmod = binary_probit(list1, Z, pdinfo, popt, prn);
+    probmod = binary_probit(list1, dset, popt, prn);
     if (probmod.errcode) {
 	goto bailout;
     }
 
     if (prn != NULL) {
 	probmod.aux = AUX_BIPROB;
-	printmodel(&probmod, pdinfo, OPT_NONE, prn);
+	printmodel(&probmod, dset, OPT_NONE, prn);
     }
 
     bp->ll0 = probmod.lnL;
@@ -386,14 +358,14 @@ static int biprobit_first_pass (bp_container *bp, MODEL *olsmod,
     set_reference_missmask_from_model(&probmod);
 
     clear_model(&probmod);
-    probmod = binary_probit(list2, Z, pdinfo, popt, prn);
+    probmod = binary_probit(list2, dset, popt, prn);
     if (probmod.errcode) {
 	goto bailout;
     }
 
     if (prn != NULL) {
 	probmod.aux = AUX_BIPROB;
-	printmodel(&probmod, pdinfo, OPT_NONE, prn);
+	printmodel(&probmod, dset, OPT_NONE, prn);
     }
 
     bp->ll0 += probmod.lnL;
@@ -404,10 +376,10 @@ static int biprobit_first_pass (bp_container *bp, MODEL *olsmod,
 
     for (i=0; i<T; i++) {
 	u2[i] = probmod.uhat[probmod.t1 + i];
+	bp->mask[i] = (na(u1[i]) || na(u2[i]));
     }
 
-    bp->arho = match_residuals(T, u1, u2, bp->mask);
-    bp->arho = atanh(bp->arho);
+    bp->arho = 0;
     
  bailout:
 
@@ -427,22 +399,23 @@ static int biprobit_first_pass (bp_container *bp, MODEL *olsmod,
 */
 
 static int bp_container_fill (bp_container *bp, MODEL *olsmod, 
-			      double **Z, DATAINFO *pdinfo,
-			      PRN *prn)
+			      DATASET *dset, PRN *prn)
 {
     int err;
 
+    /* sample beginning and end */
     bp->t1 = olsmod->t1;
     bp->t2 = olsmod->t2;
 
-    err = bp_base_info(bp, (const double **) Z, pdinfo);
+    /* lists and number of parameters per equation */
+    err = bp_base_info(bp, dset);
 
     if (!err) {
-	err = biprobit_first_pass(bp, olsmod, Z, pdinfo, prn);
+	err = biprobit_first_pass(bp, olsmod, dset, prn);
     }
 
     if (!err) {
-	err = bp_cont_fill_data(bp, (const double **) Z);
+	err = bp_cont_fill_data(bp, dset);
     }
 
     if (!err) {
@@ -513,7 +486,7 @@ static int *prelim_list (const int *list)
    the two probit models to be estimated subsequently
 */
 
-MODEL bp_preliminary_ols (const int *list, double **Z, DATAINFO *pdinfo) 
+MODEL bp_preliminary_ols (const int *list, DATASET *dset) 
 {
 #if BIPDEBUG
     PRN *prn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
@@ -529,14 +502,14 @@ MODEL bp_preliminary_ols (const int *list, double **Z, DATAINFO *pdinfo)
 	return mod;
     }
 
-    mod = lsq(tmplist, Z, pdinfo, OLS, OPT_A);
+    mod = lsq(tmplist, dset, OLS, OPT_A);
     if (gretl_model_get_data(&mod, "droplist")) {
 	gretl_model_destroy_data_item(&mod, "droplist");
     }
     free(tmplist);
 
 #if BIPDEBUG
-    printmodel(&mod, pdinfo, OPT_NONE, prn);
+    printmodel(&mod, dset, OPT_NONE, prn);
     gretl_print_destroy(prn);
 #endif
 
@@ -557,7 +530,7 @@ static int biprob_prelim (const double *theta, bp_container *bp)
 
     bp->arho = theta[bp->npar - 1];
 
-    if (fabs(bp->arho) > 16) {
+    if (fabs(bp->arho) > 18) {
 	return 1; /* suitable error code? */
     }
 
@@ -655,27 +628,20 @@ static int biprob_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
 	d1 = exp(-0.5*a*a) * normal_cdf(u_ba) / (P * SQRT_2_PI);
 	d2 = exp(-0.5*b*b) * normal_cdf(u_ab) / (P * SQRT_2_PI);
 	da = f / (P*ca*ca);
+
+	d1 = bp->s1[i] ? d1 : -d1;
+	d2 = bp->s2[i] ? d2 : -d2;
 	da = eqt ? da : -da;
 
 	for (j=0; j<bp->k1; j++) {
-	    if (bp->s1[i]) {
-		tmp = gretl_matrix_get(bp->reg1, i, j) * d1;
-	    } else {
-		tmp = -gretl_matrix_get(bp->reg1, i, j) * d1;
-	    }
-	    
+	    tmp = gretl_matrix_get(bp->reg1, i, j) * d1;
 	    gretl_matrix_set(bp->score, i, j, tmp);
 	    tmp += gretl_vector_get(bp->sscore, j);
 	    gretl_vector_set(bp->sscore, j, tmp);
 	}
 	
 	for (j=0; j<bp->k2; j++) {
-	    if (bp->s2[i]) {
-		tmp = gretl_matrix_get(bp->reg2, i, j) * d2;
-	    } else {
-		tmp = -gretl_matrix_get(bp->reg2, i, j) * d2;
-	    }
-
+	    tmp = gretl_matrix_get(bp->reg2, i, j) * d2;
 	    gretl_matrix_set(bp->score, i, bp->k1 + j, tmp);
 	    tmp += gretl_vector_get(bp->sscore, bp->k1 + j);
 	    gretl_vector_set(bp->sscore, bp->k1 + j, tmp);
@@ -696,13 +662,25 @@ static int biprob_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
     return 0;
 }
 
-/* analytical hessian */
+/* 
+   Analytical Hessian for the biprobit model; it is assumed that 
+   the score matrix has already been computed and is safely stored 
+   in the container. This is important because we use a neat trick:
+
+   \frac{\partial^2 ln f(x)}{\partial x \partial x'} =
+   \frac{\partial^2 f(x)}{\partial x \partial x'} - gg'
+
+   where g = \frac{\partial ln f(x)}{\partial x}.
+   In practice, we just compute explicitly the difference between
+   the OPG and the actual Hessain, and then reconstruct H from 
+   there.
+ */
 
 int biprobit_hessian (double *theta, gretl_matrix *H, void *ptr)
 {
     bp_container *bp = (bp_container *) ptr;
     double a, b, P, f, d1, d2, da, tmp, u_ab, u_ba;
-    double ca, sa, ssa, x;
+    double ca, sa, ssa, rho, x;
     double h11 = 0;
     double h12 = 0;
     double h13 = 0;
@@ -723,6 +701,8 @@ int biprobit_hessian (double *theta, gretl_matrix *H, void *ptr)
 	gretl_matrix_zero(bp->sscore);
 	gretl_matrix_block_zero(bp->B);
 
+	/* first, put the OPG matrix into H */
+
 	err = gretl_matrix_multiply_mod(bp->score, GRETL_MOD_TRANSPOSE,
 					bp->score, GRETL_MOD_NONE,
 					H, GRETL_MOD_NONE);
@@ -733,6 +713,15 @@ int biprobit_hessian (double *theta, gretl_matrix *H, void *ptr)
     }
 
     for (t=0; t<bp->nobs; t++) {
+
+	/* Warning: it is important to handle properly the four
+	   cases (00, 01, 10, 11) and the associated sign switches;
+	   this may be very tricky. The code below should be bug-free
+	   from this point of view, but you never know. Kudos to
+	   Claudia Pigini for going through this with incredible 
+	   perseverance.
+	*/
+
 	a = gretl_vector_get(bp->fitted1, t);
 	b = gretl_vector_get(bp->fitted2, t);
 	
@@ -740,35 +729,32 @@ int biprobit_hessian (double *theta, gretl_matrix *H, void *ptr)
 	b = bp->s2[t] ? b : -b;
 	eqt = (bp->s1[t] == bp->s2[t]);
 	ssa = eqt ? sa : -sa;
+	rho = ssa/ca;
+	P = bvnorm_cdf(rho, a, b);
 	
-	P = bvnorm_cdf(ssa/ca, a, b);
-	
-	/* score */
+	/* score (for atan(rho) we use the precomputed one) */
 	
 	u_ba = (ca*b - ssa*a);
 	u_ab = (ca*a - ssa*b);
-	f = ca/M_2PI * exp(-0.5* (a*a + u_ba*u_ba));
 
 	d1 = exp(-0.5*a*a) * normal_cdf(u_ba) / (P * SQRT_2_PI);
-	d2 = exp(-0.5*b*b) * normal_cdf(u_ab) / (P * SQRT_2_PI);
-	da = f / (P*ca*ca);
-	da = eqt ? da : -da;
+	d2 = exp(-0.5*b*b) * normal_cdf(u_ab) / (P * SQRT_2_PI);       
+
+	f = ca/M_2PI * exp(-0.5* (a*a + u_ba*u_ba));
+
+	da = gretl_matrix_get(bp->score,t, k1+k2);
+
+	/* hessian + opg */
 
 	h11 = -(a * d1 + sa*ca*da);
-
-	tmp = f/P;
-	h12 = (eqt ? tmp : -tmp);
-
-	tmp = -da * (ca * u_ab);
-	h13 = (bp->s1[t] ? tmp : -tmp);
-
+	h12 = (eqt ? f/P : -f/P);
+	h13 = (bp->s1[t] ? -da * (ca * u_ab) : da * (ca * u_ab));
 	h22 = -(b*d2 + sa*ca*da);
-
-	tmp = -da * (ca * u_ba);
-	h23 = (bp->s2[t] ? tmp : -tmp);
+	h23 = (bp->s2[t] ? -da * (ca * u_ba) : da * (ca * u_ba));
 
 	tmp = (eqt ? u_ba*u_ab : -u_ba*u_ab);
-	h33 += da * (ca * tmp - sa);
+	tmp = da * (ca * tmp - sa) /ca;
+	h33 += tmp;
 
 	for (i=0; i<bp->k1; i++) {
 	    x = gretl_matrix_get(bp->reg1, t, i);
@@ -814,7 +800,7 @@ int biprobit_hessian (double *theta, gretl_matrix *H, void *ptr)
     gretl_matrix_print(bp->H13, "H13");
     gretl_matrix_print(bp->H22, "H22");
     gretl_matrix_print(bp->H23, "H23");
-    fprintf(stderr, "h33 = %12.6f\n", h33);
+    fprintf(stderr, "ca = %g, sa = %g, h33 = %12.6f\n", ca, sa, h33);
     gretl_matrix_print(H, "OPG");
 #endif
 
@@ -1198,7 +1184,7 @@ static int bp_add_hat_matrices (MODEL *pmod, bp_container *bp,
     return err;
 }
 
-static int biprobit_fill_model (MODEL *pmod, bp_container *bp, DATAINFO *pdinfo,
+static int biprobit_fill_model (MODEL *pmod, bp_container *bp, DATASET *dset,
 				gretlopt opt)
 {
     gretl_matrix *V;
@@ -1221,12 +1207,12 @@ static int biprobit_fill_model (MODEL *pmod, bp_container *bp, DATAINFO *pdinfo,
     }
 
     for (i=0; i<bp->k1; i++) {
-	strcpy(pmod->params[i], pdinfo->varname[bp->X1list[i+1]]);
+	strcpy(pmod->params[i], dset->varname[bp->X1list[i+1]]);
 	pmod->coeff[i] = gretl_vector_get(bp->beta, i);
     }
 	
     for (i=0; i<bp->k2; i++) {
-	strcpy(pmod->params[i+bp->k1], pdinfo->varname[bp->X2list[i+1]]);
+	strcpy(pmod->params[i+bp->k1], dset->varname[bp->X2list[i+1]]);
 	pmod->coeff[i + bp->k1] = gretl_vector_get(bp->gama, i);
     }
 
@@ -1249,7 +1235,7 @@ static int biprobit_fill_model (MODEL *pmod, bp_container *bp, DATAINFO *pdinfo,
     pmod->t2 = bp->t2;
     pmod->nobs = bp->nobs;
 
-    gretl_model_set_coeff_separator(pmod, pdinfo->varname[bp->depvar2], bp->k1);
+    gretl_model_set_coeff_separator(pmod, dset->varname[bp->depvar2], bp->k1);
 
     V = biprobit_vcv(bp, opt, &err);
 
@@ -1284,7 +1270,7 @@ static int biprobit_fill_model (MODEL *pmod, bp_container *bp, DATAINFO *pdinfo,
    The driver function for the plugin.
 */
 
-MODEL biprobit_estimate (const int *list, double **Z, DATAINFO *pdinfo, 
+MODEL biprobit_estimate (const int *list, DATASET *dset, 
 			 gretlopt opt, PRN *prn) 
 {
     PRN *vprn = NULL;
@@ -1308,7 +1294,7 @@ MODEL biprobit_estimate (const int *list, double **Z, DATAINFO *pdinfo,
 	return mod;
     }
 
-    mod = bp_preliminary_ols(list, Z, pdinfo);
+    mod = bp_preliminary_ols(list, dset);
     if (mod.errcode) {
 	return mod;
     } 
@@ -1317,7 +1303,7 @@ MODEL biprobit_estimate (const int *list, double **Z, DATAINFO *pdinfo,
 	vprn = prn;
     }
 
-    mod.errcode = bp_container_fill(bp, &mod, Z, pdinfo, vprn);
+    mod.errcode = bp_container_fill(bp, &mod, dset, vprn);
     if (mod.errcode) {
 	return mod;
     } 
@@ -1325,7 +1311,7 @@ MODEL biprobit_estimate (const int *list, double **Z, DATAINFO *pdinfo,
     mod.errcode = bp_do_maxlik(bp, opt, prn);
 
     if (!mod.errcode) {
-	biprobit_fill_model(&mod, bp, pdinfo, opt);
+	biprobit_fill_model(&mod, bp, dset, opt);
     }
 
     bp_container_destroy(bp);
