@@ -17,7 +17,7 @@
  * 
  */
 
-/* import data from Eviews workfiles */
+/* import data from EViews workfiles */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,11 +29,11 @@
 # include "swap_bytes.h"
 #endif
 
-#define EDEBUG 0
+#define EVDEBUG 0
 
 #define WF1_NA 1e-37
 
-static void bin_error (int *err)
+static void wf1_error (int *err)
 {
     fputs("binary read error\n", stderr);
     *err = 1;
@@ -44,7 +44,7 @@ static int read_int (FILE *fp, int *err)
     int i;
 
     if (fread(&i, sizeof i, 1, fp) != 1) {
-	bin_error(err);
+	wf1_error(err);
     }
 #if WORDS_BIGENDIAN
     reverse_int(i);
@@ -66,10 +66,10 @@ static int read_short (FILE *fp, int *err)
     fread(&(sc.c[0]), 1, 1, fp);
     i = sc.s;
 #else
-    short s;
+    unsigned short s;
 
     if (fread(&s, sizeof s, 1, fp) != 1) {
-	bin_error(err);
+	wf1_error(err);
     } 
     i = s;
 #endif
@@ -77,18 +77,18 @@ static int read_short (FILE *fp, int *err)
     return i;
 }
 
-static long read_long (FILE *fp, int *err)
+static unsigned int read_unsigned (FILE *fp, int *err)
 {
-    long int l;
+    unsigned int u;
 
-    if (fread(&l, sizeof l, 1, fp) != 1) {
-	bin_error(err);
+    if (fread(&u, sizeof u, 1, fp) != 1) {
+	wf1_error(err);
     }
 #if WORDS_BIGENDIAN
-    reverse_int(l);
+    reverse_int(u);
 #endif
 
-    return l;
+    return u;
 }
 
 static double read_double (FILE *fp, int *err)
@@ -96,7 +96,7 @@ static double read_double (FILE *fp, int *err)
     double x;
 
     if (fread(&x, sizeof x, 1, fp) != 1) {
-	bin_error(err);
+	wf1_error(err);
     }
 #if WORDS_BIGENDIAN
     reverse_double(x);
@@ -105,9 +105,76 @@ static double read_double (FILE *fp, int *err)
     return (x == WF1_NA)? NADBL : x;
 }
 
-static int get_data (FILE *fp, long pos, double **Z, int i, int n)
+static int wf1_read_history (FILE *fp, unsigned pos, 
+			     DATASET *dset, int i)
+{
+    char *htxt;
+    unsigned hpos;
+    int len, err = 0;
+
+#if EVDEBUG
+    fseek(fp, pos, SEEK_SET);
+    fprintf(stderr, "first short: %d\n", read_short(fp, &err));
+#endif
+
+    fseek(fp, pos + 2, SEEK_SET);
+    len = read_int(fp, &err);
+    if (err) {
+	return 1;
+    }
+
+#if EVDEBUG
+    fprintf(stderr, "history length: %d\n", len);
+    fprintf(stderr, "next int: %d\n", read_int(fp, &err));
+#endif
+
+    fseek(fp, pos + 10, SEEK_SET);
+    hpos = read_unsigned(fp, &err);
+    if (err) {
+	return 1;
+    }  
+
+    htxt = calloc(len + 1, 1);
+    if (htxt != NULL) {
+	fseek(fp, hpos, SEEK_SET);
+	if (fread(htxt, 1, len, fp) == len) {
+	    char *targ = VARLABEL(dset, i);
+
+	    *targ = '\0';
+	    strncat(targ, htxt, MAXLABEL - 1);
+	    fprintf(stderr, "%s\n", htxt);
+	}
+	free(htxt);
+    }
+
+    return 0;
+}
+
+#if EVDEBUG
+
+static void print_bytes (FILE *fp, unsigned pos, int n)
+{
+    unsigned char u;
+    int i;
+
+    fprintf(stderr, "%d bytes at 0x%x: ", n, pos);
+    fseek(fp, pos, SEEK_SET);
+
+    for (i=0; i<n; i++) {
+	fread(&u, sizeof u, 1, fp);
+	fprintf(stderr, " %03d", (int) u);
+    }	
+    fputc('\n', stderr);
+}
+
+#endif
+
+static int wf1_read_values (FILE *fp, int ftype, 
+			    unsigned pos, unsigned sz,
+			    DATASET *dset, int i, int k)
 {
     double x;
+    unsigned xpos;
     int t, nobs = 0;
     int err = 0;
 
@@ -117,141 +184,185 @@ static int get_data (FILE *fp, long pos, double **Z, int i, int n)
 	return 1;
     }
 
-    /* should we be able to handle an offset here? */
-    if (nobs != n) {
+    fprintf(stderr, "got nobs = %d\n", nobs);
+    if (nobs != dset->n) {
 	fputs("problem: this does not match the specification "
 	      " for the dataset\n", stderr);
     }
 
-    fseek(fp, pos + 22, SEEK_SET);
-    for (t=0; t<nobs; t++) {
+    if (sz != nobs * sizeof(double)) {
+	fprintf(stderr, "trouble: data size only %d bytes (at most %d doubles)\n", 
+		(int) sz, (int) sz / sizeof(double));
+#if EVDEBUG
+	/* try soldiering on */
+	nobs = sz / sizeof(double);
+#else
+	/* give up */
+	return 1;
+#endif
+    }    
+
+#if EVDEBUG
+    fprintf(stderr, "following ints:");
+    for (t=0; t<4; t++) {
+	fprintf(stderr, " %d", read_int(fp, &err));
+    }
+    fprintf(stderr, ", %d", read_short(fp, &err));
+    fputc('\n', stderr);
+#endif
+
+    xpos = pos + 22;
+
+    if (nobs > dset->n) {
+	/* don't overflow the Z array */
+	nobs = dset->n;
+    }
+
+#if EVDEBUG
+    if (k != 11) {
+	print_bytes(fp, xpos, 8);
+    }
+#endif
+
+    fseek(fp, xpos, SEEK_SET);
+    fprintf(stderr, "reading doubles at 0x%x (%d)\n", xpos, (int) xpos);
+    for (t=0; t<nobs && !err; t++) {
 	x = read_double(fp, &err);
-	if (err) {
-	    break;
+	if (!err) {
+	    dset->Z[i][t] = x;
 	}
-	Z[i][t] = x;
     }
 
     return err;
 }
 
-static int read_history (FILE *fp, long pos, DATAINFO *pdinfo, int i)
+static int read_wf1_variables (FILE *fp, int ftype, unsigned pos, 
+			       DATASET *dset, int *nvread, 
+			       PRN *prn)
 {
-    char *htxt;
-    int hpos, len, err = 0;
-
-    fseek(fp, pos + 2, SEEK_SET);
-    len = read_int(fp, &err);
-    if (err) {
-	return 1;
-    }
-
-    fseek(fp, pos + 10, SEEK_SET);
-    hpos = read_long(fp, &err);
-    if (err) {
-	return 1;
-    }  
-
-    htxt = calloc(len + 1, 1);
-    if (htxt != NULL) {
-	fseek(fp, hpos, SEEK_SET);
-	if (fread(htxt, 1, len, fp) == len) {
-	    char *targ = VARLABEL(pdinfo, i);
-
-	    *targ = '\0';
-	    strncat(targ, htxt, MAXLABEL - 1);
-	    fprintf(stderr, "history: '%s'\n", htxt);
-	}
-	free(htxt);
-    }
-
-    return 0;
-}
-
-static int read_wf1_variables (FILE *fp, long pos, double **Z,
-			       DATAINFO *dinfo, int *nvread, PRN *prn)
-{
-    int nv = dinfo->v + 1; /* RESID */
-    int msg_done = 0;
+    int nv = dset->v + 1; /* allow for "RESID" */
     char vname[32];
     short code = 0;
-    long u;
-    int i, j = 0;
+    unsigned u, sz;
+    int i, k, j = 0;
+    int obj = 1;
     int err = 0;
 
     fseek(fp, pos + 62, SEEK_SET);
     code = read_short(fp, &err);
     if (code == 0) {
-	fprintf(stderr, "Did not get sensible code: trying skipping forward 32 bytes\n");
+	fprintf(stderr, "Did not get sensible code: skipping 32 bytes\n");
 	pos += 32;
     }
 
     for (i=0; i<nv && !err; i++, pos += 70) {
+	int discard = 0;
+
 	/* read the 'code' for the 'object' (should be 44 for a regular
 	   variable?) */
 	fseek(fp, pos + 62, SEEK_SET);
 	code = read_short(fp, &err);
-	if (code == 43) {
-	    /* constant: skip */
-	    continue;
-	} else if (code != 44) {
-	    if (!msg_done) {
-		pprintf(prn, "byte %ld: unknown object code %d\n", 
-			pos + 62, (int) code);
-		msg_done = 1;
-	    }
-	    continue;
+	if (code != 44) {
+	    discard = 1;
 	}
 
-	/* grab the variable name */
+	/* grab the object name */
 	fseek(fp, pos + 22, SEEK_SET);
 	fscanf(fp, "%31s", vname);
 	if (!strcmp(vname, "C") || !strcmp(vname, "RESID")) {
+	    discard = 1;
+	}
+
+	fprintf(stderr, "\n*** object %d (type %d), '%s', at 0x%x (%d)\n", obj++, 
+		(int) code, vname, pos, (int) pos);
+
+	if (discard) {
+	    fprintf(stderr, "Discarding '%s'\n", vname);
 	    continue;
 	}
-	fprintf(stderr, "Got variable %d, '%s'\n", j + 1, vname);
-	dinfo->varname[++j][0] = 0;
-	strncat(dinfo->varname[j], vname, VNAMELEN - 1);
+
+	/* storage type code? */
+	fseek(fp, pos + 4, SEEK_SET);
+	k = read_short(fp, &err);
+	if (k != 11) {
+#if EVDEBUG
+	    fprintf(stderr, "unknown data storage method %d\n", (int) k);
+#else
+	    pputs(prn, "unknown data storage method, skipping\n");
+	    continue;
+#endif
+	}
+
+	dset->varname[++j][0] = 0;
+	strncat(dset->varname[j], vname, VNAMELEN - 1);
+
+	fseek(fp, pos + 6, SEEK_SET);
+	sz = read_unsigned(fp, &err);
+	fprintf(stderr, "data record size: %d\n", (int) sz);
+	fseek(fp, pos + 10, SEEK_SET);
+	sz = read_unsigned(fp, &err);
+	fprintf(stderr, "data block size: %d\n", (int) sz);
+
+#if EVDEBUG
+	fprintf(stderr, "first 6 bytes as shorts:");
+	fseek(fp, pos, SEEK_SET);
+	for (k=0; k<3; k++) {
+	    fprintf(stderr, " %d", read_short(fp, &err));
+	}
+	fputc('\n', stderr);
+	fprintf(stderr, "last 6 bytes as shorts:");
+	fseek(fp, pos + 64, SEEK_SET);
+	for (k=0; k<3; k++) {
+	    fprintf(stderr, " %d", read_short(fp, &err));
+	}
+	fputc('\n', stderr);
+#endif
 
 	/* get stream position for the data */
 	fseek(fp, pos + 14, SEEK_SET);
-	u = read_long(fp, &err);
+	u = read_unsigned(fp, &err);
 	if (u > 0) {
 	    /* follow up at the pos given above, if non-zero */
-	    err = get_data(fp, u, Z, j, dinfo->n);
+	    fprintf(stderr, "data record at 0x%x (%d)\n", u, (int) u);
+	    err = wf1_read_values(fp, ftype, u, sz, dset, j, k);
 	} else {
 	    fputs("Couldn't find the data: skipping this variable\n", stderr);
+	    continue;
 	}
 
-	/* stream pos for history */
-	fseek(fp, pos + 54, SEEK_SET);
-	u = read_long(fp, &err);
-	if (u > 0) {
-	    read_history(fp, u, dinfo, j);
+	if (!err) {
+	    /* stream position for history */
+	    fseek(fp, pos + 54, SEEK_SET);
+	    u = read_unsigned(fp, &err);
+	    if (u > 0) {
+		fprintf(stderr, "Reading history block at 0x%x\n", (unsigned) u);
+		wf1_read_history(fp, u, dset, j);
+	    }
 	}
     }
 
     *nvread = j;
 
     fprintf(stderr, "actual number of variables read = %d\n", *nvread);
+
     if (*nvread == 0) {
 	pputs(prn, _("No variables were read\n"));
 	err = E_DATA;
-    }
+    } 
 
     return err;
 }
 
-#if EDEBUG
+#if EVDEBUG > 1
 
 static void analyse_mystery_vals (FILE *fp)
 {
     union {
 	unsigned char s[8];
-	short i2[4];
-	int i4[2];
+	unsigned short i2[4];
+	unsigned int i4[2];
     } u;
-    short k;
+    unsigned short k;
     int err = 0;
 
     fseek(fp, 122, SEEK_SET);
@@ -271,15 +382,16 @@ static void analyse_mystery_vals (FILE *fp)
 
 #endif
 
-static int parse_wf1_header (FILE *fp, DATAINFO *dinfo, long *offset)
+static int parse_wf1_header (FILE *fp, int ftype, DATASET *dset, 
+			     unsigned *offset)
 {
     int nvars = 0, nobs = 0, startyr = 0;
     short pd = 0, startper = 0;
-    long off;
+    unsigned off;
     int err = 0;
 
     fseek(fp, 80, SEEK_SET);
-    off = read_long(fp, &err);
+    off = read_unsigned(fp, &err);
     *offset = off + 26;
 
     fseek(fp, 114, SEEK_SET);
@@ -288,23 +400,16 @@ static int parse_wf1_header (FILE *fp, DATAINFO *dinfo, long *offset)
     fseek(fp, 124, SEEK_SET);
     pd = read_short(fp, &err);
 
-#if 0 /* old approach, seems to be wrong? */
-    fseek(fp, 126, SEEK_SET);
-    startper = read_short(fp, &err);
-#endif
-
     fseek(fp, 128, SEEK_SET);
     startyr = read_int(fp, &err);
 
-#if 1
     fseek(fp, 132, SEEK_SET);
     startper = read_short(fp, &err);
-#endif
 
     fseek(fp, 140, SEEK_SET);
     nobs = read_int(fp, &err);
 
-#if EDEBUG
+#if EVDEBUG > 1
     analyse_mystery_vals(fp);
 #endif
 
@@ -312,7 +417,7 @@ static int parse_wf1_header (FILE *fp, DATAINFO *dinfo, long *offset)
 	startper = 0;
     }
 
-    if (nvars <= 2 || nobs <= 0 || startyr <= 0 ||
+    if (nvars <= 2 || nobs <= 0 || startyr < 0 ||
 	pd <= 0 || startper < 0) {
 	err = E_DATA;
 	fprintf(stderr, "header info:\n"
@@ -324,9 +429,9 @@ static int parse_wf1_header (FILE *fp, DATAINFO *dinfo, long *offset)
 		nvars, nobs, (int) pd,
 		startyr, (int) startper);
     } else {
-	dinfo->v = nvars - 2; /* skip C and RESID */
-	dinfo->n = nobs;
-	dinfo->pd = pd;
+	dset->v = nvars - 2; /* skip C and RESID */
+	dset->n = nobs;
+	dset->pd = pd;
 
 	fprintf(stderr, "header info:\n"
 		" number of variables = %d\n"
@@ -334,7 +439,7 @@ static int parse_wf1_header (FILE *fp, DATAINFO *dinfo, long *offset)
 		" data frequency = %d\n"
 		" starting year or major = %d\n"
 		" starting sub-period or minor = %d\n",
-		dinfo->v, dinfo->n, dinfo->pd,
+		dset->v, dset->n, dset->pd,
 		startyr, (int) startper);
     }
 
@@ -346,103 +451,113 @@ static int parse_wf1_header (FILE *fp, DATAINFO *dinfo, long *offset)
 		char fmt[16];
 
 		sprintf(fmt, "%%d:%%0%dd", p10 + 1);
-		sprintf(dinfo->stobs, fmt, startyr, startper);
+		sprintf(dset->stobs, fmt, startyr, startper);
 	    } else {
-		sprintf(dinfo->stobs, "%d:%d", startyr, startper);
+		sprintf(dset->stobs, "%d:%d", startyr, startper);
 	    }
 	} else {
-	    sprintf(dinfo->stobs, "%d", startyr);
+	    sprintf(dset->stobs, "%d", startyr);
 	}
 
-	if (dinfo->pd > 1 || startyr > 10) {
-	    dinfo->structure = TIME_SERIES;
+	if (dset->pd > 1 || startyr > 10) {
+	    dset->structure = TIME_SERIES;
 	}
 
-	dinfo->sd0 = get_date_x(dinfo->pd, dinfo->stobs);
-	ntodate(dinfo->endobs, dinfo->n - 1, dinfo);
+	dset->sd0 = get_date_x(dset->pd, dset->stobs);
+	ntodate(dset->endobs, dset->n - 1, dset);
     }
 
     return err;
 }
 
-static int check_file_type (FILE *fp)
+static int wf1_check_file_type (FILE *fp)
 {
     char s[22] = {0};
-    int err = 0;
+    int n = fread(s, 1, 21, fp);
+    int ftype = -1;
 
-    if (fread(s, 1, 21, fp) != 21) {
-	err = 1;
-    } else if (strcmp(s, "New MicroTSP Workfile")) {
-	err = 1;
+    if (n == 21) {
+	if (!strcmp(s, "New MicroTSP Workfile")) {
+	    /* old-style EViews file, OK */
+	    ftype = 0;
+	} else if (!strncmp(s, "EViews File V01", 15)) {
+	    /* new style */
+	    ftype = 1;
+	}		
     }
 
-    return err;
+    return ftype;
 }
 
-int wf1_get_data (const char *fname, 
-		  double ***pZ, DATAINFO *pdinfo,
+int wf1_get_data (const char *fname, DATASET *dset,
 		  gretlopt opt, PRN *prn)
 {
     FILE *fp;
-    double **newZ = NULL;
-    DATAINFO *newinfo = NULL;
-    long offset;
-    int nvread, err = 0;
+    DATASET *newset = NULL;
+    unsigned offset;
+    int nvread, ftype;
+    int err = 0;
 
     fp = gretl_fopen(fname, "rb");
     if (fp == NULL) {
 	return E_FOPEN;
     }
 
-    if (check_file_type(fp)) {
+    ftype = wf1_check_file_type(fp);
+    
+    if (ftype < 0) {
 	fclose(fp);
-	pputs(prn, "This file does not seem to be an Eviews workfile");
+	pputs(prn, "This file does not seem to be an EViews workfile\n");
 	return E_DATA;
     }
 
-    newinfo = datainfo_new();
-    if (newinfo == NULL) {
+    if (ftype == 1) {
+	pputs(prn, "EViews 7+ file: expect problems!\n");
+    }
+
+    newset = datainfo_new();
+    if (newset == NULL) {
 	pputs(prn, _("Out of memory\n"));
 	fclose(fp);
 	return E_ALLOC;
     }
 
-    err = parse_wf1_header(fp, newinfo, &offset);
+    err = parse_wf1_header(fp, ftype, newset, &offset);
     if (err) {
 	pputs(prn, _("Error reading workfile header\n"));
-	free_datainfo(newinfo);
+	free_datainfo(newset);
 	fclose(fp);
 	return err;
     }
 
-    err = start_new_Z(&newZ, newinfo, 0);
+    err = start_new_Z(newset, 0);
     if (err) {
 	pputs(prn, _("Out of memory\n"));
-	free_datainfo(newinfo);
+	free_datainfo(newset);
 	fclose(fp);
 	return E_ALLOC;
     }	
 
-    err = read_wf1_variables(fp, offset, newZ, newinfo, &nvread, prn);
+    err = read_wf1_variables(fp, ftype, offset, newset, &nvread, prn);
 
     if (err) {
-	destroy_dataset(newZ, newinfo);
+	destroy_dataset(newset);
     } else {
-	int merge = (*pZ != NULL);
-	int nvtarg = newinfo->v - 1;
+	int merge = (dset->Z != NULL);
+	int nvtarg = newset->v - 1;
 
 	if (nvread < nvtarg) {
-	    dataset_drop_last_variables(nvtarg - nvread, &newZ, newinfo);
+	    dataset_drop_last_variables(nvtarg - nvread, newset);
 	}
 
-	if (fix_varname_duplicates(newinfo)) {
+	if (fix_varname_duplicates(newset)) {
 	    pputs(prn, _("warning: some variable names were duplicated\n"));
 	}
 
-	err = merge_or_replace_data(pZ, pdinfo, &newZ, &newinfo, opt, prn);
+	err = merge_or_replace_data(dset, &newset, opt, prn);
 
 	if (!err && !merge) {
-	    dataset_add_import_info(pdinfo, fname, GRETL_WF1);
+	    dataset_add_import_info(dset, fname, GRETL_WF1);
 	}
     }
 
