@@ -27,9 +27,210 @@ static void invalid_varname (PRN *prn)
     pputs(prn, _("\nPlease rename this variable and try again"));
 }
 
-#ifndef ODS_IMPORTER
+#if defined(ODS_IMPORTER) || defined(XLSX_IMPORTER)
 
-static int worksheet_start_dataset (double ***pZ, DATAINFO *newinfo)
+static char *get_absolute_path (const char *fname)
+{
+    char buf[FILENAME_MAX];
+    char *s, *ret = NULL;
+
+    s = getcwd(buf, sizeof buf - strlen(fname) - 2);
+    if (s != NULL) {
+	ret = g_strdup_printf("%s/%s", s, fname);
+    }
+
+    return ret;
+}
+
+static void remove_temp_dir (char *dname)
+{
+    const char *udir = gretl_dotdir();
+
+#ifdef G_OS_WIN32
+    char *fullpath = g_strdup_printf("%s%s", udir, dname);
+
+    gretl_chdir(udir);
+    win32_delete_dir(fullpath);
+    g_free(fullpath);
+#else
+    gretl_chdir(udir);
+    gretl_deltree(dname);
+#endif
+}
+
+# ifdef G_OS_WIN32
+
+static int gretl_make_tempdir (char *dname)
+{
+    strcpy(dname, ".gretl-ssheet-XXXXXX");
+    mktemp(dname);
+
+    if (*dname == '\0') {
+	return E_FOPEN;
+    } else {
+	return gretl_mkdir(dname);
+    }
+}
+
+# else
+
+static int gretl_make_tempdir (char *dname)
+{
+    char *s;
+    int err = 0;
+
+    strcpy(dname, ".gretl-ssheet-XXXXXX");
+    s = mkdtemp(dname);
+
+    if (s == NULL) {
+	gretl_errmsg_set_from_errno("gretl_make_tempdir");
+	err = E_FOPEN;
+    } 
+
+    return err;
+}
+
+# endif /* G_OS_WIN32 or not */
+
+/* for ODS and XLSX: unzip the target file in the user's
+   "dotdir". On successful completion @dname holds the
+   name of the temporary directory, in the dotdir, holding
+   the contents of the zipfile
+*/
+
+static int open_import_zipfile (const char *fname, char *dname,
+				PRN *prn)
+{
+    int (*gretl_unzip_file)(const char *, GError **);
+    const char *udir = gretl_dotdir();
+    char *abspath = NULL;
+    void *handle = NULL;
+    FILE *fp;
+    GError *gerr = NULL;
+    int err = 0;
+
+    errno = 0;
+    *dname = '\0';
+
+    fp = gretl_fopen(fname, "r");
+    if (fp == NULL) {
+	return E_FOPEN;
+    }
+
+    fclose(fp);
+
+    /* by doing chdir, we may lose track of the file if 
+       its path is relative */
+    if (!g_path_is_absolute(fname)) {
+	abspath = get_absolute_path(fname);
+    }
+
+    /* cd to user dir */
+    if (gretl_chdir(udir)) {
+	gretl_errmsg_set_from_errno(udir);
+	err = E_FOPEN;
+    } else {
+	err = gretl_make_tempdir(dname);
+	if (!err) {
+	    err = gretl_chdir(dname);
+	    if (err) {
+		gretl_remove(dname);
+	    }
+	}
+    }
+    
+    if (!err) {
+	gretl_unzip_file = get_plugin_function("gretl_unzip_file", 
+					       &handle);
+	if (gretl_unzip_file == NULL) {
+	    gretl_remove(dname);
+	    err = E_FOPEN;
+	}
+    }
+
+    if (!err) {
+	if (abspath != NULL) {
+	    err = (*gretl_unzip_file)(abspath, &gerr);
+	} else {
+	    err = (*gretl_unzip_file)(fname, &gerr);
+	}
+	if (gerr != NULL) {
+	    pprintf(prn, "gretl_unzip_file: '%s'\n", gerr->message);
+	    g_error_free(gerr);
+	} else if (err) {
+	    pprintf(prn, "gretl_unzip_file: err = %d\n", err);
+	}
+    }
+
+    free(abspath);
+
+    if (handle != NULL) {
+	close_plugin(handle);
+    }
+
+    return err;
+}
+
+/* check for spurious empty columns at the right of the sheet */
+
+static int import_prune_columns (DATASET *dset)
+{
+    int allmiss = 1, ndel = 0;
+    int i, t, err = 0;
+
+    for (i=dset->v-1; i>0 && allmiss; i--) {
+	for (t=0; t<dset->n; t++) {
+	    if (!na(dset->Z[i][t])) {
+		allmiss = 0;
+		break;
+	    }
+	}
+	if (allmiss) ndel++;
+    }
+
+    if (ndel == dset->v - 1) {
+	gretl_errmsg_set(_("No numeric data were found"));
+	err = E_DATA;
+    } else if (ndel > 0) {
+	fprintf(stderr, "Sheet has %d trailing empty variables\n", ndel);
+	err = dataset_drop_last_variables(ndel, dset);
+    }
+
+    return err;
+}
+
+static void import_ts_check (DATASET *dset)
+{
+    PRN *prn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
+    int reversed = 0;
+    int mpd = -1;
+
+    mpd = test_markers_for_dates(dset, &reversed, NULL, prn);
+
+    if (mpd > 0) {
+	pputs(prn, _("taking date information from row labels\n\n"));
+	if (dset->markers != DAILY_DATE_STRINGS) {
+	    dataset_destroy_obs_markers(dset);
+	}
+	if (reversed) {
+	    reverse_data(dset, prn);
+	}
+    } 
+
+#if ODEBUG
+    fprintf(stderr, "xinfo->dset->pd = %d\n", xinfo->dset->pd);
+#endif
+
+    if (dset->pd != 1 || strcmp(dset->stobs, "1")) { 
+        dset->structure = TIME_SERIES;
+    }
+
+    gretl_print_destroy(prn);
+}
+
+#else /* !ODS, !XLSX */
+
+static int worksheet_start_dataset (DATASET *newinfo)
 {
     if (newinfo->v == 1) {
 	/* only the constant is present! */
@@ -37,7 +238,7 @@ static int worksheet_start_dataset (double ***pZ, DATAINFO *newinfo)
 	return E_DATA;
     } else {
 	/* create import dataset */
-	return start_new_Z(pZ, newinfo, 0);
+	return start_new_Z(newinfo, 0);
     }
 }
 
@@ -50,7 +251,7 @@ static int worksheet_start_dataset (double ***pZ, DATAINFO *newinfo)
 static int 
 importer_dates_check (int row_offset, int col_offset, 
 		      BookFlag *pflags, char **labels, 
-		      DATAINFO *newinfo, PRN *prn, int *err)
+		      DATASET *newinfo, PRN *prn, int *err)
 {
     int d, t;
     char dstr[12];
@@ -88,7 +289,7 @@ importer_dates_check (int row_offset, int col_offset,
     if (!*err) {
 	int reversed = 0;
 
-	ret = test_markers_for_dates(NULL, newinfo, &reversed, NULL, prn);
+	ret = test_markers_for_dates(newinfo, &reversed, NULL, prn);
 	if (reversed) {
 	    *pflags |= BOOK_DATA_REVERSED;
 	}
