@@ -2772,14 +2772,14 @@ static int Jgradient (double *theta, double *gr, int n,
 }
 
 static int max_beta_vname (GRETL_VAR *v, 
-			   const DATAINFO *pdinfo)
+			   const DATASET *dset)
 {
     int r = gretl_matrix_rows(v->jinfo->Beta);
     const char *s;
     int i, ni, n = 0;
 
     for (i=0; i<r; i++) {    
-	s = beta_vname(v, pdinfo, i);
+	s = beta_vname(v, dset, i);
 	ni = strlen(s);
 	if (ni > n) {
 	    n = ni;
@@ -2793,7 +2793,7 @@ static int max_beta_vname (GRETL_VAR *v,
 
 static int printres (Jwrap *J, GRETL_VAR *jvar, 
 		     gretl_restriction *rset,
-		     const DATAINFO *pdinfo,
+		     const DATASET *dset,
 		     PRN *prn)
 {
     const gretl_matrix *c = J->beta;
@@ -2836,11 +2836,11 @@ static int printres (Jwrap *J, GRETL_VAR *jvar,
     }
     pputs(prn, "\n\n");
 
-    nwid = max_beta_vname(jvar, pdinfo) + 1;
+    nwid = max_beta_vname(jvar, dset) + 1;
     sprintf(namefmt, "%%-%ds", nwid);
 
     for (i=0; i<J->p1; i++) {
-	pprintf(prn, namefmt, beta_vname(jvar, pdinfo, i));
+	pprintf(prn, namefmt, beta_vname(jvar, dset, i));
 
 	for (j=0; j<J->r; j++) {
 	    pprintf(prn, "%#12.5g ", gretl_matrix_get(c, i, j));
@@ -2870,7 +2870,7 @@ static int printres (Jwrap *J, GRETL_VAR *jvar,
     pputs(prn, "\n\n");
 
     for (i=0; i<J->p; i++) {
-	sprintf(vname, "%s", pdinfo->varname[jvar->ylist[i+1]]);
+	sprintf(vname, "%s", dset->varname[jvar->ylist[i+1]]);
 	pprintf(prn, namefmt, vname);
 	for (j=0; j<J->r; j++) {
 	    pprintf(prn, "%#12.5g ", gretl_matrix_get(c, i, j));
@@ -2892,67 +2892,71 @@ static int printres (Jwrap *J, GRETL_VAR *jvar,
     return 0;
 }
 
-/* simulated annealing: can help when the standard deterministic
+static void set_up_matrix (gretl_matrix *m, double *val, 
+			   int rows, int cols)
+{
+    m->val = val;
+    m->rows = rows;
+    m->cols = cols;
+    m->t1 = m->t2 = 0;
+}
+
+/* Simulated annealing: can help when the standard deterministic
    initialization (theta_0) leads to a local maximum.
-   We use ll-best point in case of improvement, else the 
-   last value.
+   We use the loglik-best point in case of improvement, else 
+   the last point visited.
+
+   This should be moved into gretl_bfgs.c (with some general-
+   ization possible) and exposed for users.
 */
 
-static int simann (Jwrap *J, gretlopt opt, PRN *prn)
+int gretl_simann (double *theta, int n, int maxit,
+		  BFGS_CRIT_FUNC cfunc, int crittype,
+		  void *data, gretlopt opt, PRN *prn)
 {
-    gretl_matrix *b = J->theta;
-    int i, SAiter = 4096;
-    double f0, f1;
-    double fbest, fworst;
-    double rndu;
-    int jump;
-
+    gretl_matrix b;
     gretl_matrix *b0 = NULL;
     gretl_matrix *b1 = NULL;
     gretl_matrix *bstar = NULL;
     gretl_matrix *d = NULL;
-
+    double f0, f1;
+    double fbest, fworst;
     double Temp = 1.0;
     double radius = 1.0;
     int improved = 0;
-    int err = 0;
+    int i, err = 0;
 
-    if (b == NULL) {
-	fprintf(stderr, "simann: J->theta = NULL\n");
-	return E_DATA;
-    }
+    set_up_matrix(&b, theta, n, 1);
 
-    b0 = gretl_matrix_copy(b);
-    b1 = gretl_matrix_copy(b);
-    bstar = gretl_matrix_copy(b);
-    d = gretl_column_vector_alloc(b->rows);
+    b0 = gretl_matrix_copy(&b);
+    b1 = gretl_matrix_copy(&b);
+    bstar = gretl_matrix_copy(&b);
+    d = gretl_column_vector_alloc(n);
 
     if (b0 == NULL || b1 == NULL || bstar == NULL || d == NULL) {
 	err = E_ALLOC;
 	goto bailout;
     }
 
-    f0 = fbest = fworst = Jloglik(b0->val, J);
+    f0 = fbest = fworst = cfunc(b.val, data);
 
     if (opt & OPT_V) {
 	pprintf(prn, "\nSimulated annealing: initial function value = %.8g\n",
 		f0);
-    }    
+    }
 
-    for (i=0; i<SAiter; i++) {
+    /* Question: should the initial radius be a function of the scale
+       of the initial parameter vector?
+    */
+
+    for (i=0; i<maxit; i++) {
 	gretl_matrix_random_fill(d, D_NORMAL);
 	gretl_matrix_multiply_by_scalar(d, radius);
 	gretl_matrix_add_to(b1, d);
-	f1 = Jloglik(b1->val, J);
+	f1 = cfunc(b1->val, data);
 
-	if (f1 > f0) {
-	    jump = 1;
-	} else {
-	    rndu = ((double) rand()) / RAND_MAX;
-	    jump = (Temp < rndu);
-	}
-
-	if (jump) {
+	if (f1 > f0 || gretl_rand_01() < Temp) {
+	    /* jump to the new point */
 	    f0 = f1;
 	    gretl_matrix_copy_values(b0, b1);
 	    if (f0 > fbest) {
@@ -2971,6 +2975,7 @@ static int simann (Jwrap *J, gretlopt opt, PRN *prn)
 		fworst = f0;
 	    }
 	} else {
+	    /* revert to where we were */
 	    gretl_matrix_copy_values(b1, b0);
 	    f1 = f0;
 	}
@@ -2980,21 +2985,21 @@ static int simann (Jwrap *J, gretlopt opt, PRN *prn)
     }
 
     if (improved) {
-	gretl_matrix_copy_values(J->theta, bstar);
+	/* use the best point */
+	gretl_matrix_copy_values(&b, bstar);
     } else {
-	gretl_matrix_copy_values(J->theta, b0);
+	/* use the last point */
+	gretl_matrix_copy_values(&b, b0);
     }
 
-    sync_with_theta(J, b->val);
-
     if (improved) {
 	if (opt & OPT_V) pputc(prn, '\n');
     } else {
-	pprintf(prn, "No improvement found in %d iterations\n\n", SAiter);
+	pprintf(prn, "No improvement found in %d iterations\n\n", maxit);
     }
     
     if (fbest - fworst < 1.0e-9) {
-	pprintf(prn, "*** warning: likelihood seems to be flat\n");
+	pprintf(prn, "*** warning: surface seems to be flat\n");
     }
 
  bailout:
@@ -3167,7 +3172,7 @@ static int do_bfgs (Jwrap *J, gretlopt opt, PRN *prn)
 
 int general_vecm_analysis (GRETL_VAR *jvar, 
 			   gretl_restriction *rset,
-			   const DATAINFO *pdinfo,
+			   const DATASET *dset,
 			   PRN *prn)
 {
     Jwrap *J = NULL;
@@ -3230,7 +3235,14 @@ int general_vecm_analysis (GRETL_VAR *jvar,
     }
 
     if (!err && do_simann) {
-	err = simann(J, opt, prn);
+	int n = gretl_vector_get_length(J->theta);
+
+	err = gretl_simann(J->theta->val, n, 4096,
+			   Jloglik, C_LOGLIK, J,
+			   opt, prn);
+	if (!err) {
+	    sync_with_theta(J, J->theta->val);
+	}
     }
 
     if (!err) {
@@ -3257,7 +3269,7 @@ int general_vecm_analysis (GRETL_VAR *jvar,
 	} else if (opt & OPT_B) {
 	    transcribe_to_jvar(J, jvar, OPT_B);
 	} else {
-	    printres(J, jvar, rset, pdinfo, prn);
+	    printres(J, jvar, rset, dset, prn);
 	}
     } 
 
