@@ -307,8 +307,10 @@ static void finalize_mail_settings (GtkWidget *w, struct mail_dialog *md)
     /* recipient */
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
     recip = my_combo_box_get_active_text(GTK_COMBO_BOX(md->recip_combo));
-#else
+#elif GTK_MAJOR_VERSION < 3
     recip = gtk_combo_box_get_active_text(GTK_COMBO_BOX(md->recip_combo));
+#else
+    recip = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(md->recip_combo));    
 #endif
 
     if (recip != NULL && *recip != '\0') {
@@ -562,7 +564,12 @@ static void set_combo_strings_from_list (GtkComboBox *box, GList *list)
     GList *mylist = list;
 
     while (mylist != NULL) {
+#if GTK_MAJOR_VERSION >= 3
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(box), 
+				       mylist->data);
+#else
 	gtk_combo_box_append_text(box, mylist->data);
+#endif
 	mylist = mylist->next;
     }
 }
@@ -600,7 +607,9 @@ mail_to_dialog (const char *fname, struct mail_info *minfo,
 
     gtk_window_set_title(GTK_WINDOW(md.dlg), _("gretl: send mail"));
     set_dialog_border_widths(md.dlg);
+#if GTK_MAJOR_VERSION < 3    
     gtk_dialog_set_has_separator(GTK_DIALOG(md.dlg), FALSE);
+#endif    
     gtk_window_set_position(GTK_WINDOW(md.dlg), GTK_WIN_POS_MOUSE);
 
     nb = gtk_notebook_new();
@@ -636,7 +645,11 @@ mail_to_dialog (const char *fname, struct mail_info *minfo,
 	gtk_table_attach(GTK_TABLE(tbl), lbl, 0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0);
 
 	if (i == 0) {
+#if GTK_MAJOR_VERSION >= 3
+	    w = gtk_combo_box_text_new_with_entry();
+#else
 	    w = gtk_combo_box_entry_new_text();
+#endif
 	    if (minfo->addrs != NULL) {
 		set_combo_strings_from_list(GTK_COMBO_BOX(w), minfo->addrs);
 	    } 
