@@ -39,22 +39,6 @@
 
 #define ODEBUG 1
 
-static void remove_temp_dir (char *dname)
-{
-    const char *udir = gretl_dotdir();
-
-#ifdef G_OS_WIN32
-    char *fullpath = g_strdup_printf("%s%s", udir, dname);
-
-    gretl_chdir(udir);
-    win32_delete_dir(fullpath);
-    g_free(fullpath);
-#else
-    gretl_chdir(udir);
-    gretl_deltree(dname);
-#endif
-}
-
 enum {
     ODS_NONE,     
     ODS_NUMERIC, 
@@ -87,8 +71,7 @@ struct ods_sheet_ {
     int seltab;            /* number of selected table */
     int xoffset;           /* col offset chosen by user */
     int yoffset;           /* row offset chosen by user */
-    double **Z;            /* data array */
-    DATAINFO *dinfo;       /* dataset info */
+    DATASET *dset;         /* dataset struct */
 };
 
 static ods_table *ods_table_new (xmlNodePtr node, int *err)
@@ -151,8 +134,7 @@ static ods_sheet *ods_sheet_new (xmlDocPtr doc, int *err)
 	sheet->seltab = -1;
 	sheet->xoffset = 0;
 	sheet->yoffset = 0;
-	sheet->Z = NULL;
-	sheet->dinfo = NULL;
+	sheet->dset = NULL;
     } else {
 	*err = E_ALLOC;
     }
@@ -178,10 +160,9 @@ static void ods_sheet_free (ods_sheet *sheet)
 
 	if (sheet->doc != NULL) {
 	    xmlFreeDoc(sheet->doc);
-	    xmlCleanupParser();
 	}
 
-	destroy_dataset(sheet->Z, sheet->dinfo);
+	destroy_dataset(sheet->dset);
 
 	free(sheet);
     }
@@ -419,8 +400,8 @@ static int ods_error (ods_sheet *sheet,
     } else {
 	int v = (sheet->flags & BOOK_OBS_LABELS)? j : j + 1;
 
-	if (v > 0 && v < sheet->dinfo->v) {
-	    pprintf(prn, " (\"%s\"):\n", sheet->dinfo->varname[v]);
+	if (v > 0 && v < sheet->dset->v) {
+	    pprintf(prn, " (\"%s\"):\n", sheet->dset->varname[v]);
 	} else {
 	    pputs(prn, ":\n");
 	}
@@ -449,7 +430,7 @@ static int real_read_cell (xmlNodePtr cur,
     v = jread + 1 - obscol;
     t = iread - vnames;
 
-    if (v >= sheet->dinfo->v || t >= sheet->dinfo->n) {
+    if (v >= sheet->dset->v || t >= sheet->dset->n) {
 	fprintf(stderr, "v = %d, t = %d: out of bounds?\n", v, t);
 	return E_DATA;
     }
@@ -475,11 +456,11 @@ static int real_read_cell (xmlNodePtr cur,
 	if (vtype == ODS_STRING) {
 	    val = get_ods_string_value(cur);
 	    if (val != NULL) {
-		*sheet->dinfo->varname[v] = '\0';
-		strncat(sheet->dinfo->varname[v],
+		*sheet->dset->varname[v] = '\0';
+		strncat(sheet->dset->varname[v],
 			val, VNAMELEN - 1);
 		fprintf(stderr, " varname: '%s'\n", val);
-		if (check_varname(sheet->dinfo->varname[v])) {
+		if (check_varname(sheet->dset->varname[v])) {
 		    invalid_varname(prn);
 		    err = 1;
 		}
@@ -528,7 +509,7 @@ static int real_read_cell (xmlNodePtr cur,
 	}
 
 	if (!err) {
-	    strncat(sheet->dinfo->S[t], val, OBSLEN - 1);
+	    strncat(sheet->dset->S[t], val, OBSLEN - 1);
 	}
 
 	free(val);
@@ -562,8 +543,8 @@ static int real_read_cell (xmlNodePtr cur,
     if (err) {
 	ods_error(sheet, iread, jread, ODS_NUMERIC, vtype, prn);
     } else {
-	for (j=0, vj=v; j<nr && vj<sheet->dinfo->v; j++, vj++) {
-	    sheet->Z[vj][t] = x;
+	for (j=0, vj=v; j<nr && vj<sheet->dset->v; j++, vj++) {
+	    sheet->dset->Z[vj][t] = x;
 	}
     }	    
 
@@ -620,14 +601,14 @@ static int sheet_allocate_data (ods_sheet *sheet,
     fprintf(stderr, "sheet_allocate_data: n=%d, v=%d\n",
 	    n, v);
 
-    sheet->dinfo = create_new_dataset(&sheet->Z, v, n, labels);
-    if (sheet->dinfo == NULL) {
+    sheet->dset = create_new_dataset(v, n, labels);
+    if (sheet->dset == NULL) {
 	return E_ALLOC;
     }
 
     /* write fallback variable names */
     for (i=1; i<v; i++) {
-	sprintf(sheet->dinfo->varname[i], "v%d", i);
+	sprintf(sheet->dset->varname[i], "v%d", i);
     }
 
     return 0;
@@ -747,17 +728,17 @@ static int repeat_data_row (ods_sheet *sheet, int iread,
     int vnames = (sheet->flags & BOOK_AUTO_VARNAMES)? 0 : 1;
     int i, t = iread - vnames;
 
-    if (t < 1 || t >= sheet->dinfo->n) {
+    if (t < 1 || t >= sheet->dset->n) {
 	pprintf(prn, "Found a repeated row in the wrong place\n");
 	return E_DATA;
     }
 
-    for (i=1; i<sheet->dinfo->v; i++) {
-	sheet->Z[i][t] = sheet->Z[i][t-1];
+    for (i=1; i<sheet->dset->v; i++) {
+	sheet->dset->Z[i][t] = sheet->dset->Z[i][t-1];
     }
 
-    if (sheet->dinfo->S != NULL) {
-	strcpy(sheet->dinfo->S[t], sheet->dinfo->S[t-1]);
+    if (sheet->dset->S != NULL) {
+	strcpy(sheet->dset->S[t], sheet->dset->S[t-1]);
     }
 
     return 0;
@@ -907,8 +888,6 @@ static ods_sheet *ods_read_content (PRN *prn, int *err)
     xmlNodePtr cur = NULL;
     xmlNodePtr c1, c2;
 
-    LIBXML_TEST_VERSION xmlKeepBlanksDefault(0);
-
     *err = gretl_xml_open_doc_root("content.xml", 
 				   "document-content", 
 				   &doc, &cur);
@@ -922,7 +901,6 @@ static ods_sheet *ods_read_content (PRN *prn, int *err)
     sheet = ods_sheet_new(doc, err);
     if (sheet == NULL) {
 	xmlFreeDoc(doc);
-	xmlCleanupParser();
 	return NULL;
     }
 
@@ -975,73 +953,6 @@ static int check_mimetype (PRN *prn)
     return err;
 }
 
-#ifdef G_OS_WIN32
-
-static int gretl_make_tempdir (char *dname)
-{
-    strcpy(dname, ".gretl-ssheet-XXXXXX");
-    mktemp(dname);
-
-    if (*dname == '\0') {
-	return E_FOPEN;
-    } else {
-	return gretl_mkdir(dname);
-    }
-}
-
-#else
-
-static int gretl_make_tempdir (char *dname)
-{
-    char *s;
-    int err = 0;
-
-    strcpy(dname, ".gretl-ssheet-XXXXXX");
-    s = mkdtemp(dname);
-
-    if (s == NULL) {
-	gretl_errmsg_set_from_errno("gretl_make_tempdir");
-	err = E_FOPEN;
-    } 
-
-    return err;
-}
-
-#endif
-
-static int ts_check (ods_sheet *sheet, PRN *prn)
-{
-    int reversed = 0;
-    int mpd = -1;
-
-    if (sheet->dinfo->S == NULL) {
-	return 0;
-    }
-
-    mpd = test_markers_for_dates(&sheet->Z, sheet->dinfo, &reversed,
-				 NULL, prn);
-
-    if (mpd > 0) {
-	pputs(prn, _("taking date information from row labels\n\n"));
-	if (sheet->dinfo->markers != DAILY_DATE_STRINGS) {
-	    dataset_destroy_obs_markers(sheet->dinfo);
-	}
-	if (reversed) {
-	    sheet->flags |= BOOK_DATA_REVERSED;
-	}
-    } 
-
-#if ODEBUG
-    fprintf(stderr, "sheet->dinfo->pd = %d\n", sheet->dinfo->pd);
-#endif
-
-    if (sheet->dinfo->pd != 1 || strcmp(sheet->dinfo->stobs, "1")) { 
-        sheet->dinfo->structure = TIME_SERIES;
-    }
-
-    return 0;
-}
-
 static int ods_min_offset (wbook *book, int k)
 {
     ods_sheet *sheet = book->data;
@@ -1183,145 +1094,36 @@ static int ods_sheet_dialog (ods_sheet *sheet, int *err)
     return book.selected;
 }
 
-/* check for spurious empty columns at the right of the sheet */
-
-static int ods_prune_columns (ods_sheet *sheet)
-{
-    int allmiss = 1, ndel = 0;
-    int i, t, err = 0;
-
-    for (i=sheet->dinfo->v-1; i>0 && allmiss; i--) {
-	for (t=0; t<sheet->dinfo->n; t++) {
-	    if (!na(sheet->Z[i][t])) {
-		allmiss = 0;
-		break;
-	    }
-	}
-	if (allmiss) ndel++;
-    }
-
-    if (ndel > 0) {
-	fprintf(stderr, "Sheet has %d trailing empty variables\n", ndel);
-	err = dataset_drop_last_variables(ndel, &sheet->Z, sheet->dinfo);
-    }
-
-    return err;
-}
-
-static int finalize_ods_import (double ***pZ, DATAINFO *pdinfo,
-				ods_sheet *sheet, gretlopt opt,
+static int finalize_ods_import (DATASET *dset,
+				ods_sheet *sheet, 
+				gretlopt opt,
 				PRN *prn)
 {
-    PRN *tprn;
-    int err;
+    int err = import_prune_columns(sheet->dset);
 
-    err = ods_prune_columns(sheet);
-
-    if (!err && sheet->dinfo->v == 1) {
-	gretl_errmsg_set(_("No numeric data were found"));
-	err = E_DATA;
+    if (!err && sheet->dset->S != NULL) {
+	import_ts_check(sheet->dset);
     }
 
     if (!err) {
-	tprn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
-	ts_check(sheet, tprn);
-	if (sheet->flags & BOOK_DATA_REVERSED) {
-	    reverse_data(sheet->Z, sheet->dinfo, tprn);
-	}
-	gretl_print_destroy(tprn);
-    }
-
-    if (!err) {
-	err = merge_or_replace_data(pZ, pdinfo, &sheet->Z, 
-				    &sheet->dinfo, opt, prn);
+	err = merge_or_replace_data(dset, &sheet->dset, opt, prn);
     }  
 
     return err;
 }
 
-static char *get_absolute_path (const char *fname)
-{
-    char buf[FILENAME_MAX];
-    char *s, *ret = NULL;
-
-    s = getcwd(buf, sizeof buf - strlen(fname) - 2);
-    if (s != NULL) {
-	ret = g_strdup_printf("%s/%s", s, fname);
-    }
-
-    return ret;
-}
-
 int ods_get_data (const char *fname, int *list, char *sheetname,
-		  double ***pZ, DATAINFO *pdinfo,
-		  gretlopt opt, PRN *prn)
+		  DATASET *dset, gretlopt opt, PRN *prn)
 {
     int gui = (opt & OPT_G);
     ods_sheet *sheet = NULL;
-    int (*gretl_unzip_file)(const char *, GError **);
-    const char *udir = gretl_dotdir();
-    char *abspath = NULL;
-    void *handle;
     char dname[32];
-    FILE *fp;
-    GError *gerr = NULL;
-    int err = 0;
+    int err;
 
-    errno = 0;
-
-    /* test-open the file */
-    fp = gretl_fopen(fname, "r");
-    if (fp == NULL) {
-	return E_FOPEN;
-    }
-    fclose(fp);
-
-    /* by doing chdir, we may lose track of the ods file if 
-       its path is relative */
-    if (!g_path_is_absolute(fname)) {
-	abspath = get_absolute_path(fname);
-    }
-
-    /* cd to user dir */
-    if (gretl_chdir(udir)) {
-	gretl_errmsg_set_from_errno(udir);
-	return E_FOPEN;
-    }
-
-    err = gretl_make_tempdir(dname);
-    if (!err) {
-	err = gretl_chdir(dname);
-	if (err) {
-	    gretl_remove(dname);
-	}
-    }
-    
+    err = open_import_zipfile(fname, dname, prn);
     if (err) {
 	return err;
-    }
-
-    gretl_unzip_file = get_plugin_function("gretl_unzip_file", 
-					   &handle);
-    if (gretl_unzip_file == NULL) {
-	gretl_remove(dname);
-        return E_FOPEN;
-    }
-
-    if (abspath == NULL) {
-	err = (*gretl_unzip_file)(fname, &gerr);
-    } else {
-	err = (*gretl_unzip_file)(abspath, &gerr);
-	free(abspath);
-    }
-
-    if (gerr != NULL) {
-	pprintf(prn, "gretl_unzip_file: '%s'\n", gerr->message);
-	g_error_free(gerr);
-    } else if (err) {
-	pprintf(prn, "gretl_unzip_file: err = %d\n", err);
-    }
-
-    close_plugin(handle);
+    }    
 
     if (!err) {
 	err = check_mimetype(prn);
@@ -1348,16 +1150,12 @@ int ods_get_data (const char *fname, int *list, char *sheetname,
 
 	    if (resp < 0) {
 		/* canceled */
-		ods_sheet_free(sheet);
-		return -1;
+		err = -1;
+		goto bailout;
 	    } 
-	} else if (list != NULL && list[0] == 3) {
-	    err = set_ods_params_from_cli(sheet, list, sheetname);
 	} else {
-	    sheet->seltab = 0;
-	    sheet->xoffset = sheet->tables[0]->xoffset;
-	    sheet->yoffset = sheet->tables[0]->yoffset;
-	}
+	    err = set_ods_params_from_cli(sheet, list, sheetname);
+	} 
     }
 
     if (!err) {
@@ -1365,12 +1163,14 @@ int ods_get_data (const char *fname, int *list, char *sheetname,
     }
 
     if (!err) {
-	err = finalize_ods_import(pZ, pdinfo, sheet, opt, prn); 
+	err = finalize_ods_import(dset, sheet, opt, prn); 
 	if (!err && gui) {
 	    record_ods_params(sheet, list);
 	}
     }
 
+ bailout:
+
     ods_sheet_free(sheet);
 
     return err;
