@@ -115,6 +115,82 @@ static int push_series_name (const char *s)
     return err;
 }
 
+static char **whitenames;
+static int n_white;
+
+static int allocate_whitenames (int n)
+{
+    int i, err = 0;
+
+    whitenames = malloc(n * sizeof *whitenames);
+    if (whitenames == NULL) {
+	err = 1;
+    } else {
+	for (i=0; i<n && !err; i++) {
+	    whitenames[i] = malloc(16);
+	    if (whitenames[i] == NULL) {
+		err = 1;
+	    }
+	}
+    }
+
+    return err;
+}
+
+/* This is used if we want to restrict the series in a new
+   database to just those that were in an earlier one.
+   First run ./mkwhite on the old .idx file to make
+   a whitelist file, e.g.
+
+   ./mkwhite < fedstl.idx > fedstl.whitelist
+
+   Then the whitelist file will be picked up and series
+   not in that file will be discarded.
+*/
+
+static int maybe_read_whitelist_file (const char *fname)
+{
+    FILE *fp = fopen(fname, "r");
+    int err = 0;
+
+    if (fp != NULL) {
+	char line[32], vname[16];
+	int i, n = 0;
+
+	while (fgets(line, sizeof line, fp)) {
+	    if (sscanf(line, "%15s", vname) == 1) {
+		n++;
+	    }
+	}
+
+	if (n > 0) {
+	    err = allocate_whitenames(n);
+	    if (!err) {
+		rewind(fp);
+		i = 0;
+		while (fgets(line, sizeof line, fp)) {
+		    if (sscanf(line, "%15s", vname) == 1) {
+			*whitenames[i] = '\0';
+			strncat(whitenames[i++], vname, 15);
+		    }
+		}
+		n_white = i;
+		if (n_white > 4) {
+		    printf("whitelisted %d series:\n ", n_white);
+		    for (i=0; i<4; i++) {
+			printf("%s, ", whitenames[i]);
+		    }
+		    printf("%s...\n", whitenames[i]);
+		}
+	    }
+	}	
+
+	fclose(fp);
+    }
+
+    return err;
+}
+
 static FREDbuf *FREDbuf_new (FREDtask task, int catid)
 {
     FREDbuf *fb = malloc(sizeof *fb);
@@ -326,6 +402,36 @@ static int series_is_duplicate (const char *idstr, int *err)
     return dup;
 }
 
+static int skip_this_series (const char *vname)
+{
+    int skip = 0;
+
+    if (strlen(vname) > 15) {
+	/* for now we'll skip series with excessively long names:
+	   these are unlikely to be "major" data (?)
+	*/
+	printf("%s: too long, skipping\n", vname);
+	skip = 1;
+    } else if (whitenames != NULL) {
+	char test[16];
+	int i;
+
+	lower(test, vname);
+	skip = 1;
+	for (i=0; i<n_white; i++) {
+	    if (!strcmp(test, whitenames[i])) {
+		skip = 0;
+		break;
+	    }
+	}
+	if (skip) {
+	    printf("%s: not whitelisted, skipping\n", test);
+	}
+    } 
+
+    return skip;
+}
+
 /* Read the relevant details from a FRED series record. If we're 
    doing a real data write, follow up by getting the observations on
    the series.
@@ -333,9 +439,11 @@ static int series_is_duplicate (const char *idstr, int *err)
 
 static int get_series_info (xmlNodePtr n, FILE *fidx, FILE *fbin)
 {
-    xmlChar *idstr, *title, *units, *freq, *adj;
-    xmlChar *start, *stop;
-    int pd = 0, err = 0;
+    xmlChar *idstr = NULL, *title = NULL;
+    xmlChar *units = NULL, *freq = NULL, *adj = NULL;
+    xmlChar *start = NULL, *stop = NULL;
+    int pd = 0, skipit = 0;
+    int err = 0;
 
     freq = xmlGetProp(n, (XUC) "frequency_short");
     if (freq == NULL) {
@@ -348,19 +456,26 @@ static int get_series_info (xmlNodePtr n, FILE *fidx, FILE *fbin)
 	free(freq);
 	return 0;
     }
-		
+
     idstr = xmlGetProp(n, (XUC) "id");
+
     if (idstr != NULL && fidx != NULL) {
-	if (series_is_duplicate((const char *) idstr, &err)) {
+	if (skip_this_series((const char *) idstr)) {
+	    skipit = 1;
+	} else if (series_is_duplicate((const char *) idstr, &err)) {
 	    if (!err) {
 		printf("duplicate series %s: skipping\n", (const char *) idstr);
 	    }
-	    free(freq);
-	    free(idstr);
-	    return err;
-	}
+	    skipit = 1;
+	}	    
     }
 
+    if (skipit) {
+	free(freq);
+	free(idstr);
+	return err;
+    }	
+
     title = xmlGetProp(n, (XUC) "title");
     units = xmlGetProp(n, (XUC) "units_short");
     adj =   xmlGetProp(n, (XUC) "seasonal_adjustment_short");
@@ -375,6 +490,7 @@ static int get_series_info (xmlNodePtr n, FILE *fidx, FILE *fbin)
 	FREDbuf *fb;
 
 	fb = fredget(FRED_OBS, 0, (const char *) idstr, fidx, &err);
+
 	if (!err) {
 	    fb->pd = pd;
 	    strncat(fb->sername, (const char *) idstr, MAXNAME - 1);
@@ -384,6 +500,7 @@ static int get_series_info (xmlNodePtr n, FILE *fidx, FILE *fbin)
 	}
 
 	if (!err) {
+	    /* finalize the index entry */
 	    char stobs[16], endobs[16], sername[32];
 
 	    lower(sername, (const char *) idstr);
@@ -461,7 +578,20 @@ static int skipcat (int id)
     } else if (id == 32251) {
 	/* highly detailed asset stats */
 	skip = 1;
-    }	
+    } else if (id == 32360 || id == 32414 || id == 32436) {
+	/* Business Lending, Bond Market Indexes, Construction */
+	skip = 1;
+    } else if (id == 32429) {
+	/* "Manufacturing", extra rel. to 2011-04-20 */
+	skip = 1;
+    } else if (id == 32406 || id == 32361 || id == 32370 ||
+	       id == 32379 || id == 32388 || id == 32397) {
+	/* more loans details */
+	skip = 1;
+    } else if (id == 32440 || id == 32439) {
+	/* delinquency rates, charge-offs */
+	skip = 1;
+    }
 
     return skip;
 }
@@ -545,7 +675,23 @@ static int fred_recurse (xmlNodePtr node, FREDbuf *fb,
 
     err = get_categories_info(node->xmlChildrenNode, fb);
 
+    /* note that some FRED categories have both sub-categories
+       under them and "loose" series (e.g. Consumer Prices)
+    */
+
     if (!err) {
+	/* first get any top-level series, if we're reading data */
+	if (fidx != NULL && fbin != NULL) {
+	    if (!skipcat(fb->catid)) {
+		fbnext = fredget(FRED_SERIES, fb->catid, NULL, fidx, &err);
+		if (!err) {
+		    err = parse_fred_xml(fbnext, fidx, fbin);
+		    FREDbuf_free(fbnext);
+		}
+	    }
+	}
+
+	/* then get any sub-categories */
 	if (fb->catlist != NULL) {
 	    int i;
 
@@ -559,14 +705,6 @@ static int fred_recurse (xmlNodePtr node, FREDbuf *fb,
 		    }
 		}
 	    }
-	} else if (fidx != NULL && fbin != NULL) {
-	    if (!skipcat(fb->catid)) {
-		fbnext = fredget(FRED_SERIES, fb->catid, NULL, fidx, &err);
-		if (!err) {
-		    err = parse_fred_xml(fbnext, fidx, fbin);
-		    FREDbuf_free(fbnext);
-		}
-	    }
 	}
     }
 
@@ -697,16 +835,16 @@ static FREDbuf *fredget (FREDtask task, int catid, const char *sername,
 	    sprintf(url, "%s/category/children?category_id=%d&api_key=%s", 
 		    FRED_SERVER, catid, API_KEY);
 	    if (fidx != NULL) {
-		printf("Finding children for category id %d...\n", catid);
+		printf("Finding children for category id %d\n", catid);
 	    }
 	} else if (task == FRED_SERIES) {
 	    sprintf(url, "%s/category/series?category_id=%d&api_key=%s", 
 		    FRED_SERVER, catid, API_KEY);
-	    printf("Finding series under category id %d...\n", catid);
+	    printf("Finding series under category id %d\n", catid);
 	} else if (task == FRED_OBS) {
 	    sprintf(url, "%s/series/observations?series_id=%s&api_key=%s", 
 		    FRED_SERVER, sername, API_KEY);
-	    printf("Getting observations for series %s...\n", sername);
+	    printf("Getting observations for series %s\n", sername);
 	}	    
 
 	curl_easy_setopt(curl, CURLOPT_URL, url);
@@ -863,15 +1001,20 @@ int main (int argc, char **argv)
     }
 
     if (!cats_only) {
+	const char *whitename = NULL;
+
 	if (db_opt == DB_INTL) {
 	    fidx = fopen("fred_intl.idx", "w");
 	    fbin = fopen("fred_intl.bin", "wb");
+	    whitename = "fred_intl.whitelist";
 	} else if (db_opt == DB_JOLTS) {
 	    fidx = fopen("jolts.idx", "w");
 	    fbin = fopen("jolts.bin", "wb");
+	    whitename = "jolts.whitelist";
 	} else {
 	    fidx = fopen("fedstl.idx", "w");
 	    fbin = fopen("fedstl.bin", "wb");
+	    whitename = "fedstl.whitelist";
 	}
 
 	if (fidx == NULL || fbin == NULL) {
@@ -884,6 +1027,8 @@ int main (int argc, char **argv)
 	} else {
 	    fputs("# St Louis Fed (various series, large)\n", fidx);
 	}
+
+	maybe_read_whitelist_file(whitename);
     }
 
     xmlKeepBlanksDefault(0);
